//--------------------------------------------------------------------------//
// Title:       hsmc_loopback.v                                             //
// Author:		San Diego High Speed Design Team   
// Rev:         Rev 0.1                                                      //
// Description: Stratix IV GX PCI Express Board NIOS BTS                    //
//              HSMC Loopback + PCIe Board to Board Loopback + TX Eye       //
//              HSMA:	x4 XCVR + x17 LVDS + x3 Parallel Loopback           //
//				HSMB:	x4 XCVR + ~x80 Parallel Loopback           			//
//              VOD, Preemphasis, and Equalization DPRIO via NIOS BTS       //
//--------------------------------------------------------------------------//
// Revision History:                                                        //
// 1: Initial															//
//------ 1 ------- 2 ------- 3 ------- 4 ------- 5 ------- 6 ------- 7 ------7
//------ 0 ------- 0 ------- 0 ------- 0 ------- 0 ------- 0 ------- 0 ------8
//Copyright  2009 Altera Corporation. All rights reserved.  Altera products
//are protected under numerous U.S. and foreign patents, maskwork rights,
//copyrights and other intellectual property laws.
//                 
//This reference design file, and your use thereof, is subject to and
//governed by the terms and conditions of the applicable Altera Reference
//Design License Agreement.  By using this reference design file, you
//indicate your acceptance of such terms and conditions between you and
//Altera Corporation.  In the event that you do not agree with such terms and
//conditions, you may not use the reference design file. Please promptly                         
//destroy any copies you have made.
//
//This reference design file being provided on an "as-is" basis and as an
//accommodation and therefore all warranties, representations or guarantees
//of any kind (whether express, implied or statutory) including, without
//limitation, warranties of merchantability, non-infringement, or fitness for
//a particular purpose, are specifically disclaimed.  By making this
//reference design file available, Altera expressly does not recommend,
//suggest or require that this reference design file be used in combination 
//with any other product not provided by Altera
//----------------------------------------------------------------------------

// Comment in for BTS GUI:
`define	enable_sopc_builder_system

//`define	ddr3_dev
//`define	ddr2_dimm
//`define	enet 
//`define	fsm


`define	hsma_xcvrs
`define	hsma_lvds
`define	hsma_parallel
// HSMB only populated for the EP2AGX260F1152 device
// Current plan is no lvds on HSMB
//`define	hsmb_xcvrs
////`define	hsmb_lvds
//`define	hsmb_parallel   
//`define	pcie_edge_basic



`ifdef	hsma_xcvrs
	`define	use_clkin_ref_q2
	`define	hsma_xcvr_refclk	 clkin_ref_q2_p
`endif

`ifdef	hsmb_xcvrs
	`define	use_clkin_ref_q2
	`define	hsmb_xcvr_refclk	 clkin_ref_q2_p
`endif

`ifdef	pcie_edge_basic
	`define	pcie_refclk	pcie_refclk_p
	`define	use_pcie_refclk	
`endif

`ifdef	hsma_lvds
	`define	lvds_refclk_a		 clkin_top_p
`endif	

`ifdef	hsmb_lvds
	`define	lvds_refclk_b		 clkin_top_p
`endif	

/* 	Arria II GX FPGA Host Development Kit

	// Pushbutton, Dip Switch, and LED Definitions
	FPGA_RST		Resets the Board Test System
	PB0				1st press:	Enable Comma Detect
					2nd press:	Enable Channel Bond
					3rd press:	Start transmitting PRBS data
					4th press:	Create an error in the transmitter data stream
	PB1				Initially LED results are for xcvrs, lvds, and parallel.
					1st press:	xcvrs only
					2nd press:	lvds only
					3rd press:	parallel only
	USER_DIP[0]		HSMA Transceivers, Source Synchronous, and Parallel Loopback
	USER_DIP[1]		HSMB Transceivers, Source Synchronous, and Parallel Loopback
					HSMB is disabled until A2GX260 silicon is available
	USER_DIP[2]		PCIe Edge Connector
	USER_DIP[3]		HSMA & HSMB Parallel Clock Phase Control:
	               	0=0 degrees  1=180 degrees (disabled for NIOS designs)
	//  LED indicators combine transceiver, source synchronous, and low speed interfaces, for each test enabled by the user_dipsw
	LED0			PLLs are locked.
	LED1			Pattern Sync Acquired (Word aligned, Channel Bonded, 1st PRBS Data Received).
	LED2			Test Complete  
	LED2			Error 
	HSMA_TX			HSMA TX PLLs Locked
	HSMA_RX			HSMA RX PLLs Locked
	HSMB_TX			HSMB TX PLLs Locked (Disabled until A2GX260 silicon is available)
	HSMB_RX			HSMB RX PLLs Locked (Disabled until A2GX260 silicon is available)
	
*/
/* Arria II GX PCIe Edge Interface Test + HSMA XCVR & LVDS & Parallel Loopback Tests:
   
		The Arria II GX PCIE Edge Interface test requires two boards:
  			
				->	An Arria II GX FPGA Development Kit Board, this is
					interface under test referred to as the "PCIE Edge Board".
			   
				->	A Stratix IV FPGA Development Kit, this board is 
					referred to as the "PCIE Host Board". 
				
			Boards Setup and Programming:
			               
				A)	Turn the power switch to OFF on the PCIE Edge Board and remove
					the AC Power Adaptor.
				B)	Turn the power switch to OFF on the PCIE Host Board and remove
					any AC Power Adaptor.
				C)	Plug the HSMC PCIE Host Adaptor Daughter Card into
					the PCIE Host Board HSMC Port A.
				D)	Plug in an HSMC Daughter Card into the PCIE Edge Boards HSMC Port A.
				E)	If necessary, put in two 1" standoffs and adjustment screws into
					the HSMC PCIE Host Adaptor Daughter Card.
				F)	Adjust the screws so that the PCIE Host Adaptor Daughter Card is level
				G)	Set jumper J5 to the "PLL BYPASS" position on the HSMC PCIE Host Adaptor Daughter Card
				H)	Set Jumper J6 away from the "X_EMI_ON" position on the HSMC PCIE Host Adaptor Daughter Card
				I)	Carefully plug in the PCIE Edge Board's PCIE Edge Connector into the 
				    HSMC PCIE Host Adaptor Daughter Card's PCIE female connector
				J)	Do not plug any power into the banana jacks on the HSMC PCIE Host Adaptor Daughter Card
				K)	Plug the AC adaptor into the PCIE Host Board
				L)	Don't provide any power to the PCIE Edge Board, power is provided by the PCIE Host Board
					through the PCIE Edge Interface
				M)	On the PCIE Host Board, set user dipswitch SW3[8:1] = [00000001]
				N)	On the PCIE Edge Board, set USER_DIP[0:3]=[1010] (0=ON on the dipswitch silk screen)
				O)	Turn the POWER switch to ON on the PCIE Host Board
				P)	Confirm the Power LEDs illuminate on both the PCIE Host Board
					and the PCIE Edge Board.  The fan on both boards should also turn on
				Q)	Plug in the USB cable from the PC into the USB connector on the PCIE Host Board.
				R)	Program the PCIE Host Board with "hsmc_loopback_host.sof".
					The source code for this SOF is located in the /pld/pcie/pcie_host_s4gx_fpga_dev_kit folder.
					
						Note, you may need to do the following:
							1)	Set the PCIE Edge Board's JTAG CHAIN jumpers to enable
								MAX_EN and disable HSMA_EN, HSMB_EN, and PCIE_EN.
								(jumper removed is enable and jumper installed is disabled on Rev A)
							2)	Set the PCIE Host Board's JTAG SW6=[ON ON ON ON].  ON silkscreen means
								disabled, ON is toward the HSMC connector and OFF is toward the PCIe
								edge connector.
							
				S)	Do not power down either board.
					Holding the PCIE Host Board stable, carefully unplug
					the USB cable from the PCIE Host Board.			
				T)	Holding the PCIE Edge Board stable, plug the USB cable into
					the USB connector on the PCIE Edge Board.
				U)	Program the PCIE Edge Board with "hsmc_loopback_edge.sof.
					
						Note, you may need to do the following:
							1)	Set the PCIE Edge Board's JTAG CHAIN jumpers to disable
								MAX_EN, HSMA_EN, HSMB_EN, and PCIE_EN.
							2)	Set the PCIE Host Board's JTAG SW6=[OFF OFF ON ON]
							
			Test Instructions:

                1)	Press and release the RESET on the PCIE Host Board FPGA_RST 
					on the PCIE Ege Board pushbuttons at the same time.
					LEDs 4-15 will display the "heartbeat" pattern on the PCIE Host Board,
					and PCIE x1, x4, and x8 LEDs will display the "heartbeat" pattern on
					the PCIE Edge Board.  This indicates both board's FPGA state machines
					are running.                          
                 2)	Press and release PB0 at the same time on both boards.
                 3)	Press and release PB0 on the PCIE Edge Board and PB1 on the PCIE Host Board
					at the same time.
                 4)	Press and release PB0 on the PCIE Edge Board and PB2 on the PCIE Host Board
					at the same time.
                 5)	Confirm LEDs 0, 1 and 2 illuminate on both boards, and LED 3 is not illuminated
					on either board
                 6)	Press both PB1 and PB2 on the PCIE Host Board, then release both to transmit an error
					to the PCIE Edge Board.
                 7)	Confirm LED 3 is illuminated on the PCIE Edge Board.  
                 8)	Press and release PB0 on the PCIE Edge Board
					to transmit an error to the PCIE Host Board.                   
                 9)	Confirm LED 3 is illuminated on the PCIE Host Board.  		  
                 10) Repeat steps 1-9 to confirm the test results.
                 
	NOTE:  If the test does not pass, try repeating steps 1-10 for
		   the 180 degree phase of the forwarded clock vs parallel data
		   by setting USER_DIP[3] = 1                 
		   
*/
	   
/* 	Arria II GX FPGA Host Development Kit
	HSMA XCVR, LVDS, and Parallel Interface Tests Only
 	1)	Plug in the HSMA loopback card.
	2)	Set USER_DIP[0:3]=[1000] (0=ON on the dipswitch silk screen)
	3)	Plug in the USB Blaster cable to the JTAG connector.
	4)	Power on the board.
	5)	Using the Quartus Programmer, program the hsmc_loopback.sof into the
	    Arria IV GX FPGA Development Kit.
	6)	Press and release the FPGA_RST pushbutton.  	
		LEDs {pcie_led_x1, pcie_led_x4,pcie_led_x8}
		will display the "heartbeat pattern" indicating the
		Arria II GX test design is running, and
		HSMA_TX & HSMA_RX should also illuminate.
		Also the HSMA_PRSTN LED illuminates
		whenever the loopback card is installed.
	7)	Press and release PB0.(rx enable comma detect)
	8)	Press and release PB0.(rx channel bond)
	9)	Press and release PB0.(tx start transmitting prbs data)
	10)	Confirm LEDS 0, 1 and 2 illuminate, and LED 3 is not illuminated.
	11)	Press and release PB0 (create tx error)
	12)	Confirm LED3 is illuminated.
    13) Repeat steps 6-12 to confirm the results
	NOTE:  If the test does not pass, try repeating steps 6-12 for
		the 180 degree phase of the forwarded clock vs parallel data
		by setting USER_DIP[3] = 1.
*/

module hsmc_loopback (
    //wired through XCVR blocks, all AC-coupled)


`ifdef	use_clkin_ref_q1_1 
      input           clkin_ref_q1_1_p     //LVDS    //default 125 MHz  //????adj. defaut 100.000 MHz osc
`endif												  //buffered copies to clkin_ref_q1_2, clkin_ref_q3, and clkin_top
`ifdef	use_clkin_ref_q1_2 
      input           clkin_ref_q1_2_p,     //LVDS    //default 125 MHz  //???? adj. defaut 125.000 MHz osc
`endif
`ifdef	use_clkin_ref_q2
      input           clkin_ref_q2_p,     //LVDS      //default 100 MHz osc // ??? default 125.000 MHz osc
													  //buffered copy to clkin_bot													  //Driven by sma ext_clk_p if clk_sel=1
`endif
`ifdef	use_clkin_ref_q3
      input           clkin_ref_q3_p,     //LVDS      //adj. default 125.000 MHz osc
`endif
`ifdef	use_refclk_155M
    input   	    clkin_155_p,	   //LVPECL    //155.520 MHz osc 
`endif
    input           clkin_bot_p,       //LVDS      //ADJ default 100.000 MHz osc or sma in (Requires external termination.)
    input           clkin_top_p,       //LVDS      //ADJ default 125.000 MHz osc (Requires external termination.)
    output          clkout_sma,        //1.8V      //PLL CLK sma out

	
////DDR3-SDRAM-PORTS  -> 64Mx16 Interface ---------------------//49 pins
`ifdef	ddr3_dev 
    output [14:0]  ddr3_a,          //SSTL15    //Address (1Gb max)
    output [2:0]   ddr3_ba,         //SSTL15    //Bank address
    inout  [15:0]  ddr3_dq,         //SSTL15    //Data
    inout  [1:0]   ddr3_dqs_p,      //SSTL15    //Strobe Pos
    inout  [1:0]   ddr3_dqs_n,      //SSTL15    //Strobe Neg
    output [1:0]   ddr3_dm,         //SSTL15    //Byte write mask
    output         ddr3_wen,        //SSTL15    //Write enable
    output         ddr3_rasn,       //SSTL15    //Row address select
    output         ddr3_casn,       //SSTL15    //Column address select
    output         ddr3_ck_p,       //SSTL15    //System Clock Pos
 //   output         ddr3_ck_n,       //SSTL15    //System Clock Neg
    output         ddr3_cke,        //SSTL15    //Clock Enable
    output         ddr3_csn,        //SSTL15    //Chip Select
    output         ddr3_resetn,     //SSTL15    //Reset
    output         ddr3_odt,        //SSTL15    //On-die termination enable
 `else
    output         ddr3_wen,        //SSTL15    //Write enable
    output         ddr3_cke,        //SSTL15    //Clock Enable
    output         ddr3_csn,        //SSTL15    //Chip Select
    output         ddr3_resetn,     //SSTL15    //Reset
    output         ddr3_odt,        //SSTL15    //On-die termination enable
 `endif
 //DDR2 SDRAM SoDIMM -------------------------------------//x64 -> 117 pins (Default)
	//x64 -> 125 pins
`ifdef	ddr2_dimm
    output [15:0]  ddr2_dimm_a,	    //SSTL18    //Address		OK
    output [2:0]   ddr2_dimm_ba,    //SSTL18    //Bank address  OK
    inout  [63:0]  ddr2_dimm_dq,         //SSTL18    //Data x64 SODIMM		OK
    inout  [7:0]   ddr2_dimm_dqs_p,      //SSTL18    //Strobe Pos			OK
    inout  [7:0]   ddr2_dimm_dqs_n,      //SSTL18    //Strobe Neg			OK
    output [7:0]   ddr2_dimm_dm,         //SSTL18    //Byte write mask  OK
    output [0:0]   ddr2_dimm_cke,    //SSTL18   //System Clock Enable  OK
    output [1:0]   ddr2_dimm_ck_p,   //SSTL18   //System Clock Pos     OK
 //   output [1:0]   ddr2_dimm_ck_n,   //SSTL18    //System Clock Neg		OK
    output         ddr2_dimm_wen,         //SSTL18    //Write enable		OK
    output         ddr2_dimm_rasn,       //SSTL18    //Row address select		OK
    output         ddr2_dimm_casn,       //SSTL18    //Column address select  OK
   output	[1:0]  ddr2_dimm_csn,        //SSTL18    //Chip Select           OK
    output         ddr2_dimm_resetn,     //SSTL18    //Reset
    output  [1:0]  ddr2_dimm_odt,        //SSTL18    //On-die termination enable	OK
`else
   output          ddr2_dimm_wen,         //SSTL18    //Write enable		OK
   output   [0:0]  ddr2_dimm_cke,    //SSTL18   //System Clock Enable  OK
   output	[1:0]  ddr2_dimm_csn,        //SSTL18    //Chip Select           OK
   output          ddr2_dimm_resetn,     //SSTL18    //Reset
   output   [1:0]  ddr2_dimm_odt,        //SSTL18    //On-die termination enable	OK
`endif
////////////////////////////////////////////////////////////////// 
`ifdef enet
//ETHERNET-10/100/1000-RGMII-----------
    output  	   enet_gtx_clk,      //2.5V  //RGMII Transmit Clock
    output [3:0]   enet_tx_d,        //2.5V  //TX to PHY
    input  [3:0]   enet_rx_d,        //2.5V  //RX from PHY
    output         enet_tx_en,       //2.5V  //RGMII Transmit Control
    input	       enet_rx_clk,      //2.5V  //Derived Received Clock
    input          enet_rx_dv,       //2.5V  //RGMII Receive Control 
    output         enet_resetn,        //2.5V      //Reset to PHY (TR=0)
    output         enet_mdc,           //2.5V      //MDIO Control (TR=0)
    inout          enet_mdio,          //2.5V      //MDIO Data (TR=0)
    input          enet_intn,           //2.5V      //MDIO Interrupt (TR=0)
`endif
///////////////////////////////////////////////////////////////////

//FLASH-SRAM-MAX-------------FSM-Bus---//90 pins
//`ifdef fsm
    output [25:0]  fsm_a,              //2.5V      //FSM Address Bus (1Gb Flash)
    inout  [31:0]  fsm_d,              //2.5V      //FSM Data Bus
//    output         flash_clk,          //2.5V  
//    output         flash_cen,          //2.5V  
//    output         flash_oen,          //2.5V
//    output         flash_wen,          //2.5V
//    output         flash_advn,         //2.5V
//    input          flash_rdybsyn,      //2.5V
//    output         flash_resetn,       //2.5V     // (TR=0)
//    output         sram_clk,           //2.5V
//    output         sram_cen,           //2.5V
//    inout  [3:0]   sram_dqp,           //2.5V     //Parity bits only go to SRAM
//    output [3:0]   sram_bwn,           //2.5V
//    output         sram_gwn,           //2.5V
//    output         sram_bwen,          //2.5V
//    output         sram_oen,           //2.5V
//    output         sram_advn,          //2.5V
//    output         sram_adspn,         //2.5V
//    output         sram_adscn,         //2.5V
//    output         sram_zz,            //2.5V     // (TR=0)
    output         max2_clk,           //1.8V
    output         max2_csn,           //1.8V
    output [3:0]   max2_ben,           //1.8V
    output         max2_oen,           //1.8V
    output         max2_wen,           //1.8V
//`endif
////LCD----------------------------------//11 pins
    inout  [7:0]   lcd_data,           //2.5V
    output         lcd_d_cn,           //2.5V
    output         lcd_wen,            //2.5V
    output         lcd_csn,            //2.5V
//
////User-IO------------------------------//22 pins
    input  [3:0]   user_dipsw,         //1.8V/2.5V     // (TR=0)
//    output [7:0]   user_led,           //2.5V
    output [3:0]   user_led,           //2.5V
    input  [1:0]   user_pb,            //1.8V/2.5V     // (TR=0)
//    input  [1:0]   user_pb,            //1.8V/2.5V     // (TR=0)
    input          cpu_resetn,         //2.5V (DEV_CLRn)    // (TR=0)
  
// //PCI-EXPRESS-EDGE---------------------
`ifdef	use_pcie_refclk
    input          pcie_refclk_p,      //HCSL
`endif
`ifdef	pcie_edge_basic
    output [7:0]   pcie_tx_p,          //1.4V PCML
    input  [7:0]   pcie_rx_p,          //1.4V PCML
`endif
    input          pcie_smbclk,        //2.5V     // (TR=0)
    inout          pcie_smbdat,        //2.5V     // (TR=0)
    input          pcie_perstn,        //2.5V     // (TR=0)
    output         pcie_waken,         //2.5V     // (TR=0)
    output         pcie_led_x1,        //2.5V
    output         pcie_led_x4,        //2.5V
    output         pcie_led_x8,        //2.5V
//    output         pcie_led_g2,        //2.5V
//HIGH-SPEED-MEZZANINE-CARD------------//198 pins (HSMB is only connected on EP2AGX260 devices)
    //Port A -->   single samtec conn  //107 pins  //------------------
`ifdef	hsma_xcvrs
    output [3:0]   hsma_tx_p,    	   //1.4V PCML
    input  [3:0]   hsma_rx_p,    	   //1.4V PCML
`endif
      //Enable below for CMOS HSMC     
      //inout  [79:0]  hsma_d,           //2.5V
      //Enable below for LVDS HSMC
`ifdef	hsma_lvds
    output [16:0]  hsma_tx_d_p,        //LVDS  //69 pins
    input  [16:0]  hsma_rx_d_p,        //LVDS
    input          hsma_clk_in_p1,     //LVDS //Requires external termination  
    output         hsma_clk_out_p1,    //LVDS
    input          hsma_clk_in_p2,     //LVDS //Requires external termination
    output         hsma_clk_out_p2,    //LVDS    
`endif
    //inout  [3:0]   hsma_d,             //2.5V
    input		   hsma_d_0,
    input		   hsma_d_2,
    output		   hsma_d_1,
    output		   hsma_d_3,
    input          hsma_clk_in0,       //2.5V
    output reg     hsma_clk_out0,      //2.5V

    //inout          hsma_sda,           //2.5V     // (TR=0)
    input		   hsma_sda,
    output         hsma_scl,           //2.5V     // (TR=0)
    output         hsma_tx_led,        //2.5V
    output         hsma_rx_led,        //2.5V
    input          hsma_prsntn ,       //2.5V     // (TR=0)
//    //Port B -->   single samtec conn  //107 pins  //------------------
`ifdef	hsmb_xcvrs //xcvrs & lvds not supported on HSMB
	output [3:0]   hsmb_tx_p,    	 //1.4V PCML   
	input  [3:0]   hsmb_rx_p,    	 //1.4V PCML   
`endif

//      //Enable below for CMOS HSMC     
//      //inout  [79:0]  hsmb_d,           //2.5V
//      //Enable below for LVDS HSMC  
`ifdef	hsmb_lvds
      output [16:0]  hsmb_tx_d_p,        //LVDS   
      input  [16:0]  hsmb_rx_d_p,        //LVDS   
`endif
//    inout  [75:0]   hsmb_d,             //2.5V
    input          hsmb_clk_in0,       //2.5V   
    output   reg      hsmb_clk_out0       //2.5V   
//    output         hsmb_clk_out_p2,    //LVDS   
//    inout          hsmb_sda,           //2.5V     // (TR=0)   
//    output         hsmb_scl,           //2.5V     // (TR=0)   
//    output         hsmb_tx_led,        //2.5V                 
//    output         hsmb_rx_led,        //2.5V                 
//    input          hsmb_prsntn         //2.5V     // (TR=0)  
);  

	wire	hsma_xcvr_reset_n		=	reset_n && xcvr_ctrl 		&& user_dipsw[0];
	wire	hsma_ss_reset_n			=	reset_n && ss_ctrl			&& user_dipsw[0];
	wire	hsma_parallel_reset_n	=	reset_n && parallel_ctrl	&& user_dipsw[0];
	wire	hsmb_xcvr_reset_n		=	reset_n && xcvr_ctrl		&& user_dipsw[1];
	wire	hsmb_ss_reset_n			=	reset_n && ss_ctrl			&& user_dipsw[1];
	wire	hsmb_parallel_reset_n	=	reset_n && parallel_ctrl	&& user_dipsw[1];
	wire	pcie_edge_reset_n		=	reset_n && user_dipsw[2];

			
	wire	tests_disabled_n	=	(									//Force the LED off if no tests are active:	
										hsma_xcvr_reset_n 		|
										hsma_ss_reset_n 		|
										hsma_parallel_reset_n 	|  
										hsmb_xcvr_reset_n 		|
										hsmb_ss_reset_n 		|
										hsmb_parallel_reset_n   |
										pcie_edge_reset_n
									);
									
	wire	hsma_tx_rx_led_disable_n	=	hsma_ss_reset_n | hsma_xcvr_reset_n;  
	wire	hsmb_tx_rx_led_disable_n	=	hsmb_ss_reset_n | hsmb_xcvr_reset_n;
		
								
	assign	user_led[0]	=	(!(
								(hsma_xcvr_plls_locked  				|| (!hsma_xcvr_reset_n)) 		&&
								(hsma_ss_plls_locked  					|| (!hsma_ss_reset_n)) 			&&
								(hsma_parallel_plls_locked 				|| (!hsma_parallel_reset_n))	&& 
								(hsmb_xcvr_plls_locked  				|| (!hsmb_xcvr_reset_n)) 		&&
								(hsmb_ss_plls_locked  					|| (!hsmb_ss_reset_n)) 			&&
								(hsmb_parallel_plls_locked 				|| (!hsmb_parallel_reset_n))		&&
								(pcie_edge_plls_locked 					|| (!pcie_edge_reset_n))		
							)) || (!tests_disabled_n);		 
							                                  
	assign	user_led[1]	=	(!(
								(hsma_xcvr_pattern_sync_acquired  		|| (!hsma_xcvr_reset_n))   		&&
								(hsma_ss_pattern_sync_acquired  		|| (!hsma_ss_reset_n))   		&&
								(hsma_parallel_pattern_sync_acquired	|| (!hsma_parallel_reset_n))	&&
								(hsmb_xcvr_pattern_sync_acquired  		|| (!hsmb_xcvr_reset_n))   		&&
								(hsmb_ss_pattern_sync_acquired  		|| (!hsmb_ss_reset_n))   		&&
								(hsmb_parallel_pattern_sync_acquired	|| (!hsmb_parallel_reset_n))	&&
								(pcie_edge_pattern_sync_acquired		|| (!pcie_edge_reset_n))
							)) || (!tests_disabled_n) ;	
									                                    				
	assign	user_led[2]	=	(!(
								(hsma_xcvr_test_complete  				|| (!hsma_xcvr_reset_n))   		&&
								(hsma_ss_test_complete  				|| (!hsma_ss_reset_n))   		&&
								(hsma_parallel_test_complete			|| (!hsma_parallel_reset_n))	&&
								(hsmb_xcvr_test_complete  				|| (!hsmb_xcvr_reset_n))   		&&
								(hsmb_ss_test_complete  				|| (!hsmb_ss_reset_n))   		&&
								(hsmb_parallel_test_complete			|| (!hsmb_parallel_reset_n))	&&
								(pcie_edge_test_complete				|| (!pcie_edge_reset_n))		
							)) || (!tests_disabled_n) ;		                                    

							
	assign	user_led[3]	=	(!(
								(hsma_xcvr_error_latch 					&& hsma_xcvr_reset_n) 			||
								(hsma_ss_error_latch 					&& hsma_ss_reset_n) 			||
								(hsma_parallel_error_latch				&& hsma_parallel_reset_n)		||
								(hsmb_xcvr_error_latch 					&& hsmb_xcvr_reset_n) 			||
								(hsmb_ss_error_latch 					&& hsmb_ss_reset_n) 			||
								(hsmb_parallel_error_latch				&& hsmb_parallel_reset_n)		||
								(pcie_edge_error_latch 					&& pcie_edge_reset_n)		
							)) || (!tests_disabled_n) ;	

 
							
	assign	  hsma_tx_led	=	(
									!(hsma_ss_led_tx & hsma_xcvr_led_tx) 
								)	|| (!hsma_tx_rx_led_disable_n);   
	assign	  hsma_rx_led	=	(
									!(hsma_ss_led_rx & hsma_xcvr_led_rx)
								)	|| (!hsma_tx_rx_led_disable_n);    
	 
	assign	  hsmb_tx_led	=	(
									!(hsmb_ss_led_tx & hsmb_xcvr_led_tx)
								)	|| (!hsmb_tx_rx_led_disable_n);   
	assign	  hsmb_rx_led	=	(
									!(hsmb_ss_led_rx & hsmb_xcvr_led_rx)
								)	|| (!hsmb_tx_rx_led_disable_n);    
								
	assign	clkin_parallel		=	clkin_top_p;                                
	assign	cal_blk_clk			=	lf_core_clk;
	assign	nios_clk			=	lf_core_clk;  
	assign	nios_reset_n		=	reset_n;

	assign	clk_parallel_0deg	=	clkin_parallel;
	assign	clk_parallel_180deg	=	!clkin_parallel;
	assign	pll_locked_parallel	=	1;

	reg			lf_core_clk;     
	reg	[1:0]	lf_core_clk_ctr;
	always	@(posedge clkin_parallel)
	begin
		lf_core_clk_ctr	<=	lf_core_clk_ctr + 1;
		case	(lf_core_clk_ctr)
			0,2:		lf_core_clk	<=	!lf_core_clk;
			default:	;
		endcase	
	end
	
     
			
	// Metastabilizer for asynchronous reset signal     
	// 1st parameter is the preset value
	// 2nd parameter is the debounce period
	parameter	debounce_prd			=	200000;
	switch_debouncer	#(0,debounce_prd)	switch_debouncer_reset_n
	(
		.reset_n	(1'b1),
		.clk		(!clkin_parallel),
		.data_in	(cpu_resetn),
		.data_out	(reset_n)
	);

	wire	[3:0]	user_led_w;	
	defparam		heartbeat_x4_i.heartbeat_mode	=	1;	// Selects LED pattern
	heartbeat_x4	heartbeat_x4_i
	(
		.reset_n		(reset_n),
		.clk			(clkin_parallel),
		.led_pattern	(user_led_w[3:0])
	);
   
//    assign	{pcie_led_x1,pcie_led_x4,pcie_led_x8} = user_led_w[2:0];
    assign	{pcie_led_x4,pcie_led_x8} = user_led_w[1:0];
	
	// user_pb[0] will drive 1) en_cdet, channel_bond, start, then create_error:
	switch_debouncer	#(0,debounce_prd)	switch_debouncer_pb0
	(
		.reset_n(reset_n),
		.clk(clkin_bot_p),
		.data_in(!user_pb[0]),
		.data_out(pb_ctrl)
	);

	reg			en_cdet_ctrl;
	reg			channel_bond_ctrl;
	reg			create_error_ctrl;
	reg			start_ctrl;
	reg	[3:0]	pb_state;
	always	@(negedge pb_ctrl or negedge reset_n)
	begin
		if	(!reset_n)
		begin
			en_cdet_ctrl		=	1;
			channel_bond_ctrl	=	0;
			create_error_ctrl	=	0;
			start_ctrl			=	0;
			pb_state			=	0;
		end else begin
			case	(pb_state)
				0:			begin
								pb_state			<=	1;
								en_cdet_ctrl		<=	0;
								channel_bond_ctrl	<=	1;
							end
				1:			begin
								pb_state			<=	2;
								channel_bond_ctrl	<=	0;
								start_ctrl			<=	1;
							end
				2:			begin
								pb_state			<=	3;
								start_ctrl			<=	0;
								create_error_ctrl	<=	1;
							end
				default:	begin
								create_error_ctrl	<=	0;
							end
			endcase
		end
	end
		
	assign	en_cdet			=	en_cdet_ctrl		&	pb_ctrl;
	assign	channel_bond	=	channel_bond_ctrl	&	pb_ctrl;
	assign	start			=	start_ctrl			&	pb_ctrl;
	assign	create_error	=	create_error_ctrl	&	pb_ctrl;

// user_pb[0] will drive 1) en_cdet, channel_bond, start, then create_error:
	switch_debouncer	#(0,debounce_prd)	switch_debouncer_comb
	(
		.reset_n(reset_n),
		.clk(clkin_bot_p),
		.data_in(!user_pb[1]),
		.data_out(comb_ctrl)
	);

	// reset to report xcvrs, ss lvds, and parallel simultaneously
	// press pb1 to test xcvrs only, press again for lvds only, press again for parallel only  


	
	reg			xcvr_ctrl;
	reg			ss_ctrl;
	reg			parallel_ctrl;
	reg	[3:0]	comb_state;
	always	@(posedge comb_ctrl or negedge reset_n)
	begin
		if	(!reset_n)
		begin
			comb_state		<=	0;
			xcvr_ctrl		<=	1;
			ss_ctrl			<=	1;
			parallel_ctrl	<=	1;
		end else begin
			case	(comb_state)
				0:			begin
								comb_state		<=	1;
								xcvr_ctrl		<=	1;
								ss_ctrl			<=	0;
								parallel_ctrl	<=	0;
							end
				1:			begin
								comb_state		<=	2;
								xcvr_ctrl		<=	0;
								ss_ctrl			<=	1;
								parallel_ctrl	<=	0;
							end
				default:	begin
								xcvr_ctrl		<=	0;
								ss_ctrl			<=	0;
								parallel_ctrl	<=	1;
							end
			endcase
		end
	end
 
`ifdef	enable_sopc_builder_system

	always	@(clk_parallel_0deg)
	begin
		hsma_clk_out0	=	clk_parallel_0deg;
	end

	always	@(clk_parallel_0deg)
	begin
		hsmb_clk_out0	=	clk_parallel_0deg;
	end


	nios_bts nios_bts_inst
    	(
    	  .avalon_dip_to_the_nios_bts_port_lvds_x17_1G_hsma_0                       (user_dipsw									),
    	  .avalon_dip_to_the_nios_bts_port_parallel_loopback_x3_hsma_0              (user_dipsw                             	),
    	  .avalon_dip_to_the_nios_bts_port_xcvr_x4_hsma_0                           (user_dipsw                             	),
    	  .avalon_led_from_the_nios_bts_port_lvds_x17_1G_hsma_0                     (                                       	),
    	  .avalon_led_from_the_nios_bts_port_parallel_loopback_x3_hsma_0            (                                       	),
    	  .avalon_led_from_the_nios_bts_port_xcvr_x4_hsma_0                         (                                       	),
    	  .avalon_pb_to_the_nios_bts_port_lvds_x17_1G_hsma_0                        ({hsma_prsntn,hsmb_prsntn,3'b000,user_pb}	),
    	  .avalon_pb_to_the_nios_bts_port_parallel_loopback_x3_hsma_0               ({hsma_prsntn,hsmb_prsntn,3'b000,user_pb}	),
    	  .avalon_pb_to_the_nios_bts_port_xcvr_x4_hsma_0                            ({hsma_prsntn,hsmb_prsntn,3'b000,user_pb}	),
    	  .cal_blk_clk_sig_to_the_nios_bts_port_lvds_x17_1G_hsma_0                  (cal_blk_clk                            	),
    	  .cal_blk_clk_sig_to_the_nios_bts_port_parallel_loopback_x3_hsma_0         (cal_blk_clk                            	),
    	  .cal_blk_clk_sig_to_the_nios_bts_port_xcvr_x4_hsma_0                      (cal_blk_clk                            	),
    	  .channel_bond_pb_to_the_nios_bts_port_lvds_x17_1G_hsma_0                  (channel_bond                           	),
    	  .channel_bond_pb_to_the_nios_bts_port_parallel_loopback_x3_hsma_0         (channel_bond                           	),
    	  .channel_bond_pb_to_the_nios_bts_port_xcvr_x4_hsma_0                      (channel_bond                           	),
    	  .clk_0                                                                    (nios_clk                               	),
    	  .create_error_pb_to_the_nios_bts_port_lvds_x17_1G_hsma_0                  (create_error                           	),
    	  .create_error_pb_to_the_nios_bts_port_parallel_loopback_x3_hsma_0         (create_error                           	),
    	  .create_error_pb_to_the_nios_bts_port_xcvr_x4_hsma_0                      (create_error                           	),
    	  .en_cdet_pb_to_the_nios_bts_port_lvds_x17_1G_hsma_0                       (en_cdet                                	),
    	  .en_cdet_pb_to_the_nios_bts_port_parallel_loopback_x3_hsma_0              (en_cdet                                	),
    	  .en_cdet_pb_to_the_nios_bts_port_xcvr_x4_hsma_0                           (en_cdet                                	),
    	  .error_latch_from_the_nios_bts_port_lvds_x17_1G_hsma_0                    (hsma_ss_error_latch                    	),
    	  .error_latch_from_the_nios_bts_port_parallel_loopback_x3_hsma_0           (hsma_parallel_error_latch              	),
    	  .error_latch_from_the_nios_bts_port_xcvr_x4_hsma_0                        (hsma_xcvr_error_latch                  	),
    	  .led_rx_from_the_nios_bts_port_lvds_x17_1G_hsma_0                         (hsma_ss_led_rx                         	),
    	  .led_rx_from_the_nios_bts_port_parallel_loopback_x3_hsma_0                (                                       	),
    	  .led_rx_from_the_nios_bts_port_xcvr_x4_hsma_0                             (hsma_xcvr_led_rx                       	),
    	  .led_tx_from_the_nios_bts_port_lvds_x17_1G_hsma_0                         (hsma_ss_led_tx                         	),
    	  .led_tx_from_the_nios_bts_port_parallel_loopback_x3_hsma_0                (                                       	),
    	  .led_tx_from_the_nios_bts_port_xcvr_x4_hsma_0                             (hsma_xcvr_led_tx                       	),
    	  .pattern_sync_acquired_from_the_nios_bts_port_lvds_x17_1G_hsma_0          (hsma_ss_pattern_sync_acquired          	),
    	  .pattern_sync_acquired_from_the_nios_bts_port_parallel_loopback_x3_hsma_0 (hsma_parallel_pattern_sync_acquired    	),
    	  .pattern_sync_acquired_from_the_nios_bts_port_xcvr_x4_hsma_0              (hsma_xcvr_pattern_sync_acquired        	),
    	  .plls_in_locked_to_the_nios_bts_port_lvds_x17_1G_hsma_0                   (1'b1                                   	),
    	  .plls_in_locked_to_the_nios_bts_port_parallel_loopback_x3_hsma_0          (pll_locked_parallel                    	),
    	  .plls_in_locked_to_the_nios_bts_port_xcvr_x4_hsma_0                       (1'b1                                   	),
    	  .plls_locked_from_the_nios_bts_port_lvds_x17_1G_hsma_0                    (hsma_ss_plls_locked                    	),
    	  .plls_locked_from_the_nios_bts_port_parallel_loopback_x3_hsma_0           (hsma_parallel_plls_locked             	 	),
    	  .plls_locked_from_the_nios_bts_port_xcvr_x4_hsma_0                        (hsma_xcvr_plls_locked                  	),
    	  .refclk_p_to_the_nios_bts_port_lvds_x17_1G_hsma_0                         (`lvds_refclk_a                         	),
    	  .refclk_p_to_the_nios_bts_port_parallel_loopback_x3_hsma_0                (clk_parallel_0deg                      	),
    	  .refclk_p_to_the_nios_bts_port_xcvr_x4_hsma_0                             (`hsma_xcvr_refclk                      	),
    	  .reset_n                                                                  (nios_reset_n                           	),
    	  .reset_n_to_the_nios_bts_port_lvds_x17_1G_hsma_0                          (reset_n                                	),
    	  .reset_n_to_the_nios_bts_port_parallel_loopback_x3_hsma_0                 (reset_n                                	),
    	  .reset_n_to_the_nios_bts_port_xcvr_x4_hsma_0                              (reset_n                                	),
    	  .rx_p_to_the_nios_bts_port_lvds_x17_1G_hsma_0                             (hsma_rx_d_p                            	),
    	  .rx_p_to_the_nios_bts_port_parallel_loopback_x3_hsma_0                    ({hsma_sda,hsma_d_2,hsma_d_0}           	),
    	  .rx_p_to_the_nios_bts_port_xcvr_x4_hsma_0                                 (hsma_rx_p                              	),
    	  .rx_ss_clk_in_to_the_nios_bts_port_lvds_x17_1G_hsma_0                     (`lvds_refclk_a                         	),
    	  .rx_ss_clk_in_to_the_nios_bts_port_parallel_loopback_x3_hsma_0            (hsma_clk_in0                           	),
    	  .rx_ss_clk_in_to_the_nios_bts_port_xcvr_x4_hsma_0                         (0                                      	),
    	  .seriallpbken_dip_to_the_nios_bts_port_lvds_x17_1G_hsma_0                 (0                                      	),
    	  .seriallpbken_dip_to_the_nios_bts_port_parallel_loopback_x3_hsma_0        (0                                     	 	),
    	  .seriallpbken_dip_to_the_nios_bts_port_xcvr_x4_hsma_0                     (0                                     	 	),
    	  .start_pb_to_the_nios_bts_port_lvds_x17_1G_hsma_0                         (start                                  	),
    	  .start_pb_to_the_nios_bts_port_parallel_loopback_x3_hsma_0                (start                                  	),
    	  .start_pb_to_the_nios_bts_port_xcvr_x4_hsma_0                             (start                                  	),
    	  .test_complete_from_the_nios_bts_port_lvds_x17_1G_hsma_0                  (hsma_ss_test_complete                  	),
    	  .test_complete_from_the_nios_bts_port_parallel_loopback_x3_hsma_0         (hsma_parallel_test_complete            	),
    	  .test_complete_from_the_nios_bts_port_xcvr_x4_hsma_0                      (hsma_xcvr_test_complete                	),
    	  .tx_p_from_the_nios_bts_port_lvds_x17_1G_hsma_0                           (hsma_tx_d_p                            	),
    	  .tx_p_from_the_nios_bts_port_parallel_loopback_x3_hsma_0                  ({hsma_scl,hsma_d_3,hsma_d_1}           	),
    	  .tx_p_from_the_nios_bts_port_xcvr_x4_hsma_0                               (hsma_tx_p                              	),
    	  .tx_ss_clk_out_from_the_nios_bts_port_lvds_x17_1G_hsma_0                  (hsma_clk_out_p2                        	),
    	  .tx_ss_clk_out_from_the_nios_bts_port_parallel_loopback_x3_hsma_0         (           								),	// hsma_clk_out0
    	  .tx_ss_clk_out_from_the_nios_bts_port_xcvr_x4_hsma_0                      (                                       	),
    	  .xcvr_reset_pb_n_to_the_nios_bts_port_lvds_x17_1G_hsma_0                  (reset_n                                	),
    	  .xcvr_reset_pb_n_to_the_nios_bts_port_parallel_loopback_x3_hsma_0         (reset_n                                	),
    	  .xcvr_reset_pb_n_to_the_nios_bts_port_xcvr_x4_hsma_0                      (reset_n                                	),
    	  
      .adsc_n_to_the_max2_inf                                                   (),
      .bw_n_to_the_max2_inf                                                     (max2_ben),
      .bwe_n_to_the_max2_inf                                                    (max2_wen),
      .chipenable1_n_to_the_max2_inf                                            (max2_csn),
      .outputenable_n_to_the_max2_inf                                           (max2_oen),
      .tri_state_bridge_0_address                                               (fsm_a),
      .tri_state_bridge_0_data                                                  (fsm_d),

      .led_from_the_heartbeat_0                                                 (pcie_led_x1)
    	  
    	);     

		assign max2_clk = nios_clk;

    	
    	// Make compatible in case some or all of these interfaces are
    	// available on future boards for larger Arria II GX devices
    	assign	hsmb_ss_plls_locked               		=	1'b1;
		assign	hsmb_ss_pattern_sync_acquired     		=	1'b1;
		assign	hsmb_ss_test_complete             		=	1'b1;
		assign	hsmb_ss_error_latch               		=	1'b0;	
		assign	hsmb_ss_led_rx							=	1'b1;
		assign	hsmb_ss_led_tx							=	1'b1;   
	    assign	hsmb_xcvr_plls_locked               	=	1'b1;
		assign	hsmb_xcvr_pattern_sync_acquired     	=	1'b1;
		assign	hsmb_xcvr_test_complete             	=	1'b1;
		assign	hsmb_xcvr_error_latch               	=	1'b0;	
		assign	hsmb_xcvr_led_rx						=	1'b1;
		assign	hsmb_xcvr_led_tx						=	1'b1;	
		assign	hsmb_parallel_plls_locked               =	1'b1;
		assign	hsmb_parallel_pattern_sync_acquired     =	1'b1;
		assign	hsmb_parallel_test_complete             =	1'b1;
		assign	hsmb_parallel_error_latch               =	1'b0;		
	    assign	pcie_edge_plls_locked              		=	1'b1;
		assign	pcie_edge_pattern_sync_acquired    		=	1'b1;
		assign	pcie_edge_test_complete            		=	1'b1;
		assign	pcie_edge_error_latch              		=	1'b0;					
		                                                                                                          

`else 

	// Clock Mux to control phase of HSMA Parallel Loopback forwarded clock
	always	@(*)
	begin
		case	(user_dipsw[3])
			0:			hsma_clk_out0	=	clk_parallel_0deg;
			1:			hsma_clk_out0	=	clk_parallel_180deg;
			default;
		endcase
	end
	
	// Clock Mux to control phase of HSMB Parallel Loopback forwarded clock
	always	@(*)
	begin
		case	(user_dipsw[3])
			0:			hsmb_clk_out0	=	clk_parallel_0deg;
			1:			hsmb_clk_out0	=	clk_parallel_180deg;
			default;
		endcase
	end     



  	       
	`ifdef	hsma_lvds
		nios_bts_port_lvds_x17_1G_hsma nios_bts_port_lvds_x17_1G_hsma_i
		(
		    .reset_n                     (reset_n && hsma_ss_reset_n		),
			.en_cdet_pb                  (en_cdet							),
			.channel_bond_pb             (channel_bond						),
			.start_pb                    (start								),
			.create_error_pb             (create_error						),
			.xcvr_reset_pb_n             (hsma_ss_reset_n					),
		    .refclk_p                    (`lvds_refclk_a					),
		    .tx_p                        (hsma_tx_d_p						),
		    .rx_p                        (hsma_rx_d_p						),
		 	.cal_blk_clk_sig             (cal_blk_clk						),
			.seriallpbken_dip            (0									),                                                                              
		    .led_tx                      (hsma_ss_led_tx					),
		    .led_rx                      (hsma_ss_led_rx     				),
			//.rx_ss_clk_in              (hsma_clk_in_p2					),
			.rx_ss_clk_in                (`lvds_refclk_a					),
		    .tx_ss_clk_out               (hsma_clk_out_p2					),
			.plls_in_locked              (1'b1								),
			.plls_locked                 (hsma_ss_plls_locked               ),
			.pattern_sync_acquired       (hsma_ss_pattern_sync_acquired     ),
			.test_complete               (hsma_ss_test_complete             ),
			.error_latch                 (hsma_ss_error_latch               ),
			.error_irq                   (                                  ),
			.nios_clk                    (0                                 ),
			.nios_address                (0                                 ),
		  	.nios_chipselect             (0                                 ),
		  	.nios_read                   (0                                 ),
		  	.nios_readdata               (                                  ),
		  	.nios_reset_n                (reset_n                           ),
		  	.nios_write                  (0                                 ),
		 	.nios_writedata              (0                                 ),
			.nios_waitrequest            (                                  ),
			.avalon_pb                   (user_pb                           ),
			.avalon_dip                  (user_dipsw                        ),
			.avalon_led                  (                                  )
		); 
	`else
		assign	hsma_ss_plls_locked               		=	1'b1;
		assign	hsma_ss_pattern_sync_acquired     		=	1'b1;
		assign	hsma_ss_test_complete             		=	1'b1;
		assign	hsma_ss_error_latch               		=	1'b0;
		assign	hsma_ss_led_rx							=	1'b1;
		assign	hsma_ss_led_tx							=	1'b1;			
	`endif
	
	// Current plan is for no lvds on hsmb - confirm and delete:
	`ifdef	hsmb_lvds
		nios_bts_port_lvds_x17_1G_hsmb nios_bts_port_lvds_x17_1G_hsmb_i
		(
		    .reset_n                     (reset_n && hsmb_ss_reset_n		),
			.en_cdet_pb                  (en_cdet							),
			.channel_bond_pb             (channel_bond						),
			.start_pb                    (start								),
			.create_error_pb             (create_error						),
			.xcvr_reset_pb_n             (hsmb_ss_reset_n					),
		    .refclk_p                    (`lvds_refclk_b					),
		    .tx_p                        (hsmb_tx_d_p						),
		    .rx_p                        (hsmb_rx_d_p						),
		 	.cal_blk_clk_sig             (cal_blk_clk						),
			.seriallpbken_dip            (0									),
		    .led_tx                      (hsmb_ss_led_tx					),
		    .led_rx                      (hsmb_ss_led_rx     				),
			.rx_ss_clk_in                (`lvds_refclk_b					),
		    .tx_ss_clk_out               (hsmb_clk_out_p2					),
			.plls_in_locked              (1'b1								),
			.plls_locked                 (hsmb_ss_plls_locked               ),
			.pattern_sync_acquired       (hsmb_ss_pattern_sync_acquired     ),
			.test_complete               (hsmb_ss_test_complete             ),
			.error_latch                 (hsmb_ss_error_latch               ),
			.error_irq                   (                                  ),
			.nios_clk                    (0                                 ),
			.nios_address                (0                                 ),   
		  	.nios_chipselect             (0                                 ),
		  	.nios_read                   (0                                 ),
		  	.nios_readdata               (                                  ),
		  	.nios_reset_n                (reset_n                           ),
		  	.nios_write                  (0                                 ),
		 	.nios_writedata              (0                                 ),
			.nios_waitrequest            (                                  ),
			.avalon_pb                   (user_pb                           ),
			.avalon_dip                  (user_dipsw                        ),
			.avalon_led                  (				                    )
		); 
	`else
	    assign	hsmb_ss_plls_locked               		=	1'b1;
		assign	hsmb_ss_pattern_sync_acquired     		=	1'b1;
		assign	hsmb_ss_test_complete             		=	1'b1;
		assign	hsmb_ss_error_latch               		=	1'b0;	
		assign	hsmb_ss_led_rx							=	1'b1;
		assign	hsmb_ss_led_tx							=	1'b1;			
	`endif

	`ifdef	hsma_xcvrs
		nios_bts_port_xcvr_x4_hsma	nios_bts_port_xcvr_x4_hsma_i
		(
    		.reset_n                           	(reset_n && hsma_xcvr_reset_n       ),
			.en_cdet_pb                        	(en_cdet                            ),
			.channel_bond_pb                   	(channel_bond                       ),
			.start_pb                          	(start                              ),
			.create_error_pb           			(create_error                       ),
			.xcvr_reset_pb_n	               	(hsma_xcvr_reset_n                  ),
    		.refclk_p                          	(`hsma_xcvr_refclk                  ),
    		.tx_p    	                       	(hsma_tx_p                          ),
    		.rx_p                              	(hsma_rx_p                          ),
 			.cal_blk_clk_sig                   	(cal_blk_clk                        ),
			.seriallpbken_dip                  	(0                                  ),
    		.led_tx                            	(hsma_xcvr_led_tx                   ),
    		.led_rx                            	(hsma_xcvr_led_rx                   ),
			.rx_ss_clk_in			           	(0                                  ),
    		.tx_ss_clk_out       	           	(                                   ),
			.plls_in_locked			           	(1'b1                               ),
			.plls_locked                       	(hsma_xcvr_plls_locked              ),
			.pattern_sync_acquired             	(hsma_xcvr_pattern_sync_acquired    ),
			.test_complete                     	(hsma_xcvr_test_complete            ),
			.error_latch                       	(hsma_xcvr_error_latch              ),
			.error_irq                         	(                                   ),
			.nios_clk                          	(0                                  ),
			.nios_address                      	(0                                  ),
  			.nios_chipselect                   	(0                                  ),
  			.nios_read                         	(0                                  ),
  			.nios_readdata                     	(                                   ),
  			.nios_reset_n                      	(reset_n                            ),
  			.nios_write                        	(0                                  ),
 			.nios_writedata                    	(0                                  ),
			.nios_waitrequest                  	(                                   ),
			.avalon_pb                         	(user_pb                            ),
			.avalon_dip                        	(user_dipsw                         ),
			.avalon_led							(				                    )
		);	
	`else
		assign	hsma_xcvr_plls_locked               	=	1'b1;
		assign	hsma_xcvr_pattern_sync_acquired     	=	1'b1;
		assign	hsma_xcvr_test_complete             	=	1'b1;
		assign	hsma_xcvr_error_latch               	=	1'b0;	
		assign	hsma_xcvr_led_rx						=	1'b1;
		assign	hsma_xcvr_led_tx						=	1'b1;			
	`endif
	
	`ifdef	hsmb_xcvrs
		nios_bts_port_xcvr_x4_hsmb	nios_bts_port_xcvr_x4_hsmb_i
		(
    		.reset_n                           	(reset_n && hsmb_xcvr_reset_n       ),
			.en_cdet_pb                        	(en_cdet                            ),
			.channel_bond_pb                   	(channel_bond                       ),
			.start_pb                          	(start                              ),
			.create_error_pb           			(create_error                       ),
			.xcvr_reset_pb_n	               	(hsmb_xcvr_reset_n                  ),
    		.refclk_p                          	(`hsmb_xcvr_refclk_ql               ),
    		.tx_p    	                       	(hsmb_tx_p[3:0]                     ),
    		.rx_p                              	(hsmb_rx_p[3:0]                     ),
 			.cal_blk_clk_sig                   	(cal_blk_clk                        ),
			.seriallpbken_dip                  	(0                                  ),
    		.led_tx                            	(hsmb_xcvr_led_tx                   ),
    		.led_rx                            	(hsmb_xcvr_led_rx                   ),
			.rx_ss_clk_in			           	(0                                  ),
    		.tx_ss_clk_out       	           	(                                   ),
			.plls_in_locked			           	(1'b1                               ),
			.plls_locked                       	(hsmb_xcvr_plls_locked              ),
			.pattern_sync_acquired             	(hsmb_xcvr_pattern_sync_acquired    ),
			.test_complete                     	(hsmb_xcvr_test_complete            ),
			.error_latch                       	(hsmb_xcvr_error_latch              ),
			.error_irq                         	(                                   ),
			.nios_clk                          	(0                                  ),
			.nios_address                      	(0                                  ),
  			.nios_chipselect                   	(0                                  ),
  			.nios_read                         	(0                                  ),
  			.nios_readdata                     	(                                   ),
  			.nios_reset_n                      	(reset_n                            ),
  			.nios_write                        	(0                                  ),
 			.nios_writedata                    	(0                                  ),
			.nios_waitrequest                  	(                                   ),
			.avalon_pb                         	(user_pb                            ),
			.avalon_dip                        	(user_dipsw                         ),
			.avalon_led							(				                    )
		);
		
	`else
	    	assign	hsmb_xcvr_plls_locked               	=	1'b1;
			assign	hsmb_xcvr_pattern_sync_acquired     	=	1'b1;
			assign	hsmb_xcvr_test_complete             	=	1'b1;
			assign	hsmb_xcvr_error_latch               	=	1'b0;	
			assign	hsmb_xcvr_led_rx						=	1'b1;
			assign	hsmb_xcvr_led_tx						=	1'b1;			
	`endif	
	
	`ifdef	hsma_parallel
		nios_bts_port_parallel_loopback_x3_hsma	nios_bts_port_parallel_loopback_x3_hsma_i
		(
    		.reset_n                   (reset_n	&& hsma_parallel_reset_n   	),
			.en_cdet_pb                (en_cdet								),
			.channel_bond_pb           (channel_bond						),
			.start_pb                  (start								),
			.create_error_pb           (create_error						),	
			.xcvr_reset_pb_n	       (hsma_parallel_reset_n				),
    		.refclk_p                  (clk_parallel_0deg					),
    		.tx_p    	               ({hsma_scl,hsma_d_3,hsma_d_1}		),
    		.rx_p                      ({hsma_sda,hsma_d_2,hsma_d_0}		),
 			.cal_blk_clk_sig           (0									),
			.seriallpbken_dip          (0									),
    		.led_tx                    (									),
    		.led_rx                    (									),
			.rx_ss_clk_in			   (hsma_clk_in0						),   
			// Use dipswitches to control phase for forwarded parallel clocks:
    		//.tx_ss_clk_out       	   (hsma_clk_out0						),
			.plls_in_locked			   (pll_locked_parallel					),
			.plls_locked               (hsma_parallel_plls_locked			),
			.pattern_sync_acquired     (hsma_parallel_pattern_sync_acquired	),
			.test_complete             (hsma_parallel_test_complete			),
			.error_latch               (hsma_parallel_error_latch			),
			.error_irq                 (									),
			.nios_clk                  (0									),
			.nios_address              (0									),
  			.nios_chipselect           (0									),
  			.nios_read                 (0									),
  			.nios_readdata             (									),
  			.nios_reset_n              (reset_n								),
  			.nios_write                (0									),
 			.nios_writedata            (0									),
			.nios_waitrequest          (									),
			.avalon_pb                 (user_pb								),
			.avalon_dip                (user_dipsw							),
			.avalon_led				   (									)
		);                        
	`else
		assign	hsma_parallel_plls_locked               =	1'b1;
		assign	hsma_parallel_pattern_sync_acquired     =	1'b1;
		assign	hsma_parallel_test_complete             =	1'b1;
		assign	hsma_parallel_error_latch               =	1'b0;	
	`endif
	
	`ifdef	hsmb_parallel
		nios_bts_port_parallel_loopback_x80_hsmb	nios_bts_port_parallel_loopback_x80_hsmb_i
		(
    		.reset_n                   (reset_n	&& hsmb_parallel_reset_n	),
			.en_cdet_pb                (en_cdet								),
			.channel_bond_pb           (channel_bond						),
			.start_pb                  (start								),
			.create_error_pb           (create_error						),	
			.xcvr_reset_pb_n	       (hsmb_parallel_reset_n				),
    		.refclk_p                  (clk_parallel_0deg					),
    		.tx_p    	               ({hs mb_scl,hsmb_d_3,hsmb_d_1}		),
    		.rx_p                      ({hs mb_sda,hsmb_d_2,hsmb_d_0}		),
 			.cal_blk_clk_sig           (0									),
			.seriallpbken_dip          (0									),
    		.led_tx                    (									),
    		.led_rx                    (									),
			.rx_ss_clk_in			   (hsmb_clk_in0						),   
			// Use dipswitches to control phase for forwarded parallel clocks:
    		//.tx_ss_clk_out       	   (hsmb_clk_out0						),
			.plls_in_locked			   (pll_locked_parallel					),
			.plls_locked               (hsmb_parallel_plls_locked			),
			.pattern_sync_acquired     (hsmb_parallel_pattern_sync_acquired	),
			.test_complete             (hsmb_parallel_test_complete			),
			.error_latch               (hsmb_parallel_error_latch			),
			.error_irq                 (									),
			.nios_clk                  (0									),
			.nios_address              (0									),
  			.nios_chipselect           (0									),
  			.nios_read                 (0									),
  			.nios_readdata             (									),
  			.nios_reset_n              (reset_n								),
  			.nios_write                (0									),
 			.nios_writedata            (0									),
			.nios_waitrequest          (									),
			.avalon_pb                 (user_pb								),
			.avalon_dip                (user_dipsw							),
			.avalon_led				   (									)
		);  
	`else
		assign	hsmb_parallel_plls_locked               =	1'b1;
		assign	hsmb_parallel_pattern_sync_acquired     =	1'b1;
		assign	hsmb_parallel_test_complete             =	1'b1;
		assign	hsmb_parallel_error_latch               =	1'b0;	
	`endif		
	
	
		// Use basic mode to drive PCIe Edge Connector
	`ifdef	pcie_edge_basic
	
		// Split PCIe into two x4 8.5G XCVRs (can use hsma component if not using NIOS)
		nios_bts_port_xcvr_x4_pcie_1	nios_bts_port_xcvr_x4_pcie_1_i
		(
    		.reset_n                           	(reset_n && pcie_edge_reset_n       ),
			.en_cdet_pb                        	(en_cdet                            ),
			.channel_bond_pb                   	(channel_bond                       ),
			.start_pb                          	(start                              ),
			.create_error_pb           			(create_error                       ),
			.xcvr_reset_pb_n	               	(pcie_edge_reset_n                  ),
    		.refclk_p                          	(`pcie_refclk                  ),
    		.tx_p    	                       	(pcie_tx_p[3:0]                     ),
    		.rx_p                              	(pcie_rx_p[3:0]                     ),
 			.cal_blk_clk_sig                   	(cal_blk_clk                        ),
			.seriallpbken_dip                  	(0                                  ),
			.led_tx                            	(	                	    ),
			.led_rx                            	(		                    ),
			.rx_ss_clk_in			        	(0                                  ),
			.tx_ss_clk_out       	           	(                                   ),
			.plls_in_locked			        	(1'b1                               ),
			.plls_locked                       	(pcie_edge_plls_locked_1            ),
			.pattern_sync_acquired             	(pcie_edge_pattern_sync_acquired_1  ),
			.test_complete                     	(pcie_edge_test_complete_1          ),
			.error_latch                       	(pcie_edge_error_latch_1            ),
			.error_irq                         	(                                   ),
			.nios_clk                          	(0                                  ),
			.nios_address                      	(0                                  ),
  			.nios_chipselect                   	(0                                  ),
  			.nios_read                         	(0                                  ),
  			.nios_readdata                     	(                                   ),
  			.nios_reset_n                      	(reset_n                            ),
  			.nios_write                        	(0                                  ),
 			.nios_writedata                    	(0                                  ),
			.nios_waitrequest                  	(                                   ),
			.avalon_pb                         	(user_pb                            ),
			.avalon_dip                        	(user_dipsw                         ),
			.avalon_led							(				    )
		);       
		
 		nios_bts_port_xcvr_x4_pcie_2	nios_bts_port_xcvr_x4_pcie_2_i
		(
    			.reset_n                           	(reset_n && pcie_edge_reset_n       ),
			.en_cdet_pb                        	(en_cdet                            ),
			.channel_bond_pb                   	(channel_bond                       ),
			.start_pb                          	(start                              ),
			.create_error_pb           			(create_error                       ),
			.xcvr_reset_pb_n	               	(pcie_edge_reset_n                  ),
    		.refclk_p                       	(`pcie_refclk                  ),
    		.tx_p    	                    	(pcie_tx_p[7:4]                     ),
    		.rx_p                           	(pcie_rx_p[7:4]                     ),
 			.cal_blk_clk_sig                   	(cal_blk_clk                        ),
			.seriallpbken_dip                  	(0                                  ),
			.led_tx                            	(	                	    ),
			.led_rx                            	(		                    ),
			.rx_ss_clk_in			        	(0                                  ),
			.tx_ss_clk_out       	           	(                                   ),
			.plls_in_locked			        	(1'b1                               ),
			.plls_locked                       	(pcie_edge_plls_locked_2            ),
			.pattern_sync_acquired             	(pcie_edge_pattern_sync_acquired_2  ),
			.test_complete                     	(pcie_edge_test_complete_2          ),
			.error_latch                       	(pcie_edge_error_latch_2            ),
			.error_irq                         	(                                   ),
			.nios_clk                          	(0                                  ),
			.nios_address                      	(0                                  ),
  			.nios_chipselect                   	(0                                  ),
  			.nios_read                         	(0                                  ),
  			.nios_readdata                     	(                                   ),
  			.nios_reset_n                      	(reset_n                            ),
  			.nios_write                        	(0                                  ),
 			.nios_writedata                    	(0                                  ),
			.nios_waitrequest                  	(                                   ),
			.avalon_pb                         	(user_pb                            ),
			.avalon_dip                        	(user_dipsw                         ),
			.avalon_led							(				    )
		);       
		
	    assign	pcie_edge_plls_locked                	=	pcie_edge_plls_locked_1           	&&	pcie_edge_plls_locked_2           ;
		assign	pcie_edge_pattern_sync_acquired      	=	pcie_edge_pattern_sync_acquired_1	&&	pcie_edge_pattern_sync_acquired_2 ;
		assign	pcie_edge_test_complete              	=	pcie_edge_test_complete_1         	&&	pcie_edge_test_complete_2         ;
		assign	pcie_edge_error_latch                	=	pcie_edge_error_latch_1           	||	pcie_edge_error_latch_2           ;
			
	`else
	    assign	pcie_edge_plls_locked              	=	1'b1;
		assign	pcie_edge_pattern_sync_acquired    	=	1'b1;
		assign	pcie_edge_test_complete            	=	1'b1;
		assign	pcie_edge_error_latch              	=	1'b0;	
	`endif
`endif

// Disable unused interfaces:
`ifndef	ddr3_dev
    assign	ddr3_wen		=	1'b1;
    assign	ddr3_cke		=	1'b0;
	assign	ddr3_csn		=	1'b1;
    assign	ddr3_resetn		=	1'b0;
    assign	ddr3_odt		=	1'b0;
`endif

`ifndef	ddr2_dimm
   assign       ddr2_dimm_wen    = 1'b1;   //SSTL18    //Write enable		OK
   assign   	ddr2_dimm_cke    = 1'b0;   //SSTL18    //System Clock Enable  OK
   assign	  	ddr2_dimm_csn    = 2'b11;  //SSTL18    //Chip Select           OK
   assign       ddr2_dimm_resetn = 1'b0;   //SSTL18    //Reset
   assign		ddr2_dimm_odt    = 2'b00;  //SSTL18    //On-die termination enable	OK
`endif

	
endmodule

