module nios_bts_port_parallel_loopback_x3_hsma_top (
	// Resets:
    input          					reset_n,
	input							en_cdet_pb,
	input							channel_bond_pb,
	input							start_pb,
	input							create_error_pb,           				
	input		   					xcvr_reset_pb_n,	
    input          					refclk_p,
    output [io_width-1:0]   		tx_p,    	  
    input  [io_width-1:0]   		rx_p,
 	input							cal_blk_clk_sig,
	input  							seriallpbken_dip,    
    output         					led_tx,     
    output         					led_rx,
	input							rx_ss_clk_in,			
    output							tx_ss_clk_out,       	
	input							plls_in_locked,				
	output		    				plls_locked, 
	output							pattern_sync_acquired,
	output							test_complete,
	output							error_latch,
	output							error_irq,
	input							nios_clk,
	input	[nios_map_width-1:0]	nios_address,
  	input	      					nios_chipselect,
  	input	      					nios_read,
  	output	[nios_width-1:0]		nios_readdata,
  	input	      					nios_reset_n,
  	input	      	    		    nios_write,
 	input	[nios_width-1:0]		nios_writedata,
	output							nios_waitrequest,
	input	[7:0]					avalon_pb,
	input	[7:0]					avalon_dip,
	output	[7:0]					avalon_led											
);  
 
                  
// Board Test System for HSMA Interface: 
parameter	component_type					=	"PARALLEL";
parameter	nios_width						=	32;
parameter	bts_idle						=  	0;	// Only PRBS pattern which won't occur after word alignment -- (tx inserts these if tx_ready is deasserted) 
parameter	debounce_prd					=	100000;
parameter	delayed_rst_prd					=	100000;
parameter	delayed_rst_pw					=	32;				// Delayed reset pulse width
parameter	test_length_bts					=	'h800000;		// Number of data_gen_readdata to transmit before asserting testcomplete
parameter	test_complete_ctr_width			=	24;				// > log2(test_length_bts)	  
parameter	tx_wrapper_mode					=	6;			    // (<= (1<<sssc_spi42_width)  - 2) for parallel; // Repeating pattern: lower 1/2 word incrementing, dc balance with word align & byte order patterns  
parameter	byte_width						=	8;               
parameter	deserializer_width				=	8;				// Unused
parameter	serial_channels 				= 	1;        		
parameter	io_width						=	3;				// Width of serial or parallel bus 
parameter	deskew_depth					= 	16;				// For BTS Channel Bonder
parameter	deskew_depth_log2				= 	4;				// log2(deskew_depth)
parameter	sync_pattern_width				= 	8;				// Unused 
parameter	mux_sel_width					= 	3;				// Unused
parameter	num_quads						= 	1; 				// 1 Quad (1 xcvr block) used for 4 serial channel (defines # of tx plls)
parameter	channel_width					= 	io_width;		// (I/O Width for parallel data)
parameter	channel_width_log2				= 	2;
// Width should be >= sync_pattern_width
// word_align_pattern[sync_pattern_width-1:0] used by word aligner
// Full pattern output by tx_wrapper_bts, suggest make dc balanced if ac coupling.
// Usually make {serial_channels{word_align_pattern}} different from prbs_seed_param
// bc = K28.5  (sync) fb = (byte ordering)        
parameter	byte_order_pattern				=	3'b000;	// Unused in rx - need something to tranmit   
parameter	word_align_pattern 				=	3'b000;
parameter	prbs_depth						=	3;				// data_width should be an integer multiple of prbs_depth
parameter	prbs_coeff_param				=	3'b011;         // Should equal data_width
parameter	prbs_seed_param					=	3'b111;
// DPRIO XCVRS                                        
parameter	dc_bits							=	3;
parameter	eq_bits							=	4;
parameter	pre_bits						=	5;
parameter	vod_bits						=	3; 
// Skew delay across serial channels:                             
parameter	xcvr_skew_num_barrels			=	2;				// >=2
parameter	xcvr_skew_num_barrels_m1_log2	=	1;				// ceil(log2(skew_num_barrels-1)) (ceil means round up)
// xs_wrapper_bts major controls
parameter	rx_fifo_enable					=	0;
parameter	word_aligner_enable				=	0;
parameter	channel_bond_enable				=	0;
parameter	convert_endian					=	0;      
parameter	flip_data_bits					=	0;  	
parameter	skew_msb_1st					=	0;				// Unused skew_msb_1st=1 (Source Synchronous LVDS are MSB 1st. skew_msb_1st=0 (XCVRS are LSB 1st)
// Avalon Control Interface         		
parameter	map_dprio						=	0;   
parameter	nios_map_width					=	20;         
parameter	addrs_per_reg					=	'h200;
parameter	addrs_per_reg_log2				=	9;   
parameter	enable_nios_bts 				=	1;
parameter	tec_words						=	1;				// Transaction/Error Counter Nios words
parameter	num_lanes						=	1;				// Typically set to serial_channels;
																// Number of error lanes (eg 9 for 72 bit DDR2 with 9 byte lanes)
parameter	ssd_bits						=	8;				// Number of bits per SSD
parameter	num_ssds						=	4;				// Number of Seven Segment Display addresses
parameter	bts_id1							=	32'h0000_0200;	// Set lower bits to addrs_per_reg
parameter	bts_id2							=	32'hace1_ace2;
parameter	bts_id3							=	32'h89ab_cdef;
parameter	bts_id4							=	32'h5555_5555;
parameter	bts_id5							=	32'hcccc_cccc;
parameter	bts_id6							=	32'hbabe_1234;
parameter	bts_id7							=	32'hfeed_baba;
parameter	bts_id8							=	32'habcd_beef;
parameter	board_id	    					=	"Arria II GX FPGA Dev Kit        ";
parameter	port_id							=	"HSMA x3 Single Ended Loopback   ";
parameter	generic_id						=	"NIOS Board Test System          ";
parameter	id_words	    				=	8;  
                    
 
defparam	nios_bts_port_core_i.component_type					=	  component_type				;
defparam	nios_bts_port_core_i.nios_width						=	  nios_width					;	
defparam	nios_bts_port_core_i.bts_idle						=  	  bts_idle						;
defparam	nios_bts_port_core_i.debounce_prd					=	  debounce_prd					;
defparam	nios_bts_port_core_i.delayed_rst_prd				=	  delayed_rst_prd				;	
defparam	nios_bts_port_core_i.delayed_rst_pw					=	  delayed_rst_pw				;	
defparam	nios_bts_port_core_i.test_length_bts				=	  test_length_bts				;	
defparam	nios_bts_port_core_i.test_complete_ctr_width		=	  test_complete_ctr_width		;	
defparam	nios_bts_port_core_i.tx_wrapper_mode				=	  tx_wrapper_mode				;	
defparam	nios_bts_port_core_i.byte_width						=	  byte_width					;	
defparam	nios_bts_port_core_i.deserializer_width				=	  deserializer_width			;	
defparam	nios_bts_port_core_i.serial_channels 				= 	  serial_channels 				;
defparam	nios_bts_port_core_i.io_width						= 	  io_width						;
defparam	nios_bts_port_core_i.deskew_depth					= 	  deskew_depth					;
defparam	nios_bts_port_core_i.deskew_depth_log2				= 	  deskew_depth_log2				;
defparam	nios_bts_port_core_i.sync_pattern_width				= 	  sync_pattern_width			;	
defparam	nios_bts_port_core_i.mux_sel_width					= 	  mux_sel_width					;
defparam	nios_bts_port_core_i.num_quads						= 	  num_quads						;
defparam	nios_bts_port_core_i.channel_width					= 	  channel_width					;
defparam	nios_bts_port_core_i.channel_width_log2				= 	  channel_width_log2			;	
defparam	nios_bts_port_core_i.byte_order_pattern				=	  byte_order_pattern			;	
defparam	nios_bts_port_core_i.word_align_pattern 			=	  word_align_pattern 			;	
defparam	nios_bts_port_core_i.prbs_depth						=	  prbs_depth					;	
defparam	nios_bts_port_core_i.prbs_coeff_param				=	  prbs_coeff_param				;
defparam	nios_bts_port_core_i.prbs_seed_param				=	  prbs_seed_param				;	
defparam	nios_bts_port_core_i.dc_bits						=	  dc_bits						;	
defparam	nios_bts_port_core_i.eq_bits						=	  eq_bits						;	
defparam	nios_bts_port_core_i.pre_bits						=	  pre_bits						;
defparam	nios_bts_port_core_i.vod_bits						=	  vod_bits						;
defparam	nios_bts_port_core_i.xcvr_skew_num_barrels			=	  xcvr_skew_num_barrels			;
defparam	nios_bts_port_core_i.xcvr_skew_num_barrels_m1_log2	=	  xcvr_skew_num_barrels_m1_log2	;
defparam	nios_bts_port_core_i.rx_fifo_enable					=	  rx_fifo_enable				;	
defparam	nios_bts_port_core_i.word_aligner_enable			=	  word_aligner_enable			;	
defparam	nios_bts_port_core_i.channel_bond_enable			=	  channel_bond_enable			;	
defparam	nios_bts_port_core_i.convert_endian					=	  convert_endian				;	
defparam	nios_bts_port_core_i.flip_data_bits					=	  flip_data_bits				;	
defparam	nios_bts_port_core_i.skew_msb_1st					=	  skew_msb_1st					;
defparam	nios_bts_port_core_i.map_dprio						=	  map_dprio						;
defparam	nios_bts_port_core_i.nios_map_width					=	  nios_map_width				;	
defparam	nios_bts_port_core_i.addrs_per_reg					=	  addrs_per_reg					;
defparam	nios_bts_port_core_i.addrs_per_reg_log2				=	  addrs_per_reg_log2			;	
defparam	nios_bts_port_core_i.enable_nios_bts 				=	  enable_nios_bts 				;
defparam	nios_bts_port_core_i.tec_words						=	  tec_words						;
defparam	nios_bts_port_core_i.num_lanes						=	  num_lanes						;
defparam	nios_bts_port_core_i.ssd_bits						=	  ssd_bits						;
defparam	nios_bts_port_core_i.num_ssds						=	  num_ssds						;
defparam	nios_bts_port_core_i.bts_id1						=	  bts_id1						;	
defparam	nios_bts_port_core_i.bts_id2						=	  bts_id2						;	
defparam	nios_bts_port_core_i.bts_id3						=	  bts_id3						;	
defparam	nios_bts_port_core_i.bts_id4						=	  bts_id4						;	
defparam	nios_bts_port_core_i.bts_id5						=	  bts_id5						;	
defparam	nios_bts_port_core_i.bts_id6						=	  bts_id6						;	
defparam	nios_bts_port_core_i.bts_id7						=	  bts_id7						;	
defparam	nios_bts_port_core_i.bts_id8						=	  bts_id8						;	
defparam	nios_bts_port_core_i.board_id	    				=	  board_id	    				;
defparam	nios_bts_port_core_i.port_id						=	  port_id						;	
defparam	nios_bts_port_core_i.generic_id						=	  generic_id					;	
defparam	nios_bts_port_core_i.id_words	    				=	  id_words	    				;
                                                
nios_bts_port_core	nios_bts_port_core_i
(
    .reset_n                            (reset_n                ), 
	.en_cdet_pb                         (en_cdet_pb             ),
	.channel_bond_pb                    (channel_bond_pb        ),
	.start_pb                           (start_pb               ),
	.create_error_pb           			(create_error_pb        ),
	.xcvr_reset_pb_n                    (xcvr_reset_pb_n        ),
    .refclk_p                           (refclk_p               ),
    .tx_p    	                        (tx_p    	            ),
    .rx_p                               (rx_p                   ),
 	.cal_blk_clk_sig                    (cal_blk_clk_sig        ),
	.seriallpbken_dip                   (seriallpbken_dip       ),
    .led_tx                             (led_tx                 ),
    .led_rx     		                (led_rx     		    ),
	.rx_ss_clk_in			            (rx_ss_clk_in	        ),
    .tx_ss_clk_out       	            (tx_ss_clk_out          ),
	.plls_in_locked			            (plls_in_locked	        ),  
	.plls_locked                        (plls_locked            ),
	.pattern_sync_acquired              (pattern_sync_acquired  ),
	.test_complete                      (test_complete          ),
	.error_latch                        (error_latch            ),
	.error_irq                          (error_irq              ), 
	.nios_clk	                        (nios_clk	            ),
	.nios_address                       (nios_address           ),
  	.nios_chipselect                    (nios_chipselect        ),
  	.nios_read                          (nios_read              ),
  	.nios_readdata                      (nios_readdata          ),
  	.nios_reset_n                       (nios_reset_n           ),
  	.nios_write                         (nios_write             ),
 	.nios_writedata	                    (nios_writedata	        ),
	.nios_waitrequest					(nios_waitrequest		),
	.avalon_pb                          (avalon_pb              ),
	.avalon_dip                         (avalon_dip             ),
	.avalon_led							(avalon_led				)						
);  



endmodule