
//--------------------------------------------------------------------------//
// Title:       nios_bts_port_xcvr_x4_hsma_top.v                              //
// Rev:         Rev 1                                                       //
// Author		San Diego - High Speed Design Group                         //
//--------------------------------------------------------------------------//
// Description: NIOS BTS Port For Transceivers                              //
//----------------------------------------------------------------------------
//Copyright  2011 Altera Corporation. All rights reserved.  Altera products
//are protected under numerous U.S. and foreign patents, maskwork rights,
//copyrights and other intellectual property laws.
//                                                                                                     
//This reference design file, and your use thereof, is subject to and
//governed by the terms and conditions of the applicable Altera Reference
//Design License Agreement.  By using this reference design file, you
//indicate your acceptance of such terms and conditions between you and
//Altera Corporation.  In the event that you do not agree with such terms and
//conditions, you may not use the reference design file. Please promptly                         
//destroy any copies you have made.
//
//This reference design file being provided on an "as-is" basis and as an
//accommodation and therefore all warranties, representations or guarantees
//of any kind (whether express, implied or statutory) including, without
//limitation, warranties of merchantability, non-infringement, or fitness for
//a particular purpose, are specifically disclaimed.  By making this
//reference design file available, Altera expressly does not recommend,
//suggest or require that this reference design file be used in combination 
//with any other product not provided by Altera

module  nios_bts_port_xcvr_x4_hsma_top
(
    input          					reset_n,
	input							en_cdet_pb,
	input							channel_bond_pb,
	input							start_pb,
	input							create_error_pb,           				
	input		   					xcvr_reset_pb_n,	
    input          					refclk_p,
    output [serial_channels-1:0]   	tx_p,    	  
    input  [serial_channels-1:0]   	rx_p,
 	input							cal_blk_clk_sig,
	input  [serial_channels-1:0]	seriallpbken_dip,    
    output         					led_tx,     
    output         					led_rx,
	input							rx_ss_clk_in,			
    output							tx_ss_clk_out,       	
	input							plls_in_locked,			
	output		    				plls_locked, 
	output							pattern_sync_acquired,
	output							test_complete,
	output							error_latch,
	output							error_irq,
	input							nios_clk,
	input	[nios_map_width-1:0]	nios_address,
  	input	      					nios_chipselect,
  	input	      					nios_read,
  	output	[nios_width-1:0]		nios_readdata,
  	input	      					nios_reset_n,
  	input	      	    		    nios_write,
 	input	[nios_width-1:0]		nios_writedata,
	output							nios_waitrequest,
	input	[31:0]					avalon_pb,
	input	[31:0]					avalon_dip,
	output	[31:0]					avalon_led											
);  
 

// Board Test System for HSMA Interface: 
parameter	component_type					=	"A2GX_HSM_XCVR";
parameter	serdes_2d						=	0;	
parameter	nios_width						=	32;
parameter	bts_idle						=  	20'hccccc;			// (tx inserts these if tx_ready is deasserted) 
parameter	debounce_prd					=	100000;
parameter	delayed_rst_prd					=	100000;
parameter	delayed_rst_pw					=	32;				// Delayed reset pulse width
parameter	test_length_bts					=	'h800000;		// Number of data_gen_readdata to transmit before asserting testcomplete
parameter	test_complete_ctr_width			=	24;				// > log2(test_length_bts)	  
parameter	tx_wrapper_mode					=	16;				// Repeating pattern: lower 1/2 word incrementing, upper 1/2 word complement of lower half word so DC balanced, followed by word_align_pattern   
parameter	byte_width						=	8;               
parameter	deserializer_width				=	20;				// PCS deserializer width (assuming XCVR encoding is hidden)				
parameter	serial_channels 				= 	4;
parameter	io_width						=	4;				// Width of serial or parallel bus         		
parameter	deskew_depth					= 	8;				// For BTS Channel Bonder
parameter	deskew_depth_log2				= 	3;				// log2(deskew_depth)
parameter	sync_pattern_width				= 	20;				// Typically set the channel_width, fewer bits for less logic in word aligner
parameter	mux_sel_width					= 	5;				// log2(channel_width) (round any fraction up)   
parameter	mux_sel_step					=   1;				// 1 for word aligner & byte ordering in core, set to deserializer_width for byte order in core & word aligner in alt2gxb
parameter	num_quads						= 	1; 				// 1 Quad (1 xcvr block) used for 4 serial channels 
parameter	channel_width					= 	20;				// Channel Width (parallel data) at the FPGA interface
parameter	channel_width_log2				= 	5;
// Width should be >= sync_pattern_width
// word_align_pattern[sync_pattern_width-1:0] used by word aligner
// Full pattern output by tx_wrapper_bts, suggest make dc balanced if ac coupling.
// Usually make {serial_channels{word_align_pattern}} different from prbs_seed_param
// bc = K28.5  (sync) fb = (byte ordering)        
parameter	byte_order_pattern				=	20'h33333;	// d pad + k byte order pattern    
parameter	word_align_pattern 				=	20'haaaaa;	// d pad + k28.5 aligner pattern 
/* PRBS Generators Parameter Settings:

	PRBS MEM is similar to the ddr2 megacore's example_driver.  The outputs of
	all shift registers are output as PRBS data, and multiple prbs generates are 
	concatenated across the parallel bus.  See prbs_generate.v module.

	PRBS COM provides data for input to a serializer, a realtime scope can
	detect the LFRS length.
	
	enable_nios_bts=0:	Pre-NIOS / Avalon Control Interface.  PRBS MEM only
							is available.
							prbs_depth:		SR PRBS polynomial size.  
							prbs_type:		Don't care, acts like =prbs_mem
							prbs_width:		Set = ()
							eg:  serial_channels=4, channel_width_32, prbs_depth=8
								 (4*32/8) = 16 prbs8 generators are concatenated
								 Concatenated seeds and coefficients are set by
								 prbs_seed and prbs_coeff, respectively.
								 (channel_width * serial_channels / prbs_depth)
								 should be an integer
								 			 					 
	enable_nios_bts=1:	Fixed PRBS Type & PRBS Depth.  PRBS COM or PRBS MEM is
						set prior to Quartus compilation
							prbs_type:		Selects either prbs_com or prbs_mem
							prbs_depth:		SR PRBS MEM polynomial size if
											prbs_type=prbs_mem
											SR PRBS COM polynomial size if
											prbs_type=prbs_com
							prbs_width:		Set = (channel_width * serial_channels)
											if prbs_type = prbs_mem
											Set = (prbs_depth * serial_channels)
											if prbs_type = prbs_com
	enable_nios_bts=2:	NIOS dynamically controls PRBS Type (com or mem) and
						PRBS Depth.  Both PRBS COM and PRBS MEM are selectable
						dynamically by NIOS after Quartus compilation.
							prbs_depth:	 	The max SR PRBS polynomial size
											for the prbs com. 
											Also is the default prbs depth
											out of reset for both mem and com,
											then uses aci_prd controlled by NIOS.
											If prbs_type=prbs_mem, 
											(channel_width * serial_channels /
											prbs_depth) should be an integer
							prbs_type:		Default out of reset, then uses aci_prt
											controlled by NIOS.  Typically set 
											prbs_type=prbs_com
							prbs_width:		Set = max(prbs_depth,channel_width) 
											* serial_channels)
											Width of the aci_prs and aci_prc
											registers, make large enough for 	
											either mem or com prbs generator types
*/   
//	enable_nios_bts=0:	Pre-NIOS / Avalon Control Interface PRBS MEM
//					1:	Parameters select either PRBS COM or PRBS MEM
//					2:	NIOS dynamically controls PRBS type COM/MEM and Depth
//					3:  Subset of 2, allows selection between 4 PRBS generators (eg 7, 23, 15, 31)
parameter	enable_nios_bts 					=	3;	// prbs_generator.v

// PRBS 7:                                                      	
//parameter	prbs_depth						=	7;	// PRBS Shift Register Size (If enable_nios_bts=2, set to the max PRBS COM Shift Register Polynomial Size).
//parameter	prbs_width						=	serial_channels * channel_width; // Set to serial_channels * max{prbs_depth,channel_width)
//parameter	prbs_coeff_param					=	{serial_channels{7'b000_0011}};	
//parameter	prbs_seed_param						=	{serial_channels{7'b011_1010}}; // keep identical so can swap xcvr loopbacks 
// PRBS 15:                                                     	
//parameter	prbs_depth						=	15;	// PRBS Shift Register Size (If enable_nios_bts=2, set to the max PRBS COM Shift Register Polynomial Size).
//parameter	prbs_width						=	serial_channels * channel_width; // Set to serial_channels * max{prbs_depth,channel_width)
//parameter	prbs_coeff_param					=	{serial_channels{15'b000_0000_0000_0011}};	
//parameter	prbs_seed_param						=	{serial_channels{15'b011_0101_1010_1011}}; // keep identical so can swap xcvr loopbacks 
// PRBS 31                                                      	
parameter	prbs_depth						=	31;	// PRBS Shift Register Size (If enable_nios_bts=2, set to the max PRBS COM Shift Register Polynomial Size).
parameter	prbs_width						=	serial_channels * channel_width; // Set to serial_channels * max{prbs_depth,channel_width)
parameter	prbs_coeff_param					=	{serial_channels{31'b000_0000_0000_0000_0000_0000_0000_1001}};	
parameter	prbs_seed_param						=	{serial_channels{31'b011_0101_1010_1011_1100_0000_1010_0101}}; // keep identical so can swap xcvr loopbacks 
                                                                	
parameter	prbs_type						=	prbs_com;
parameter	prt_init						=	1;	// 0=PRBS7, 1=PRBS15, 2=PRBS23, 3=PRBS31, 40 for PCIe Compliance

// Use PRBS_MEM to meet timing:                                 	
//parameter	prbs_depth						=	8;				// data_width should be an integer multiple of prbs_depth
//parameter	prbs_width						=	serial_channels * channel_width;
//parameter	prbs_coeff_param					=	{(serial_channels*channel_width/prbs_depth){8'h1d}};
//parameter	prbs_seed_param					=	80'h12345_6789a_bcd31_41592; // {(serial_channels*channel_width/prbs_depth){8'hA5}};
//parameter	prbs_seed_param						=	{(serial_channels*channel_width/prbs_depth){8'hA5}};
//parameter	prbs_type						=	prbs_mem;

parameter	prbs_mem						=	0;				// PRBS mimimizes area and timing uses full shift register as parallel data, like example_driver in DDR2 megacores
																// All test designs before HSMC SDI XCVR daughtercard operate in this mode
parameter	prbs_com						=	1;				// PRBS calculated for serializer, uses extra LE's and has less timing margin 
// DPRIO XCVRS                                        
parameter	enable_gdn_pdn				=	1;			// Set to 1 to enable dynamic ALTGX and PLL power down control from NIOS
parameter	dc_bits							=	2;	
parameter	eq_bits							=	4;
parameter	pre_bits						=	5;
parameter	vod_bits						=	3; 
// Skew delay across serial channels:                             
parameter	xcvr_skew_num_barrels			=	2;				// >=2
parameter	xcvr_skew_num_barrels_m1_log2	=	1;				// ceil(log2(skew_num_barrels-1)) (ceil means round up)
// xs_wrapper_bts major controls
parameter	rx_fifo_enable					=	1;				// user rx fifo inside altgx
parameter	word_aligner_enable				=	1;				// For byte order
parameter	channel_bond_enable				=	1;
parameter	convert_endian					=	1;				// word aligner assumes msb 1st - xcvrs are lsb 1st -- LVDS SERDES are msb 1st     
parameter	flip_data_bits					=	1;	
parameter	skew_msb_1st					=	1;	
// Avalon Control Interface         		
parameter	map_dprio						=	1;   			// For NIOS control of serial loopback, preemphasis, etc
parameter	nios_map_width					=	20;         
parameter	addrs_per_reg						=	32'h200;
parameter	addrs_per_reg_log2				=	9;   
parameter	tec_words						=	1;				// Transaction/Error Counter Nios words
parameter	num_lanes						=	serial_channels;	// Typically set to serial_channels;
																// Number of error lanes (eg 9 for 72 bit DDR2 with 9 byte lanes)
parameter	ssd_bits						=	8;				// Number of bits per SSD
parameter	num_ssds						=	4;				// Number of Seven Segment Display addresses
parameter	bts_id1							=	addrs_per_reg;	// Addr per reg
parameter	bts_id2							=	32'h0000_0001;
parameter	bts_id3							=	32'h0000_0002;
parameter	bts_id4							=	32'h0000_0003;
parameter	bts_id5							=	32'hffff_fffc;
parameter	bts_id6							=	32'hffff_fffd;
parameter	bts_id7							=	32'hffff_fffe;
parameter	bts_id8							=	32'hffff_ffff;
parameter	board_id	    				=	"Arria II GX FPGA Dev Kit        ";
parameter	port_id							=	"HSMA x4 Transceivers [0..3]     ";
parameter	generic_id						=	"NIOS Board Test System          ";
parameter	id_words	    				=	8;  
parameter	enable_sdm_dip_override			=	0;				// Avalon dips to mask sync detect of unused xcvrs in block
parameter	enable_edm_dip_override			=	0;				// Avalon dips to maks error detect of unused masks in xcvr block
 
defparam	nios_bts_port_core_i.component_type					=	  component_type				;
defparam	nios_bts_port_core_i.serdes_2d						=	  serdes_2d						;
defparam	nios_bts_port_core_i.nios_width						=	  nios_width					;	
defparam	nios_bts_port_core_i.bts_idle						=  	  bts_idle						;
defparam	nios_bts_port_core_i.debounce_prd					=	  debounce_prd					;
defparam	nios_bts_port_core_i.delayed_rst_prd				=	  delayed_rst_prd				;	
defparam	nios_bts_port_core_i.delayed_rst_pw					=	  delayed_rst_pw				;	
defparam	nios_bts_port_core_i.test_length_bts				=	  test_length_bts				;	
defparam	nios_bts_port_core_i.test_complete_ctr_width		=	  test_complete_ctr_width		;	
defparam	nios_bts_port_core_i.tx_wrapper_mode				=	  tx_wrapper_mode				;	
defparam	nios_bts_port_core_i.byte_width						=	  byte_width					;	
defparam	nios_bts_port_core_i.deserializer_width				=	  deserializer_width			;	
defparam	nios_bts_port_core_i.serial_channels 				= 	  serial_channels 				;
defparam	nios_bts_port_core_i.io_width 						= 	  io_width		 				;
defparam	nios_bts_port_core_i.deskew_depth					= 	  deskew_depth					;
defparam	nios_bts_port_core_i.deskew_depth_log2				= 	  deskew_depth_log2				;
defparam	nios_bts_port_core_i.sync_pattern_width				= 	  sync_pattern_width			;	
defparam	nios_bts_port_core_i.mux_sel_width					= 	  mux_sel_width					;
defparam	nios_bts_port_core_i.mux_sel_step					= 	  mux_sel_step					;
defparam	nios_bts_port_core_i.num_quads						= 	  num_quads						;
defparam	nios_bts_port_core_i.channel_width					= 	  channel_width					;
defparam	nios_bts_port_core_i.channel_width_log2				= 	  channel_width_log2			;	
defparam	nios_bts_port_core_i.byte_order_pattern				=	  byte_order_pattern			;	
defparam	nios_bts_port_core_i.word_align_pattern 			=	  word_align_pattern 			;	
defparam	nios_bts_port_core_i.prbs_depth						=	  prbs_depth					;	
defparam	nios_bts_port_core_i.prbs_coeff_param				=	  prbs_coeff_param				;
defparam	nios_bts_port_core_i.prbs_seed_param				=	  prbs_seed_param				;
defparam	nios_bts_port_core_i.prbs_type						=	  prbs_type						;
defparam	nios_bts_port_core_i.prt_init					=	 prt_init;
defparam	nios_bts_port_core_i.prbs_width						=	  prbs_width					;
defparam	nios_bts_port_core_i.enable_gdn_pdn					=	  enable_gdn_pdn				;
defparam	nios_bts_port_core_i.dc_bits						=	  dc_bits						;	
defparam	nios_bts_port_core_i.eq_bits						=	  eq_bits						;	
defparam	nios_bts_port_core_i.pre_bits						=	  pre_bits						;
defparam	nios_bts_port_core_i.vod_bits						=	  vod_bits						;
defparam	nios_bts_port_core_i.xcvr_skew_num_barrels			=	  xcvr_skew_num_barrels			;
defparam	nios_bts_port_core_i.xcvr_skew_num_barrels_m1_log2	=	  xcvr_skew_num_barrels_m1_log2	;
defparam	nios_bts_port_core_i.rx_fifo_enable					=	  rx_fifo_enable				;	
defparam	nios_bts_port_core_i.word_aligner_enable			=	  word_aligner_enable			;	
defparam	nios_bts_port_core_i.channel_bond_enable			=	  channel_bond_enable			;	
defparam	nios_bts_port_core_i.convert_endian					=	  convert_endian				;	
defparam	nios_bts_port_core_i.flip_data_bits					=	  flip_data_bits				;	
defparam	nios_bts_port_core_i.skew_msb_1st					=	  skew_msb_1st					;
defparam	nios_bts_port_core_i.map_dprio						=	  map_dprio						;
defparam	nios_bts_port_core_i.nios_map_width					=	  nios_map_width				;	
defparam	nios_bts_port_core_i.addrs_per_reg					=	  addrs_per_reg					;
defparam	nios_bts_port_core_i.addrs_per_reg_log2				=	  addrs_per_reg_log2			;	
defparam	nios_bts_port_core_i.enable_nios_bts 				=	  enable_nios_bts 				;
defparam	nios_bts_port_core_i.tec_words						=	  tec_words						;
defparam	nios_bts_port_core_i.num_lanes						=	  num_lanes						;
defparam	nios_bts_port_core_i.ssd_bits						=	  ssd_bits						;
defparam	nios_bts_port_core_i.num_ssds						=	  num_ssds						;
defparam	nios_bts_port_core_i.bts_id1						=	  bts_id1						;	
defparam	nios_bts_port_core_i.bts_id2						=	  bts_id2						;	
defparam	nios_bts_port_core_i.bts_id3						=	  bts_id3						;	
defparam	nios_bts_port_core_i.bts_id4						=	  bts_id4						;	
defparam	nios_bts_port_core_i.bts_id5						=	  bts_id5						;	
defparam	nios_bts_port_core_i.bts_id6						=	  bts_id6						;	
defparam	nios_bts_port_core_i.bts_id7						=	  bts_id7						;	
defparam	nios_bts_port_core_i.bts_id8						=	  bts_id8						;	
defparam	nios_bts_port_core_i.board_id	    				=	  board_id	    				;
defparam	nios_bts_port_core_i.port_id						=	  port_id						;	
defparam	nios_bts_port_core_i.generic_id						=	  generic_id					;	
defparam	nios_bts_port_core_i.id_words	    				=	  id_words	    				;
defparam	nios_bts_port_core_i.enable_sdm_dip_override		=	  enable_sdm_dip_override		;
defparam	nios_bts_port_core_i.enable_edm_dip_override		=	  enable_edm_dip_override		;
                                                
nios_bts_port_core	nios_bts_port_core_i
(
    .reset_n                            (reset_n                ), 
	.en_cdet_pb                         (en_cdet_pb             ),
	.channel_bond_pb                    (channel_bond_pb        ),
	.start_pb                           (start_pb               ),
	.create_error_pb           			(create_error_pb        ),
	.xcvr_reset_pb_n                    (xcvr_reset_pb_n        ),
    .refclk_p                           (refclk_p               ),
    .tx_p    	                        (tx_p    	            ),
    .rx_p                               (rx_p                   ),
 	.cal_blk_clk_sig                    (cal_blk_clk_sig        ),
	.seriallpbken_dip                   (seriallpbken_dip       ),
    .led_tx                             (led_tx                 ),
    .led_rx     		                (led_rx     		    ),
	.rx_ss_clk_in			            (rx_ss_clk_in	        ),
    .tx_ss_clk_out       	            (tx_ss_clk_out          ),
	.plls_in_locked			            (plls_in_locked	        ),
	.plls_locked                        (plls_locked            ),
	.pattern_sync_acquired              (pattern_sync_acquired  ),
	.test_complete                      (test_complete          ),
	.error_latch                        (error_latch            ),
	.error_irq                          (error_irq              ),
	.nios_clk	                        (nios_clk	            ),
	.nios_address                       (nios_address           ),
  	.nios_chipselect                    (nios_chipselect        ),
  	.nios_read                          (nios_read              ),
  	.nios_readdata                      (nios_readdata          ),
  	.nios_reset_n                       (nios_reset_n           ),
  	.nios_write                         (nios_write             ),
 	.nios_writedata	                    (nios_writedata	        ),
	.nios_waitrequest					(nios_waitrequest		),
	.avalon_pb                          (avalon_pb              ),
	.avalon_dip                         (avalon_dip             ),
	.avalon_led							(avalon_led				)						
);  



endmodule