/*
Copyright  2006 Altera Corporation. All rights reserved.  Altera products 
are protected under numerous U.S. and foreign patents, maskwork rights, 
copyrights and other intellectual property laws.  

This reference design file, and your use thereof, is subject to and governed
by the terms and conditions of the applicable Altera Reference Design 
License Agreement.  By using this reference design file, you indicate your
acceptance of such terms and conditions between you and Altera Corporation.
In the event that you do not agree with such terms and conditions, you may
not use the reference design file. Please promptly destroy any copies you 
have made.

This reference design file being provided on an "as-is" basis and as an 
accommodation and therefore all warranties, representations or guarantees
of any kind (whether express, implied or statutory) including, without 
limitation, warranties of merchantability, non-infringement, or fitness for
a particular purpose, are specifically disclaimed.  By making this reference
design file available, Altera expressly does not recommend, suggest or 
require that this reference design file be used in combination with any 
other product not provided by Altera
*/
  
/*******************************************************************  
* NOTICE: The information contained in this file is proprietary    *
* to Altera and is being made available to Altera's customers      *
* under strict non-disclosure agreements.  Use or disclosure of    *
* this information is permissible only under the terms of          *
* existing non-disclosure agreements.                              *
********************************************************************/  

/*----------------------------------------------------------------------------
--                                                                          --
-- Title:       altera_bts.v                                                --
-- Rev:         Rev 1                                                       --
-- Modified:    October 12, 2005                                            --
-- Author:      David Johnson                                               --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Description: Altera Board Test System                                    --
-- Data Generator / Data Comparator with Avalon control bus for NIOS,       --
-- The high speed data path is Avalon Compliant, but will be connected      --
-- externally to maximize performance                                       --
-- [January 8, 2006] dtj - Added support for prbs seed & coefficients from  --
--                         BTS NIOS System                                  --
------------------------------------------------------------------------------
--                                                                          --
-- Revision History:                                                        --
-- Rev 1 - Original code                                                    --
------------------------------------------------------------------------------
*/                                                                                
module	altera_bts		(
	                                                                                                                	
	// Generator data written to external device or interface
	data_gen_reset_n,
	data_gen_clk,
	data_gen_chipselect,
	data_gen_read,
	data_gen_address,             
	data_gen_readdata,
	
	// Compare data read from external device or interface    
	data_actual_reset_n,                                                                                                                                                                                
	data_actual_clk,
	data_actual_chipselect,
	data_actual_write,
	data_actual_address,             
	data_actual_writedata,
	data_actual_waitrequest,
	             
	// Signals for Signaltap, IRQs, etc
	error_latch,
	data_error,
	test_complete,
	error_mask,
	aci_prs,
	aci_prd,
	aci_prc,
	aci_prt,
	aci_ttc,
	aci_rtc,
	aci_wec,
	aci_lec,
	aci_ips
);   
     
	assign	aci_ttc	=	data_gen_transaction_ctr; 	 // Only bottom 32 bits for now
	assign	aci_rtc	=	data_actual_transaction_ctr; // Only bottom 32 bits for now
	
	parameter	data_width							=	256;
	parameter	nios_width							=	32;
//	parameter	sync_pattern_length					=	4;		// # of patterns to capture	
//	parameter	sync_pattern_depth					=	2;		// log2(sync_pattern_length) 
	parameter	test_length_bts						=	100000;
	parameter	test_complete_ctr_width				=	64;		// > log2(test_length_bts)	                        			         
	parameter	tec_words							=	2;      // # of NIO Words for tranaction and error counters
	parameter	num_lanes							=	8;      // Number of error lanes - Scalable up to 64 lanes, due to memory reset      
	parameter	lane_width							=	data_width / num_lanes;
	parameter	enable_nios_bts						=	0;
	parameter	aci_tectr_width						=	tec_words * nios_width;	// Transaction and Error counter width 
	parameter	prbs_type							=	0;
	parameter	prbs_width							=	data_width;	// for com, set to channel_width * prbs_depth
	parameter	channel_width						=	4;
	parameter	serial_channels						=	1;
	parameter	cw_div2								=	channel_width / 2;
	parameter	cw_div4								=	channel_width / 4;
	parameter	cw_div8								=	channel_width / 8;
	parameter	hf2									=	{cw_div2{2'b10}};
	parameter	hf4									=	{cw_div4{4'b1100}};
	parameter	hf8									=	{cw_div8{8'b1111_0000}};
	parameter	lf									=	{{cw_div2{1'b1}},{cw_div2{1'b0}}};
	parameter	pcie_compliance40					=	40'b0011111010_1010101010_1100000101_0101010101;
	parameter	prbs_com							=	1;		// Create PRBS for input to serializer of SERDES or XCVR - uses more logic
	parameter	prbs_mem							=	0;		// Use full PRBS shift register as parallel data, as is
																// done in Altera's DDR2 Megacore example_driver.v - uses minimal logic


	input											data_gen_reset_n;
	input											data_gen_clk;
	input											data_gen_chipselect;
	input											data_gen_read;
	input											data_gen_address; 	// Any access will read a new prbs data word            
	output	[data_width-1:0]						data_gen_readdata;
	                                            	
	input											data_actual_reset_n;                                                                                                                                                                                
	input											data_actual_clk;
	input											data_actual_chipselect;
	input											data_actual_write;
	input											data_actual_address;             
	input	[data_width-1:0]						data_actual_writedata;
	output											data_actual_waitrequest;
															 
	// Signals for Signaltap, IRQs etc          	
	output	reg										error_latch;
	output	reg										data_error;  
	output	reg										test_complete;
	input		[data_width-1:0]					error_mask;        
	// Nios BTS                                 	
	input		[prbs_width-1:0]					aci_prs;
	input		[nios_width-1:0]					aci_prd;	
	input		[prbs_width-1:0]					aci_prc;
	input		[nios_width-1:0]					aci_prt;	
	output		[aci_tectr_width-1:0]				aci_ttc;
	output		[aci_tectr_width-1:0]				aci_rtc;
	output	reg	[aci_tectr_width-1:0]				aci_wec;
	output	reg	[(aci_tectr_width*num_lanes)-1:0] 	aci_lec; 
	input											aci_ips;
    
	
	wire		[prbs_width-1:0]					prbs_coeff;
	wire		[prbs_width-1:0]					prbs_seed;
	wire		[aci_tectr_width-1:0]				data_gen_transaction_ctr;
	wire		[aci_tectr_width-1:0]				data_actual_transaction_ctr;
	wire		[data_width-1:0]					data_actual_readdata;
	wire											data_actual_read; 
	reg			[aci_tectr_width-1:0]				aci_lec_mem				[0:num_lanes-1];
	reg			[aci_tectr_width-1:0]				aci_wec_rg1;	
	reg			[num_lanes-1:0]						lane_err;
	reg			[15:0]								data_gen_ctr;
	reg			[15:0]								data_actual_ctr;
// Avalon control register
     
 
// PRBS data generator  
    
parameter	prbs_coeff1_param		=	{32{8'h1d}};   			// coeff1-4 are maximal length coefficients for 8, 16, 32, and 128 LFSRs
parameter	prbs_coeff2_param		=	{16{16'h100b}} ;
parameter	prbs_coeff3_param		=	{8{32'hc000_0401}};
parameter	prbs_coeff4_param		=	{2{128'h0000_0000_0000_0000_0000_0000_2800_0005}};	
parameter	prbs_depth1_param		=	8;
parameter	prbs_depth2_param		=	16;
parameter	prbs_depth3_param		=	32;
parameter	prbs_depth4_param		=	128;
parameter	prbs_seed256_param		=	256'hace1_ace2_2233_f7a5_5555_2138_2128_3003_bc21_1c32_1242_e2c1_2525_3148_1124_3208;
parameter	prbs_seed64_param		=	 64'hace3_abc4_1023_1188;
parameter	prbs_seed320_param		=	{prbs_seed256_param,prbs_seed64_param};         

parameter	prbs_seed_param			=	prbs_seed320_param;
parameter	prbs_depth				=	prbs_depth3_param;
parameter	prbs_coeff_param		=	prbs_coeff3_param; // WARNING! For prbs_type=prbs_mem, set so data_width is a multiple of prbs_depth

// Internal data structures
reg		[test_complete_ctr_width-1:0]	test_complete_ctr;
wire	[nios_width-1:0]				prbs_type_ctrl;	// Dynamic control of prbs type (mem or com) - rfu (reserve for future use)
wire	[nios_width-1:0]				prbs_depth_ctrl;	// Dynamic control of prbs depth -- rfu


generate
	// Legacy before Avalon Control Interface / NIOS was part of BTS:
	if	(enable_nios_bts == 0)	assign	prbs_coeff		=	prbs_coeff_param; // fixed
	if	(enable_nios_bts == 0)	assign	prbs_seed		=	prbs_seed_param;  // fixed
	if	(enable_nios_bts == 0)	assign	prbs_type_ctrl	=	0;				  // fixed legacy memory prbs 
	if	(enable_nios_bts == 0)	assign	prbs_depth_ctrl	=	prbs_depth;		  // fixed
	// Fixed PRBS Depth and PRBS Type (mem or com) as a parameter
	// Use if a large PRBS size, or only one PRBS depth required, and only COM or MEM type
	// Also supports legacy after NIOS added to BTS, up to HSMC XCVR SDI board.  
	//if	(enable_nios_bts == 1)	assign	prbs_coeff		=	aci_prc; 	// dynamic
	//if	(enable_nios_bts == 1)	assign	prbs_seed		=	aci_prs;	// dynamic
	if	(enable_nios_bts == 1)	assign	prbs_coeff		=	prbs_coeff_param; // fixed
	if	(enable_nios_bts == 1)	assign	prbs_seed		=	prbs_seed_param;  // fixed
	if	(enable_nios_bts == 1)	assign	prbs_type_ctrl	=	prbs_type;	// fixed
	if	(enable_nios_bts == 1)	assign	prbs_depth_ctrl	=	prbs_depth;	// fixed
	// Dynamic PRBS Depth & PRBS Type -- Uses more logic
	if	(enable_nios_bts == 2)	assign	prbs_coeff		=	aci_prc; 	// dynamic
	if	(enable_nios_bts == 2)	assign	prbs_seed		=	aci_prs;	// dynamic
	if	(enable_nios_bts == 2)	assign	prbs_type_ctrl	=	aci_prt;	// dynamic
	if	(enable_nios_bts == 2)	assign	prbs_depth_ctrl	=	aci_prd;	// dynamic
	// Static 1 of 4 PRBS generators
	if	(enable_nios_bts > 2)
	begin
		assign	prbs_coeff		=	prbs_coeff_param; // fixed
		assign	prbs_seed		=	prbs_seed_param;  // fixed
		assign	prbs_type_ctrl	=	prbs_type;	// fixed
		assign	prbs_depth_ctrl	=	prbs_depth;	// fixed
	end
endgenerate

/*
// PRBS 7:
//parameter	prbs_depth						=	7;	// PRBS Shift Register Size (If enable_nios_bts=2, set to the max PRBS COM Shift Register Polynomial Size).
//parameter	prbs_width						=	serial_channels * channel_width; // Set to serial_channels * max{prbs_depth,channel_width)
//parameter	prbs_coeff_param				=	{serial_channels{7'b000_0011}};	
//parameter	prbs_seed_param					=	{serial_channels{7'b011_1010}}; // keep identical so can swap xcvr loopbacks 
// PRBS 15:
//parameter	prbs_depth						=	15;	// PRBS Shift Register Size (If enable_nios_bts=2, set to the max PRBS COM Shift Register Polynomial Size).
//parameter	prbs_width						=	serial_channels * channel_width; // Set to serial_channels * max{prbs_depth,channel_width)
//parameter	prbs_coeff_param				=	{serial_channels{15'b000_0000_0000_0011}};	
//parameter	prbs_seed_param					=	{serial_channels{15'b011_0101_1010_1011}}; // keep identical so can swap xcvr loopbacks 
// PRBS 31
//parameter	prbs_depth						=	31;	// PRBS Shift Register Size (If enable_nios_bts=2, set to the max PRBS COM Shift Register Polynomial Size).
//parameter	prbs_width						=	serial_channels * channel_width; // Set to serial_channels * max{prbs_depth,channel_width)
//parameter	prbs_coeff_param				=	{serial_channels{31'b000_0000_0000_0000_0000_0000_0000_1001}};	
//parameter	prbs_seed_param					=	{serial_channels{31'b011_0101_1010_1011_1100_0000_1010_0101}}; // keep identical so can swap xcvr loopbacks 

*/
      


// For enable_nios_bts > 2, multi-prbs generators:
wire	[data_width-1:0]	data_gen_readdata_7;
wire	[data_width-1:0]	data_gen_readdata_15;
wire	[data_width-1:0]	data_gen_readdata_23;
wire	[data_width-1:0]	data_gen_readdata_31;
wire	[data_width-1:0]	data_actual_readdata_7;
wire	[data_width-1:0]	data_actual_readdata_15;
wire	[data_width-1:0]	data_actual_readdata_23;
wire	[data_width-1:0]	data_actual_readdata_31;
reg		[data_width-1:0]	data_gen_readdata_reg;
reg		[data_width-1:0]	data_actual_readdata_reg;

generate     
	if	(enable_nios_bts > 2)
	begin                     
		// NOTE:	Use until enable_nios_bts = 2 (dynamic scalable PRBS generator is supported)
		//		This portion assumes channel_width > prbs_depth
		//		If not true, use function for max for prbs_width = serial_channels * max{prbs_depth,channel_width);
		// PRBS7:
		defparam	prbs_generator_gen_7.data_width				=	data_width;
		defparam	prbs_generator_gen_7.nios_width				=	nios_width; 
		defparam	prbs_generator_gen_7.prbs_depth				=	7;	// Maximum prbs size
		defparam	prbs_generator_gen_7.tec_words				=	tec_words;
		defparam	prbs_generator_gen_7.prbs_type				=	prbs_type;
		defparam	prbs_generator_gen_7.prbs_width				=	serial_channels * 7; //channel_width; // Set to serial_channels * max{prbs_depth,channel_width);
		defparam	prbs_generator_gen_7.channel_width			=	channel_width;
		defparam	prbs_generator_gen_7.serial_channels			=	serial_channels;
		defparam	prbs_generator_gen_7.prbs_coeff				=	{serial_channels{7'b000_0011}};
		prbs_generator	prbs_generator_gen_7		(
			//********nios side********** 
			// slave prbs interface    
			.prbs_reset_n			(data_gen_reset_n && (!aci_ips) ),
			.prbs_clk				(data_gen_clk       			),
			.prbs_chipselect  		(data_gen_chipselect			),
			.prbs_read        		(data_gen_read      			),
			.prbs_address     		(data_gen_address   			),
			.prbs_readdata    		(data_gen_readdata_7  			),
			.prbs_start				(0								),    
			.prbs_transaction_ctr   (data_gen_transaction_ctr		),
			//.prbs_coeff             ({serial_channels{7'b000_0011}}	),
			.prbs_seed              ({serial_channels{7'b011_1010}}	)
			////.prbs_type_ctrl			(prbs_com						)
			////.prbs_depth_ctrl		(7					)	// Change for smaller prbs polynomials
		);	
	
		defparam	prbs_generator_actual_7.data_width			=	data_width;
		defparam	prbs_generator_actual_7.nios_width			=	nios_width; 
		defparam	prbs_generator_actual_7.prbs_depth			=	7;
		defparam	prbs_generator_actual_7.tec_words			=	tec_words;
		defparam	prbs_generator_actual_7.prbs_type			=	prbs_type;
		defparam	prbs_generator_actual_7.prbs_width			=	serial_channels * 7; // channel_width; // Set to serial_channels * max{prbs_depth,channel_width);
		defparam	prbs_generator_actual_7.channel_width			=	channel_width;
		defparam	prbs_generator_actual_7.serial_channels			=	serial_channels;
		defparam	prbs_generator_actual_7.prbs_coeff			=	{serial_channels{7'b000_0011}};
		prbs_generator	prbs_generator_actual_7
		(
			//********nios side********** 
			// slave prbs interface    
			.prbs_reset_n			(data_actual_reset_n && (!aci_ips)	),
			.prbs_clk			(data_actual_clk       			),
			.prbs_chipselect  		(data_actual_chipselect  		),
			.prbs_read        		(data_actual_write    			),	// Writes to module should match prbs
			.prbs_address     		(data_actual_address   			),
			.prbs_readdata    		(data_actual_readdata_7  			),
			.prbs_start			(0					),    
			.prbs_transaction_ctr   	(data_actual_transaction_ctr		),
			//.prbs_coeff             	({serial_channels{7'b000_0011}}		),
			.prbs_seed              	({serial_channels{7'b011_1010}}		)
			////prbs_type_ctrl			(prbs_com						)
			////.prbs_depth_ctrl		(7					)
		);        

		// PRBS15:
		defparam	prbs_generator_gen_15.data_width				=	data_width;
		defparam	prbs_generator_gen_15.nios_width				=	nios_width; 
		defparam	prbs_generator_gen_15.prbs_depth				=	15;	// Maximum prbs size
		defparam	prbs_generator_gen_15.tec_words					=	tec_words;
		defparam	prbs_generator_gen_15.prbs_type					=	prbs_type;
		defparam	prbs_generator_gen_15.prbs_width				=	serial_channels * 15; // channel_width; // Set to serial_channels * max{prbs_depth,channel_width);
		defparam	prbs_generator_gen_15.channel_width				=	channel_width;
		defparam	prbs_generator_gen_15.serial_channels				=	serial_channels;
		defparam	prbs_generator_gen_15.prbs_coeff				=	{serial_channels{15'b000_0000_0000_0011}};

		prbs_generator	prbs_generator_gen_15		(
			//********nios side********** 
			// slave prbs interface    
			.prbs_reset_n			(data_gen_reset_n && (!aci_ips) ),
			.prbs_clk			(data_gen_clk       				),
			.prbs_chipselect  		(data_gen_chipselect				),
			.prbs_read        		(data_gen_read      				),
			.prbs_address     		(data_gen_address   				),
			.prbs_readdata    		(data_gen_readdata_15				),
			.prbs_start			(0						),    
			//.prbs_transaction_ctr   	(data_gen_transaction_ctr			),
			//.prbs_coeff             	({serial_channels{15'b000_0000_0000_0011}}	),
			.prbs_seed              	({serial_channels{15'b011_0101_1010_1011}}	)
			////.prbs_type_ctrl			(prbs_com						)
			////.prbs_depth_ctrl		(15						)	// Change for smaller prbs polynomials
		);	
	
		defparam	prbs_generator_actual_15.data_width			=	data_width;
		defparam	prbs_generator_actual_15.nios_width			=	nios_width; 
		defparam	prbs_generator_actual_15.prbs_depth			=	15;
		defparam	prbs_generator_actual_15.tec_words			=	tec_words;
		defparam	prbs_generator_actual_15.prbs_type			=	prbs_type;
		defparam	prbs_generator_actual_15.prbs_width			=	serial_channels * 15; // channel_width; // Set to serial_channels * max{prbs_depth,channel_width);
		defparam	prbs_generator_actual_15.channel_width			=	channel_width;
		defparam	prbs_generator_actual_15.serial_channels		=	serial_channels;
		defparam	prbs_generator_actual_15.prbs_coeff			=	{serial_channels{15'b000_0000_0000_0011}};
		prbs_generator	prbs_generator_actual_15
		(
			//********nios side********** 
			// slave prbs interface    
			.prbs_reset_n			(data_actual_reset_n && (!aci_ips)		),
			.prbs_clk			(data_actual_clk       				),
			.prbs_chipselect  		(data_actual_chipselect  			),
			.prbs_read        		(data_actual_write    				),	// Writes to module should match prbs
			.prbs_address     		(data_actual_address   				),
			.prbs_readdata    		(data_actual_readdata_15			),
			.prbs_start			(0						),    
			//.prbs_transaction_ctr   	(data_actual_transaction_ctr			),
			//.prbs_coeff             	({serial_channels{15'b000_0000_0000_0011}}	),
			.prbs_seed              	({serial_channels{15'b011_0101_1010_1011}}	)
			////.prbs_type_ctrl			(prbs_com						)
			////.prbs_depth_ctrl		(15						)
		);        
 
		// PRBS23:
		defparam	prbs_generator_gen_23.data_width				=	data_width;
		defparam	prbs_generator_gen_23.nios_width				=	nios_width; 
		defparam	prbs_generator_gen_23.prbs_depth				=	23;	// Maximum prbs size
		defparam	prbs_generator_gen_23.tec_words					=	tec_words;
		defparam	prbs_generator_gen_23.prbs_type					=	prbs_type;
		defparam	prbs_generator_gen_23.prbs_width				=	serial_channels * 23; // channel_width; // Set to serial_channels * max{prbs_depth,channel_width);
		defparam	prbs_generator_gen_23.channel_width				=	channel_width;
		defparam	prbs_generator_gen_23.serial_channels				=	serial_channels;
		defparam	prbs_generator_gen_23.prbs_coeff				=	{serial_channels{23'b000_0000_0000_0000_0010_0001}};
		prbs_generator	prbs_generator_gen_23		(
			//********nios side********** 
			// slave prbs interface    
			.prbs_reset_n			(data_gen_reset_n && (!aci_ips) ),
			.prbs_clk			(data_gen_clk       					),
			.prbs_chipselect  		(data_gen_chipselect					),
			.prbs_read        		(data_gen_read      					),
			.prbs_address     		(data_gen_address   					),
			.prbs_readdata    		(data_gen_readdata_23					),
			.prbs_start			(0							),    
			//.prbs_transaction_ctr   	(data_gen_transaction_ctr				),
			//.prbs_coeff             	({serial_channels{23'b000_0000_0000_0000_0010_0001}}	),
			.prbs_seed              	({serial_channels{23'b011_0101_1010_1011_1111_0000}}	)
			////.prbs_type_ctrl			(prbs_com						)
			////.prbs_depth_ctrl		(23							)	// Change for smaller prbs polynomials
		);	
	
		defparam	prbs_generator_actual_23.data_width			=	data_width;
		defparam	prbs_generator_actual_23.nios_width			=	nios_width; 
		defparam	prbs_generator_actual_23.prbs_depth			=	23;
		defparam	prbs_generator_actual_23.tec_words			=	tec_words;
		defparam	prbs_generator_actual_23.prbs_type			=	prbs_type;
		defparam	prbs_generator_actual_23.prbs_width			=	serial_channels * 23; // channel_width; // Set to serial_channels * max{prbs_depth,channel_width);
		defparam	prbs_generator_actual_23.channel_width			=	channel_width;
		defparam	prbs_generator_actual_23.serial_channels		=	serial_channels;
		defparam	prbs_generator_actual_23.prbs_coeff			=	{serial_channels{23'b000_0000_0000_0000_0010_0001}};
		prbs_generator	prbs_generator_actual_23
		(
			//********nios side********** 
			// slave prbs interface    
			.prbs_reset_n			(data_actual_reset_n && (!aci_ips)			),
			.prbs_clk			(data_actual_clk       					),
			.prbs_chipselect  		(data_actual_chipselect  				),
			.prbs_read        		(data_actual_write    					),	// Writes to module should match prbs
			.prbs_address     		(data_actual_address   					),
			.prbs_readdata    		(data_actual_readdata_23					),
			.prbs_start			(0							),    
			//.prbs_transaction_ctr   	(data_actual_transaction_ctr				),
			//.prbs_coeff             	({serial_channels{23'b000_0000_0000_0000_0010_0001}}	),
			.prbs_seed              	({serial_channels{23'b011_0101_1010_1011_1111_0000}}	)
			////.prbs_type_ctrl			(prbs_com						)
			////.prbs_depth_ctrl		(23							)
		);        
   
		// PRBS31:
		defparam	prbs_generator_gen_31.data_width				=	data_width;
		defparam	prbs_generator_gen_31.nios_width				=	nios_width; 
		defparam	prbs_generator_gen_31.prbs_depth				=	31;	// Maximum prbs size
		defparam	prbs_generator_gen_31.tec_words					=	tec_words;
		defparam	prbs_generator_gen_31.prbs_type					=	prbs_type;
		defparam	prbs_generator_gen_31.prbs_width				=	serial_channels * 31; // channel_width; // Set to serial_channels * max{prbs_depth,channel_width);
		defparam	prbs_generator_gen_31.channel_width				=	channel_width;
		defparam	prbs_generator_gen_31.serial_channels				=	serial_channels;
		defparam	prbs_generator_gen_31.prbs_coeff				=	{serial_channels{31'b000_0000_0000_0000_0000_0000_0000_1001}};
		prbs_generator	prbs_generator_gen_31		(
			//********nios side********** 
			// slave prbs interface    
			.prbs_reset_n			(data_gen_reset_n && (!aci_ips) ),
			.prbs_clk			(data_gen_clk       						),
			.prbs_chipselect  		(data_gen_chipselect						),
			.prbs_read        		(data_gen_read      						),
			.prbs_address     		(data_gen_address   						),
			.prbs_readdata    		(data_gen_readdata_31						),
			.prbs_start			(0								),    
			//.prbs_transaction_ctr   	(data_gen_transaction_ctr					),
			//.prbs_coeff             	({serial_channels{31'b000_0000_0000_0000_0000_0000_0000_1001}}	),
			.prbs_seed              	({serial_channels{31'b011_0101_1010_1011_1100_0000_1010_0101}}	)
			////.prbs_type_ctrl			(prbs_com						)
			////.prbs_depth_ctrl		(31								)	// Change for smaller prbs polynomials
		);	
	
		defparam	prbs_generator_actual_31.data_width			=	data_width;
		defparam	prbs_generator_actual_31.nios_width			=	nios_width; 
		defparam	prbs_generator_actual_31.prbs_depth			=	31;
		defparam	prbs_generator_actual_31.tec_words			=	tec_words;
		defparam	prbs_generator_actual_31.prbs_type			=	prbs_type;
		defparam	prbs_generator_actual_31.prbs_width			=	serial_channels * 31; //  channel_width; // Set to serial_channels * max{prbs_depth,channel_width);
		defparam	prbs_generator_actual_31.channel_width			=	channel_width;
		defparam	prbs_generator_actual_31.serial_channels		=	serial_channels;
		defparam	prbs_generator_actual_31.prbs_coeff			=	{serial_channels{31'b000_0000_0000_0000_0000_0000_0000_1001}};
		prbs_generator	prbs_generator_actual_31
		(
			//********nios side********** 
			// slave prbs interface    
			.prbs_reset_n			(data_actual_reset_n && (!aci_ips)				),
			.prbs_clk			(data_actual_clk       						),
			.prbs_chipselect  		(data_actual_chipselect  					),
			.prbs_read        		(data_actual_write    						),	// Writes to module should match prbs
			.prbs_address     		(data_actual_address   						),
			.prbs_readdata    		(data_actual_readdata_31						),
			.prbs_start			(0								),    
			//.prbs_transaction_ctr   	(data_actual_transaction_ctr					),
			//.prbs_coeff             	({serial_channels{31'b000_0000_0000_0000_0000_0000_0000_1001}}	),
			.prbs_seed              	({serial_channels{31'b011_0101_1010_1011_1100_0000_1010_0101}}	)
			////.prbs_type_ctrl			(prbs_com						)
			////.prbs_depth_ctrl		(31								)
		);  
		
		
		always	@(posedge data_gen_clk or negedge data_gen_reset_n)
		begin
			if	(!data_gen_reset_n)
			begin
				data_gen_ctr	<=	0;
			end else begin
				data_gen_ctr	<=	data_gen_ctr + 1;
			end
		end

		always	@(posedge data_actual_clk or negedge data_actual_reset_n)
		begin
			if	(!data_actual_reset_n)
			begin
				data_actual_ctr	<=	0;
			end else begin
				data_actual_ctr	<=	data_actual_ctr + 1;
			end
		end


		always @(*)
		begin
			case(aci_prt) // Note: defaults to prbs_type (0=bts_mem, 1=bts_com)
				0:			begin
								data_gen_readdata_reg		=	data_gen_readdata_7;
								data_actual_readdata_reg	=	data_actual_readdata_7;
							end
				1:			begin
								data_gen_readdata_reg		=	data_gen_readdata_15;
								data_actual_readdata_reg	=	data_actual_readdata_15;
							end	
				2:			begin
								data_gen_readdata_reg		=	data_gen_readdata_23;
								data_actual_readdata_reg	=	data_actual_readdata_23;
							end	
				3:			begin
								data_gen_readdata_reg		=	data_gen_readdata_31;
								data_actual_readdata_reg	=	data_actual_readdata_31;
							end
				4:			begin	// High Frequency
								data_gen_readdata_reg		=	{serial_channels{hf2}};
								data_actual_readdata_reg	=	{serial_channels{hf2}};
							end
				5:			begin	// High Frequency / 2
								data_gen_readdata_reg		=	{serial_channels{hf4}};
								data_actual_readdata_reg	=	{serial_channels{hf4}};
							end
				6:			begin	// High Frequency / 4
								data_gen_readdata_reg		=	{serial_channels{hf8}};
								data_actual_readdata_reg	=	{serial_channels{hf8}};
							end
				7:			begin	// Low Frequency
								data_gen_readdata_reg		=	{serial_channels{lf}};
								data_actual_readdata_reg	=	{serial_channels{lf}};
							end
				40:			begin
								if	(channel_width == 20)
								begin
									case	(data_gen_ctr[0])
										0:		data_gen_readdata_reg		=	{serial_channels{pcie_compliance40[39:20]}};
										1:		data_gen_readdata_reg		=	{serial_channels{pcie_compliance40[19:0]}};
										default;
									endcase
									case	(data_actual_ctr[0])
										0:		data_actual_readdata_reg		=	{serial_channels{pcie_compliance40[39:20]}};
										1:		data_actual_readdata_reg		=	{serial_channels{pcie_compliance40[19:0]}};
										default;
									endcase
								end								
								if	(channel_width == 40)
								begin
									data_gen_readdata_reg		=	{serial_channels{pcie_compliance40}};
									data_actual_readdata_reg	=	{serial_channels{pcie_compliance40}};
								end								
							end	
				default:	begin
								data_gen_readdata_reg		=	data_gen_readdata_7;
								data_actual_readdata_reg	=	data_actual_readdata_7;
							end				
			endcase
		end

		
		assign		data_gen_readdata		=	data_gen_readdata_reg;
		assign		data_actual_readdata	=	data_actual_readdata_reg;		
		
	end else begin
		defparam	prbs_generator_gen.data_width				=	data_width;
		defparam	prbs_generator_gen.nios_width				=	nios_width; 
		defparam	prbs_generator_gen.prbs_depth				=	prbs_depth;	// Maximum prbs size
		defparam	prbs_generator_gen.tec_words				=	tec_words;
		defparam	prbs_generator_gen.prbs_type				=	prbs_type;
		defparam	prbs_generator_gen.prbs_width				=	prbs_width;
		defparam	prbs_generator_gen.channel_width			=	channel_width;
		defparam	prbs_generator_gen.serial_channels			=	serial_channels;
		defparam	prbs_generator_gen.prbs_coeff				=	prbs_coeff_param;
		prbs_generator	prbs_generator_gen		(
			//********nios side********** 
			// slave prbs interface    
			.prbs_reset_n			(data_gen_reset_n && (!aci_ips) ),
			.prbs_clk			(data_gen_clk       			),
			.prbs_chipselect  		(data_gen_chipselect			),
			.prbs_read        		(data_gen_read      			),
			.prbs_address     		(data_gen_address   			),
			.prbs_readdata    		(data_gen_readdata  			),
			.prbs_start			(0					),    
			.prbs_transaction_ctr   	(data_gen_transaction_ctr		),
			//.prbs_coeff             	(prbs_coeff				),
			.prbs_seed              	(prbs_seed				)
			////.prbs_type_ctrl			(prbs_type_ctrl				)
			////.prbs_depth_ctrl		(prbs_depth_ctrl			)	// Change for smaller prbs polynomials
		);	
	
		defparam	prbs_generator_actual.data_width			=	data_width;
		defparam	prbs_generator_actual.nios_width			=	nios_width; 
		defparam	prbs_generator_actual.prbs_depth			=	prbs_depth;
		defparam	prbs_generator_actual.tec_words				=	tec_words;
		defparam	prbs_generator_actual.prbs_type				=	prbs_type;
		defparam	prbs_generator_actual.prbs_width			=	prbs_width;
		defparam	prbs_generator_actual.channel_width			=	channel_width;
		defparam	prbs_generator_actual.serial_channels			=	serial_channels;
		defparam	prbs_generator_actual.prbs_coeff			=	prbs_coeff_param;
		prbs_generator	prbs_generator_actual
		(
			//********nios side********** 
			// slave prbs interface    
			.prbs_reset_n			(data_actual_reset_n && (!aci_ips)	),
			.prbs_clk			(data_actual_clk       			),
			.prbs_chipselect  		(data_actual_chipselect  		),
			.prbs_read        		(data_actual_write    			),	// Writes to module should match prbs
			.prbs_address     		(data_actual_address   			),
			.prbs_readdata    		(data_actual_readdata  			),
			.prbs_start			(0					),    
			.prbs_transaction_ctr   	(data_actual_transaction_ctr		),
			//.prbs_coeff             	(prbs_coeff				),
			.prbs_seed              	(prbs_seed				)
			////.prbs_type_ctrl			(prbs_type_ctrl				)
			////.prbs_depth_ctrl		(prbs_depth_ctrl			)
		);
	end
endgenerate

// Comparator
reg	[data_width-1:0]	data_actual_writedata_pl1;
reg				data_actual_write_pl1;
reg				data_actual_chipselect_pl1;

always	@(posedge data_actual_clk or negedge data_actual_reset_n)
begin
	if	(!data_actual_reset_n)
	begin
		error_latch					<=	0;	
		data_error					<=	0;   
		data_actual_writedata_pl1	<=	0;
		data_actual_write_pl1		<=	0;
		data_actual_chipselect_pl1	<=	0;
		test_complete_ctr			<=	test_length_bts;
		test_complete				<=	0; 
		aci_wec_rg1					<=	0;				 
		aci_wec						<=	0; 	// Pipeline to line up with aci_lec 
		aci_lec						<=	0;
		lane_err					<=	0;
	end else begin
		if	(data_actual_write_pl1 && data_actual_chipselect_pl1)
		begin
			if	(test_complete_ctr == 0)
			begin
				test_complete		<=	1;
			end else begin
				test_complete_ctr	<=	test_complete_ctr - 1;
			end
			data_error				<=	((data_actual_readdata | error_mask) != (data_actual_writedata_pl1 | error_mask));
			if	((data_actual_readdata | error_mask) != (data_actual_writedata_pl1 | error_mask))
			begin                                                                         
				error_latch				<=	1;
			end	
			// Word error counter:
			if	((data_actual_readdata | error_mask) != (data_actual_writedata_pl1 | error_mask))
			begin   
				//if	(aci_wec_rg1 != {aci_tectr_width{1'b1}})	// Don't rollover
				begin	
					aci_wec_rg1			<=	aci_wec_rg1 + 1;
				end
			end	     
			lane_error_counter;
		end
		aci_wec						<=	aci_wec_rg1;	// Pipeline to match aci_lec
		//lane_error_counter_to_aci;
		data_actual_writedata_pl1	<=	data_actual_writedata;
		data_actual_write_pl1		<=	data_actual_write;
		data_actual_chipselect_pl1	<=	data_actual_chipselect;
	end
end

task	lane_error_counter;
	integer											A,B,C,D,E;
	integer											MPTR1;
	integer											MPTR2;
	reg			[aci_tectr_width-1:0]				aci_lec_reg;   
	reg			[num_lanes-1:0]						lane_err_w;			
	begin
		// Lane error counter 
		lane_err_w	=	0; 
		// Detect Error 
		lane_err_w = 0;
		for (A=0; A<num_lanes; A=A+1)
		begin
			for (B=A*lane_width; B<(A+1)*lane_width; B=B+1)
			begin
				if ((data_actual_readdata[B] | error_mask[B]) != (data_actual_writedata_pl1[B] | error_mask[B]))
				begin						
					 lane_err_w[A]	=	1;
				end																				
			end	
		end					
		lane_err	<=	lane_err_w;	// Pipeline  
		MPTR1		=	0;
		MPTR2		=	0;
		for (C=0; C<num_lanes; C=C+1)
		begin  
			for (D=0; D<aci_tectr_width; D=D+1)     
			begin 
				aci_lec_reg[D]	=	aci_lec[MPTR1];
				MPTR1			=	MPTR1 + 1;
			end  
			if	((lane_err[C]==1)) //  && (aci_lec_reg != {aci_tectr_width{1'b1}}))	// Don't rollover
			begin
				aci_lec_reg	=	aci_lec_reg + 1;
			end						
			for (E=0; E<aci_tectr_width; E=E+1)     
			begin 
				aci_lec[MPTR2]	<=	aci_lec_reg[E];
				MPTR2			=	MPTR2 + 1;
			end 
		end
	end		 
endtask

endmodule       
