//------ 1 ------- 2 ------- 3 ------- 4 ------- 5 ------- 6 ------- 7 ------7
//------ 0 ------- 0 ------- 0 ------- 0 ------- 0 ------- 0 ------- 0 ------8
//----------------------------------------------------------------------------
//Copyright  2008 Altera Corporation. All rights reserved.  Altera products  
//are protected under numerous U.S. and foreign patents, maskwork rights,     
//copyrights and other intellectual property laws.                            
//                                                                            
//This reference design file, and your use thereof, is subject to and         
//governed by the terms and conditions of the applicable Altera Reference     
//Design License Agreement.  By using this reference design file, you         
//indicate your acceptance of such terms and conditions between you and       
//Altera Corporation.  In the event that you do not agree with such terms and 
//conditions, you may not use the reference design file. Please promptly      
//destroy any copies you have made.                                           
//                                                                            
//This reference design file being provided on an "as-is" basis and as an     
//accommodation and therefore all warranties, representations or guarantees   
//of any kind (whether express, implied or statutory) including, without      
//limitation, warranties of merchantability, non-infringement, or fitness for 
//a particular purpose, are specifically disclaimed.  By making this          
//reference design file available, Altera expressly does not recommend,       
//suggest or require that this reference design file be used in combination   
//with any other product not provided by Altera.                              
//                                                                            
//-----------------------------------------------------------------------------------------//
//                                                                                         //
// Title:       prbs_generator.v                                                           //
// Rev:         Rev 2                                                                      //
// Created:     November 14, 2008                                                          //
// Author:      San Diego High Speed Design Group                                          //
//                                                                                         //
//-----------------------------------------------------------------------------------------//
//                                                                                         //
// Description:	PRBS Generators					                                           //
//             If concatenating multiple PRBS generators:                                  //
//                 A) prbs_seed initializes each prbs generator with a                     //
//                    unique seed, based on the bit range for that generator               //
//                    Concatenate the seeds for Galois Feedback Tap Notation               //
//                 B) prbs_coeff are unique coefficients for each prbs                     //
//                    generator.  Concatenate the coefficents for Galois                   //
//                    Feedback Tap Notation                                                //
//             All PRBS generators have the same size shift register, set                  //
//             by the prbs_depth_ctrl.  Uses Galois Feedback implementation,               //
//             because Galois has better distribution of logic vs Fibonaccci               //
//                                                                                         //
//       prbs_depth      PRBS shift register max size parameter				               //
//       prbs_depth_ctrl Input can be changed by NIOS for other PRBS lengths               //
//       prbs_seed       Input sets the prbs shift register's initial value,               //
//                       different bit ranges initialize different PRBSs                   //
//                       generators                                                        //
//       prbs_coeff      Input sets the prbs feedback coefficients,                        //
//                       different bit ranges for different PRBSs                          //
//       prbs_type_ctrl  Input chooses prbs_com or prbs_mem                                //
//       prbs_width      Sets the register size of the prbs coefficients &                 //
//                       prbs seed                                                         //
//     	                                                                                   //
//	PRBS MEM is similar to the ddr2 megacore's example_driver.  The outputs of             //
//	all shift registers are output as PRBS data, and multiple prbs generates are           //
//	concatenated across the parallel bus.                                                  //
//                                                                                         //
//	PRBS COM generates PRBS for parallel input to a serializer                             //
//	Assumes serializer outputs MSB 1st (True for LVDS SERDES).                             //
//	Typically set "convert_endian=1" and "flip_data_bits=1" at top level if lsb            //
//	serialized 1st (eg XCVR).  A real time scope on the serial data stream can be used     //
//  to confirm the LFSR pattern length.                                                    //  
//	                                                                                       //  
//	enable_nios_bts=0:	Pre-NIOS / Avalon Control Interface.  Uses PRBS MEM only           //
//							Generates PRBS similar to the "example_driver" included        //
//							with DDR2 Megacores. All values of the PRBS shift              //
//							register are output in parallel.  PRBS generators              //
//							are concatenated across the parallel data bus.				   //			
//							prbs_depth:		SR PRBS polynomial size.                       //
//							prbs_type:		Don't care, acts like =prbs_mem                //
//							prbs_width:		Set = serial_channels*channel_width            //
//							eg:  serial_channels=4, channel_width=32, prbs_depth=8         //
//								 (4*32/8)=16, so 16 prbs8 generators are concatenated      //
//								 Concatenated seeds and coefficients are set by            //
//								 prbs_seed and prbs_coeff, respectively.                   //
//								 (channel_width * serial_channels / prbs_depth)            //
//								 should be an integer                                      //
//								 			 					                           //
//	enable_nios_bts=1:	Parameters select either PRBS MEM or PRBS COM                      // 
//						Creates a separate PRBS generator for each serial channel          //
//							prbs_type:		Set to either prbs_com or prbs_mem             //
//							prbs_depth:		If prbs_type=prbs_mem:                         //
//										     Set = PRBS MEM shift register size and also, //
//											 (channel_width * serial_channels / prbs_depth)//
//								              should be an integer                         //
//											If prbs_type=prbs_com:                         //
//                                           Set = PRBS COM shift register size            //
//							prbs_width:		If prbs_type = prbs_mem:                       //
//                                           Set = (channel_width * serial_channels)       //
//											if prbs_type = prbs_com:                       //
//											 Set = (prbs_depth * serial_channels)          //
//	enable_nios_bts=2:	RFU (Reserved for future use)
//						NIOS dynamically controls PRBS Type (com or mem) and               //
//						PRBS Depth.  Both PRBS COM and PRBS MEM are selectable             //
//						dynamically by NIOS after Quartus compilation.                     //
//							prbs_depth:	 	The PRBS Shift Register size for PRBS COM      //
//											for the prbs com.                              //
//											Also is the default prbs depth                 //
//											out of reset for both mem and com,             //
//											then uses aci_prd controlled by NIOS.          //
//											NOTE: If prbs_type=prbs_mem,                   //
//											(channel_width * serial_channels) / prbs_depth //
//											should be an integer                           //
//											Likewise, for NIOS control if aci_prt=prbs_mem,//
//											(channel_width * serial_channels) / aci_prd    //
//											should be an integer                           //
//							prbs_type:		Set to prbs_com or prbs_mem.                  //
//                                          Default out of reset, then uses aci_prt       //
//											controlled by NIOS                            //
//							prbs_width:		Set = max(prbs_depth,channel_width)           //
//											* serial_channels)                            //
//											Width of the aci_prs and aci_prc              //
//											registers, make large enough for 	          //
//											either mem or com prbs generator types        //  
//                                                                                        //
//		enable_nios_bts=3: Same as enable_nios_bts=2, except prbs_depth and prbs_type and prbs_coeff and prbs_depth are 
//								 parameters, not dynamically changable from NIOS.
//----------------------------------------------------------------------------------------//
//                                                                                        //
// Revision History:                                                                      //
//                                                                                        //
// Rev 1:       First cut from tcl-script output  							              //
// Rev 2:		PRBS COM or MEM with coefficient, size, and seed controlled by NIOS       //              					                        				            // 
//----------------------------------------------------------------------------------------//   

module	prbs_generator	(
							// prbs generator interface 
							// Avalon type signals
							prbs_reset_n,  
							prbs_clk,
							prbs_chipselect,
							prbs_read,                                                                                                                                                        
							prbs_address,             
							prbs_readdata,
							// General signals  
							prbs_start,
							prbs_transaction_ctr,
							//prbs_coeff,
							prbs_seed
							//prbs_type_ctrl,
							//prbs_depth_ctrl		   
						);
   						
parameter	data_width				=	320;
parameter	nios_width				=	32; 			                                  
parameter	prbs_depth				=	32;
parameter	tec_words				=	2;      // # of NIO Words for tranaction and error counters
parameter	aci_tectr_width			=	tec_words * nios_width;	// Transaction and Error counter width 
parameter	prbs_com				=	1;		// Create PRBS for input to serializer of SERDES or XCVR - uses more logic
parameter	prbs_mem				=	0;		// Use full PRBS shift register as parallel data, as is
												// done in Altera's DDR2 Megacore example_driver.v - uses minimal logic
parameter	prbs_type				=	prbs_mem;
parameter	prbs_width				=	prbs_depth * serial_channels;
parameter	prbs_width_com			=	prbs_depth * serial_channels;                      
parameter	channel_width			=	1;
parameter	serial_channels			=	1;
	
parameter	prbs_type_ctrl				=	prbs_type;  // Driven from input port - RFU (Reserved for future use)
parameter	prbs_depth_ctrl				=	prbs_depth;	// Driven from input port - RFU (Reserved for future use) 
// Forced = 1 taps g0 thru gm        
/*          
Following are LFSRs for 8, 16, 32, and 128 
Concatenated for data_width = 256 
parameter	prbs_coeff1				=	{32{8'h1d};   			// coeff1-4 are maximal length coefficients for 8, 16, 32, and 128 LFSRs
parameter	prbs_coeff2				=	{16{16'h100b}} ;
parameter	prbs_coeff3				=	{8{32'hc000_0401}};
parameter	prbs_coeff4				=	{2{128'h0000_0000_0000_0000_0000_0000_2800_0005}};	
parameter	prbs1_depth				=	8;
parameter	prbs2_depth				=	16;
parameter	prbs3_depth				=	32;
parameter	prbs4_depth				=	128;
parameter	prbs_seed				=	256'hace1_ace2_2233_f7a5_5555_2138_2128_3003_bc21_1c32_1242_e2c1_2525_3148_1124_3208;
*/
parameter	prbs_coeff				=	{serial_channels{8'h1d}};
														                	          
input								prbs_reset_n;
input								prbs_clk;
input								prbs_chipselect;
input								prbs_read;
input								prbs_address;             
output	reg	[data_width-1:0]		prbs_readdata; 
input								prbs_start;  
output	reg	[aci_tectr_width-1:0]	prbs_transaction_ctr;
//input	[prbs_width-1:0]			prbs_coeff;

input	[prbs_width-1:0]			prbs_seed;

//input	[nios_width-1:0]			prbs_type_ctrl;
//input	[nios_width-1:0]			prbs_depth_ctrl;

// Internal data structures: 
reg		[data_width-1:0]			prbs_sr_mem;
reg		[data_width-1:0]			prbs_sr_mem_w;
reg		[prbs_width_com-1:0]		prbs_sr_com;
reg		[prbs_width_com-1:0]		prbs_sr_com_w;
reg		[data_width-1:0]			prbs_serdes;  // width = (channel_width * serial_channels)                             

	always	@(posedge prbs_clk or negedge prbs_reset_n)
	begin
		if	(!prbs_reset_n)
		begin
			prbs_init;
		end else begin   
			if	(prbs_start)
			begin
				prbs_init;
			end else begin
				if	(prbs_chipselect && prbs_read)	// Should match PRBS read below
				begin    				
					case	(prbs_type_ctrl)
						prbs_mem:	begin      
										build_prbs_mem;
										prbs_readdata	<=	prbs_sr_mem;														
									end
						prbs_com:	begin
										build_prbs_com;
										prbs_readdata	<=	prbs_serdes;														
									end
						default:	;
					endcase
					//if	(prbs_transaction_ctr != {aci_tectr_width{1'b1}}) // Don't rollover
					begin 
						prbs_transaction_ctr 		<=	prbs_transaction_ctr + 1;
					end
				end else begin	// No "else" section will hold the prbs data if not reading
								// This is needed due to pipeline in tx_bts_wrapper controlled by tx_ready
					//prbs_readdata				<=	0;
				end
			end
		end
	end
	
	task	prbs_init;
		begin
			prbs_sr_com						<=	prbs_seed;	// PRBS Init
			prbs_sr_mem						<=	prbs_seed;
			prbs_transaction_ctr			<=	0; 
			prbs_readdata					<=	0; 
		end
	endtask

	generate
		if	(prbs_type == prbs_com)
		begin  
			task	build_prbs_com;
			reg		[prbs_width_com-1:0]		prbs_sr_com_v;
			integer		A;
			integer		B;
			integer		C;
			integer		E;	
				begin
					prbs_sr_com_v			=	prbs_sr_com;
					for (C=0; C<channel_width; C=C+1)
					begin
						prbs_sr_com_w[0]	=	0	^	(prbs_sr_com_v[prbs_depth_ctrl-1] & prbs_coeff[0]); // coeff[0] must be 1
						B	=	prbs_depth_ctrl;
						for	(A=1; A<=prbs_width_com-1; A=A+1)
						begin
							if	((A % prbs_depth_ctrl) == 0)
							begin
								prbs_sr_com_w[A]	=	0	^	(prbs_sr_com_v[A+prbs_depth_ctrl-1] & prbs_coeff[A]); // coeff[0] must be 1
								B					=	B	+	prbs_depth_ctrl;
							end else begin
								prbs_sr_com_w[A]	=	prbs_sr_com_v[A-1] ^ (prbs_sr_com_v[B-1] & prbs_coeff[A]);
							end
						end
						prbs_sr_com_v	=	prbs_sr_com_w;
						for (E=0; E<serial_channels; E=E+1)
						begin	// Output msb 1st
							prbs_serdes[(E*channel_width)+(channel_width-C-1)] = prbs_sr_com_v[(E*prbs_depth_ctrl)+(prbs_depth_ctrl-1)];
						end
						prbs_sr_com	<=	prbs_sr_com_w;
					end
				end        
			endtask
		end else begin
			task	build_prbs_com;
				begin
					prbs_serdes	=	0;
				end
			endtask
		end
	endgenerate
	
	
	generate
		if	(prbs_type == prbs_mem)
		begin  
			task	build_prbs_mem;
			integer		A;
			integer		B;	
				begin
					prbs_sr_mem_w[0]	=	0	^	(prbs_sr_mem[prbs_depth_ctrl-1] & prbs_coeff[0]); // coeff[0] must be 1
					B	=	prbs_depth_ctrl;
					for	(A=1; A<=data_width-1; A=A+1)
					//for	(A=1; A<=prbs_width-1; A=A+1)
					begin
						if	((A % prbs_depth_ctrl) == 0)
						begin
							prbs_sr_mem_w[A]	=	0	^	(prbs_sr_mem[A+prbs_depth_ctrl-1] & prbs_coeff[A]); // coeff[0] must be 1
							B			=	B	+	prbs_depth_ctrl;
						end else begin
							prbs_sr_mem_w[A]	=	prbs_sr_mem[A-1] ^ (prbs_sr_mem[B-1] & prbs_coeff[A]);
						end
					end
					prbs_sr_mem	<=	prbs_sr_mem_w;
				end        
			endtask 
		end else begin
			task	build_prbs_mem;
				begin
					prbs_sr_mem	<=	0;
				end
			endtask
		end
	endgenerate


endmodule       
