//
//Copyright  2006 Altera Corporation. All rights reserved.  Altera products 
//are protected under numerous U.S. and foreign patents, maskwork rights, 
//copyrights and other intellectual property laws.  
//
//This reference design file, and your use thereof, is subject to and governed
//by the terms and conditions of the applicable Altera Reference Design 
//License Agreement.  By using this reference design file, you indicate your
//acceptance of such terms and conditions between you and Altera Corporation.
//In the event that you do not agree with such terms and conditions, you may
//not use the reference design file. Please promptly destroy any copies you 
//have made.
//
//This reference design file being provided on an "as-is" basis and as an 
//accommodation and therefore all warranties, representations or guarantees
//of any kind (whether express, implied or statutory) including, without 
//limitation, warranties of merchantability, non-infringement, or fitness for
//a particular purpose, are specifically disclaimed.  By making this reference
//design file available, Altera expressly does not recommend, suggest or 
//require that this reference design file be used in combination with any 
//other product not provided by Altera

/* NOTICE: The information contained in this file is proprietary   *
* to Altera and is being made available to Altera's customers      *
* under strict non-disclosure agreements.  Use or disclosure of    *
* this information is permissible only under the terms of          *
* existing non-disclosure agreements.                              *
********************************************************************/

/*------------------------------------------------------------------------------------------
--                                                                                        --
-- Title:       tx_wrapper_bts.v                                                          --
-- Rev:         Rev 1                                                                     --
-- Modified:    1/12/2006                                                                 --
-- Author:      David Johnson                                                             --
--                                                                                        --
--------------------------------------------------------------------------------------------
-- Wrapper for the TX Transceiver and Source Synchronous Serializer Portion of the BTS    --
-- Wrapper allows for pll lock, generates word alignment, channel bonding,				  --
-- and error creation       															  --
--                                                                                        --
-- After "reset" is asserted, outputs repeating dc balanced pattern + bit align pattern   --                                                                                        --
-- After "start" is asserted, outputs data created by the data generator                  --
--------------------------------------------------------------------------------------------
--                                                                                        --
-- Revision History:                                                                      --
--                                                                                        --
-- Rev 1:       Created                                                                   --
--                                                                                        --
--------------------------------------------------------------------------------------------
*/


module tx_wrapper_bts	(
                         	reset_n,			
							clk,				
							start,				
							create_error,
							tx_ready,
							data_gen_readdata,
							data_gen_read,
							tx_ctrlenable,															// Asserts during alignment pattern, use to control k_ctrl of 8B/10B encoder 
							tx_datain,
							test_complete,
							aci_cem
						);    
						
parameter									serial_channels			=	2;
parameter									serial_channels_div2	=	serial_channels / 2;
parameter									serial_channels_div2_o	=	(serial_channels - 1) / 2;	// Use if odd # serial channels	
parameter									word_align_pattern		=	16'hf628; 					// Little Endian (compatible with ALTGXB)
parameter									channel_width			=	16;							// Make even   
parameter									test_length_bts			=	'h100000;					// Number of data_gen_readdata to transmit before asserting test_complete
parameter									test_complete_ctr_width	=	64;							// > log2(test_length_bts)	  
parameter									tx_wrapper_mode			=	0;							// Mode 0, truncated to 8 for simulation
parameter									channel_width_div2		=	channel_width / 2;  
parameter									bonded_word_width		=	channel_width * serial_channels;
parameter									tx_datain_bts_idle		=	0;
parameter									byte_order_pattern		=	{{channel_width_div2{1'b1}},{channel_width_div2{1'b0}}};
parameter									byte_width				=	8;
parameter									tx_ctrlenable_width		=	serial_channels * channel_width / byte_width;
parameter									tw						=	tx_ctrlenable_width / serial_channels;
parameter									tx_data_ctr_width		=	channel_width;
parameter									component_type			=	"MY_COMPONENT";
parameter									serdes_2d				=	0;	
parameter									hf_striping_1			=	{channel_width{1'b1}};
parameter									hf_striping_0			=	{channel_width{1'b0}};

input                  						reset_n;			
input										clk;				
input										start;				
input										create_error;
input										tx_ready;
input		[bonded_word_width-1:0]			data_gen_readdata;  
output	reg									data_gen_read;
output	reg	[tx_ctrlenable_width-1:0]		tx_ctrlenable;					// For out of band signaling (eg K characters)
output	reg	[bonded_word_width-1:0]			tx_datain;
output	reg									test_complete;
input		[bonded_word_width-1:0]			aci_cem;

// Internal data structures            
reg			[tx_ctrlenable_width-1:0]		tx_ctrlenable_0;
reg			[bonded_word_width-1:0]			tx_in_0;
reg			[bonded_word_width-1:0]			tx_in;
reg			[tx_data_ctr_width-1:0]			tx_data_ctr;      
integer										bts_state;
reg			[test_complete_ctr_width-1:0]	test_complete_ctr;
reg											toggle_tx_in;
//. For XPAK or X2 Module:
reg			[7:0]							count;       // packet size
reg			[3:0]							ipg_count;   // ipg size   
reg											create_error_reg; 
wire		[bonded_word_width-1:0]			create_error_bits;
reg			[bonded_word_width-1:0]			hf_striping;

assign  create_error_bits       =       (aci_cem & ({bonded_word_width{create_error_reg}}));


parameter	xaui_packet_size	=	255;	// 63;
parameter	ipg_packet_size		=	7;		// 6;
// Establish XPAK or X2 Link  
 /*  7/25/2005 Description of Packet

	Lane              0     1     2     3
    ||K|| = Sync =  K28.5 K28.5 K28.5 K28.5  XGMII CTRL={1 1 1 1} DATA={07h 07h 07h 07h}
    ||R|| = Skip =  K28.0 K28.0 K28.0 K28.0  XGMII CTRL={1 1 1 1} DATA={07h 07h 07h 07h}                                                                                                                       
    ||A|| = Align = K28.3 K28.3 K28.3 K28.3  XGMII CTRL={1 1 1 1} DATA={07h 07h 07h 07h}
	||S|| = Start = K27.7 DX.Y  DX.7  DX.Y   XGMII CTRL={1 0 0 0} Data={FBh D1h D2h D3h} = D1-3 Preamble data bytes 
    ||T|| = Terminate Lane 0 = K29.7 K28.5 K28.5 K28.5  K29.7 <--> CTRL=1 DATA=FDh             
	        Terminate Lane 1 = DX.Y  K29.7 K28.5 K28.5
			Terminate Lane 2 = DX.Y  DX.Y  K29.7 K28.5
			Terminate Lane 3 = DX.Y  DX.Y  DX.Y  K29.7
	/E/     Any column(s), Error CTRL=1 DATA=FEh
	Data Generator - User byte serializer (separated by "|"}

	When not enabled, transmit:
							    Lane                 3         2         1         0
							    CTRL=FFh	DATA={07h 07h | 07h 07h | 07h 07h | 07h 07h}		||K|| or ||R|| or ||A||

	When enabled send back to back packets:			
		PACKET START:
							    Lane                 3         2         1         0		
			Packet Start		CTRL=01h	DATA={D5h 55h | 55h 55h | 55h 55h | 55h FBh}		||S||   
			Packet Data 1   	CTRL=00h	DATA={01h 01h | 01h 01h | 01h 01h | 01h 01h}        Packet Data 1
			Packet Data 2   	CTRL=00h	DATA={02h 02h | 02h 02h | 02h 02h | 02h 02h}        Packet Data 2
			     ...           ...                            ...                                 ...
			Packet Data 62  	CTRL=00h	DATA={3Eh 3Eh | 3Eh 3Eh | 3Eh 3Eh | 3Eh 3Eh}        Packet Data 62  
			Terminate Lane 0    CTRL=FFh    DATA={07h 07h | 07h 07h | 07h 07h | 07h FDh}         ||T||             
			IPG 1          		CTRL=FFh    DATA={07h 07h | 07h 07h | 07h 07h | 07h 07h}        Interpacket Gap 1
			IPG 2          		CTRL=FFh    DATA={07h 07h | 07h 07h | 07h 07h | 07h 07h}        Interpacket Gap 2
			IPG 3          		CTRL=FFh    DATA={07h 07h | 07h 07h | 07h 07h | 07h 07h}        Interpacket Gap 3
			IPG 4          		CTRL=FFh    DATA={07h 07h | 07h 07h | 07h 07h | 07h 07h}        Interpacket Gap 4
			IPG 5          		CTRL=FFh    DATA={07h 07h | 07h 07h | 07h 07h | 07h 07h}        Interpacket Gap 5
			IPG 6          		CTRL=FFh    DATA={07h 07h | 07h 07h | 07h 07h | 07h 07h}        Interpacket Gap	6 
		PACKET END
*/       

// XAUI & X2 Protocol
parameter	idle_k		=	8'hFF;
parameter	idle_d		=	64'h07_07_07_07_07_07_07_07;
parameter	start_k		=	8'h01;
parameter	start_d		=	64'hD5_55_55_55_55_55_55_FB;
parameter	terminate_k	=	8'hFF;
parameter	terminate_d	=	64'h07_07_07_07_07_07_07_FD; 



generate      
	if	(
			(component_type == "XPAK")	||
			(component_type == "X2")	||
			(component_type	== "X2_S4GX") ||
			(component_type	== "X2_A2GX")
		)    
	// Ported from Alan Winters code on SGX Dev Kit XPAK Packet Generator
	// Not scalable - hard coded for channel width=16 and serial channels = 4  
	   

	

	
	begin
		always @ (posedge clk)
		begin
			if (!reset_n)
			begin
				tx_datain 			<=	0;
				tx_ctrlenable 		<=	0;
				count				<=	0;    
				ipg_count 			<=	0; 
				bts_state			<=	0;
				test_complete_ctr	<=	0;
				test_complete		<=	0; 
				data_gen_read		<=	0;  
				create_error_reg	<=	0;  	
			end else if (!tx_ready)
			begin// idle mode
				tx_datain     <= idle_d; 
				tx_ctrlenable <= idle_k;
				count <= 0;    
				ipg_count <=0;  
			end else /* tx_ready = 1*/ if (ipg_count == 0) //tx_ready = 1
			// data packet
			begin
				if (count == xaui_packet_size) // Adjust for packet size, # * 8
				begin           
					//tx_datain <= 64'h07_07_07_07_07_07_07_FD; // Worked on X2 & XPAK
					tx_datain		<= terminate_d;	// End of data packet terminates immediately after XAUI data packet
					tx_ctrlenable	<= terminate_k;	//
					count <= 0;  
					ipg_count <= 1; // Start IPG on next clock
				end else if (count == 8'h00)
				begin   
					tx_datain		<= start_d;	// Packet preamble
					tx_ctrlenable	<= start_k;
					count <= count + 1;
					case	(bts_state)
						0:			begin
										if (start)
										begin
											bts_state			<=	1;
											data_gen_read		<=	1;
										end
										test_complete_ctr		<=	test_length_bts; 
										test_complete			<=	0;									
									end
						default:	begin
										data_gen_read			<=	1;
									end
					endcase
				end else begin
					count <= count + 1;   						
					case	(bts_state)
						0:			begin
										tx_datain <= ({count,count,count,count,count,count,count,count}); // Data Packets
										tx_ctrlenable <= 8'h00;
									end     
						1:			begin
										tx_ctrlenable			<=	0;
										if	(tx_ready)
										begin
											test_complete_ctr	<=	test_complete_ctr - 1;
											tx_datain			<=	data_gen_readdata;
											if	(test_complete_ctr == 1)
											begin
												bts_state		<=	2;					// Assert test complete flag, continue sending data & checking for errors forever
												test_complete	<=	1;
											end
										end else begin
											tx_datain			<=	tx_datain_bts_idle;
										end  
										data_gen_read = (count != (xaui_packet_size-1)); // Adjust for packet size, # * 8
									end						
						2:			begin												// After test complete, continue writing PRBS data  - can now insert errors
										tx_ctrlenable			<=	0;    
										if	(create_error)
										begin
											create_error_reg    <=      1;
										end else if (tx_ready)
										begin
  	                            		    create_error_reg   <=      0;
										end																		
										if	(tx_ready)
										begin
											tx_datain			<=	data_gen_readdata ^ create_error_bits;
										end else begin
											tx_datain			<=	tx_datain_bts_idle;
										end
										data_gen_read = (count != (xaui_packet_size-1)); // Adjust for packet size, # * 8
									end
						default:	;
					endcase
				end
			end else begin   // start ipg 
				if (ipg_count < ipg_packet_size)  // set IPG packet size
				begin
					tx_datain 		<= 	idle_d;	// 64'h07_07_07_07_07_07_07_07;
					tx_ctrlenable 	<=	idle_k;	// 8'hFF;
					ipg_count <= ipg_count + 1;
				end else begin // end of IPG 
					tx_datain		<=	idle_d;	// <= 64'h07_07_07_07_07_07_07_07;
					tx_ctrlenable	<=	idle_k;	// 8'hFF;
					ipg_count <= 0;
				end
			end  // end ipg
		end
	end 
endgenerate	


// For SERDES with LVDS parallel data inputs
// (eg HSMC SDI Daughter card, a partial SERDES uses LVDS inputs as parallel data).
// If the link pattern is low frequency, the CD (carrier detect) can mute the SDI RX equalizer).
// Therefore, "stripe" the replicated LVDS channels during link establishment with high frequency content.
// NOTE:  serial_channels refers to, for example, the # of LVDS channels.
//        For a Partial SERDES using LVDS inputs, # serial_channels still refers the # of lvds channels,
//        however these are parallel inputs to a partial SERDES and must be >=2. 
//        hf_striping is not supported if serial_channels = 1	
	task	hf_striping_init;
		begin
			case	(serdes_2d)
				1,2:		begin
				if	(serial_channels[0] == 0) // Even # serial channels
				begin
					hf_striping		<=	{serial_channels_div2{hf_striping_1,hf_striping_0}};
				end else begin
					hf_striping		<=	{{(serial_channels_div2_o){hf_striping_1,hf_striping_0}},hf_striping_1};
				end
							end
				default:	begin
				hf_striping			<=	0;
			end	
			endcase		
		end
	endtask	   

// Legacy link establishment (no insert / delete ordered sets)    
generate
	if	(
			(component_type != "XPAK")	&&
			(component_type != "X2")	&&
			(component_type != "X2_S4GX") &&
			(component_type != "X2_A2GX")
		)  
	begin  	
    
	// Generate repeating DC balanced pattern followed by word alignment pattern on all channels after device is reset
	// This will allow the pma plls to lock and the pcs to word align and channel bond.
	always @(posedge clk or negedge reset_n)
	begin
		if	(!reset_n)
		begin  
			tx_data_ctr		<=	0;      
			tx_ctrlenable_0	<=	0;
			tx_in			<=	{serial_channels{word_align_pattern}};
			toggle_tx_in	<=	0;
			hf_striping_init;
		end else begin
			case	(1'b1)
				(tx_wrapper_mode == 0):	tx_data_ctr <=	tx_data_ctr + 1;
				(tx_wrapper_mode >0):	if	(tx_data_ctr == tx_wrapper_mode)
										begin
											tx_data_ctr <= 0;
										end else begin
											tx_data_ctr	<=	tx_data_ctr + 1;   
										end
				//default:				tx_data_ctr		<=	tx_data_ctr + 1; // case never occurs 
			endcase
			case (tx_data_ctr)       
				0:			begin    // If AC coupled, should DC Balance word_align_pattern and byte_order_pattern 
								tx_ctrlenable_0	<=	{serial_channels{{(tw-1){1'b0}},1'b1}};	
								tx_in			<=	{serial_channels{word_align_pattern}};  	// Each serial channel has upper pad bits are "D", word alignment is "K"
								toggle_tx_in	<=	~toggle_tx_in;								// Every other cycle invert sequence (for dc balance)
							end						
				4:			begin
								tx_ctrlenable_0	<=	{serial_channels{{(tw-1){1'b0}},1'b1}};		// Each serial channel has upper pad bits are "D", byte order is "K"
								if (serdes_2d == 1)
								begin
									// Keep it DC balanced for 2d SERDES:  eg LVDS SERDES + SDI XCVRS
									// See lane_order_rotate.  Use byte_order_pattern for the lane_order_pattern
									// lane_order_rotate assumes lane_order_pattern is tranmitted in lane 0, 
									// and ~lane_order_pattern is tranmitted in the remaining lanes
									tx_in			<=	{{(serial_channels-1){~byte_order_pattern}},byte_order_pattern};
								end else begin
									tx_in			<=	{serial_channels{byte_order_pattern}};
								end
							end
				default:	begin	// DC balanced derived from incrementing data transmitted during pll lock, word alignment, & channel bonding 
									// To simplify, use the word alignment pattern as the channel bonding align pattern
								tx_ctrlenable_0	<=	{tx_ctrlenable_width{1'b0}}; 
								if	(channel_width[0] == 0 && (tx_data_ctr_width == channel_width))
								begin
									tx_in_0		=	{serial_channels{{(~tx_data_ctr[channel_width_div2-1:0]),tx_data_ctr[channel_width_div2-1:0]}}};  
								end else begin // If channel_width to large or odd, use a simpler method:
									tx_in_0		=	tx_data_ctr; // [bonded_word_width-1:0];
								end
								case	(serdes_2d)
									1:			begin
									tx_in			<=	({bonded_word_width{toggle_tx_in}}) ^ tx_in_0 ^ hf_striping;	// Toggle polarity every cycle
												end
									2:			begin
													tx_in			<=	({bonded_word_width{toggle_tx_in}}) ^ ({serial_channels{byte_order_pattern}}) ^ hf_striping;	// Toggle polarity every cycle
												end
									default:	begin
									tx_in			<=	({bonded_word_width{toggle_tx_in}}) ^ tx_in_0;
								end
								endcase
							end
			endcase
		end
	end	              
	
	always	@(posedge clk or negedge reset_n)
	begin
		if (!reset_n)
		begin
			bts_state			<=	0;
			test_complete_ctr	<=	0;
			test_complete		<=	0; 
			data_gen_read		<=	0; 
			tx_ctrlenable		<=	0;
			tx_datain			<=	{serial_channels{word_align_pattern}}; 
			create_error_reg	<=	0;  	
		end else begin 
			case	(bts_state)
				0:			begin
								if	(start)										// Start test
								begin
									bts_state			<=	1;
									data_gen_read		<=	1;
								end
								test_complete_ctr		<=	test_length_bts;
								test_complete			<=	0;    
								tx_ctrlenable			<=	tx_ctrlenable_0;
								tx_datain				<=	tx_in;				// Until bts starts, tx_ctrlenable xcvrs     
							end
				1:			begin
								tx_ctrlenable			<=	0;
								data_gen_read			<=	1;
								if	(tx_ready)
								begin
									test_complete_ctr	<=	test_complete_ctr - 1;
									tx_datain			<=	data_gen_readdata;
									if	(test_complete_ctr == 1)
									begin
										bts_state		<=	2;					// Assert test complete flag, continue sending data & checking for errors forever
										test_complete	<=	1;
									end
								end else begin
									tx_datain			<=	tx_datain_bts_idle;
								end
							end
				2:			begin												// After test complete, continue writing PRBS data  - can now insert errors
								tx_ctrlenable			<=	0;  
								if	(create_error)
								begin
									create_error_reg    <=      1;
								end else if (tx_ready)
								begin
  	                                create_error_reg   <=      0;
								end								
								if	(tx_ready)
								begin
									tx_datain			<=	data_gen_readdata ^ create_error_bits;
								end else begin
									tx_datain			<=	tx_datain_bts_idle;
								end
								data_gen_read			<=	1;
							end
				default:	begin												// Should never get here  
								tx_ctrlenable			<=	0;
								tx_datain				<=	tx_in;
							end
			endcase
		end
	end                 
 
 	end 
endgenerate

endmodule

