//--------------------------------------------------------------------------//
// Title:       heartbeat_x4.v                                              //
// Rev:         Rev 1                                                       //
// Author		Altera High Speed Design Group - San Diego                  //
//--------------------------------------------------------------------------//
// Description: // Drives LEDs with a "heartbeat" pattern (board is alive)  // 
//----------------------------------------------------------------------------
//Copyright  2006 Altera Corporation. All rights reserved.  Altera products
//are protected under numerous U.S. and foreign patents, maskwork rights,
//copyrights and other intellectual property laws.
//                                                                                                     
//This reference design file, and your use thereof, is subject to and
//governed by the terms and conditions of the applicable Altera Reference
//Design License Agreement.  By using this reference design file, you
//indicate your acceptance of such terms and conditions between you and
//Altera Corporation.  In the event that you do not agree with such terms and
//conditions, you may not use the reference design file. Please promptly                         
//destroy any copies you have made.
//
//This reference design file being provided on an "as-is" basis and as an
//accommodation and therefore all warranties, representations or guarantees
//of any kind (whether express, implied or statutory) including, without
//limitation, warranties of merchantability, non-infringement, or fitness for
//a particular purpose, are specifically disclaimed.  By making this
//reference design file available, Altera expressly does not recommend,
//suggest or require that this reference design file be used in combination 
//with any other product not provided by Altera

// Designed to drop into the legacy bts system
// The BTS legacy system creates tx_digitalreset and rx_digitalreset
// external to this module
module xcvr_reset_sequencer_bts
(
	input							reset_n,
	input							clk,			
	input							busy,
	input	[num_quads-1:0]			pll_locked_in,
	output	[num_quads-1:0]			pll_locked_out,
	input	[serial_channels-1:0]	rx_freqlocked_in,
	output	[serial_channels-1:0]	rx_freqlocked_out,
	output	[num_quads-1:0]			pll_powerdown,
	output	[serial_channels-1:0]	rxanalog_reset
);

parameter	num_quads					=	1;
parameter	serial_channels				=	1;

parameter	clk_freq_mhz				=	50;
parameter	pll_powerdown_counts		=	clk_freq_mhz * 2;	// Make 1us (clock frequency in MHz * 2 for 2x margin) 
parameter	busy_to_analog_reset_counts	=	16;
parameter	pll_pd_width				=	10;					// Make wide enough to count up to pll_powerdown_counts
parameter	btoa_rst_width				=	8;					// Make wide enough to count up to busy_to_analog_reset_counts
reg			pll_powerdown_rg;
reg			rx_analogreset_rg;
reg	[pll_pd_width-1:0]		pll_powerdown_ctr;
reg	[btoa_rst_width-1:0]	btoa_reset_ctr;							

							
					
	always	@(posedge clk or negedge reset_n)
	begin
		if	(!reset_n)
		begin
			pll_powerdown_rg		<=	1;
			rx_analogreset_rg		<=	1;	
			pll_powerdown_ctr		<=	pll_powerdown_counts;
			btoa_reset_ctr			<=	busy_to_analog_reset_counts;
		end else begin
			if	(pll_powerdown_ctr != 0)
			begin
				pll_powerdown_ctr	<=	pll_powerdown_ctr - 1;
			end else begin
				pll_powerdown_rg	<=	0;	// TX PLL should start locking here
			end
			if (pll_powerdown_rg == 0)
			begin
				if	(busy_sync)
				begin
					btoa_reset_ctr		<=	busy_to_analog_reset_counts;
					//rx_analogreset_rg	<=	1;		
				end else begin
					if	(btoa_reset_ctr != 0)
					begin
						btoa_reset_ctr		<=	btoa_reset_ctr - 1;
					end else begin
						rx_analogreset_rg	<=	0;
					end
				end
			end
		end
	end			

assign	pll_powerdown		=	{num_quads{pll_powerdown_rg}};
assign	pll_locked_out		=	{num_quads{tx_plls_locked_sync}};
assign	rx_freqlocked_out	=	{serial_channels{rx_tx_plls_locked_sync}};
assign	rxanalog_reset		=	{serial_channels{rx_analogreset_rg}};
assign	tx_plls_locked		= 	&pll_locked_in;
assign	rx_tx_plls_locked	= 	(&rx_freqlocked_in) && (&pll_locked_out);	


// Metastability synchronizers for XCVR
// 1st parameter is synchonizer depth, 2nd parameter is the preset value
// TX PLL
synchronizer	#(4,0)		synchonizer_pll_locked_sync	(.reset_n(1'b1),.clk(clk),.data_in(tx_plls_locked & (~pll_powerdown_rg)),.data_out(tx_plls_locked_sync));
// RX Analog Reset:
synchronizer	#(4,1)		synchonizer_busy_sync		(.reset_n(1'b1),.clk(clk),.data_in(busy),.data_out(busy_sync));
// TX + RX PLL:
synchronizer	#(4,0)		synchonizer_rx_fifo_rst_n	(.reset_n(1'b1),.clk(clk),.data_in(rx_tx_plls_locked & (~rx_analogreset_rg)),.data_out(rx_tx_plls_locked_sync));

/*


// Push Buttons Debouncers for XCVR
switch_debouncer	#(0,debounce_prd)	switch_debouncer_xcvr_fpga_en_cdet	(.reset_n(xcvr_reset_n),.clk(rx_pld_clk),.data_in(en_cdet),.data_out(en_cdet_sync));
switch_debouncer	#(0,debounce_prd)	switch_debouncer_xcvr_chn_bond		(.reset_n(xcvr_reset_n),.clk(rx_pld_clk),.data_in(channel_bond),.data_out(channel_bond_sync));
switch_debouncer	#(0,debounce_prd)	switch_debouncer_xcvr_bts_start		(.reset_n(xcvr_reset_n),.clk(tx_pld_clk),.data_in(start),.data_out(bts_start_sync));
switch_debouncer	#(0,debounce_prd)	switch_debouncer_xcvr_create_error	(.reset_n(xcvr_reset_n),.clk(tx_pld_clk),.data_in(create_error),.data_out(create_error_sync)); 
// 1st parameter enables positive edge detect
// 2nd parameter enables negative edge detect
// 3rd parameter is pulse polarity (1=positive, 0=negative)
// Creates single clock pulse from edge
edge_detector		#(1,0,1)			edge_detector_create_single_error(.reset_n(xcvr_reset_n),.clk(tx_pld_clk),.data_in(create_error_sync),.data_out(create_single_error)); // Single error event

// Metastability synchronizers for XCVR
// 1st parameter is synchonizer depth, 2nd parameter is the preset value
synchronizer	#(4,0)		synchonizer_pll_locked_sync	(.reset_n(1'b1),.clk(tx_pld_clk),.data_in(plls_in_locked && tx_plls_locked),.data_out(tx_plls_locked_sync));
synchronizer	#(4,0)		synchonizer_rx_fifo_rst_n	(.reset_n(1'b1),.clk(tx_pld_clk),.data_in(plls_in_locked && rx_tx_plls_locked),.data_out(rx_tx_plls_locked_sync));

// XCVR Auto Reset (eg Allow PLLs to lock, then reset phase comp FIFOs)
assign	tx_plls_locked		= &pll_locked_sig;
assign	rx_tx_plls_locked	= (&rx_freqlocked_sig) && (&pll_locked_sig);



// 1st parameter is the asserted value, 2nd is pulse width, 3rd is delay from rising edge on trig
// On rising edge of trig, a pulse is asserted, otherwise data_in passes thru.  On reset the asserted value passes thru. 
// Adds delay so PLLs can fully phase lock:
// Trig only assert once, so use + trigger
delayed_reset	#(1,delayed_rst_pw,delayed_rst_prd)	delayed_reset_txdigitalreset 	(.clk(tx_pld_clk),.reset_n(xcvr_reset_n),.trig(tx_plls_locked_sync),.data_in(!tx_plls_locked_sync),.data_out(tx_digital_reset_sync_dly));
// Trig only assert once, so use + trigger
delayed_reset	#(1,delayed_rst_pw,delayed_rst_prd)	delayed_reset_rxdigitalreset	(.clk(tx_pld_clk),.reset_n(xcvr_reset_n),.trig(rx_tx_plls_locked_sync),.data_in(!rx_tx_plls_locked_sync),.data_out(rx_digital_reset_sync_dly));
// Use negative edge of XCVR reset pulse to trigger next pulse                                                                                                                                                
delayed_reset	#(0,delayed_rst_pw,delayed_rst_prd)	delayed_reset_rxfifo_rst_n		(.clk(tx_pld_clk),.reset_n(xcvr_reset_n),.trig(!rx_digital_reset_sync_dly),.data_in(!rx_digital_reset_sync_dly),.data_out(rx_fifos_rst_sync_dly_n));

*/

				
endmodule