/******************************************************************************
*Copyright  2006 Altera Corporation. All rights reserved.  Altera products   *
*are protected under numerous U.S. and foreign patents, maskwork rights,      *
*copyrights and other intellectual property laws.                             *
*                                                                             *
*This reference design file, and your use thereof, is subject to and governed *
*by the terms and conditions of the applicable Altera Reference Design        *
*License Agreement.  By using this reference design file, you indicate your   *
*acceptance of such terms and conditions between you and Altera Corporation.  *
*In the event that you do not agree with such terms and conditions, you may   *
*not use the reference design file. Please promptly destroy any copies you    *
*have made.                                                                   *
*                                                                             *
*This reference design file being provided on an "as-is" basis and as an      *
*accommodation and therefore all warranties, representations or guarantees    *
*of any kind (whether express, implied or statutory) including, without       *
*limitation, warranties of merchantability, non-infringement, or fitness for  *
*a particular purpose, are specifically disclaimed.  By making this reference *
*design file available, Altera expressly does not recommend, suggest or       *
*require that this reference design file be used in combination with any      *
*other product not provided by Altera                                         */

/******************************************************************************
* NOTICE: The information contained in this file is proprietary   			  *
* to Altera and is being made available to Altera's customers     			  *
* under strict non-disclosure agreements.  Use or disclosure of   			  *
* this information is permissible only under the terms of         			  *
* existing non-disclosure agreements.                             			  *
*******************************************************************************/    

/*                  
-------------------------------------------------------------------------------------------
--                                                                                       --
-- Title:       xs_wrapper_bts.v                                                         --
-- Rev:         Rev 1                                                                    --
-- Modified:    January 30, 2006                                                         --
-- Author:      David Johnson                                                            --
--  Wraps around transceivers & SERDES - Performs synchonization up to channel bonding   --                                                                                       --

--------------------------------------------------------------------------------------------  
--                                                                                       --
-- Revision History:                                                                     --  
--                                                                                       --
-- Rev 1:       Created                                                                  --
--                                                                                       --
--------------------------------------------------------------------------------------------
*/

module	xs_wrapper_bts		(
								reset_n,					// Asynchronous reset of module
								rx_fifos_rst_n,				// RX FIFO reset for XCVR designs
								tx_pld_clk,		            // Transmit/Write PLD Clock
								tx_datain,	                // Transmit/Write Data     
								tx_ctrlenable,				// For out of band signaling (eg 8B/10B K28.5)
								tx_ready,					// Ready Signal for Transmit/Write data                
								rx_pld_clk,					// Receive/Read PLD Clock
								rx_clkout,					// Receive Clock(s) (for XCVR & SERDES designs)
								en_cdet,					// Enable Comma Detect (for XCVR & SERDES designs)
								channel_bond,				// Enable Channel Bond (for XCVR & SERDES designs)
								start,						// Start Transmit/Write Data Generator
								create_error,				// Create Error in Transmit/Write Data Generator   
								rx_patterndetect,			// If using channel bonder in bts and word_aligner + byte_order in pcs
								rx_dataout,					// Receive/Read Data
								rx_ready,					// Ready Signal for Receive/Read Data (Currently fixed latency only - eg: no RX FIFOs, support xpak on sgx dev kit)
								error_latch,				// Persistently asserts on 1st Error detected
								error_irq,					// Unused - keep port for reverse compatibility with legacy designs
								data_error,					// Asserts on each error for 1 rx_pld_clk period
								pattern_sync_acquired,		// Asserts when receiver detects sync pattern
								test_complete,				// Asserts after RX side of BTS receives "test_length_bts" words  
								data_skew,					// Add data skew delay on transmitter between the serial channels
								error_mask,					// Error mask -  mask bits for error detection - set  parameter to enable_error_mask = 1
								sync_mask,					// Sync mask - mask bits for pattern_sync_acquired - requires parameter to enable_sync_mask = 1
								aci_prs,                	// PRBS Seed
								aci_prd,  					// PRBS Depth
								aci_prc,  					// PRBS Coefficients
								aci_prt,					// PRBS Type (com or mem)
								aci_cem,  					// Create Error Mask
								aci_ttc,  					// Transmitter / Write Transaction Counter
								aci_rtc,  					// Receiver / Read Transaction Counter
								aci_wec,  					// Word Error Counter
								aci_lec,  					// Lane Error Counter
	  							aci_ips,					// Initialize PRBS Seed
	  							aci_cla,					// Channel Lock Acquired (persistent)
								aci_cls,					// Channel Lock Sync (Pulses on each channel alignment pattern)
	  							aci_pass					// Test passed					
							);         
parameter	stratix_gx_dev_kit_xcvrs	=   0;					// Setting=1 applies wire twist to HSSDC2 if using all 20 xcvrs
parameter	rx_fifo_enable				=	1;					// Enables RX FIFOs (use for XCVR designs)
parameter	rx_delay_bts_start			=	0;					// Delay detection of pattern_sync_acquired during rx transceiver initialization (use for XCVR designs)
parameter	word_aligner_enable			=	1;					// Enables Word Aligner (use for XCVR & SERDES designs)
parameter	channel_bond_enable			=	1;					// Enable Channel Bonder (use for XCVR & SERDES designs)
parameter	convert_endian				=	1;					// For XCVRS wired little endian (use for little endian XCVR & SERDES designs)
parameter	flip_data_bits				=	0;					// Set to 1 if MSB sent 1st (SONET), set to 0 lsb transmitted 1st (8B/10B)			=	0;			// Set to 1 if MSB sent 1st (SONET), set to 0 lsb transmitted 1st (8B/10B)
parameter	cw							=	16;					// Channel Width - If channel width is odd, make certain to override byte_order_pattern parameter
																// You can still dc balance by selecting word_align_pattern and byte_order_pattern to balance if both are transmitted (tx_wrapper_mode > 4)
parameter	cw_div2                		=	cw / 2;				
parameter	num_endians					=	1;					// cw / deserializer_width (normally defined as cw/byte_width)	// Number of endians (use for XCVR & SERDES designs)
parameter	sync_pattern_width			=	16;					// Word Aligner Sync Pattern Width
parameter	mux_sel_width				=	4;					// set to log2(cw)
parameter	mux_sel_step				=	1;					// Set to 1 normally, set to deserializer_width if using word aligner for byte reorder function
parameter	word_align_pattern			=	'hf628; 			// word aligner pattern (big endian)
// Transmitted as part of tx_wrapper_bts sequence for xcvr byte order block
parameter	byte_order_pattern			=	{{cw_div2{1'b1}},{cw_div2{1'b0}}};
parameter	pattern_sync_prbs			=	prbs_seed_param;	 	// Unused - keep for legacy designs
																	// 1ts prbs data now directly captured from TX PRBS generator
parameter	bonded_word_width			=	cw * serial_channels;  	// Bus Width  (Channel Width * Number of Serial Channels for XCVRs & SERDES)
parameter	tx_ctrlenable_width			=	cw * serial_channels / byte_width;
parameter	tw							=	tx_ctrlenable_width / serial_channels; // Single channel control word width
parameter	byte_width					=	8;	
parameter	rfcw						=	cw / byte_width;		// rx_pattern_detect through rx fifo if using pcs word aligner & bts channel_bonder  
parameter	rx_patterndetect_width		= 	serial_channels * cw / byte_width;       
parameter	prbs_type					=	0;					// Controls if BTS generates true PRBS for COMM component's serializer,
																// or minimizes area using full shift register as parallel data
parameter	prbs_com					=	1;
parameter	prbs_mem					=	0;	
parameter	prbs_width					=	data_width;			// For com, set to cw * prbs_depth;
// rx fifo control bits - Increase to include k ctrl, disp error, codeword violation, etc
//defparam	bts_xcvr_wrapper_i.
								
// altera_bts:
parameter	prbs_coeff2					=	2'b11;
parameter	prbs_coeff3					=	3'b011;
parameter	prbs_coeff4					=	4'b0011;
parameter	prbs_coeff5					=	5'b00101;
parameter	prbs_coeff6					=	6'b000011;
parameter	prbs_coeff7					=	7'b0000011;
parameter	prbs_coeff8					=	8'h1d;
parameter	prbs_coeff16				=	16'h100b;
parameter	prbs_coeff32				=	32'hc000_0401;
parameter	prbs_coeff128				=	128'h0000_0000_0000_0000_0000_0000_2800_0005;
                                    	
parameter	nios_width					=	32;		 					// NIOS Datapath Width For transaction counter
parameter	data_width					=	bonded_word_width; 			// Make equal to bonded_word_width!
parameter	prbs_depth					=	32; 						// PRBS shift register depth
parameter	prbs_seed_param				=	64'hace3_abc4_1023_1188;	// PRBS seed PRBS Seed - set to all 1's if using bts without SERDES (eg USB or memory controller)
parameter	prbs_coeff_param			=	{8{prbs_coeff32}};     	// PRBS coefficients          
// tx_wrapper_bts:                  	
parameter	serial_channels				=	4;							// Serial Channels for XCVR or SERDES
parameter	tx_wrapper_mode				=	0;							// Creates tx pattern for XCVRs & SERDES to bit align, word align, and channel bond
parameter	tx_data_ctr_width			=	cw;							// Set >= ceil(log2(tx_wrapper_mode+1). (tx_wrapper_mode=0, set to cw which is the default)
// channel_aligner:                 	
parameter	deskew_depth				=	32;							// Channel Bonding Word Deskew Capability
parameter	deskew_ptr_width			= 	5;							// log2(deskew_depth)
                                    	
// adj_skew_bts                     	
parameter	skew_num_barrels			=	2;							// >=2 Number of barrel shifters for adding skew
parameter	channel_data_skew_width		=	cw;							// >= ceil(log2(data channel width*(skew_num_barrels-1)))  --> Not needed if serial_channels=0
parameter	data_skew_width				=	channel_data_skew_width * serial_channels;
parameter	skew_msb_1st				=	0;							// skew_msb_1st=1 (Source Synchronous LVDS are MSB 1st. skew_msb_1st=0 (XCVRS are LSB 1st) 
// misc                             	
parameter	test_length_bts				=	'h20000;					// Number of data_gen_readdata to transmit before asserting test_complete
parameter	test_complete_ctr_width		=	16;							// > log2(test_length_bts)	  
parameter	bts_idle					=	0;							// [Never occurs in PRBS]
parameter	component_type				=	"MY_COMPONENT";
parameter	serdes_2d					=	0;							// Set to 1 for 2 levels of SERDES synchronization (eg HSMC SDI board)
parameter	level_mode					=	1;							// 1 for level mode, 0 for edge mode in word aligner
// nios bts                         	
parameter	enable_nios_bts				=	0;
parameter	enable_error_mask			=	enable_nios_bts;  			// Set to 1 to use the error_mask input port (1st used in S2GX PCIE & Audio-Video boards)
parameter	enable_sync_mask			=	enable_nios_bts;    		// Set to 1 to use the sync_mask input port  (1st used in S2GX PCIE & Audio-Video boards)
parameter	tec_words					=	2;							// Transaction/Error Counter Nios words (eg tec_words=2, w
parameter	num_lanes					=	serial_channels;			// eg: Set to 9 for 72 bits DDR2 with 9 byte lanes
		                                       


input											reset_n;
input											rx_fifos_rst_n;	// RX FIFO Reset (After PLLs are phase locked)	
input											tx_pld_clk;	                 
output	[data_width-1:0]						tx_datain; 
output	[tx_ctrlenable_width-1:0]				tx_ctrlenable; 
input											tx_ready;	 
input											rx_pld_clk; 
input	[serial_channels-1:0]					rx_clkout;
input											en_cdet;
input											channel_bond;
input											start;
input											create_error;
input	[rx_patterndetect_width-1:0]			rx_patterndetect;                            	
input	[data_width-1:0]						rx_dataout;	
input											rx_ready;
output											error_latch;
output											error_irq;		// Unused - keep port for legacy designs.  Actual function moved
																// into the avalon control interface
output											data_error; 
output											pattern_sync_acquired;
output											test_complete; 
input	[data_skew_width-1:0]					data_skew;
input	[data_width-1:0]						error_mask;
input	[data_width-1:0]						sync_mask;   
input	[prbs_width-1:0]						aci_prs;
input	[nios_width-1:0]						aci_prd;
input	[prbs_width-1:0]						aci_prc;
input	[nios_width-1:0]						aci_prt;
input	[data_width-1:0]						aci_cem;
output	[(nios_width*tec_words)-1:0]			aci_ttc;
output	[(nios_width*tec_words)-1:0]			aci_rtc;
output	[(nios_width*tec_words)-1:0]			aci_wec;
output	[(nios_width*tec_words*num_lanes)-1:0] 	aci_lec; 
input				  							aci_ips;
output				  							aci_cla;
output	reg										aci_cls;
output	reg			  							aci_pass;	

wire											reset_n;
wire											rx_fifos_rst_n;
wire											tx_pld_clk;	                 
wire	[data_width-1:0]						tx_datain; 
wire	[tx_ctrlenable_width-1:0]				tx_ctrlenable; 
wire											tx_ready;	
wire											rx_pld_clk; 
wire	[serial_channels-1:0]					rx_clkout;
wire											en_cdet;
wire											channel_bond;
wire											start;
wire											create_error;
wire	[rx_patterndetect_width-1:0]			rx_patterndetect;                            	
wire	[data_width-1:0]						rx_dataout;	
wire											rx_ready;
wire											error_latch;
wire											data_error; 
wire											pattern_sync_acquired;
wire											test_complete; 
wire	[data_skew_width-1:0]					data_skew;
wire	[data_width-1:0]						error_mask_1;
wire	[data_width-1:0]						sync_mask_1;
wire	[data_width-1:0]						aci_cem_1;                                                    
// Internal data structures
       
wire	[data_width-1:0]						tx_datain_0;
wire	[data_width-1:0]						tx_datain_skewed;  
wire	[tx_ctrlenable_width-1:0]				tx_ctrlenable_0; 
wire	[tx_ctrlenable_width-1:0]				tx_ctrlenable_skewed;  
wire											channel_aligned;                                                                                                                                                                  
wire											deskew_overflow;
                                            	
wire	[bonded_word_width-1:0]					rx_data_out;			// Out of phase comp fifo (if XCVR or SERDES little endian)
wire	[bonded_word_width-1:0]					rx_data_out_be;			// Out of phase comp fifo - big endian  
wire	[bonded_word_width-1:0]					rx_out_word_aligner;    // Out of word aligner - big endian   
wire	[bonded_word_width-1:0]					rx_out_bonded;   		// Out of channel aligner - big endian
wire	[bonded_word_width-1:0]					rx_out_bonded_le;		// Bonded data - little endian
wire	[bonded_word_width-1:0]					rx_out_bonded_le_2d;	// Bonded data after 2d byte_order_rotate (if 2d_serdes=1)
reg		[bonded_word_width-1:0]					rx_out_bonded_le_pl1;	// Extra pipeline to meet timing (this is redundant - pipelined in channel aligner)
reg		[bonded_word_width-1:0]					rx_out_bonded_le_pl2;	// Pipeline for start of pattern detect
wire	[serial_channels-1:0]					pattern_detect;	 
// Assume only asserts in the lsb - enable byte ordering in the pcs
wire	[rx_patterndetect_width-1:0]			rx_patterndetect_out;	 
wire	[serial_channels-1:0]					resync;                                            	
wire											data_gen_read;       
wire	[data_width-1:0]						data_gen_readdata; 
reg												data_actual_write;                                          	
wire											enable_pattern_sync_prbs_detect;
reg												test_complete_prev;
reg												rx_ready_pl1;
reg												rx_ready_pl2; 
reg		[data_width-1:0]						pattern_sync;
reg		[serial_channels-1:0]					channel_mask;
assign	error_irq	=	0;					// Unused - keep port for reverse compatibility with legacy designs
      
//  Use for debug to functionally remove bits from the sync_pattern_acquired detector and the data_error related detectors
// Set the respective _mask_enable = 0 to ignore the mask.
assign sync_mask_1 			= (enable_sync_mask != 0)	? sync_mask : 0;
assign error_mask_1 		= (enable_error_mask != 0) 	? error_mask : 0;
assign aci_cem_1			= (enable_nios_bts !=0) 	? aci_cem : ~0;


							
// For bonding all XCVRs or Source Synchronous SERDES,
// get all 20 XCVR RX channels data & control onto one clock domain: 
generate
	if (serial_channels >= 1  && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_1 	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[0 ]),.datain({rx_patterndetect[(rfcw*1 )-1:rfcw*0 ],rx_dataout[(cw*1 )-1:cw*0 ]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*1 )-1:rfcw*0 ],rx_data_out[(cw*1 )-1:cw*0 ]}));
	if (serial_channels >= 2  && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_2 	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[1 ]),.datain({rx_patterndetect[(rfcw*2 )-1:rfcw*1 ],rx_dataout[(cw*2 )-1:cw*1 ]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*2 )-1:rfcw*1 ],rx_data_out[(cw*2 )-1:cw*1 ]}));
	if (serial_channels >= 3  && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_3 	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[2 ]),.datain({rx_patterndetect[(rfcw*3 )-1:rfcw*2 ],rx_dataout[(cw*3 )-1:cw*2 ]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*3 )-1:rfcw*2 ],rx_data_out[(cw*3 )-1:cw*2 ]}));
	if (serial_channels >= 4  && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_4 	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[3 ]),.datain({rx_patterndetect[(rfcw*4 )-1:rfcw*3 ],rx_dataout[(cw*4 )-1:cw*3 ]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*4 )-1:rfcw*3 ],rx_data_out[(cw*4 )-1:cw*3 ]}));
	if (serial_channels >= 5  && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_5 	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[4 ]),.datain({rx_patterndetect[(rfcw*5 )-1:rfcw*4 ],rx_dataout[(cw*5 )-1:cw*4 ]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*5 )-1:rfcw*4 ],rx_data_out[(cw*5 )-1:cw*4 ]}));
	if (serial_channels >= 6  && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_6 	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[5 ]),.datain({rx_patterndetect[(rfcw*6 )-1:rfcw*5 ],rx_dataout[(cw*6 )-1:cw*5 ]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*6 )-1:rfcw*5 ],rx_data_out[(cw*6 )-1:cw*5 ]}));
	if (serial_channels >= 7  && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_7 	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[6 ]),.datain({rx_patterndetect[(rfcw*7 )-1:rfcw*6 ],rx_dataout[(cw*7 )-1:cw*6 ]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*7 )-1:rfcw*6 ],rx_data_out[(cw*7 )-1:cw*6 ]}));
	if (serial_channels >= 8  && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_8 	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[7 ]),.datain({rx_patterndetect[(rfcw*8 )-1:rfcw*7 ],rx_dataout[(cw*8 )-1:cw*7 ]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*8 )-1:rfcw*7 ],rx_data_out[(cw*8 )-1:cw*7 ]}));
	if (serial_channels >= 9  && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_9 	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[8 ]),.datain({rx_patterndetect[(rfcw*9 )-1:rfcw*8 ],rx_dataout[(cw*9 )-1:cw*8 ]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*9 )-1:rfcw*8 ],rx_data_out[(cw*9 )-1:cw*8 ]}));
	if (serial_channels >= 10 && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_10	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[9 ]),.datain({rx_patterndetect[(rfcw*10)-1:rfcw*9 ],rx_dataout[(cw*10)-1:cw*9 ]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*10)-1:rfcw*9 ],rx_data_out[(cw*10)-1:cw*9 ]}));
	if (serial_channels >= 11 && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_11	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[10]),.datain({rx_patterndetect[(rfcw*11)-1:rfcw*10],rx_dataout[(cw*11)-1:cw*10]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*11)-1:rfcw*10],rx_data_out[(cw*11)-1:cw*10]}));
	if (serial_channels >= 12 && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_12	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[11]),.datain({rx_patterndetect[(rfcw*12)-1:rfcw*11],rx_dataout[(cw*12)-1:cw*11]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*12)-1:rfcw*11],rx_data_out[(cw*12)-1:cw*11]}));
	if (serial_channels >= 13 && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_13	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[12]),.datain({rx_patterndetect[(rfcw*13)-1:rfcw*12],rx_dataout[(cw*13)-1:cw*12]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*13)-1:rfcw*12],rx_data_out[(cw*13)-1:cw*12]}));
	if (serial_channels >= 14 && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_14	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[13]),.datain({rx_patterndetect[(rfcw*14)-1:rfcw*13],rx_dataout[(cw*14)-1:cw*13]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*14)-1:rfcw*13],rx_data_out[(cw*14)-1:cw*13]}));
	if (serial_channels >= 15 && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_15	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[14]),.datain({rx_patterndetect[(rfcw*15)-1:rfcw*14],rx_dataout[(cw*15)-1:cw*14]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*15)-1:rfcw*14],rx_data_out[(cw*15)-1:cw*14]}));
	if (serial_channels >= 16 && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_16	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[15]),.datain({rx_patterndetect[(rfcw*16)-1:rfcw*15],rx_dataout[(cw*16)-1:cw*15]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*16)-1:rfcw*15],rx_data_out[(cw*16)-1:cw*15]}));
	if (serial_channels >= 17 && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_17	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[16]),.datain({rx_patterndetect[(rfcw*17)-1:rfcw*16],rx_dataout[(cw*17)-1:cw*16]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*17)-1:rfcw*16],rx_data_out[(cw*17)-1:cw*16]}));
	if (serial_channels >= 18 && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_18	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[17]),.datain({rx_patterndetect[(rfcw*18)-1:rfcw*17],rx_dataout[(cw*18)-1:cw*17]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*18)-1:rfcw*17],rx_data_out[(cw*18)-1:cw*17]}));
	if (serial_channels >= 19 && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_19	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[18]),.datain({rx_patterndetect[(rfcw*19)-1:rfcw*18],rx_dataout[(cw*19)-1:cw*18]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*19)-1:rfcw*18],rx_data_out[(cw*19)-1:cw*18]}));
	if (serial_channels >= 20 && rx_fifo_enable)	phasecomp_fifo	#(cw+rfcw)	phasecomp_fifo_20	(.reset_n(rx_fifos_rst_n),.inclk(rx_clkout[19]),.datain({rx_patterndetect[(rfcw*20)-1:rfcw*19],rx_dataout[(cw*20)-1:cw*19]}),.outclk(rx_pld_clk),.dataout({rx_patterndetect_out[(rfcw*20)-1:rfcw*19],rx_data_out[(cw*20)-1:cw*19]}));
	if	(!rx_fifo_enable)	assign  {rx_patterndetect_out,rx_data_out}	=	{rx_patterndetect,rx_dataout}; 
endgenerate    
// Convert Endian if required (Word aligner is big endian, Altera SERDES & XCVRs are little endian):        
defparam	endian_connect_word_aligner_data.convert_endian				=	convert_endian;	
defparam	endian_connect_word_aligner_data.serial_channels			=	serial_channels;
defparam	endian_connect_word_aligner_data.channel_width				=	cw;
defparam	endian_connect_word_aligner_data.num_endians				=	num_endians;
defparam	endian_connect_word_aligner_data.flip_data_bits				=	flip_data_bits;
endian_connect	endian_connect_word_aligner_data		(
															.data_in	(rx_data_out),
															.data_out	(rx_data_out_be)
														);    								

generate
	if (serial_channels >= 1  && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_1 	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*1 )-1:cw*0 ]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[0 ]),.resync(resync[0 ]),.data_out(rx_out_word_aligner[(cw*1 )-1:cw*0 ]));
	if (serial_channels >= 2  && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_2 	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*2 )-1:cw*1 ]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[1 ]),.resync(resync[1 ]),.data_out(rx_out_word_aligner[(cw*2 )-1:cw*1 ]));
	if (serial_channels >= 3  && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_3 	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*3 )-1:cw*2 ]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[2 ]),.resync(resync[2 ]),.data_out(rx_out_word_aligner[(cw*3 )-1:cw*2 ]));
	if (serial_channels >= 4  && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_4 	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*4 )-1:cw*3 ]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[3 ]),.resync(resync[3 ]),.data_out(rx_out_word_aligner[(cw*4 )-1:cw*3 ]));
	if (serial_channels >= 5  && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_5 	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*5 )-1:cw*4 ]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[4 ]),.resync(resync[4 ]),.data_out(rx_out_word_aligner[(cw*5 )-1:cw*4 ]));
	if (serial_channels >= 6  && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_6 	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*6 )-1:cw*5 ]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[5 ]),.resync(resync[5 ]),.data_out(rx_out_word_aligner[(cw*6 )-1:cw*5 ]));
	if (serial_channels >= 7  && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_7 	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*7 )-1:cw*6 ]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[6 ]),.resync(resync[6 ]),.data_out(rx_out_word_aligner[(cw*7 )-1:cw*6 ]));
	if (serial_channels >= 8  && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_8 	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*8 )-1:cw*7 ]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[7 ]),.resync(resync[7 ]),.data_out(rx_out_word_aligner[(cw*8 )-1:cw*7 ]));
	if (serial_channels >= 9  && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_9 	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*9 )-1:cw*8 ]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[8 ]),.resync(resync[8 ]),.data_out(rx_out_word_aligner[(cw*9 )-1:cw*8 ]));
	if (serial_channels >= 10 && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_10	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*10)-1:cw*9 ]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[9 ]),.resync(resync[9 ]),.data_out(rx_out_word_aligner[(cw*10)-1:cw*9 ]));
	if (serial_channels >= 11 && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_11	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*11)-1:cw*10]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[10]),.resync(resync[10]),.data_out(rx_out_word_aligner[(cw*11)-1:cw*10]));
	if (serial_channels >= 12 && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_12	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*12)-1:cw*11]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[11]),.resync(resync[11]),.data_out(rx_out_word_aligner[(cw*12)-1:cw*11]));
	if (serial_channels >= 13 && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_13	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*13)-1:cw*12]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[12]),.resync(resync[12]),.data_out(rx_out_word_aligner[(cw*13)-1:cw*12]));
	if (serial_channels >= 14 && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_14	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*14)-1:cw*13]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[13]),.resync(resync[13]),.data_out(rx_out_word_aligner[(cw*14)-1:cw*13]));
	if (serial_channels >= 15 && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_15	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*15)-1:cw*14]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[14]),.resync(resync[14]),.data_out(rx_out_word_aligner[(cw*15)-1:cw*14]));
	if (serial_channels >= 16 && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_16	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*16)-1:cw*15]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[15]),.resync(resync[15]),.data_out(rx_out_word_aligner[(cw*16)-1:cw*15]));
	if (serial_channels >= 17 && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_17	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*17)-1:cw*16]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[16]),.resync(resync[16]),.data_out(rx_out_word_aligner[(cw*17)-1:cw*16]));
	if (serial_channels >= 18 && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_18	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*18)-1:cw*17]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[17]),.resync(resync[17]),.data_out(rx_out_word_aligner[(cw*18)-1:cw*17]));
	if (serial_channels >= 19 && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_19	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*19)-1:cw*18]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[18]),.resync(resync[18]),.data_out(rx_out_word_aligner[(cw*19)-1:cw*18]));
	if (serial_channels >= 20 && word_aligner_enable)	word_aligner	#(cw,sync_pattern_width,mux_sel_width,mux_sel_step,level_mode)	word_aligner_20	(.reset_n(reset_n),.clk(rx_pld_clk),.en_cdet(en_cdet),.data_in(rx_data_out_be[(cw*20)-1:cw*19]),.sync_pattern(word_align_pattern),.pattern_detect(pattern_detect[19]),.resync(resync[19]),.data_out(rx_out_word_aligner[(cw*20)-1:cw*19]));                                                                                                                   
	if (!word_aligner_enable)							assign rx_out_word_aligner = rx_data_out_be;                                                                                                                                                  
	// Can use for loop to accomplish fully scalable version - assume byte ordering in the rx pcs is operational:       
	if (serial_channels >= 1  && (!word_aligner_enable))  assign	pattern_detect[0 ]	=	rx_patterndetect_out[rfcw * 0 ];   
	if (serial_channels >= 2  && (!word_aligner_enable))  assign	pattern_detect[1 ]	=	rx_patterndetect_out[rfcw * 1 ];
	if (serial_channels >= 3  && (!word_aligner_enable))  assign	pattern_detect[2 ]	=	rx_patterndetect_out[rfcw * 2 ];
	if (serial_channels >= 4  && (!word_aligner_enable))  assign	pattern_detect[3 ]	=	rx_patterndetect_out[rfcw * 3 ];
	if (serial_channels >= 5  && (!word_aligner_enable))  assign	pattern_detect[4 ]	=	rx_patterndetect_out[rfcw * 4 ];
	if (serial_channels >= 6  && (!word_aligner_enable))  assign	pattern_detect[5 ]	=	rx_patterndetect_out[rfcw * 5 ];
	if (serial_channels >= 7  && (!word_aligner_enable))  assign	pattern_detect[6 ]	=	rx_patterndetect_out[rfcw * 6 ];
	if (serial_channels >= 8  && (!word_aligner_enable))  assign	pattern_detect[7 ]	=	rx_patterndetect_out[rfcw * 7 ];
	if (serial_channels >= 9  && (!word_aligner_enable))  assign	pattern_detect[8 ]	=	rx_patterndetect_out[rfcw * 8 ];
	if (serial_channels >= 10 && (!word_aligner_enable))  assign	pattern_detect[9 ]	=	rx_patterndetect_out[rfcw * 9 ];
	if (serial_channels >= 11 && (!word_aligner_enable))  assign	pattern_detect[10]	=	rx_patterndetect_out[rfcw * 10];
	if (serial_channels >= 12 && (!word_aligner_enable))  assign	pattern_detect[11]	=	rx_patterndetect_out[rfcw * 11];
	if (serial_channels >= 13 && (!word_aligner_enable))  assign	pattern_detect[12]	=	rx_patterndetect_out[rfcw * 12];
	if (serial_channels >= 14 && (!word_aligner_enable))  assign	pattern_detect[13]	=	rx_patterndetect_out[rfcw * 13];
	if (serial_channels >= 15 && (!word_aligner_enable))  assign	pattern_detect[14]	=	rx_patterndetect_out[rfcw * 14];
	if (serial_channels >= 16 && (!word_aligner_enable))  assign	pattern_detect[15]	=	rx_patterndetect_out[rfcw * 15];
	if (serial_channels >= 17 && (!word_aligner_enable))  assign	pattern_detect[16]	=	rx_patterndetect_out[rfcw * 16];
	if (serial_channels >= 18 && (!word_aligner_enable))  assign	pattern_detect[17]	=	rx_patterndetect_out[rfcw * 17];
	if (serial_channels >= 19 && (!word_aligner_enable))  assign	pattern_detect[18]	=	rx_patterndetect_out[rfcw * 18];
	if (serial_channels >= 20 && (!word_aligner_enable))  assign	pattern_detect[19]	=	rx_patterndetect_out[rfcw * 19];
endgenerate                       
	
                                                            
                                                           
// If error_mask and sync_mask for a serial channel is all 1's (channel if fully ignored),
// set channel_mask for that serial channel to 1
// Channel bonder will ignore the fully masked channel
integer	A1,A2,A3;
always	@(*)
begin
	A1	=	0;
	for	(A2=0; A2<serial_channels; A2=A2+1)
	begin
		channel_mask[A2]	=	1;
		for	(A3=0; A3<cw; A3=A3+1)
		begin
			if	((error_mask[A1] == 0) || (sync_mask[A1] == 0))
			begin
				channel_mask[A2]	=	1'b0;
			end
			A1	=	A1 + 1;	
		end
	end
end


generate
	if	(channel_bond_enable && (serial_channels > 1)) // Must have at least 2 channels to bond together
	begin
		defparam	channel_aligner_i.channel_width		=	cw;
		defparam	channel_aligner_i.deskew_depth		=	deskew_depth;		// 32;
		defparam	channel_aligner_i.deskew_ptr_width	= 	deskew_ptr_width;	// 5;	// log2(deskew_depth)
		defparam	channel_aligner_i.serial_channels	=	serial_channels;                                                                
		channel_aligner	channel_aligner_i	(
												.reset_n			(reset_n),
												.clk				(rx_pld_clk),
												.data_in			(rx_out_word_aligner),
												.channel_mask		(channel_mask),
												.channel_bond		(channel_bond),
												.pattern_detect		(pattern_detect),
												.channel_aligned	(channel_aligned),
												.deskew_overflow	(deskew_overflow),
												.data_out			(rx_out_bonded)
											);
		// After pll bit align, word align, & channel bond, enable detection of pattern_sync
		assign	enable_pattern_sync_prbs_detect	=	channel_aligned;
		assign	aci_cla							=	channel_aligned;
	end else begin
		assign	rx_out_bonded = rx_out_word_aligner;	
		// Currently always enabled if not using channel bonder.
		// Therefore make word_align_pattern different from pattern_sync_prbs
		assign	enable_pattern_sync_prbs_detect	=	1'b1;
		assign	aci_cla							=	1'b0;										
	end
endgenerate

 
// Converts to little endian - includes HSSDC loopback wire twist on the Stratix GX Development Board        
generate  
	if	((serial_channels == 20) && (stratix_gx_dev_kit_xcvrs == 1))  // XCVR GX Board (HSSDC2 twist)
	begin
		assign	rx_out_bonded_le	=	{ 
											{rx_out_bonded[(cw*19)-1:(19*cw)-cw]},	// HSSDC twist
											{rx_out_bonded[(cw*20)-1:(20*cw)-cw]},	// HSSDC twist
											{rx_out_bonded[(cw*17)-1:(17*cw)-cw]},	// HSSDC twist
											{rx_out_bonded[(cw*18)-1:(18*cw)-cw]},	// HSSDC twist									
											{rx_out_bonded[(cw*16)-1:(16*cw)-cw]},
											{rx_out_bonded[(cw*15)-1:(15*cw)-cw]},
											{rx_out_bonded[(cw*14)-1:(14*cw)-cw]},
											{rx_out_bonded[(cw*13)-1:(13*cw)-cw]},
											{rx_out_bonded[(cw*12)-1:(12*cw)-cw]},
											{rx_out_bonded[(cw*11)-1:(11*cw)-cw]},
											{rx_out_bonded[(cw*10)-1:(10*cw)-cw]},
											{rx_out_bonded[(cw* 9)-1:( 9*cw)-cw]},
											{rx_out_bonded[(cw* 8)-1:( 8*cw)-cw]},
											{rx_out_bonded[(cw* 7)-1:( 7*cw)-cw]},
											{rx_out_bonded[(cw* 6)-1:( 6*cw)-cw]},
											{rx_out_bonded[(cw* 5)-1:( 5*cw)-cw]},
											{rx_out_bonded[(cw* 4)-1:( 4*cw)-cw]},
											{rx_out_bonded[(cw* 3)-1:( 3*cw)-cw]},
											{rx_out_bonded[(cw* 2)-1:( 2*cw)-cw]},
											{rx_out_bonded[(cw* 1)-1:( 1*cw)-cw]} 
	
										};
	end else begin
		assign	rx_out_bonded_le	=	rx_out_bonded;
	end
									
endgenerate

generate
	if	(serdes_2d==1)
	begin
		defparam	lane_order_rotate_i.data_width		=	bonded_word_width;
		defparam	lane_order_rotate_i.lane_width		=	cw;
		defparam	lane_order_rotate_i.serial_channels	=	serial_channels;
		lane_order_rotate	lane_order_rotate_i
		(
			.reset_n			(reset_n),
			.clk				(rx_pld_clk),
			.en_lane_rotate		(channel_bond),
			.lane_order_pattern	(byte_order_pattern),
			.data_in			(rx_out_bonded_le),
			.lane_sync			(),
			.data_out			(rx_out_bonded_le_2d)
		);
	end else begin
		assign	rx_out_bonded_le_2d	=	rx_out_bonded_le;
	end
endgenerate
                                     
// Controls data_actual_write to start compare of bts data flow
// Includes error counter

assign	pattern_sync_acquired = data_actual_write;

// 1st parameter is synchronizer depth, 2nd parameter is the preset value
// Use synchonizer to add delay - make depth larger than bts rx fifo depth.
synchronizer	#(32,0)		synchonizer_rx_bts_reset_n	(.reset_n(reset_n),.clk(tx_pld_clk),.data_in((rx_fifos_rst_n || (!rx_delay_bts_start)) && enable_pattern_sync_prbs_detect),.data_out(rx_bts_reset_n));

// When the 1st TX PRBS data is sent, capture this data (pattern_sync) and activate search on
// rx for the 1st prbs data (pattern_sync_capture).
// Assume latency through the channel is > then the synchonizer latency, otherwise
// the pattern must repeat. 
// Alternate to this solution is to set pattern_sync value as a parameter OR calculate it on reset OR acknowledge it pseudo-static
parameter data_gen_pl	=	2;
//synchronizer	#(2,0)		synchonizer_pattern_sync	(.reset_n(reset_n),.clk(rx_pld_clk),.data_in((pattern_sync_capture == data_gen_pl)),.data_out(pattern_sync_capture_rx));
//pseudo static tx_pld_clk to rx_pld_clk domains:
assign pattern_sync_capture_rx=   (pattern_sync_capture == data_gen_pl);

reg	[1:0]	pattern_sync_capture;	// Capture from the PRBS TX generator
always	@(posedge tx_pld_clk or negedge	reset_n)
begin
	if	(!reset_n)
	begin
		//pattern_sync			<=	0; // don't wipe out pattern_sync on reset
		pattern_sync_capture	<=	0;
	end else begin
		if	((pattern_sync_capture != data_gen_pl) && (data_gen_read_bts))
		begin
			pattern_sync			<=	data_gen_readdata;
			pattern_sync_capture	<=	pattern_sync_capture + 1;
		end
	end
end

	
	
always	@(posedge rx_pld_clk or negedge rx_bts_reset_n)
begin
	if	(!rx_bts_reset_n)
	begin
		data_actual_write		<=	0;
		aci_cls					<=	0;
		aci_pass				<=	0;
		rx_out_bonded_le_pl1	<=	0;  
		rx_out_bonded_le_pl2	<=	0;
		test_complete_prev		<=	0;
		rx_ready_pl1			<=	0;
		rx_ready_pl2			<=	0;
	end else begin
	if
		(
				((rx_out_bonded_le_pl1 | sync_mask_1) == (pattern_sync | sync_mask_1))  &&
				(
					pattern_sync_capture_rx 		|| 
					(component_type == "X2_S4GX")	||
					(component_type == "X2_A2GX")
				)	// May need to run design twice, 1st to capture pattern_sync
		)		
		begin
			data_actual_write	<=	1;	// Test has started
		end
		if	(serial_channels > 1)
		begin 
			aci_cls				<=	(rx_out_bonded_le_pl1[bonded_word_width-1:0] == {serial_channels{word_align_pattern}});
		end
		rx_out_bonded_le_pl1	<=	rx_out_bonded_le_2d;
		rx_out_bonded_le_pl2	<=	rx_out_bonded_le_pl1;
		if (test_complete && (!test_complete_prev))
		begin
			aci_pass		<=	!error_latch;
		end
		test_complete_prev	<=	test_complete;		
		rx_ready_pl1		<=	rx_ready;
		rx_ready_pl2		<=	rx_ready_pl1;	
	end
end

defparam	tx_wrapper_bts_i.serial_channels			=	serial_channels;	
defparam	tx_wrapper_bts_i.word_align_pattern			=	word_align_pattern;	
defparam	tx_wrapper_bts_i.channel_width				=	cw;							// Make even   
defparam	tx_wrapper_bts_i.test_length_bts			=	test_length_bts;			// create_error inserts errors after test_length_bts bts words are transmitted
defparam	tx_wrapper_bts_i.test_complete_ctr_width	=	test_complete_ctr_width;	// > log2(test_length_bts)	  
defparam	tx_wrapper_bts_i.tx_wrapper_mode			=	tx_wrapper_mode;
defparam	tx_wrapper_bts_i.tx_datain_bts_idle			=	bts_idle;
defparam	tx_wrapper_bts_i.byte_order_pattern			=	byte_order_pattern;
defparam	tx_wrapper_bts_i.tx_data_ctr_width			=	tx_data_ctr_width;
defparam	tx_wrapper_bts_i.component_type				=	component_type;
defparam	tx_wrapper_bts_i.serdes_2d					=	serdes_2d;					// Set to 2 levels of SERDES synchronization
tx_wrapper_bts	tx_wrapper_bts_i	(
                         				.reset_n				(reset_n),			
										.clk					(tx_pld_clk),				
										.start					(start),				
										.create_error			(create_error),
										.tx_ready				(tx_ready),
										.data_gen_readdata		(data_gen_readdata),
										.data_gen_read			(data_gen_read),
										.tx_ctrlenable			(tx_ctrlenable_0),
										.tx_datain				(tx_datain_0),
										.test_complete          (),			// tx side test_complete 
										.aci_cem                (aci_cem_1)
									);  




   
// If using 8B/10B, send through data & control - can only delay by a multiple of "byte_width"
// Add skew delay on parallel tx side:
defparam	skew_adjust_bts_data_i.serial_channels		=	serial_channels;  
defparam	skew_adjust_bts_data_i.channel_width		=	cw;							// cw = channel width
defparam	skew_adjust_bts_data_i.channel_skew_width	=	channel_data_skew_width;	// ceil(log2(channel_width*(num_barrels-1)))
defparam	skew_adjust_bts_data_i.num_barrels			=	skew_num_barrels;			// Number of barrel shifters per channel	
defparam	skew_adjust_bts_data_i.skew_msb_1st			=	skew_msb_1st;				// skew_msb_1st=1 (Source Synchronous LVDS are MSB 1st. skew_msb_1st=0 (XCVRS are LSB 1st) 
skew_adjust_bts	skew_adjust_bts_data_i	(
										.reset_n		(reset_n),
										.clk			(tx_pld_clk),
										.datain			(tx_datain_0),
										.dataout		(tx_datain_skewed),
										.skew			(data_skew)
									);         

parameter	channel_ctrl_skew_width	=						channel_data_skew_width - 3;  // 1 control word per byte, log2(8 bits) = 3
reg			[(channel_ctrl_skew_width*serial_channels)-1:0]	 ctrl_skew;

   
// data_skew / 8 = ctrl_skew (on a per channel basis)   
integer	A,B;
always	@(data_skew)
begin  
	ctrl_skew = 0;
	for	(A=0; A<serial_channels; A=A+1)
	begin
		for (B=0; B<channel_ctrl_skew_width; B=B+1)   // Divide by 8 (1 control character per byte)
		begin 
			ctrl_skew[(A*channel_ctrl_skew_width)+B] = data_skew[(A*channel_data_skew_width)+B+3];
		end
	end
end

defparam	skew_adjust_bts_ctrl_i.serial_channels		=	serial_channels;  
defparam	skew_adjust_bts_ctrl_i.channel_width		=	tw;							// tw = control character width
defparam	skew_adjust_bts_ctrl_i.channel_skew_width	=	channel_ctrl_skew_width;	// ceil(log2(tw*(num_barrels-1)))
defparam	skew_adjust_bts_ctrl_i.num_barrels			=	skew_num_barrels;			// Number of barrel shifters per channel	
defparam	skew_adjust_bts_ctrl_i.skew_msb_1st			=	skew_msb_1st;
skew_adjust_bts	skew_adjust_bts_ctrl_i	(
										.reset_n		(reset_n),
										.clk			(tx_pld_clk),
										.datain			(tx_ctrlenable_0),
										.dataout		(tx_ctrlenable_skewed),
										.skew			(ctrl_skew)
									); 							
  
defparam	endian_connect_tx_ctrl.convert_endian		=	convert_endian;
defparam	endian_connect_tx_ctrl.serial_channels		=	serial_channels;
defparam	endian_connect_tx_ctrl.channel_width		=	tw;
defparam	endian_connect_tx_ctrl.num_endians			=	num_endians;
defparam	endian_connect_tx_ctrl.flip_data_bits		=	0;
endian_connect	endian_connect_tx_ctrl					(
															.data_in	(tx_ctrlenable_skewed),
															.data_out	(tx_ctrlenable)
														);									
 
// Word aligner is big endian, SERDES are little endian:        
defparam	endian_connect_tx_data.convert_endian		=	convert_endian;
defparam	endian_connect_tx_data.serial_channels		=	serial_channels;
defparam	endian_connect_tx_data.channel_width		=	cw;
defparam	endian_connect_tx_data.num_endians			=	num_endians;
defparam	endian_connect_tx_data.flip_data_bits		=	flip_data_bits;
endian_connect	endian_connect_tx_data					(
															.data_in	(tx_datain_skewed),
															.data_out	(tx_datain)
														);									

assign	data_gen_read_bts		=	data_gen_read && tx_ready;	
assign	data_actual_write_bts	=	data_actual_write && rx_ready_pl2;								
defparam	altera_bts_inst.nios_width				=	nios_width;		 	// For transaction counter
defparam	altera_bts_inst.data_width				=	data_width;
defparam	altera_bts_inst.prbs_depth				=	prbs_depth; 		// Use as seed for PRBS
defparam	altera_bts_inst.prbs_seed_param			=	prbs_seed_param;
defparam	altera_bts_inst.prbs_coeff_param		=	prbs_coeff_param;
defparam	altera_bts_inst.test_length_bts			=	test_length_bts;
defparam	altera_bts_inst.test_complete_ctr_width	=	test_complete_ctr_width;	// > log2(test_length_bts)	  
defparam	altera_bts_inst.tec_words				=	tec_words;
defparam	altera_bts_inst.num_lanes				=	num_lanes;
defparam	altera_bts_inst.enable_nios_bts			=	enable_nios_bts;
defparam	altera_bts_inst.prbs_type				=	prbs_type;
defparam	altera_bts_inst.prbs_width				=	prbs_width;
defparam	altera_bts_inst.channel_width			=	cw;
defparam	altera_bts_inst.serial_channels			=	serial_channels;
altera_bts	altera_bts_inst		(   
									// Compatible with avalon component
									.data_gen_reset_n       	(reset_n             			),
									.data_gen_clk           	(tx_pld_clk             		),
									.data_gen_chipselect    	(1'b1 							), // (Unused) Required by Avalon
									.data_gen_read          	(data_gen_read_bts				),
									.data_gen_address       	(0    							), // (Unused) Required by Avalon
									.data_gen_readdata      	(data_gen_readdata   			),									
									// Compare data read from external device or interface    
									.data_actual_reset_n        (rx_bts_reset_n             	),                                                                                                                                                                        
									.data_actual_clk            (rx_pld_clk			   			),
									.data_actual_chipselect     (1'b1							), // (Unused) Required by Avalon
									.data_actual_write          (data_actual_write_bts			), 
									.data_actual_address        (0								), // (Unused) Required by Avalon     
									.data_actual_writedata      (rx_out_bonded_le_pl2 			),
									.data_actual_waitrequest    (	                  			),												 
									// Signals for Signaltap, IRQs, etc
									.error_latch                (error_latch         			),
									.data_error                 (data_error          			),
									.test_complete				(test_complete					),
									.error_mask					(error_mask_1					),
									// NIOS BTS:
									.aci_prs                    (aci_prs						),
									.aci_prd                    (aci_prd						),
									.aci_prc                    (aci_prc						),
									.aci_prt					(aci_prt						),
									.aci_ttc                    (aci_ttc						),
									.aci_rtc                    (aci_rtc						),
									.aci_wec                    (aci_wec						),
									.aci_lec                    (aci_lec						),
									.aci_ips					(aci_ips						)
								);                             
endmodule
