/********************************************************************
* Module:    delayed_reset
* Author:    David T. Johnson
* Function:  Creates delayed pulse on each rising input edge
* Revision:  1.0  October 20, 2005
********************************************************************/

//Copyright  2005 Altera Corporation. All rights reserved.  Altera products 
//are protected under numerous U.S. and foreign patents, maskwork rights, 
//copyrights and other intellectual property laws.  
//
//This reference design file, and your use thereof, is subject to and governed
//by the terms and conditions of the applicable Altera Reference Design 
//License Agreement.  By using this reference design file, you indicate your
//acceptance of such terms and conditions between you and Altera Corporation.
//In the event that you do not agree with such terms and conditions, you may
//not use the reference design file. Please promptly destroy any copies you 
//have made.
//
//This reference design file being provided on an "as-is" basis and as an 
//accommodation and therefore all warranties, representations or guarantees
//of any kind (whether express, implied or statutory) including, without 
//limitation, warranties of merchantability, non-infringement, or fitness for
//a particular purpose, are specifically disclaimed.  By making this reference
//design file available, Altera expressly does not recommend, suggest or 
//require that this reference design file be used in combination with any 
//other product not provided by Altera

module delayed_reset
(
	clk,
	reset_n,
	trig,
	data_in,
	data_out			
);
						
	parameter				assert_val 		= 1;		// Asserted value of pulse (1 or 0)
	parameter				pulse_width		= 32;		// Width of pulse
	parameter 				counter_max 	= 100000; 	// Delay from rising edge on trig,
														// until data_out asserts
														// Eg 10 ns clock, 100000 is 1 ms
	parameter				counter_width	=	32;		// Set = ceil(log2(counter_max))					
	input					clk;
	input					reset_n;
	input					trig;
	input					data_in;
	output					data_out;
										
	reg						data_out;									
// Internal data structures:
	reg						trig_prev;
	reg	[counter_width-1:0]	counter;
	
	always	@(posedge clk or negedge reset_n)
	begin
		if	(!reset_n)
		begin
			data_out		<=	assert_val;
			counter			<=	0;
			trig_prev		<=	0;
		end else begin
			if	(!trig_prev && trig) // + edge
			begin
				counter		<=	counter_max;
				data_out	<=	data_in;
			end else if	(counter == pulse_width)	// Start pulse
			begin
				counter		<=	counter - 1;
				data_out	<=	assert_val;
			end else if (counter == 0)				// End pulse
			begin
				data_out	<=	data_in;
			end else begin
				counter	<=	counter - 1;
				data_out	<=	data_in;
			end
			trig_prev	<=	trig;
		end
	end
				
					

endmodule
