/******************************************************************************
*Copyright  2006 Altera Corporation. All rights reserved.  Altera products   *
*are protected under numerous U.S. and foreign patents, maskwork rights,      *
*copyrights and other intellectual property laws.                             *
*                                                                             *
*This reference design file, and your use thereof, is subject to and governed *
*by the terms and conditions of the applicable Altera Reference Design        *
*License Agreement.  By using this reference design file, you indicate your   *
*acceptance of such terms and conditions between you and Altera Corporation.  *
*In the event that you do not agree with such terms and conditions, you may   *
*not use the reference design file. Please promptly destroy any copies you    *
*have made.                                                                   *
*                                                                             *
*This reference design file being provided on an "as-is" basis and as an      *
*accommodation and therefore all warranties, representations or guarantees    *
*of any kind (whether express, implied or statutory) including, without       *
*limitation, warranties of merchantability, non-infringement, or fitness for  *
*a particular purpose, are specifically disclaimed.  By making this reference *
*design file available, Altera expressly does not recommend, suggest or       *
*require that this reference design file be used in combination with any      *
*other product not provided by Altera                                         */

/******************************************************************************
* NOTICE: The information contained in this file is proprietary   			  *
* to Altera and is being made available to Altera's customers     			  *
* under strict non-disclosure agreements.  Use or disclosure of   			  *
* this information is permissible only under the terms of         			  *
* existing non-disclosure agreements.                             			  *
*******************************************************************************/    

/*
-------------------------------------------------------------------------------------------
--                                                                                       -- 
-- Title:       endian_connect.v                                                       --
-- Rev:         Rev 1                                                                    --
-- Modified:    February 1, 2006                                                         --
-- Author:      David Johnson                                                            --
-- Converts Endian - Fully combinatorial "wire twists" to convert endian                 --
--                                                                                       -- 
--------------------------------------------------------------------------------------------  
--                                                                                       --
-- Revision History:                                                                     --  
--                                                                                       --
-- Rev 1:       Created                                                                  --
--                                                                                       --
--------------------------------------------------------------------------------------------
*/

// May want to modify this module if endians are not sequential   
module	endian_connect	(
								data_in,
								data_out
							);         

  
							
parameter	convert_endian		=	1;	
parameter	serial_channels		=	4;      // Quad
parameter	channel_width		=	16; 	// Channel Width
parameter	num_endians			=	channel_width / 8;  
parameter	flip_data_bits		=	0;
// Internal data structures:
		
parameter	data_width			=	channel_width * serial_channels;   
parameter	endian_width		=	channel_width / num_endians;	// Normally equals 8 (1 byte)
           
input		[data_width-1:0]	data_in;
output		[data_width-1:0]	data_out;  

wire		[data_width-1:0]	data_in;
reg			[data_width-1:0]	data_out;  
integer		A;
integer		B; 
integer		C;
integer		data_out_bit;
          
 
	always	@(data_in)
	begin  
		if	(convert_endian)	// Convert Endian
		begin
			data_out_bit	= 0;
			for	(A=0; A<serial_channels; A=A+1)
			begin
				for (B=num_endians-1; B>=0; B=B-1)
				begin   
					for (C=0; C< endian_width; C=C+1)
					begin   
						if	(flip_data_bits)
						begin
							data_out[data_out_bit]	=	data_in[(A*channel_width)+ (B*endian_width)+(endian_width-1-C)];
						end else begin
							data_out[data_out_bit]	=	data_in[(A*channel_width)+ (B*endian_width)+C];
						end
						data_out_bit = data_out_bit + 1;
					end
				end
			end	
		end else begin			// Don't Convert Endian
			data_out	=	data_in;
		end
	end

endmodule
