/********************************************************************
* Module:    phasecomp_dcfifo
* Author:    David T. Johnson
* Function:  Phase Compensation FIFO
* Revision:  1.0  July 20, 2005
*******************************************************************/

//Copyright  2005 Altera Corporation. All rights reserved.  Altera products 
//are protected under numerous U.S. and foreign patents, maskwork rights, 
//copyrights and other intellectual property laws.  
//
//This reference design file, and your use thereof, is subject to and governed
//by the terms and conditions of the applicable Altera Reference Design 
//License Agreement.  By using this reference design file, you indicate your
//acceptance of such terms and conditions between you and Altera Corporation.
//In the event that you do not agree with such terms and conditions, you may
//not use the reference design file. Please promptly destroy any copies you 
//have made.
//
//This reference design file being provided on an "as-is" basis and as an 
//accommodation and therefore all warranties, representations or guarantees
//of any kind (whether express, implied or statutory) including, without 
//limitation, warranties of merchantability, non-infringement, or fitness for
//a particular purpose, are specifically disclaimed.  By making this reference
//design file available, Altera expressly does not recommend, suggest or 
//require that this reference design file be used in combination with any 
//other product not provided by Altera

module phasecomp_fifo(
							reset_n, 
							inclk,
							datain,  
							outclk,
							dataout					
					);

	parameter	data_width 			=   8;
	parameter 	fifo_depth			=	6;				// Make even
	parameter	fifo_depth_2		=	fifo_depth/2;   // round(fifo_depth/2), centered for skew / jitter
	parameter	ptr_width			=	3;              // >= ceil(log2(fifo_depth))
	parameter	fifo_low_latency	=	fifo_depth_2;	// For low latency fifo  For QDRII_Q, calculate from
														// boards skews, drift, etc.	
	input 							reset_n; 
	input 							inclk;  
	input	[(data_width-1):0]		datain;  
	input							outclk;
	output	[(data_width-1):0] 		dataout;    

	// FIFO Queue

	
	reg	[ptr_width-1:0]		rd_ptr;
	reg	[ptr_width-1:0]		wr_ptr;
	reg	[data_width-1:0]	mem	[0:fifo_depth-1];
	reg	[data_width-1:0]	dataout_reg;
		
	integer A;
	always	@(posedge inclk or negedge reset_n)
	begin
		if	(!reset_n)
		begin
			wr_ptr	<=	0;
			for	(A=0; A<fifo_depth; A=A+1)	// async clear memory
			begin
				mem[A]	<=	0;
			end
		end else begin
			if	(wr_ptr == fifo_depth-1)
			begin
				wr_ptr	<=	0;
			end else begin
				wr_ptr	<=	wr_ptr + 1;
			end
			mem[wr_ptr]	<=	datain;
		end
	end
	
	always	@(posedge outclk or negedge reset_n)
	begin
		if	(!reset_n)
		begin
			rd_ptr	<=	fifo_low_latency;
		end else begin
			if	(rd_ptr == fifo_depth-1)
			begin
				rd_ptr	<=	0;
			end else begin
				rd_ptr	<=	rd_ptr + 1;
			end
			dataout_reg	<=	mem[rd_ptr];
		end
	end
	
	assign	dataout	=	dataout_reg;
					

endmodule
