/******************************************************************************
*                                                                             *
* License Agreement                                                           *
*                                                                             *
* Copyright (c) 2010 Altera Corporation, San Jose, California, USA.           *
* All rights reserved.                                                        *
*                                                                             *
* Permission is hereby granted, free of charge, to any person obtaining a     *
* copy of this software and associated documentation files (the "Software"),  *
* to deal in the Software without restriction, including without limitation   *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
* and/or sell copies of the Software, and to permit persons to whom the       *
* Software is furnished to do so, subject to the following conditions:        *
*                                                                             *
* The above copyright notice and this permission notice shall be included in  *
* all copies or substantial portions of the Software.                         *
*                                                                             *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
* DEALINGS IN THE SOFTWARE.                                                   *
*                                                                             *
* This agreement shall be governed in all respects by the laws of the State   *
* of California and by the laws of the United States of America.              *
* Altera does not recommend, suggest or require that this reference design    *
* file be used in conjunction or combination with any other product.          *
******************************************************************************/

/*
  bts_test.c:
  Implements functions which perform BTS tests, such as a loopback
  test on a specific interface, ddr2 memory test, etc.
*/

#include  "bts_test.h"

// COM:
int factory_test_com(struct aci_map_com *aci)
{
  (void) reset_com(aci);  
  (void) comma_detect_com(aci);
  (void) enable_channel_bond_com(aci);
  (void) start_com(aci);   
  usleep(FACTORY_DLY);
  (void) aci_rw_regs_com(aci,DISPLAY_ON,aci->base_address[0],READ_S,BCR_S); 
  return  0;
}

int optimize_xcvrs_com(struct aci_map_com *aci)
{
  alt_u8  done_flag = 0;
  int     	A;
  int		B;

  //alt_u32	xvc_err[serial_channels];


  
  while (done_flag  ==  0)
  {
    // Establish link & send bonded word data
   (void) reset_com(aci);  
   (void) comma_detect_com(aci);
   (void) enable_channel_bond_com(aci);
   (void) start_com(aci);   
    // Measure error rate
   (void)  aci_rw_regs_com(aci,DISPLAY_ON,aci->base_address[0],READ_S,LEC_S);
   // Change VOD, Preemphasis, & RX Equalization
   (void)  aci_rw_regs_com(aci,DISPLAY_ON,aci->base_address[0],READ_S,XVC_S);    
   // If error rate is low enough, set done_flag = 1
   
   // printf("optimized vod3 @ %d = %Xh\n",A,aci->xvc_vod[3]);
   //printf("serial channels = %d\n",aci->serial_channels[0]);

   (void)  get_xvc_com(aci,aci->base_address[0],aci->XVC_A); // Will initialize "*_opt" with DPRIO



	 for	(A=0; A<aci->serial_channels[0]; A++)
	 {

		 xvc_scs->xvc_dc_start[A]		=	0;
		 xvc_scs->xvc_eq_start[A]		=	0;  // ->(1 << (aci->EQ_BITS)) - 1;<-  // 0 // 5
		 xvc_scs->xvc_p0_start[A]		=	0x10;	// 0;
		 xvc_scs->xvc_p1_start[A]		=	0x0;
		 xvc_scs->xvc_p2_start[A]		=	0x10;	// 0; (convert eg x10 = 0 x01=-1)
		 xvc_scs->xvc_vod_start[A]		=	4; 		// 0;
		 xvc_scs->xvc_dc_cnt[0]			=	1;	//	1 << (aci->DC_BITS);
		 xvc_scs->xvc_eq_cnt[0]			=	1 << (aci->EQ_BITS); //6
		 xvc_scs->xvc_p0_cnt[0]			=	1;	// (1 << (aci->P0_BITS);
		 xvc_scs->xvc_p1_cnt[0]			=	1 << (aci->P1_BITS);
		 xvc_scs->xvc_p2_cnt[0]			=	1;	// 1 << (aci->P2_BITS);
		 xvc_scs->xvc_vod_cnt[0]		=	2; //// 1 << (aci->VOD_BITS);
		 xvc_scs->xvc_dc_sweep[A]		=	1;
		 xvc_scs->xvc_eq_sweep[A]		=	1 ;// ->0<- ; ///// 0;
		 xvc_scs->xvc_p0_sweep[A]		=	1;
		 xvc_scs->xvc_p1_sweep[A]		=	1;
		 xvc_scs->xvc_p2_sweep[A]		=	1;
		 xvc_scs->xvc_vod_sweep[A]		=	1;


		 /*
		 		 aci->xvc_dc_min[A]		=	0x0;
		 		 aci->xvc_dc_max[A]		=	0x0;	// (1 << (aci->VOD_BITS)) - 1;
		 		 aci->xvc_eq_min[A]		=	0;
		 		 aci->xvc_eq_max[A]		=	(1 << (aci->EQ_BITS))  - 1;
		 		 aci->xvc_p0_min[A]		=	0x10;	//0;
		 		 aci->xvc_p0_max[A]		=	0x10;	// (1 << (aci->P0_BITS))  - 1;
		 		 aci->xvc_p1_min[A]		=	0x0;
		 		 aci->xvc_p1_max[A]		=	(1 << (aci->P1_BITS))  - 1;
		 		 aci->xvc_p2_min[A]		=	0x10;	// 0;
		 		 aci->xvc_p2_max[A]		=	0x10;	// (1 << (aci->P2_BITS))  - 1;
		 		 aci->xvc_vod_min[A]	=	0;
		 		 aci->xvc_vod_max[A]	=	(1 << (aci->VOD_BITS))  - 1;
		 		 aci->xvc_err_min[A]	=	0xffffffff;
		 		 aci->xvc_err_max[A]	=	0xffffffff;
		 		 aci->xvc_status_min[A]	=	0;
		 		 aci->xvc_status_max[A]	=	0;
		 */

		 	for	(B=0; B<xvc_adapt_surfaces_dd; B++)
		 	{
				 aci->xvc_dc_opt[B].xvc_vod_adpt[A]		= 0;		// 0 thru (1 << (aci->VOD_BITS))  - 1
				 aci->xvc_dc_opt[B].xvc_eq_adpt[A]		= 0;		// 0 thru (1 << (aci->EQ_BITS))  - 1
				 aci->xvc_dc_opt[B].xvc_p0_adpt[A]		= 0;		// 0 thru (1 << (aci->P0_BITS))  - 1
				 aci->xvc_dc_opt[B].xvc_p1_adpt[A]		= 0;		// 0 thru (1 << (aci->P1_BITS))  - 1
				 aci->xvc_dc_opt[B].xvc_p2_adpt[A]		= 0;		// 0 thru (1 << (aci->P2_BITS))  - 1
				 aci->xvc_dc_opt[B].xvc_dc_adpt[A]		= 0;		// 0 thru (1 << (aci->DC_BITS))  - 1
				 aci->xvc_dc_opt[B].xvc_err_adpt[A]		= 0xFFFFFFFF;
				 aci->xvc_dc_opt[B].xvc_status_adpt[A]	= 0;
				 aci->xvc_dc_opt[B].xvc_passes_ch_adpt[A]= 0;
				 aci->xvc_dc_opt[B].xvc_passes_all_adpt[A]= 0;
/*
				 aci->xvc_eq_opt[B].xvc_vod_adpt[A]		= 0;		// 0 thru (1 << (aci->VOD_BITS))  - 1
				 aci->xvc_eq_opt[B].xvc_eq_adpt[A]		= 0;		// 0 thru (1 << (aci->EQ_BITS))  - 1
				 aci->xvc_eq_opt[B].xvc_p0_adpt[A]		= 0;		// 0 thru (1 << (aci->P0_BITS))  - 1
				 aci->xvc_eq_opt[B].xvc_p1_adpt[A]		= 0;		// 0 thru (1 << (aci->P1_BITS))  - 1
				 aci->xvc_eq_opt[B].xvc_p2_adpt[A]		= 0;		// 0 thru (1 << (aci->P2_BITS))  - 1
				 aci->xvc_eq_opt[B].xvc_dc_adpt[A]		= 0;		// 0 thru (1 << (aci->DC_BITS))  - 1
				 aci->xvc_eq_opt[B].xvc_err_adpt[A]		= 0xFFFFFFFF;
				 aci->xvc_eq_opt[B].xvc_status_adpt[A]	= 0;
				 aci->xvc_eq_opt[B].xvc_passes_ch_adpt[A]= 0;
				 aci->xvc_eq_opt[B].xvc_passes_all_adpt[A]= 0;

				 aci->xvc_p0_opt[B].xvc_vod_adpt[A]		= 0;		// 0 thru (1 << (aci->VOD_BITS))  - 1
				 aci->xvc_p0_opt[B].xvc_eq_adpt[A]		= 0;		// 0 thru (1 << (aci->EQ_BITS))  - 1
				 aci->xvc_p0_opt[B].xvc_p0_adpt[A]		= 0;		// 0 thru (1 << (aci->P0_BITS))  - 1
				 aci->xvc_p0_opt[B].xvc_p1_adpt[A]		= 0;		// 0 thru (1 << (aci->P1_BITS))  - 1
				 aci->xvc_p0_opt[B].xvc_p2_adpt[A]		= 0;		// 0 thru (1 << (aci->P2_BITS))  - 1
				 aci->xvc_p0_opt[B].xvc_dc_adpt[A]		= 0;		// 0 thru (1 << (aci->DC_BITS))  - 1
				 aci->xvc_p0_opt[B].xvc_err_adpt[A]		= 0xFFFFFFFF;
				 aci->xvc_p0_opt[B].xvc_status_adpt[A]	= 0;
				 aci->xvc_p0_opt[B].xvc_passes_ch_adpt[A]= 0;
				 aci->xvc_p0_opt[B].xvc_passes_all_adpt[A]= 0;

				 aci->xvc_p1_opt[B].xvc_vod_adpt[A]		= 0;		// 0 thru (1 << (aci->VOD_BITS))  - 1
				 aci->xvc_p1_opt[B].xvc_eq_adpt[A]		= 0;		// 0 thru (1 << (aci->EQ_BITS))  - 1
				 aci->xvc_p1_opt[B].xvc_p0_adpt[A]		= 0;		// 0 thru (1 << (aci->P0_BITS))  - 1
				 aci->xvc_p1_opt[B].xvc_p1_adpt[A]		= 0;		// 0 thru (1 << (aci->P1_BITS))  - 1
				 aci->xvc_p1_opt[B].xvc_p2_adpt[A]		= 0;		// 0 thru (1 << (aci->P2_BITS))  - 1
				 aci->xvc_p1_opt[B].xvc_dc_adpt[A]		= 0;		// 0 thru (1 << (aci->DC_BITS))  - 1
				 aci->xvc_p1_opt[B].xvc_err_adpt[A]		= 0xFFFFFFFF;
				 aci->xvc_p1_opt[B].xvc_status_adpt[A]	= 0;
				 aci->xvc_p1_opt[B].xvc_passes_ch_adpt[A]= 0;
				 aci->xvc_p1_opt[B].xvc_passes_all_adpt[A]= 0;

				 aci->xvc_p2_opt[B].xvc_vod_adpt[A]		= 0;		// 0 thru (1 << (aci->VOD_BITS))  - 1
				 aci->xvc_p2_opt[B].xvc_eq_adpt[A]		= 0;		// 0 thru (1 << (aci->EQ_BITS))  - 1
				 aci->xvc_p2_opt[B].xvc_p0_adpt[A]		= 0;		// 0 thru (1 << (aci->P0_BITS))  - 1
				 aci->xvc_p2_opt[B].xvc_p1_adpt[A]		= 0;		// 0 thru (1 << (aci->P1_BITS))  - 1
				 aci->xvc_p2_opt[B].xvc_p2_adpt[A]		= 0;		// 0 thru (1 << (aci->P2_BITS))  - 1
				 aci->xvc_p2_opt[B].xvc_dc_adpt[A]		= 0;		// 0 thru (1 << (aci->DC_BITS))  - 1
				 aci->xvc_p2_opt[B].xvc_err_adpt[A]		= 0xFFFFFFFF;
				 aci->xvc_p2_opt[B].xvc_status_adpt[A]	= 0;
				 aci->xvc_p2_opt[B].xvc_passes_ch_adpt[A]= 0;
				 aci->xvc_p2_opt[B].xvc_passes_all_adpt[A]= 0;

				 aci->xvc_vod_opt[B].xvc_vod_adpt[A]	= 0;		// 0 thru (1 << (aci->VOD_BITS))  - 1
				 aci->xvc_vod_opt[B].xvc_eq_adpt[A]		= 0;		// 0 thru (1 << (aci->EQ_BITS))  - 1
				 aci->xvc_vod_opt[B].xvc_p0_adpt[A]		= 0;		// 0 thru (1 << (aci->P0_BITS))  - 1
				 aci->xvc_vod_opt[B].xvc_p1_adpt[A]		= 0;		// 0 thru (1 << (aci->P1_BITS))  - 1
				 aci->xvc_vod_opt[B].xvc_p2_adpt[A]		= 0;		// 0 thru (1 << (aci->P2_BITS))  - 1
				 aci->xvc_vod_opt[B].xvc_dc_adpt[A]		= 0;		// 0 thru (1 << (aci->DC_BITS))  - 1
				 aci->xvc_vod_opt[B].xvc_err_adpt[A]	= 0xFFFFFFFF;
				 aci->xvc_vod_opt[B].xvc_status_adpt[A]	= 0;
				 aci->xvc_vod_opt[B].xvc_passes_ch_adpt[A]= 0;
				 aci->xvc_vod_opt[B].xvc_passes_all_adpt[A]= 0;
*/
		 	}

/*
		 // Initial with something (read DPRIO from ALTGX)
		 aci->xvc_vod_opt[A]	= aci->xvc_vod[A];
		 aci->xvc_eq_opt[A]		= aci->xvc_eq[A];
		 aci->xvc_p0_opt[A]		= aci->xvc_p0[A];
		 aci->xvc_p1_opt[A]		= aci->xvc_p1[A];
		 aci->xvc_p2_opt[A]		= aci->xvc_p2[A];
		 aci->xvc_dc_opt[A]		= aci->xvc_dc[A];
		 aci->xvc_err_opt[A]	= 0xffffffff;
		 aci->xvc_status_opt[A]	= 0;
*/
		/*
		 printf("\t\tOPT: Channel %D:\tVOD=%Xh\tP0=%Xh\tP1=%Xh\tP2=%Xh\tRX EQ=%Xh\tDC GAIN=%Xh\n",
    				 A,aci->xvc_vod_opt[A],aci->xvc_p0_opt[A],aci->xvc_p1_opt[A],aci->xvc_p2_opt[A],aci->xvc_eq_opt[A],aci->xvc_dc_opt[A]);
 		 printf("\t\tMIN: Channel %D:\tVOD=%Xh\tP0=%Xh\tP1=%Xh\tP2=%Xh\tRX EQ=%Xh\tDC GAIN=%Xh\n",
    				 A,aci->xvc_vod_min[A],aci->xvc_p0_min[A],aci->xvc_p1_min[A],aci->xvc_p2_min[A],aci->xvc_eq_min[A],aci->xvc_dc_min[A]);
 		 printf("\t\tMAX: Channel %D:\tVOD=%Xh\tP0=%Xh\tP1=%Xh\tP2=%Xh\tRX EQ=%Xh\tDC GAIN=%Xh\n",
    				 A,aci->xvc_vod_max[A],aci->xvc_p0_max[A],aci->xvc_p1_max[A],aci->xvc_p2_max[A],aci->xvc_eq_max[A],aci->xvc_dc_max[A]);
*/

	 }


	 //aci->xvc_dc_opt[6].xvc_vod_adpt[2]	=	3;
	 //printf("value = %d",aci->xvc_dc_opt[6].xvc_vod_adpt[2]);

	 (void)  optimize_xcvrs_sweep(aci,&xvc_scs[0]);
	 printf("Optimization Results:\n");
	 for	(A=0; A<aci->serial_channels[0]; A++)
    {

		 printf("\t\tdc_min:	Channel %D:\tVOD=%Xh\tP0=%Xh\tP1=%Xh\tP2=%Xh\tRX EQ=%Xh\tDC GAIN=%Xh Status=%Xh Errors=%Xh Passes-Ch=%Xh Passes-All=%Xh\n",
				 A,aci->xvc_dc_opt[dc_min].xvc_vod_adpt[A],aci->xvc_dc_opt[dc_min].xvc_p0_adpt[A],aci->xvc_dc_opt[dc_min].xvc_p1_adpt[A],
				 aci->xvc_dc_opt[dc_min].xvc_p2_adpt[A],aci->xvc_dc_opt[dc_min].xvc_eq_adpt[A],aci->xvc_dc_opt[dc_min].xvc_dc_adpt[A],
				 aci->xvc_dc_opt[dc_min].xvc_status_adpt[A],aci->xvc_dc_opt[dc_min].xvc_err_adpt[A],
				 aci->xvc_dc_opt[dc_min].xvc_passes_ch_adpt[A],aci->xvc_dc_opt[dc_min].xvc_passes_all_adpt[A]);

		 printf("\t\tdc_max:	Channel %D:\tVOD=%Xh\tP0=%Xh\tP1=%Xh\tP2=%Xh\tRX EQ=%Xh\tDC GAIN=%Xh Status=%Xh Errors=%Xh Passes-Ch=%Xh Passes-All=%Xh\n",
				 A,aci->xvc_dc_opt[dc_max].xvc_vod_adpt[A],aci->xvc_dc_opt[dc_max].xvc_p0_adpt[A],aci->xvc_dc_opt[dc_max].xvc_p1_adpt[A],
				 aci->xvc_dc_opt[dc_max].xvc_p2_adpt[A],aci->xvc_dc_opt[dc_max].xvc_eq_adpt[A],aci->xvc_dc_opt[dc_max].xvc_dc_adpt[A],
				 aci->xvc_dc_opt[dc_max].xvc_status_adpt[A],aci->xvc_dc_opt[dc_max].xvc_err_adpt[A],
				 aci->xvc_dc_opt[dc_max].xvc_passes_ch_adpt[A],aci->xvc_dc_opt[dc_max].xvc_passes_all_adpt[A]);

		 printf("\t\tdc_min_link:	Channel %D:\tVOD=%Xh\tP0=%Xh\tP1=%Xh\tP2=%Xh\tRX EQ=%Xh\tDC GAIN=%Xh Status=%Xh Errors=%Xh Passes-Ch=%Xh Passes-All=%Xh\n",
				 A,aci->xvc_dc_opt[dc_min_link].xvc_vod_adpt[A],aci->xvc_dc_opt[dc_min_link].xvc_p0_adpt[A],aci->xvc_dc_opt[dc_min_link].xvc_p1_adpt[A],
				 aci->xvc_dc_opt[dc_min_link].xvc_p2_adpt[A],aci->xvc_dc_opt[dc_min_link].xvc_eq_adpt[A],aci->xvc_dc_opt[dc_min_link].xvc_dc_adpt[A],
				 aci->xvc_dc_opt[dc_min_link].xvc_status_adpt[A],aci->xvc_dc_opt[dc_min_link].xvc_err_adpt[A],
				 aci->xvc_dc_opt[dc_min_link].xvc_passes_ch_adpt[A],aci->xvc_dc_opt[dc_min_link].xvc_passes_all_adpt[A]);

		 printf("\t\tdc_max_link:	Channel %D:\tVOD=%Xh\tP0=%Xh\tP1=%Xh\tP2=%Xh\tRX EQ=%Xh\tDC GAIN=%Xh Status=%Xh Errors=%Xh Passes-Ch=%Xh Passes-All=%Xh\n",
				 A,aci->xvc_dc_opt[dc_max_link].xvc_vod_adpt[A],aci->xvc_dc_opt[dc_max_link].xvc_p0_adpt[A],aci->xvc_dc_opt[dc_max_link].xvc_p1_adpt[A],
				 aci->xvc_dc_opt[dc_max_link].xvc_p2_adpt[A],aci->xvc_dc_opt[dc_max_link].xvc_eq_adpt[A],aci->xvc_dc_opt[dc_max_link].xvc_dc_adpt[A],
				 aci->xvc_dc_opt[dc_max_link].xvc_status_adpt[A],aci->xvc_dc_opt[dc_max_link].xvc_err_adpt[A],
				 aci->xvc_dc_opt[dc_max_link].xvc_passes_ch_adpt[A],aci->xvc_dc_opt[dc_max_link].xvc_passes_all_adpt[A]);

		 printf("\t\tdc_opt:	Channel %D:\tVOD=%Xh\tP0=%Xh\tP1=%Xh\tP2=%Xh\tRX EQ=%Xh\tDC GAIN=%Xh Status=%Xh Errors=%Xh Passes-Ch=%Xh Passes-All=%Xh\n",
				 A,aci->xvc_dc_opt[dc_opt].xvc_vod_adpt[A],aci->xvc_dc_opt[dc_opt].xvc_p0_adpt[A],aci->xvc_dc_opt[dc_opt].xvc_p1_adpt[A],
				 aci->xvc_dc_opt[dc_opt].xvc_p2_adpt[A],aci->xvc_dc_opt[dc_opt].xvc_eq_adpt[A],aci->xvc_dc_opt[dc_opt].xvc_dc_adpt[A],
				 aci->xvc_dc_opt[dc_opt].xvc_status_adpt[A],aci->xvc_dc_opt[dc_opt].xvc_err_adpt[A],
				 aci->xvc_dc_opt[dc_opt].xvc_passes_ch_adpt[A],aci->xvc_dc_opt[dc_opt].xvc_passes_all_adpt[A]);


		 printf("\t\tdc_opt_link:	Channel %D:\tVOD=%Xh\tP0=%Xh\tP1=%Xh\tP2=%Xh\tRX EQ=%Xh\tDC GAIN=%Xh Status=%Xh Errors=%Xh Passes-Ch=%Xh Passes-All=%Xh\n",
				 A,aci->xvc_dc_opt[dc_opt_link].xvc_vod_adpt[A],aci->xvc_dc_opt[dc_opt_link].xvc_p0_adpt[A],aci->xvc_dc_opt[dc_opt_link].xvc_p1_adpt[A],
				 aci->xvc_dc_opt[dc_opt_link].xvc_p2_adpt[A],aci->xvc_dc_opt[dc_opt_link].xvc_eq_adpt[A],aci->xvc_dc_opt[dc_opt_link].xvc_dc_adpt[A],
				 aci->xvc_dc_opt[dc_opt_link].xvc_status_adpt[A],aci->xvc_dc_opt[dc_opt_link].xvc_err_adpt[A],
				 aci->xvc_dc_opt[dc_opt_link].xvc_passes_ch_adpt[A],aci->xvc_dc_opt[dc_opt_link].xvc_passes_all_adpt[A]);

	 }

    // Write DPRIO with optimum setting: (for now use _max (the last passing)
	 (void)  get_xvc_com(aci,aci->base_address[0],aci->XVC_A);
	 for	(A=0; A<aci->serial_channels[0]; A++)
	 {
		 //printf("Channel %d:\tDC_CTR=%d P2_CTR=%d P1_CTR=%d P0_CTR=%d EQ_CTR=%d VOD_CTR=%d\n",
		//		 A,DC_CTR,P2_CTR,P1_CTR,P0_CTR,EQ_CTR,VOD_CTR);

		 //printf("lec %d = %d\n",A,aci->lec[A])



		 aci->xvc_vod[A] = aci->xvc_dc_opt[dc_opt_link].xvc_vod_adpt[A]; // aci->xvc_vod_max[A];
		 aci->xvc_eq[A]	 = aci->xvc_dc_opt[dc_opt_link].xvc_eq_adpt[A];	// aci->xvc_eq_max[A];
		 aci->xvc_p0[A]	 = aci->xvc_dc_opt[dc_opt_link].xvc_p0_adpt[A]; // aci->xvc_p0_max[A];
		 aci->xvc_p1[A]	 = aci->xvc_dc_opt[dc_opt_link].xvc_p1_adpt[A]; // aci->xvc_p1_max[A];
		 aci->xvc_p2[A]	 = aci->xvc_dc_opt[dc_opt_link].xvc_p2_adpt[A];	// aci->xvc_p2_max[A];
		 aci->xvc_dc[A]	 = aci->xvc_dc_opt[dc_opt_link].xvc_dc_adpt[A];	// aci->xvc_dc_max[A];


		 printf("\t\Channel %D:\tVOD=%Xh\tP0=%Xh\tP1=%Xh\tP2=%Xh\tRX EQ=%Xh\tDC GAIN=%Xh\tSerial Loopback=%X\n",
				 A,aci->xvc_vod[A],aci->xvc_p0[A],aci->xvc_p1[A],aci->xvc_p2[A],aci->xvc_eq[A],aci->xvc_dc[A],aci->xvc_slb[A]);
	 }
		 (void)  put_xvc_com(aci,aci->base_address[0],aci->XVC_A);

  	 (void) reset_com(aci);
    	 (void) comma_detect_com(aci);
    	 (void) enable_channel_bond_com(aci);
		 (void) start_com(aci);
		 (void)  aci_rw_regs_com(aci,DISPLAY_OFF,aci->base_address[0],READ_S,LEC_S);
		 io_bcr_com(aci,aci->base_address[0],aci->BCR_A); // This reads all lanes!





      done_flag = 2;   


  }

  return  0;
}


int optimize_xcvrs_sweep(struct aci_map_com *aci,struct xvc_map_scs *xvc_scs)
{


	  alt_u32	rd_data;
	  int		A;
	  int	  	VOD_CTR;
	  int	  	EQ_CTR;
	  int		P0_CTR;
	  int		P1_CTR;
	  int		P2_CTR;
	  int		DC_CTR;

	  alt_u8	VOD_CNT;
	  alt_u8	EQ_CNT;
	  alt_u8	P0_CNT;
	  alt_u8	P1_CNT;
	  alt_u8	P2_CNT;
	  alt_u8	DC_CNT;
	  alt_u32	STATUS[serial_channels_dd];
	  alt_u8	STATUS_ERR_VALID	=	5;	// (1) RX PLL + (2) TX PLL + (3) ALL PLL + (4) Channel Lock, error counters aren't running until (5) Pattern Sync Acquired
	  alt_u64	LEC_TOTAL;

	  alt_u8	xvc_dc[serial_channels_dd];
	  alt_u8	xvc_eq[serial_channels_dd];
	  alt_u8	xvc_p0[serial_channels_dd];
	  alt_u8	xvc_p1[serial_channels_dd];
	  alt_u8	xvc_p2[serial_channels_dd];
	  alt_u8	xvc_vod[serial_channels_dd];

	  DC_CNT		= xvc_scs->xvc_dc_cnt[0];
	  EQ_CNT		= xvc_scs->xvc_eq_cnt[0];
	  P0_CNT		= xvc_scs->xvc_p0_cnt[0];
	  P1_CNT		= xvc_scs->xvc_p1_cnt[0];
	  P2_CNT		= xvc_scs->xvc_p2_cnt[0];
	  VOD_CNT		= xvc_scs->xvc_vod_cnt[0];



	for	(A=0; A<aci->serial_channels[0]; A++)
		{xvc_vod[A]	=	xvc_scs->xvc_vod_start[A];}
for	(VOD_CTR=1; VOD_CTR<=VOD_CNT; VOD_CTR++)
{
	for	(A=0; A<aci->serial_channels[0]; A++)
		{xvc_eq[A]	=	xvc_scs->xvc_eq_start[A];}
for	(EQ_CTR=1; EQ_CTR<=EQ_CNT; EQ_CTR++)
{
 	for	(A=0; A<aci->serial_channels[0]; A++)
			{xvc_p0[A]	=	xvc_scs->xvc_p0_start[A];}
 for	(P0_CTR=1; P0_CTR<=P0_CNT; P0_CTR++)
 {
   for	(A=0; A<aci->serial_channels[0]; A++)
 			{xvc_p1[A]	=	xvc_scs->xvc_p1_start[A];}
  for	(P1_CTR=1; P1_CTR<=P1_CNT; P1_CTR++)
  {
 	for	(A=0; A<aci->serial_channels[0]; A++)
			{xvc_p2[A]	=	xvc_scs->xvc_p2_start[A];}
	for	(P2_CTR=1; P2_CTR<=P2_CNT; P2_CTR++)
	{
      	for	(A=0; A<aci->serial_channels[0]; A++)
   			{xvc_dc[A]	=	xvc_scs->xvc_dc_start[A];}
    for	(DC_CTR=1; DC_CTR<=DC_CNT; DC_CTR++)
    {
		 printf("\Next Sweep Value:\n");

		 (void)  get_xvc_com(aci,aci->base_address[0],aci->XVC_A);
   	 for	(A=0; A<aci->serial_channels[0]; A++)
   	 {
   		 //printf("Channel %d:\tDC_CTR=%d P2_CTR=%d P1_CTR=%d P0_CTR=%d EQ_CTR=%d VOD_CTR=%d\n",
   		//		 A,DC_CTR,P2_CTR,P1_CTR,P0_CTR,EQ_CTR,VOD_CTR);

   		 //printf("lec %d = %d\n",A,aci->lec[A])

   		 aci->xvc_vod[A] = xvc_vod[A];
   		 aci->xvc_eq[A]	= xvc_eq[A];
   		 aci->xvc_p0[A]	= xvc_p0[A];
   		 aci->xvc_p1[A]	= xvc_p1[A];
   		 aci->xvc_p2[A]	= xvc_p2[A];
   		 aci->xvc_dc[A]	= xvc_dc[A];



   		 (void)  put_xvc_com(aci,aci->base_address[0],aci->XVC_A);
   		 //printf("vod[%d]=%Xh eq[%d]=%X\n",A,aci->xvc_vod[A],A,aci->xvc_eq[A]);
		 (void)  get_xvc_com(aci,aci->base_address[0],aci->XVC_A); // confirm
		#ifdef   ALT_DEBUG

   		 printf("\t\tChannel %D:\tVOD=%Xh\tP0=%Xh\tP1=%Xh\tP2=%Xh\tRX EQ=%Xh\tDC GAIN=%Xh\tSerial Loopback=%X\n",
   				 A,aci->xvc_vod[A],aci->xvc_p0[A],aci->xvc_p1[A],aci->xvc_p2[A],aci->xvc_eq[A],aci->xvc_dc[A],aci->xvc_slb[A]);
		#endif
   	 }
   	 (void) reset_com(aci);
   	 (void) comma_detect_com(aci);
   	 (void) enable_channel_bond_com(aci);
	 (void) start_com(aci);
	 (void)  aci_rw_regs_com(aci,DISPLAY_OFF,aci->base_address[0],READ_S,LEC_S);
	 LEC_TOTAL	=	0;
   	 for	(A=0; A<aci->serial_channels[0]; A++)
   	 {
   		 LEC_TOTAL 	=	LEC_TOTAL + aci->lec[A];
   		 //printf("\tlec[%d] = %d\n",A,aci->lec[A]);
		 printf("\tlec[%d]=%xh adpt: err[min]=%xh err[max]=%xh err_adpt[min_link]=%xh err[max_link]=%xh err[opt]=%xh err[opt_link]=%xh\n",
						A,aci->lec[A],
						aci->xvc_dc_opt[dc_min].xvc_err_adpt[A],
						aci->xvc_dc_opt[dc_max].xvc_err_adpt[A],
						aci->xvc_dc_opt[dc_min_link].xvc_err_adpt[A],
						aci->xvc_dc_opt[dc_max_link].xvc_err_adpt[A],
						aci->xvc_dc_opt[dc_opt].xvc_err_adpt[A],
						aci->xvc_dc_opt[dc_opt_link].xvc_err_adpt[A]
		);



   	 }
   	 printf("LEC Total = %xh\n",LEC_TOTAL);
		 // io_bcr_com(aci,aci->base_address[0],aci->BCR_A); // This reads all lanes!

		 rd_data	= IORD(aci->base_address[0], aci->BCR_A);
		 aci->bcr[0] = rd_data;



		 ////printf("\t\tDPRIO Busy            \tDBY \t=\t%X\n",(rd_data>> aci->DBY_B)  &1);
		 //printf("\t\tChannel Lock Acquired \tCLA \t=\t%X\n",(rd_data>> aci->CLA_B)  &1);
		 //printf("\t\tRX PLLs & DPAs Locked \tPLLR\t=\t%X\n",(rd_data>> aci->PLLR_B) &1);
		 //printf("\t\tTX PLLs & DPAs Locked \tPLLT\t=\t%X\n",(rd_data>> aci->PLLT_B) &1);
		 //printf("\t\tAll PLLs Locked       \tPLLK\t=\t%X\n",(rd_data>> aci->PLLK_B) &1);
		 //printf("\t\tPattern Sync Acquired \tPSA \t=\t%X\n",(rd_data>> aci->PSA_B)  &1);
		 //printf("\t\tTest Complete         \tTSTC\t=\t%X\n",(rd_data>> aci->TSTC_B) &1);
		 //printf("\t\tError Latch           \tERRL\t=\t%X\n",(rd_data>> aci->ERRL_B) &1);
		 printf("\t\tPass/Fail             \tPASS\t=\t%X\n",(rd_data>> aci->PASS_B) &1);


   	 for	(A=0; A<aci->serial_channels[0]; A++)
   	 {

   		 STATUS[A]	=	((rd_data>> aci->PLLR_B) &1) + ((rd_data>> aci->PLLT_B) &1) + ((rd_data>> aci->PLLK_B) &1);
		 STATUS[A]	=	STATUS[A] + ((rd_data>> aci->CLA_B)  &1) + ((rd_data>> aci->PSA_B) &1) + ((rd_data>> aci->TSTC_B) &1) + ((rd_data>> aci->PASS_B) &1);


			 //if	((rd_data>> aci->PASS_B & 1) == 1 && (aci->lec[A] == 0))
			 //{
				// If # of terms change, modify ERROR_CNT_VALID
				//STATUS[A]	=	((rd_data>> aci->PLLR_B) &1) + ((rd_data>> aci->PLLT_B) &1) + ((rd_data>> aci->PLLK_B) &1);
				//STATUS[A]	=	STATUS[A] + ((rd_data>> aci->PSA_B) &1) + ((rd_data>> aci->TSTC_B) &1) + ((rd_data>> aci->PASS_B) &1);

				if ((rd_data>> aci->PASS_B & 1) == 1 && (aci->lec[A] == 0))
				 {
					 printf("-->>Channel %d:\PASSING VALUE<<-- STATUS=%d\n",A,STATUS[A]);
					 aci->xvc_dc_opt[dc_opt].xvc_vod_adpt[A]	= aci->xvc_vod[A];
					 aci->xvc_dc_opt[dc_opt].xvc_eq_adpt[A]		= aci->xvc_eq[A];
					 aci->xvc_dc_opt[dc_opt].xvc_p0_adpt[A]		= aci->xvc_p0[A];
					 aci->xvc_dc_opt[dc_opt].xvc_p1_adpt[A]		= aci->xvc_p1[A];
					 aci->xvc_dc_opt[dc_opt].xvc_p2_adpt[A]		= aci->xvc_p2[A];
					 aci->xvc_dc_opt[dc_opt].xvc_dc_adpt[A]		= aci->xvc_dc[A];
					 if	(STATUS[A] >= STATUS_ERR_VALID)
						 {aci->xvc_dc_opt[dc_opt].xvc_err_adpt[A]	= aci->lec[A];}
					 aci->xvc_dc_opt[dc_opt].xvc_status_adpt[A]	= STATUS[A];
					 if ((rd_data>> aci->PASS_B & 1) == 1 && (aci->lec[A] == 0))
						 {aci->xvc_dc_opt[dc_opt].xvc_passes_ch_adpt[A]= 1;}
					 if ((rd_data>> aci->PASS_B & 1) == 1 && (LEC_TOTAL == 0))
						 {aci->xvc_dc_opt[dc_opt].xvc_passes_all_adpt[A]= 1;}
					 if	(LEC_TOTAL == 0)
					 {
						 aci->xvc_dc_opt[dc_opt_link].xvc_vod_adpt[A]	= aci->xvc_vod[A];
						 aci->xvc_dc_opt[dc_opt_link].xvc_eq_adpt[A]		= aci->xvc_eq[A];
						 aci->xvc_dc_opt[dc_opt_link].xvc_p0_adpt[A]		= aci->xvc_p0[A];
						 aci->xvc_dc_opt[dc_opt_link].xvc_p1_adpt[A]		= aci->xvc_p1[A];
						 aci->xvc_dc_opt[dc_opt_link].xvc_p2_adpt[A]		= aci->xvc_p2[A];
						 aci->xvc_dc_opt[dc_opt_link].xvc_dc_adpt[A]		= aci->xvc_dc[A];
						 if	(STATUS[A] >= STATUS_ERR_VALID)
							 {aci->xvc_dc_opt[dc_opt_link].xvc_err_adpt[A]	= aci->lec[A];}
						 aci->xvc_dc_opt[dc_opt_link].xvc_status_adpt[A]	= STATUS[A];
						 if ((rd_data>> aci->PASS_B & 1) == 1 && (aci->lec[A] == 0))
							 {aci->xvc_dc_opt[dc_opt_link].xvc_passes_ch_adpt[A]= 1;}
						 if ((rd_data>> aci->PASS_B & 1) == 1 && (LEC_TOTAL == 0))
							 {aci->xvc_dc_opt[dc_opt_link].xvc_passes_all_adpt[A]= 1;}

					 }

				 } else {
			   		 printf("\tSTATUS[%d] = %d\n",A,STATUS[A]);
				 }

				 if	(
						 ((STATUS[A] > (aci->xvc_dc_opt[dc_min].xvc_status_adpt[A])) && (STATUS[A] <=4)) 				||  // TX RX All PLLS locked + channel lock acquired
						 ((aci->lec[A] < aci->xvc_dc_opt[dc_min].xvc_err_adpt[A]) 	 && (STATUS[A] >= STATUS_ERR_VALID))			// Error counter starts after PSA (Pattern Sync Acquired)
					 )
				 {
					 aci->xvc_dc_opt[dc_min].xvc_vod_adpt[A]	= aci->xvc_vod[A];
					 aci->xvc_dc_opt[dc_min].xvc_eq_adpt[A]		= aci->xvc_eq[A];
					 aci->xvc_dc_opt[dc_min].xvc_p0_adpt[A]		= aci->xvc_p0[A];
					 aci->xvc_dc_opt[dc_min].xvc_p1_adpt[A]		= aci->xvc_p1[A];
					 aci->xvc_dc_opt[dc_min].xvc_p2_adpt[A]		= aci->xvc_p2[A];
					 aci->xvc_dc_opt[dc_min].xvc_dc_adpt[A]		= aci->xvc_dc[A];
					 if	(STATUS[A] >= STATUS_ERR_VALID)
						 {aci->xvc_dc_opt[dc_min].xvc_err_adpt[A]	= aci->lec[A];}
					 aci->xvc_dc_opt[dc_min].xvc_status_adpt[A]	= STATUS[A];
					 if ((rd_data>> aci->PASS_B & 1) == 1 && (aci->lec[A] == 0))
						 {aci->xvc_dc_opt[dc_min].xvc_passes_ch_adpt[A]= 1;}
					 if ((rd_data>> aci->PASS_B & 1) == 1 && (LEC_TOTAL == 0))
						 {aci->xvc_dc_opt[dc_min].xvc_passes_all_adpt[A]= 1;}
				 }

				 if	(
						 ((STATUS[A] == (aci->xvc_dc_opt[dc_min].xvc_status_adpt[A])) && (STATUS[A] <=4)) 				||  // TX RX All PLLS locked + channel lock acquired
						 ((aci->lec[A] == aci->xvc_dc_opt[dc_min].xvc_err_adpt[A]) && (STATUS[A] >= STATUS_ERR_VALID))			// Error counter starts after PSA (Pattern Sync Acquired)
					 )
				 {

					 aci->xvc_dc_opt[dc_max].xvc_vod_adpt[A]	= aci->xvc_vod[A];
					 aci->xvc_dc_opt[dc_max].xvc_eq_adpt[A]		= aci->xvc_eq[A];
					 aci->xvc_dc_opt[dc_max].xvc_p0_adpt[A]		= aci->xvc_p0[A];
					 aci->xvc_dc_opt[dc_max].xvc_p1_adpt[A]		= aci->xvc_p1[A];
					 aci->xvc_dc_opt[dc_max].xvc_p2_adpt[A]		= aci->xvc_p2[A];
					 aci->xvc_dc_opt[dc_max].xvc_dc_adpt[A]		= aci->xvc_dc[A];
					 if	(STATUS[A] >= STATUS_ERR_VALID)
						 {aci->xvc_dc_opt[dc_max].xvc_err_adpt[A]	= aci->lec[A];}
					aci->xvc_dc_opt[dc_max].xvc_status_adpt[A]	= STATUS[A];
					 if ((rd_data>> aci->PASS_B & 1) == 1 && (aci->lec[A] == 0))
					 {
						 aci->xvc_dc_opt[dc_max].xvc_passes_ch_adpt[A]= 1;
						 aci->xvc_dc_opt[dc_min].xvc_passes_ch_adpt[A]= (aci->xvc_dc_opt[dc_min].xvc_passes_ch_adpt[A])+1;
					 }
					 if ((rd_data>> aci->PASS_B & 1) == 1 && (LEC_TOTAL == 0))
					 {
						 aci->xvc_dc_opt[dc_max].xvc_passes_all_adpt[A]= 1;
						 aci->xvc_dc_opt[dc_min].xvc_passes_all_adpt[A]= (aci->xvc_dc_opt[dc_min].xvc_passes_all_adpt[A])+1;
					 }

				 }
// Linked -- Min total errors
				 if	(
						 ((STATUS[A] > (aci->xvc_dc_opt[dc_min_link].xvc_status_adpt[A])) && (STATUS[A] <=4)) 				||  // TX RX All PLLS locked + channel lock acquired
						 ((LEC_TOTAL < aci->xvc_dc_opt[dc_min_link].xvc_err_adpt[A]) 	 && (STATUS[A] >= STATUS_ERR_VALID))			// Error counter starts after PSA (Pattern Sync Acquired)
					 )
				 {
					 //printf(":) link better status value found (: \n");
					 //printf("err_adpt=%xh status_adpt=%d\n",aci->xvc_dc_opt[dc_min].xvc_err_adpt[A],aci->xvc_dc_opt[dc_min].xvc_status_adpt[A]);
					 aci->xvc_dc_opt[dc_min_link].xvc_vod_adpt[A]	= aci->xvc_vod[A];
					 aci->xvc_dc_opt[dc_min_link].xvc_eq_adpt[A]		= aci->xvc_eq[A];
					 aci->xvc_dc_opt[dc_min_link].xvc_p0_adpt[A]		= aci->xvc_p0[A];
					 aci->xvc_dc_opt[dc_min_link].xvc_p1_adpt[A]		= aci->xvc_p1[A];
					 aci->xvc_dc_opt[dc_min_link].xvc_p2_adpt[A]		= aci->xvc_p2[A];
					 aci->xvc_dc_opt[dc_min_link].xvc_dc_adpt[A]		= aci->xvc_dc[A];
					 if	(STATUS[A] >= STATUS_ERR_VALID)
						 {aci->xvc_dc_opt[dc_min_link].xvc_err_adpt[A]	= LEC_TOTAL;}
					 aci->xvc_dc_opt[dc_min_link].xvc_status_adpt[A]	= STATUS[A];
					 if ((rd_data>> aci->PASS_B & 1) == 1 && (aci->lec[A] == 0))
						 {aci->xvc_dc_opt[dc_min_link].xvc_passes_ch_adpt[A]= 1;}
					 if ((rd_data>> aci->PASS_B & 1) == 1 && (LEC_TOTAL == 0))
						 {aci->xvc_dc_opt[dc_min_link].xvc_passes_all_adpt[A]= 1;}

					// printf("\tUPDATED: lec[%d]=%xh err_adpt[min]=%xh status_adpt[min]=%d\n",A,aci->lec[A],aci->xvc_dc_opt[dc_min].xvc_err_adpt[A],aci->xvc_dc_opt[dc_min].xvc_status_adpt[A]);

				 }


				 //printf("\tRECHECK: lec[%d]=%xh err_adpt[min]=%xh status_adpt[min]=%d\n",A,aci->lec[A],aci->xvc_dc_opt[dc_min].xvc_err_adpt[A],aci->xvc_dc_opt[dc_min].xvc_status_adpt[A]);

				 if	(
						 ((STATUS[A] == (aci->xvc_dc_opt[dc_min_link].xvc_status_adpt[A])) && (STATUS[A] <=4)) 				||  // TX RX All PLLS locked + channel lock acquired
						 ((LEC_TOTAL == aci->xvc_dc_opt[dc_min_link].xvc_err_adpt[A]) && (STATUS[A] >= STATUS_ERR_VALID))			// Error counter starts after PSA (Pattern Sync Acquired)
					 )
				 {
					 //printf(":> link same status value found <: \n");
					 //printf("err_adpt[min]=%xh status_adpt[min]=%d\n",aci->xvc_dc_opt[dc_min].xvc_err_adpt[A],aci->xvc_dc_opt[dc_min].xvc_status_adpt[A]);

					 aci->xvc_dc_opt[dc_max_link].xvc_vod_adpt[A]	= aci->xvc_vod[A];
					 aci->xvc_dc_opt[dc_max_link].xvc_eq_adpt[A]		= aci->xvc_eq[A];
					 aci->xvc_dc_opt[dc_max_link].xvc_p0_adpt[A]		= aci->xvc_p0[A];
					 aci->xvc_dc_opt[dc_max_link].xvc_p1_adpt[A]		= aci->xvc_p1[A];
					 aci->xvc_dc_opt[dc_max_link].xvc_p2_adpt[A]		= aci->xvc_p2[A];
					 aci->xvc_dc_opt[dc_max_link].xvc_dc_adpt[A]		= aci->xvc_dc[A];
					 if	(STATUS[A] >= STATUS_ERR_VALID)
						 {aci->xvc_dc_opt[dc_max_link].xvc_err_adpt[A]	= LEC_TOTAL;}
					 aci->xvc_dc_opt[dc_max_link].xvc_status_adpt[A]	= STATUS[A];
					 if ((rd_data>> aci->PASS_B & 1) == 1 && (aci->lec[A] == 0))
					 {
						 aci->xvc_dc_opt[dc_max_link].xvc_passes_ch_adpt[A]= 1;
						 aci->xvc_dc_opt[dc_min_link].xvc_passes_ch_adpt[A]= (aci->xvc_dc_opt[dc_min_link].xvc_passes_ch_adpt[A])+1;
					 }
					 if ((rd_data>> aci->PASS_B & 1) == 1 && (LEC_TOTAL == 0))
					 {
						 aci->xvc_dc_opt[dc_max_link].xvc_passes_all_adpt[A]= 1;
						 aci->xvc_dc_opt[dc_min_link].xvc_passes_all_adpt[A]= (aci->xvc_dc_opt[dc_min_link].xvc_passes_all_adpt[A])+1;
					 }

				 }





/*


				 if	(
						 ((STATUS[A] > (aci->xvc_dc_opt[dc_min].xvc_status_adpt[A])) && (STATUS[A] <=4)) 				||  // TX RX All PLLS locked + channel lock acquired
						 ((STATUS[A] >= STATUS_ERR_VALID) && (LEC_TOTAL < aci->xvc_dc_opt[dc_min].xvc_err_adpt[A]))			// Error counter starts after PSA (Pattern Sync Acquired)
					 )
				 {
					 printf(":) better status value found (: \n");
					 aci->xvc_dc_opt[dc_min_link].xvc_vod_adpt[A]	= aci->xvc_vod[A];
					 aci->xvc_dc_opt[dc_min_link].xvc_eq_adpt[A]		= aci->xvc_eq[A];
					 aci->xvc_dc_opt[dc_min_link].xvc_p0_adpt[A]		= aci->xvc_p0[A];
					 aci->xvc_dc_opt[dc_min_link].xvc_p1_adpt[A]		= aci->xvc_p1[A];
					 aci->xvc_dc_opt[dc_min_link].xvc_p2_adpt[A]		= aci->xvc_p2[A];
					 aci->xvc_dc_opt[dc_min_link].xvc_dc_adpt[A]		= aci->xvc_dc[A];
					 if	(STATUS[A] >= STATUS_ERR_VALID)
						 {aci->xvc_dc_opt[dc_min].xvc_err_adpt[A]	= aci->lec[A];}
					 aci->xvc_dc_opt[dc_min].xvc_status_adpt[A]	= STATUS[A];
				 }
*/



				//}
			 }


   // User interface to enter all DPRIO:
  // io_xvc_com(aci,WRITE_S,aci->base_address[0],aci->XVC_A);
   	 for	(A=0; A<aci->serial_channels[0]; A++)
   	 {
   		 if (xvc_scs->xvc_dc_sweep[A] == 1)
   		 {
				 if (xvc_dc[A] < (1 << (aci->DC_BITS)) - 1)
					 {xvc_dc[A]++;}
   		 } else {
   			 if (xvc_dc[A] > 0)
					 {xvc_dc[A]--;}
   		 }
   	 }
    }
	 for	(A=0; A<aci->serial_channels[0]; A++)
	 {
		 if (xvc_scs->xvc_p2_sweep[A] == 1)
		 {
			 if (xvc_p2[A] < (1 << (aci->P2_BITS)) - 1)
				 {xvc_p2[A]++;}
		 } else {
			 if (aci->xvc_p2[A] > 0)
				 {xvc_p2[A]--;}
		 }
	 }
   }
	for	(A=0; A<aci->serial_channels[0]; A++)
	{
		 if (xvc_scs->xvc_p1_sweep[A] == 1)
		 {
			 if (xvc_p1[A] < (1 << (aci->P1_BITS)) - 1)
				 {xvc_p1[A]++;}
		 } else {
			 if (xvc_p1[A] > 0)
				 {xvc_p1[A]--;}
		 }
	}
  }
	 for	(A=0; A<aci->serial_channels[0]; A++)
   {
	 if (xvc_scs->xvc_p0_sweep[A] == 1)
	 {
			 if (xvc_p0[A] < (1 << (aci->P0_BITS)) - 1)
				 {xvc_p0[A]++;}
	 } else {
		 if (xvc_p0[A] > 0)
				 {xvc_p0[A]--;}
	 }
   }
 }
 for	(A=0; A<aci->serial_channels[0]; A++)
  {
	 if (xvc_scs->xvc_eq_sweep[A] == 1)
	 {
			 if (xvc_eq[A] < (1 << (aci->EQ_BITS)) - 1)
				 {xvc_eq[A]++;}
	 } else {
		 if (xvc_eq[A] > 0)
				 {xvc_eq[A]--;}
	 }
  }
}
for	(A=0; A<aci->serial_channels[0]; A++)
  {
	 if (xvc_scs->xvc_vod_sweep[A] == 1)
	 {
			 if (xvc_vod[A] < (1 << (aci->VOD_BITS)) - 1)
				 {xvc_vod[A]++;}
	 } else {
		 if (xvc_vod[A] > 0)
				 {xvc_vod[A]--;}
	 }
    }
  }
  return  0;
}             

// Enable Comma Detect (RX)
int comma_detect_com(struct aci_map_com *aci)
{
  assert_bit(aci->base_address[0],aci->BCR_A,aci->ECD_M,ASSERT_HI,PULSE);
  return  0;
}

// Enable Channel Bond (RX)
int enable_channel_bond_com(struct aci_map_com *aci)
{
  assert_bit(aci->base_address[0],aci->BCR_A,aci->ECB_M,ASSERT_HI,PULSE);
  return  0;
}  

// Start Transmitting PRBS Data (TX)
int start_com(struct aci_map_com *aci)
{
  assert_bit(aci->base_address[0],aci->BCR_A,aci->STRT_M,ASSERT_HI,PULSE); 
  //assert_bit(aci->base_address[0],aci->BCR_A,aci->STRT_M,ASSERT_LO,LEVEL); // Deassert if set   
  //usleep(PB_OFF);     
  //assert_bit(aci->base_address[0],aci->BCR_A,aci->STRT_M,ASSERT_HI,LEVEL);
  return  0;
}  

// Transceiver and CMU PLL  powerdown 
int xcvr_powerdown_com(struct aci_map_com *aci)
{
  assert_bit(aci->base_address[0],aci->BCR_A,aci->GDN_M,ASSERT_HI,LEVEL);
  assert_bit(aci->base_address[0],aci->BCR_A,aci->PDN_M,ASSERT_HI,LEVEL);  
  return  0;
} 

// Transceiver & CMU PLL Powerup: reset_com will also enable the transceiver (disable gbx_powerdown and pll_powerdown)
//int xcvr_powerup_com(struct aci_map_com *aci)
//{
//  assert_bit(aci->base_address[0],aci->BCR_A,aci->GDN_M,ASSERT_LO,LEVEL);
//  assert_bit(aci->base_address[0],aci->BCR_A,aci->PDN_M,ASSERT_LO,LEVEL);  
//  return  0;
//}

// Create Error In the PRBS Data (TX)
int create_error_com(struct aci_map_com *aci)
{
  assert_bit(aci->base_address[0],aci->BCR_A,aci->CER_M,ASSERT_LO,LEVEL);
  usleep(PB_ER);    
  assert_bit(aci->base_address[0],aci->BCR_A,aci->CER_M,ASSERT_HI,LEVEL);
  return  0;
} 

// MEM
// Enable Comma Detect (RX)
int factory_test_mem(struct aci_map_mem *aci)
{   
  (void) reset_mem(aci);  
  (void) start_mem(aci);   
  usleep(FACTORY_DLY); 
  (void) aci_rw_regs_mem(aci,DISPLAY_ON,aci->base_address[0],READ_S,BCR_S); 
  return  0;
}


// Start Transmitting PRBS Data (TX)
int start_mem(struct aci_map_mem *aci)
{
  assert_bit(aci->base_address[0],aci->BCR_A,aci->STRT_M,ASSERT_LO,LEVEL); // Deassert if set   
  usleep(PB_OFF);       
  assert_bit(aci->base_address[0],aci->BCR_A,aci->STRT_M,ASSERT_HI,LEVEL);
  return  0;
}  

// Create Error In the PRBS Data (TX)
int create_error_mem(struct aci_map_mem *aci)
{
  assert_bit(aci->base_address[0],aci->BCR_A,aci->CER_M,ASSERT_LO,LEVEL);
  usleep(PB_ER);     
  assert_bit(aci->base_address[0],aci->BCR_A,aci->CER_M,ASSERT_HI,LEVEL);
  return  0;
} 



