/******************************************************************************
*                                                                             *
* License Agreement                                                           *
*                                                                             *
* Copyright (c) 2010 Altera Corporation, San Jose, California, USA.           *
* All rights reserved.                                                        *
*                                                                             *
* Permission is hereby granted, free of charge, to any person obtaining a     *
* copy of this software and associated documentation files (the "Software"),  *
* to deal in the Software without restriction, including without limitation   *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,    *
* and/or sell copies of the Software, and to permit persons to whom the       *
* Software is furnished to do so, subject to the following conditions:        *
*                                                                             *
* The above copyright notice and this permission notice shall be included in  *
* all copies or substantial portions of the Software.                         *
*                                                                             *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR  *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,    *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER      *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING     *
* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER         *
* DEALINGS IN THE SOFTWARE.                                                   *
*                                                                             *
* This agreement shall be governed in all respects by the laws of the State   *
* of California and by the laws of the United States of America.              *
* Altera does not recommend, suggest or require that this reference design    *
* file be used in conjunction or combination with any other product.          *
******************************************************************************/

/*
  cli.c:
  Implementation of a command line interface which will parse user input
  and call related BTS test functions. There should be a single void function
  which accepts a string as a parameter so that the code in the foreground loop
  can pass user input into the CLI.
*/

#include  "cli.h"
#include "time.h"
     
void cli(char cli_string[],int *exit){ 
  char      ch;
  char      line[8];
  char      aci_regs[8];
  int tmp;
   
    struct cli_struct cli_st;
    
    
    alt_u32 rx = 0;
    alt_irq_register( SC_INF_0_IRQ, &cli_st, handle_sc_interrupts );    
    alt_irq_register( HEARTBEAT_0_IRQ, NULL, handle_hb_interrupts );     
#ifdef   ALT_DEBUG
  printf("%s\n",cli_string); 
#endif
  // Initialize acim_com & acim_mem structures for all ports in the design:
  // Combined memory & comm:
  // Set num_ports_dd_com & num_ports_dd_mem in bts_defs.h to the number of ports called below 
  cli_st.port = 0;  //  // Init to the 1st memory port (since 0 comm ports)
 // (void) init_aci_com(&port_type[0],&acim_com[0],NIOS_BTS_PORT_PARALLEL_LOOPBACK_X41_HSMA_INST_BASE);
  // (void) init_aci_com(&port_type[1],&acim_com[1],NIOS_BTS_PORT_PARALLEL_LOOPBACK_X41_HSMB_INST_BASE); 
  // (void) init_aci_mem(&port_type[2],&acim_mem[0],NIOS_BTS_PORT_DDR2_SDRAM_128_BIT_INST_BASE);
  // (void) init_aci_mem(&port_type[3],&acim_mem[1],NIOS_BTS_PORT_DDR2_SDRAM_160_BIT_INST_BASE);
  (void) init_aci_com(&cli_st.port_type[0],&acim_com[0],NIOS_BTS_PORT_XCVR_X4_HSMA_0_BASE);
  (void) init_aci_com(&cli_st.port_type[1],&acim_com[1],NIOS_BTS_PORT_LVDS_X17_1G_HSMA_0_BASE);  
  (void) init_aci_com(&cli_st.port_type[2],&acim_com[2],NIOS_BTS_PORT_PARALLEL_LOOPBACK_X3_HSMA_0_BASE);

#ifdef   ALT_DEBUG
          (void)  get_port(&cli_st.port,&cli_st.port_com,&cli_st.port_mem); 
#else
           setPort(&cli_st);    // Runs the factory test also
#endif


 
  strcpy(aci_regs,ALL_S);
  ch  = "";
  while (ch != 'Q') {
    if  (cli_st.port_type[cli_st.port] == COM) 
    {
#ifdef   ALT_DEBUG
      printf("\nF>actory Test, P>ort, O>ptimize Transceivers, R>eset, C>omma Detect,\n");
      printf("B>ond Channels, S>tart, E>rror, T>est Status, D>etect Port, I>nput\n");
      printf("A>ddressing, X>CVR Powerdown, Q>uit\? ");
#endif
      fgets(line, sizeof(line), stdin);
      sscanf(line,"%c", &ch );
      switch (ch)
      {
        case  'D': // Detect Port ID
        {
          (void) aci_rw_regs_id_com(&acim_com[cli_st.port_com],DISPLAY_ON,acim_com[cli_st.port_com].base_address[0],READ_S,ALL_S); 
		      break;   
        }    
        case  'F': // Factory Test
        {
          (void) factory_test_com(&acim_com[cli_st.port_com]);       
          check_loopbackcard(&cli_st);       
          break;   
        }
        case  'O':  // Optimize VOD, Preemphasis, and RX Equalization
        { (void)  optimize_xcvrs_com(&acim_com[cli_st.port_com]);
          break;
        }    
        case  'A':  // Register to Access in the Avalon Control Register
        {
          printf("Enter Register to Address (eg PRS, PRC, ALL): ");
          fgets(line, sizeof(line), stdin);
          sscanf(line,"%s", &aci_regs );                                                                                        
          break;
        }
        case  'I': // Input Data to the Avalon Control Register
        {
          (void) aci_rw_regs_com(&acim_com[cli_st.port_com],DISPLAY_ON,acim_com[cli_st.port_com].base_address[0],WRITE_S,aci_regs);       
          break;   
        }
        case  'P': // Port
        {
          (void)  get_port(&cli_st.port,&cli_st.port_com,&cli_st.port_mem); 
          break;
        }
        case  'R': // Reset 
        {// Reset:           
              (void)    reset_com(&acim_com[cli_st.port_com]);
          break;          
        }
        case  'C':  // Enable Comma Detect:
        {
         (void) comma_detect_com(&acim_com[cli_st.port_com]);
         break;    
        }
         case  'X':  // Transceiver & CMU PLL Powerdown:
        {
         (void) xcvr_powerdown_com(&acim_com[cli_st.port_com]);
         break;    
        } 
        //case  'Y':  // Transceiver & CMU PLL Powerup: reset_com will also enable gbx_powerdown and pll_powerdown
        //{
        // (void) xcvr_powerup_com(&acim_com[cli_st.port_com]);
        // break;    
        //}                
        case 'B': // Enable Channel Bonding:
        { 
          (void) enable_channel_bond_com(&acim_com[cli_st.port_com]);
          break; 
        }
        case 'S': // Start Tests:
        {
          (void) start_com(&acim_com[cli_st.port_com]);        
          break;       
        }
         case 'E': // Error Insert:
        {
          (void) create_error_com(&acim_com[cli_st.port_com]);           
          break;       
        }    
        case 'T': // Avalon Control Register Test Status:
        { 
          (void) aci_rw_regs_com(&acim_com[cli_st.port_com],DISPLAY_ON,acim_com[cli_st.port_com].base_address[0],READ_S,ALL_S);
          break;       
        }  
        default:
        {
          break;
        }
      }
    }
    if  (cli_st.port_type[cli_st.port] == MEM) 
    {
#ifdef   ALT_DEBUG
      printf("\nF>actory Test, P>ort, R>eset, S>tart, E>rror,\n");
      printf("T>est Status, D>etect Port, I>nput, A>ddressing, Q>uit, G>UI\? ");
#endif
      fgets(line, sizeof(line), stdin);
      sscanf(line,"%c", &ch );
      switch (ch)
      {
        case  'G':
        {
            (void)  usb_gui_mem(&acim_mem[cli_st.port_mem]);
            break;
        }
        case  'D': // Detect Port ID
        {
          (void) aci_rw_regs_id_mem(&acim_mem[cli_st.port_mem],DISPLAY_ON,acim_mem[cli_st.port_mem].base_address[0],READ_S,ALL_S); 
              break;   
        }    
        case  'F': // Factory Test
        {
            factory_test_mem(&acim_mem[cli_st.port_mem]);
          break;   
        }
        case  'A':  // Register to Access in the Avalon Control Register
        {
          printf("Enter Register to Address (eg PRS, PRC, SEQ, MEM, ALL): ");
          fgets(line, sizeof(line), stdin);
          sscanf(line,"%s", &aci_regs );                                                                                                    
          break;
        }
        case  'I': // Input Data to the Avalon Control Register
        {
          (void) aci_rw_regs_mem(&acim_mem[cli_st.port_mem],DISPLAY_ON,acim_mem[cli_st.port_mem].base_address[0],WRITE_S,aci_regs);       
          break;   
        }
        case  'P': // Port
        {
          (void)  get_port(&cli_st.port,&cli_st.port_com,&cli_st.port_mem);
          break;
        }
        case  'R': // Reset 
        {// Reset:           
              (void)    reset_mem(&acim_mem[cli_st.port_mem]);
          break;          
        }
        case 'S': // Start Tests:
        {
          (void) start_mem(&acim_mem[cli_st.port_mem]);        
          break;       
        }
         case 'E': // Error Insert:
        {
          create_error_mem(&acim_mem[cli_st.port_mem]);
          break;       
        }    
        case 'T': // Avalon Control Register Test Status:
        { 
            aci_rw_regs_mem(&acim_mem[cli_st.port_mem],DISPLAY_ON,acim_mem[cli_st.port_mem].base_address[0],READ_S,ALL_S);
          break;       
        }  
        default:
        {
          break;
        }
      }          
    }  
                 
  }
  *exit  = 1;
  //return 0;
}     

// SC_INF
#define ACI_REGS_ADDR           96
#define ACI_REGS_VAL_HOST2HDL   97
#define ACI_REGS_VAL_HDL2HOST   98
void handle_sc_interrupts(void* context, alt_u32 id){
    int ch;
    int	A;
    alt_u32 datasize;
    int data_type;
    int rw_restrict;
    char burst_1or2;
    int tx,rx,er;
    int tmp;
    //alt_u32 aci_regs_addr = 0;
    char      aci_regs_addr_s[4];
    alt_u32 aci_regs_val = 0;
    alt_u32 encodedInt = 0;
    alt_u32 patternWriteToHost = 0;
    alt_u32	rd_data;
    alt_u32	wr_data;
    
    struct cli_struct *cli_st;
    cli_st = (struct cli_struct *)context;
    
 
    ch = IORD(SC_INF_0_BASE, 7); // read IRQ status it should read 1 as data
    if(ch == 0)
        return;
    ch = IORD(SC_INF_0_BASE, 0); // read command
    ch %= 0xFF;

    if  (cli_st->port_type[cli_st->port] == COM){
      switch (ch){
        case  'D': // Detect Port ID
            aci_rw_regs_id_com(&acim_com[cli_st->port_com],DISPLAY_ON,acim_com[cli_st->port_com].base_address[0],READ_S,ALL_S); 
            break;   
        case  'F': // Factory Test
            factory_test_com(&acim_com[cli_st->port_com]);
            //IOWR(SC_INF_0_BASE, 2, IORD(acim_com[cli_st->port_com].base_address[0],acim_com[cli_st->port_com].BCR_A));
            alt_u32 data = IORD(acim_com[cli_st->port_com].base_address[0],acim_com[cli_st->port_com].BCR_A); // read data at the BCR address (0x15) dont need to know
            IOWR(SC_INF_0_BASE, 2, data); // write the resulting BCR read to the special vhdl register in sc_inf(b'10')
            check_loopbackcard(cli_st);
            break;   
        case  'O':  // Optimize VOD, Preemphasis, and RX Equalization
            optimize_xcvrs_com(&acim_com[cli_st->port_com]);
            break;
        case  'I': // Input Data to the Avalon Control Register
            //aci_rw_regs_com(&acim_com[port_com],DISPLAY_ON,acim_com[port_com].base_address[0],WRITE_S,aci_regs);       
            break;   
        case  'P': // Port
            cli_st->port = IORD(SC_INF_0_BASE, 127);
            setPort(cli_st);
            IOWR(SC_INF_0_BASE, 127, cli_st->port);
            break;
        case  'R': // Reset 
            reset_com(&acim_com[cli_st->port_com]);
            break;          
        case  'C':  // Enable Comma Detect:
            comma_detect_com(&acim_com[cli_st->port_com]);
            break;    
        case  'X':  // XCVR Powerdown
            xcvr_powerdown_com(&acim_com[cli_st->port_com]);
            break;                 
        case 'B': // Enable Channel Bonding:
            enable_channel_bond_com(&acim_com[cli_st->port_com]);
            break; 
        case 'S': // Start Tests:
            start_com(&acim_com[cli_st->port_com]);        
            break;       
         case 'E': // Error Insert:
            create_error_com(&acim_com[cli_st->port_com]);
            break;       
        case 'T': // Avalon Control Register Test Status:
            // update structure values            
            aci_rw_regs_com(&acim_com[cli_st->port_com],DISPLAY_OFF,acim_com[cli_st->port_com].base_address[0],READ_S,TTC_S);// transmit transaction counter
            aci_rw_regs_com(&acim_com[cli_st->port_com],DISPLAY_OFF,acim_com[cli_st->port_com].base_address[0],READ_S,RTC_S);// rx transaction counter
            aci_rw_regs_com(&acim_com[cli_st->port_com],DISPLAY_OFF,acim_com[cli_st->port_com].base_address[0],READ_S,WEC_S);// word error counter

            // get data from the structure
            IOWR(SC_INF_0_BASE, 64, readTimer());
            IOWR(SC_INF_0_BASE, 65, *acim_com[cli_st->port_com].rtc); 
            IOWR(SC_INF_0_BASE, 66, *acim_com[cli_st->port_com].ttc); 
            IOWR(SC_INF_0_BASE, 67, *acim_com[cli_st->port_com].wec);
            break;
        case 'c': // changing data size, type, pattern, burst size
            datasize = IORD(SC_INF_0_BASE, 124); // get data size (0x7C)
            // data_type =>>>   1:PRBS  2:memory 3: math
            data_type = IORD(SC_INF_0_BASE, 125); // (0x7D)
            // rw_restrict =>>> 1:write/read  2:read only  3:write only
            rw_restrict = IORD(SC_INF_0_BASE, 126); // (0x7E)
            // burst_1or2 =>>> burst 1 is for all interfaces. 2 is only for DDR3. 
            burst_1or2 = IORD(SC_INF_0_BASE, 127); // (0x7F)
            
            reset_com(&acim_com[cli_st->port_com]); // Reset BTS
            //io_seq_com_gui(&acim_mem[port_com], datasize, data_type, rw_restrict, burst_1or2);
            //io_com_gui(&acim_com[port_com]); 
            start_com(&acim_com[cli_st->port_com]);
            break;            
          case 'A': // Addressing mode
	    // This code is specific for the Stratix V GX 28nm Test Chip Board
	    // It allows locking multiple ports:
	    // GUI should send 
        	aci_regs_val  = IORD(SC_INF_0_BASE, SC_INF_OFF93D); // read aci_regs_val
        	encodedInt = IORD(SC_INF_0_BASE, (SC_INF_OFF94D));

        	aci_regs_addr_s[0] = (encodedInt & 0xFF000000) >> 24;
        	aci_regs_addr_s[1] = (encodedInt & 0xFF0000) >> 16;
        	aci_regs_addr_s[2] = (encodedInt & 0xFF00)	>> 8;
        	aci_regs_addr_s[3] = (encodedInt & 0xFF);

			//aci_regs_addr = encodedInt % 0xFFFFFF ; // 3 chars encoded in an int
			//printf("aci_regs_addr = %s %x",aci_regs_addr,aci_regs_addr);
			if (strstr(aci_regs_addr_s,WPRT_S) != 0)
			{
				IOWR(acim_com[cli_st->port_com].base_address[0], acim_com[cli_st->port_com].PRT_A, aci_regs_val);
				aci_rw_regs_com(&acim_com[cli_st->port_com],DISPLAY_OFF,acim_com[cli_st->port_com].base_address[0],READ_S,PRT_S);
				IOWR(SC_INF_0_BASE, SC_INF_OFF97D, *acim_com[cli_st->port_com].prt); // ACI_REGS_VAL_HDL2HOST
				patternWriteToHost = *acim_com[cli_st->port_com].prt;

				reset_com(&acim_com[cli_st->port_com]); // Reset BTS
				start_com(&acim_com[cli_st->port_com]);
			}
			if (strstr(aci_regs_addr_s,RPRT_S) != 0)
			{
	            aci_rw_regs_com(&acim_com[cli_st->port_com],DISPLAY_OFF,acim_com[cli_st->port_com].base_address[0],READ_S,PRT_S);
	            IOWR(SC_INF_0_BASE, SC_INF_OFF98D, *acim_com[cli_st->port_com].prt); // ACI_REGS_VAL_HDL2HOST
	            usleep(10); // give read a chance
	            reset_com(&acim_com[cli_st->port_com]); // Reset BTS
	            start_com(&acim_com[cli_st->port_com]);
			}
            if (strstr(aci_regs_addr_s,WXVC_S) != 0){
                int l;


                int start_channel, max_channel;
                start_channel = cli_st->port;
                max_channel = 1;

                // write 1 chunk of port data
                int offset = 0;
                for(l=start_channel; l<start_channel + max_channel ; l++){
                    for (A=0; A<acim_com[l].serial_channels[0]; A=A+1){
                        rd_data = IORD(SC_INF_0_BASE, SC_INF_OFF100D+offset);
                        acim_com[l].xvc_dc[A]   = (rd_data>>(acim_com[l].DC_LSB ))  & (acim_com[l].DC_LMSK );
                        acim_com[l].xvc_eq[A]   = (rd_data>>(acim_com[l].EQ_LSB ))  & (acim_com[l].EQ_LMSK );
                        acim_com[l].xvc_p0[A]   = (rd_data>>(acim_com[l].P0_LSB ))  & (acim_com[l].P0_LMSK );
                        acim_com[l].xvc_p1[A]   = (rd_data>>(acim_com[l].P1_LSB ))  & (acim_com[l].P1_LMSK );
                        acim_com[l].xvc_p2[A]   = (rd_data>>(acim_com[l].P2_LSB ))  & (acim_com[l].P2_LMSK );
                        acim_com[l].xvc_vod[A]  = (rd_data>>(acim_com[l].VOD_LSB))  & (acim_com[l].VOD_LMSK);
                        acim_com[l].xvc_slb[A]  = (rd_data>>(acim_com[l].SLB_LSB))  & (acim_com[l].SLB_LMSK);
                        offset++;
                       }
                    put_xvc_com(&acim_com[l],acim_com[l].base_address[0],acim_com[l].XVC_A);
				  }
     	   }

            if (strstr(aci_regs_addr_s,RXVC_S) != 0){
                int l;
                int start_channel, max_channel;
                    start_channel = cli_st->port;
                    max_channel = 1;
                // read data for chunk of port and write it to SC_INF
                int offset = 0; 
                for(l=start_channel; l<start_channel + max_channel ; l++){
                    get_xvc_com(&acim_com[l],acim_com[l].base_address[0],acim_com[l].XVC_A);                    
                    
                    for (A=0; A<acim_com[l].serial_channels[0]; A=A+1){
                        wr_data=(((acim_com[l].xvc_dc[A] ) & (acim_com[l].DC_LMSK )) <<  (acim_com[l].DC_LSB ))  |
                                  (((acim_com[l].xvc_eq[A] ) & (acim_com[l].EQ_LMSK )) <<  (acim_com[l].EQ_LSB ))  |
                                  (((acim_com[l].xvc_p0[A] ) & (acim_com[l].P0_LMSK )) <<  (acim_com[l].P0_LSB ))  |
                                  (((acim_com[l].xvc_p1[A] ) & (acim_com[l].P1_LMSK )) <<  (acim_com[l].P1_LSB ))  |
                                  (((acim_com[l].xvc_p2[A] ) & (acim_com[l].P2_LMSK )) <<  (acim_com[l].P2_LSB ))  |
                                  (((acim_com[l].xvc_vod[A]) & (acim_com[l].VOD_LMSK)) <<  (acim_com[l].VOD_LSB))  |
                                  (((acim_com[l].xvc_slb[A]) & (acim_com[l].SLB_LMSK)) <<  (acim_com[l].SLB_LSB))   ;
                        IOWR(SC_INF_0_BASE, SC_INF_OFF100D+offset,wr_data);
                        offset++;
                    }
                }
         } 
            break;
       // Can remove case 'p' after "A" "RPRT" is implement in the GUI
       case 'p': // Pattern Type: 0, 1, 2, 3, 4, 5, 6, 7 or 40d maps to:
			    // PRBS7, PRBS15, PRBS23, PRBS31, 10101010, 11001100, 11110000, LF, PCIe Compliance
            aci_rw_regs_com(&acim_com[cli_st->port_com],DISPLAY_OFF,acim_com[cli_st->port_com].base_address[0],READ_S,PRT_S);

            IOWR(SC_INF_0_BASE, SC_INF_OFF98D, *acim_com[cli_st->port_com].prt); // ACI_REGS_VAL_HDL2HOST
            usleep(10); // give read a chance

            reset_com(&acim_com[cli_st->port_com]); // Reset BTS
            start_com(&acim_com[cli_st->port_com]);
            break; 
       default: break;               
      } // end of switch case
    } // end of com port

    else if  (cli_st->port_type[cli_st->port] == MEM){
      switch (ch){
        case  'D': // Detect Port ID
            aci_rw_regs_id_mem(&acim_mem[cli_st->port_mem],DISPLAY_ON,acim_mem[cli_st->port_mem].base_address[0],READ_S,ALL_S); 
            break;   
        case  'F': // Factory Test
            factory_test_mem(&acim_mem[cli_st->port_mem]);
            //factory_test_mem_gui(&acim_mem[cli_st->port_mem]);
            IOWR(SC_INF_0_BASE, 2, IORD(acim_mem[cli_st->port_mem].base_address[0],acim_mem[cli_st->port_mem].BCR_A));
            break;   
        case  'I': // Input Data to the Avalon Control Register
            //aci_rw_regs_mem(&acim_mem[port_mem],DISPLAY_ON,acim_mem[port_mem].base_address[0],WRITE_S,aci_regs);       
            break;   
        case  'P': // Port
            cli_st->port = IORD(SC_INF_0_BASE, 127);
            setPort(cli_st);
            IOWR(SC_INF_0_BASE, 127, cli_st->port);
            break;
        case  'R': // Reset 
            reset_mem(&acim_mem[cli_st->port_mem]);
            break;          
        case 'S': // Start Tests:
            start_mem(&acim_mem[cli_st->port_mem]);        
            break;       
         case 'E': // Error Insert:
            create_error_mem(&acim_mem[cli_st->port_mem]);
            break;       
        case 'T': // Avalon Control Register Test Status:
            aci_rw_regs_mem(&acim_mem[cli_st->port_mem],DISPLAY_OFF,acim_mem[cli_st->port_mem].base_address[0],READ_S,TTC_S);
            aci_rw_regs_mem(&acim_mem[cli_st->port_mem],DISPLAY_OFF,acim_mem[cli_st->port_mem].base_address[0],READ_S,RTC_S);
            aci_rw_regs_mem(&acim_mem[cli_st->port_mem],DISPLAY_OFF,acim_mem[cli_st->port_mem].base_address[0],READ_S,WEC_S);
            
            // get data from the structure
            IOWR(SC_INF_0_BASE, 64, readTimer());
            IOWR(SC_INF_0_BASE, 65, *acim_mem[cli_st->port_mem].rtc); 
            IOWR(SC_INF_0_BASE, 66, *acim_mem[cli_st->port_mem].ttc); 
            IOWR(SC_INF_0_BASE, 67, *acim_mem[cli_st->port_mem].wec);
            break;
        case 'c': // changing data size, type, pattern, burst size
            datasize = IORD(SC_INF_0_BASE, 124); // get data size
            // data_type =>>>   1:PRBS  2:memory 3: math
            data_type = IORD(SC_INF_0_BASE, 125);    
            // rw_restrict =>>> 1:write/read  2:read only  3:write only
            rw_restrict = IORD(SC_INF_0_BASE, 126);
            // burst_1or2 =>>> burst 1 is for all interfaces. 2 is only for DDR3. 
            burst_1or2 = IORD(SC_INF_0_BASE, 127);
            
            reset_mem(&acim_mem[cli_st->port_mem]); // Reset BTS
            io_seq_mem_gui(&acim_mem[cli_st->port_mem], datasize, data_type, rw_restrict, burst_1or2);
            io_mem_gui(&acim_mem[cli_st->port_mem]); 
            start_mem(&acim_mem[cli_st->port_mem]);
            break;
        default:break;
        }// end of switch case
      }// end of mem

    IOWR(SC_INF_0_BASE, 7, 0x0); // clear IRQ status
    ch = IORD(SC_INF_0_BASE, 7); // read IRQ, it should be cleared already
    if(ch != 0)
        IOWR(SC_INF_0_BASE, 7, 0x0); // clear IRQ status
}

void handle_hb_interrupts(void* context, alt_u32 id){
    // Just read this address to keep the heartbeat beating...
    int dummy_read = IORD(HEARTBEAT_0_BASE, 0);
    dummy_read++; // this is just dummy process...
    
}

alt_u32 readTimer(){
    alt_u32 rd = 0;
    
    IOWR(TIMER_0_BASE, 5, 0x1234); // write value will be ignored. Just need to issue write comand to this address
    rd = IORD(TIMER_0_BASE, 5); // read timer
    rd = rd << 16;
    rd = rd + IORD(TIMER_0_BASE, 4); // read timer

    // this value needs to go buffer
    // if the lower half has read, then put this upper half to the scj_inf register.


    IOWR(TIMER_0_BASE, 0, 0x0); // clear round flag
    IOWR(TIMER_0_BASE, 1, 0xA); // stop counter
    IOWR(TIMER_0_BASE, 2, 0xFFFF); // write max counter value
    IOWR(TIMER_0_BASE, 3, 0xFFFF); // write max counter value
    IOWR(TIMER_0_BASE, 1, 0x6); // sart counter

    return rd;
}

//void setPort(int port){
void setPort(struct cli_struct * cli_st){
            
          if ((cli_st->port >= port_com_lo) && (cli_st->port <= port_com_hi)){
            cli_st->port_com = cli_st->port - port_com_lo; // Index from 0 for COM structure 
    #ifdef   ALT_DEBUG
                        printf("port=%d  com_port=%d",cli_st->port,cli_st->port_com);   
    #endif            
            aci_rw_regs_id_com(&acim_com[cli_st->port_com],DISPLAY_ON,acim_com[cli_st->port_com].base_address[0],READ_S,ALL_S);    
              factory_test_com(&acim_com[cli_st->port_com]);
              check_loopbackcard(cli_st);
          }else if ((cli_st->port >= port_mem_lo) && (cli_st->port <= port_mem_hi)){
            cli_st->port_mem = cli_st->port - port_mem_lo; // Index from 0 for MEM structure 
            
    #ifdef   ALT_DEBUG
                        printf("port=%d  mem_port=%d",cli_st->port,cli_st->port_mem);  
                #endif            
            aci_rw_regs_id_mem(&acim_mem[cli_st->port_mem],DISPLAY_ON,acim_mem[cli_st->port_mem].base_address[0],READ_S,ALL_S);    
              factory_test_mem(&acim_mem[cli_st->port_mem]);
          }else{
    #ifdef   ALT_DEBUG
                    printf("\nwrong port number has selected\n");  
    #endif            
          }   
}

void check_loopbackcard(struct cli_struct * cli_st){
    alt_u32 rx;
    // getting HSMC loopback connector status at here            
    rx = IORD(acim_com[cli_st->port_com].base_address[0], acim_com[cli_st->port_com].BIO_A); 
    rx = rx & bio_mask_dd; // Turn off unused bits
    acim_com[cli_st->port_com].bio[0] = rx; // applying mask to the result
    IOWR(SC_INF_0_BASE, 64, rx); // writing the bio read to the SC_INF
}
