               
//`define simulation       
                              
//--------------------------------------------------------------------------//
// Title:       nios_bts_port_core.v                                        //
// Rev:         Rev 1                                                       //
// Author		David Johnson                                               //
//--------------------------------------------------------------------------//
// Description: NIOS BTS Communications Port For Transceivers,				//
//				Source Synchronous SERDES, and Parallel Data                //
//              Includes wrapper to include ALT2GXB or SERDES since S2GX    //
//----------------------------------------------------------------------------
//Copyright  2006 Altera Corporation. All rights reserved.  Altera products
//are protected under numerous U.S. and foreign patents, maskwork rights,
//copyrights and other intellectual property laws.
//                                                                                                     
//This reference design file, and your use thereof, is subject to and
//governed by the terms and conditions of the applicable Altera Reference
//Design License Agreement.  By using this reference design file, you
//indicate your acceptance of such terms and conditions between you and
//Altera Corporation.  In the event that you do not agree with such terms and
//conditions, you may not use the reference design file. Please promptly                         
//destroy any copies you have made.
//                                                      
//This reference design file being provided on an "as-is" basis and as an
//accommodation and therefore all warranties, representations or guarantees
//of any kind (whether express, implied or statutory) including, without
//limitation, warranties of merchantability, non-infringement, or fitness for
//a particular purpose, are specifically disclaimed.  By making this
//reference design file available, Altera expressly does not recommend,
//suggest or require that this reference design file be used in combination 
//with any other product not provided by Altera

module nios_bts_port_core (
	// Resets:
    input          					reset_n,
	input							en_cdet_pb,
	input							channel_bond_pb,
	input							start_pb,
	input							create_error_pb,           				
	input		   					xcvr_reset_pb_n,		// Hold in reset transceivers	
    input          					refclk_p,
    output [io_width-1:0]   		tx_p,
    input  [io_width-1:0]   		rx_p,
 	input							cal_blk_clk_sig,
	input  [serial_channels-1:0]	seriallpbken_dip,    
    output         					led_tx,     
    output         					led_rx,  
	input							rx_ss_clk_in,			// RX Source synchronous clock in
    output							tx_ss_clk_out,       	// TX Source Synchronous clock out
	input							plls_in_locked,			// Input clocks are locked
	output		    				plls_locked, 
	output							pattern_sync_acquired,
	output							test_complete,
	output							error_latch,
	output							error_irq,  
	input							nios_clk,
	input	[nios_map_width-1:0]	nios_address,    		// For multiple interfaces, add bits for address decoder to select the avalon slave
  	input	      					nios_chipselect,
  	input	      					nios_read,
  	output	[nios_width-1:0]		nios_readdata,
  	input	      					nios_reset_n,
  	input	      	    		    nios_write,
 	input	[nios_width-1:0]		nios_writedata,
	output							nios_waitrequest,
	input	[31:0]					avalon_pb,
	input	[31:0]					avalon_dip,
	output	[31:0]					avalon_led											
);  
 

// Board Test System for HSMA Interface:
parameter	component_type				=	"XCVRS";	// 1=XCVRS, 2=LVDS, 3=Parallel Data
parameter	serdes_2d					=	0;			// Set to 1 for LVDS SERDES used as parallel inputs to a transceiver,
														// and you want to compenstate for channel to channel LVDS SERDES skew.
parameter	nios_width					=	32;
parameter	bts_idle					=	0;	// Only PRBS pattern which won't occur after word alignment -- (tx inserts these if tx_ready is deasserted)
	
// Switch debouncers, metastabilizers, etc
`ifdef simulation
	parameter	debounce_prd			=	4;
	parameter	delayed_rst_prd			=	4;
	parameter	delayed_rst_pw			=	2; 
	parameter	test_length_bts			=	10;					// Number of data_gen_readdata to transmit before asserting testcomplete
	parameter	test_complete_ctr_width	=	4;					// > log2(test_length_bts)	  
	parameter	tx_wrapper_mode			=	16;					// Make larger than deskew_depth
`else                       
	parameter	debounce_prd			=	100000;
	parameter	delayed_rst_prd			=	50000;	// 100000; (reverse compatible with prior designs)
	parameter	delayed_rst_pw			=	16;			// 32;					// Delayed reset pulse width
 	parameter	test_length_bts			=	'h800000;			// Number of data_gen_readdata to transmit before asserting testcomplete
	parameter	test_complete_ctr_width	=	24;					// > log2(test_length_bts)	  
	parameter	tx_wrapper_mode			=	1024;				// Repeating pattern: lower 1/2 word incrementing, upper 1/2 word complement of lower half word so DC balanced, followed by word_align_pattern
																// Make larger than deskew_depth.	
`endif		


/*
// LFSR Polynomial Coefficients   
parameter	prbs_coeff2		=	2'b11;
parameter	prbs_coeff3		=	3'b011;
parameter	prbs_coeff4		=	4'b0011;
parameter	prbs_coeff5		=	5'b00101;
parameter	prbs_coeff6		=	6'b000011;
parameter	prbs_coeff7		=	7'b0000011;
parameter	prbs_coeff8		=	8'h1d;
parameter	prbs_coeff16	=	16'h100b;
parameter	prbs_coeff32	=	32'hc000_0401;
parameter	prbs_coeff128	=	128'h0000_0000_0000_0000_0000_0000_2800_0005;
*/

wire	[serial_channels-1:0] avalon_seriallpbken;
wire								en_cdet_sync;
wire								channel_bond_sync;
wire								bts_start_sync;
wire								create_error_sync;  
wire								create_single_error;
wire								tx_digital_reset_sync_dly; 
wire								rx_digital_reset_sync_dly; 
wire								rx_fifos_rst_sync_dly_n;  				
wire	[tx_ctrlenable_width-1:0]	tx_ctrlenable_sig;			// From bts 
wire	[tx_ctrlenable_width-1:0]	tx_ctrlenable_sig_txepcf;	// From phase comp fifo
wire	[rx_ctrldetect_width-1:0]	rx_ctrldetect_sig;			// Eg: XPAK or X2
wire	[rx_ctrldetect_width-1:0]	rx_ctrldetect_sig_0;		// Eg: XPAK or X2 if without byte order
wire								cal_blk_powerdown_sig	=	1'b0; // !xcvr_reset_pb_n; 	// 1'b0;
wire								gxb_enable_sig			=	1'b1; // xcvr_reset_pb_n;	//1'b1;     
wire	[num_quads-1:0]				gxb_powerdown_sig		=	{num_quads{aci_gdn & enable_gdn_pdn}}; // {num_quads{1'b0}}; // {num_quads{!xcvr_reset_pb_n}};
wire	[num_quads-1:0]				pll_powerdown_sig		=	{num_quads{aci_pdn & enable_gdn_pdn}}; // {num_quads{1'b0}};  // {num_quads{!xcvr_reset_pb_n}}; // {num_quads{1'b0}}; 
wire	[num_quads-1:0]				pll_powerdown_altgx;
wire	[serial_channels-1:0]		rx_locktodata_sig		=	{serial_channels{1'b0}};
wire	[serial_channels-1:0]		rx_locktorefclk_sig		=	{serial_channels{1'b0}};
wire	[serial_channels-1:0]		rx_analogreset_sig		=	{serial_channels{!xcvr_reset_n}};
wire	[serial_channels-1:0]		rx_analogreset_altgx;
wire								tx_pld_clk;
wire								rx_pld_clk; 
reg									rx_pld_clk_w;
wire								launch_edge;		// For retimer flip flop (Interlaken PMA direct mode)
assign	plls_locked			=	(&rx_freqlocked_sig) && (&pll_locked_sig) && plls_in_locked; // xcvr plls locked

// XCVR Wrapper Related   
parameter	byte_width					=  8;
parameter	deserializer_width			= 16;									// PCS deserializer width (assuming XCVR encoding is hidden)				
parameter	serial_channels 			=  6;
parameter	io_width					= serial_channels;						// Set to channel_width for parallel loopback tests
parameter	deskew_depth				=  32;									// For BTS Channel Bonder
parameter	deskew_depth_log2			=  5;									// log2(deskew_depth)

parameter	sync_pattern_width			= cw;									// (for simplity make the channel width) F628 for BTS Word Aligner
parameter	mux_sel_width				=  5;									// log2(channel_width) (round any fraction up)   
parameter	mux_sel_step				=  1;									// Set to 1 for word aligner & byte ordering in the core,
																				// Set to deserializer_width for byte ordering in the core and word aligner in alt2gxb
parameter	num_endians					= channel_width / deserializer_width;	// Eg Double Width with Byte serializer, there a 4 byte endians
parameter	rx_patterndetect_width		= serial_channels * channel_width / byte_width;       
parameter	tx_ctrlenable_width			= serial_channels * channel_width / byte_width; 
parameter	rx_ctrldetect_width			= serial_channels * channel_width / byte_width; 
parameter	tw							= tx_ctrlenable_width / serial_channels; // Single channel control word width
parameter	bonded_word_width			= channel_width * serial_channels;
parameter	cw							= channel_width;						// Abbreviate to make design more readable 
parameter	num_quads					=  2; 	// 1 Quad (1 xcvr block) used for 4 serial channels 
parameter	channel_width				= 32;	// Channel Width (parallel data) at the FPGA interface
parameter	channel_width_log2			=  5;
                        
// Width should be >= sync_pattern_width
// word_align_pattern[sync_pattern_width-1:0] used by word aligner
// Full pattern output by tx_wrapper_bts, suggest make dc balanced if ac coupling.
// Usually make {serial_channels{word_align_pattern}} different from prbs_seed_param
// bc = K28.5  (sync) fb = (byte ordering)        
parameter	byte_order_pattern			= 32'h000000fb;			// d pad + k byte order pattern    
parameter	word_align_pattern 			= 32'h000000bc;			// d pad + k28.5 aligner pattern 


wire		[serial_channels-1:0]				tx_clkout_sig;
wire		[serial_channels-1:0]				rx_clkout_sig;
wire		[serial_channels-1:0]				tx_clkout_sig_0;
wire		[serial_channels-1:0]				rx_clkout_sig_0;
wire		[bonded_word_width-1:0]				rx_dataout_sig;
wire		[bonded_word_width-1:0]				rx_dataout_sig_0;
wire		[rx_patterndetect_width-1:0]		rx_patterndetect_sig;
wire		[serial_channels-1:0]				rx_freqlocked_sig;
wire		[serial_channels-1:0]				rx_freqlocked_altgx;
wire		[num_quads-1:0]						pll_locked_sig;
wire		[num_quads-1:0]						pll_locked_altgx;
wire		[serial_channels-1:0]				rxpll_disable;
wire		[serial_channels-1:0]				rxpll_disable_w;

// data_width should match bonded_word_width
wire		[data_width-1:0]					tx_datain_sig; 			// From bts    
wire		[data_width-1:0]					tx_datain_sig_0; 		// From bts    
wire		[data_width-1:0]					tx_datain_sig_txepcf;	// from TX External phase compensation FIFO
wire		[data_width-1:0]					tx_datain_sig_txepcf_0;	// from TX External phase compensation FIFO

// XCVR altera_bts (Datagen actual & expected & Comparator)

// Works for 4 quads (64 bits) or 20 quads (320 bits) since divides into bonded width
parameter	prbs_depth				=	32;					// data_width should be an integer multiple of prbs_depth for prbs_type=prbs_mem
parameter	prbs_coeff_param		=	{serial_channels{32'hc000_0401}}; // Should equal data_width
parameter	prbs_seed_param			=	192'hace1_ace2_2233_f7a5_5555_2138_2128_3003_bc21_1c32_1242_e2c1; // prbs_seed192_param 
parameter	prbs_type				=	0;					// Legacy
parameter	prt_init				=	prbs_type;
parameter	prbs_width				=	data_width;			// For com, set to prbs_depth * channel_width
parameter	data_width				=	bonded_word_width;
parameter	enable_tx				=	1;	// For half duplex operation - only supported in some designs
parameter	enable_rx				=	1;	// For example, 3S SDI Daughtercard has 2 TX's and 1 RX
parameter	enable_sdm_dip_override	=	0;	// Allow user dip to override sync detect mask
parameter	enable_edm_dip_override	=	0;	// Allow user dip to override error detect mask

assign	led_tx	=	(&pll_locked_sig);
assign	led_rx	=	(&rx_freqlocked_sig);
 


// Transceivers with DPRIO
wire	[1023:0]						reconfig_fromgxb_sig;
wire	[1023:0]							reconfig_togxb_sig;
assign		reconfig_clk_sig		=	cal_blk_clk_sig;  
reg			lf_clk;

// DPRIO:
wire  									busy_sig;
wire  									data_valid_sig;
wire  									read_sig;
wire  									reconfig_clk_sig;
wire  	[eq_width-1:0] 					rx_eqctrl_sig;
wire   	[eq_width-1:0] 					rx_eqctrl_out_sig;
wire  	[dc_width-1:0]  				rx_eqdcgain_sig;
wire   	[dc_width-1:0]  				rx_eqdcgain_out_sig;
wire  	[pre_width-1:0] 				tx_preemp_0t_sig;
wire   	[pre_width-1:0] 				tx_preemp_0t_out_sig;
wire  	[pre_width-1:0] 				tx_preemp_1t_sig;
wire   	[pre_width-1:0] 				tx_preemp_1t_out_sig;
wire  	[pre_width-1:0] 				tx_preemp_2t_sig;
wire  	[pre_width-1:0] 				tx_preemp_2t_out_sig;
wire  	[vod_width-1:0] 				tx_vodctrl_sig;
wire   	[vod_width-1:0] 				tx_vodctrl_out_sig;
wire  									write_all_sig;			

// Error detect and sync detect avalon_dip override
generate
	begin
		if	(enable_sdm_dip_override	==	0)
		begin
			assign	aci_sdm_dip	=	0;
		end
		if	(enable_edm_dip_override	==	0)
		begin
			assign	aci_edm_dip	=	0;
		end
	end
endgenerate	



// SRIO Express Megacore
generate 
	if	((component_type == "SRIO") && (serial_channels == 4))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];
		
		alt2gxb_agx_srio_x4_2_5G	alt2gxb_agx_srio_x4_2_5G_inst (
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_enable ( gxb_enable_sig ),                                                 
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.rx_analogreset ( rx_analogreset_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),			
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_locktodata ( rx_locktodata_sig ),
			.rx_locktorefclk ( rx_locktorefclk_sig ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_ctrlenable ( tx_ctrlenable_sig_txepcf ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_ctrldetect (  ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_disperr (  ),
			.rx_errdetect (  ),
			.rx_freqlocked ( rx_freqlocked_sig ),
			.rx_patterndetect ( rx_patterndetect_sig ),
			.rx_pll_locked (  ),
			.rx_syncstatus ( rx_syncstatus_sig ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);

		// Some versions of Quartus requires separate clock on each TX in Basic Mode:
		// Pass through data and control - Replace with a direct connect
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
		if	(serial_channels >=6 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_6 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*6 )-1:tw*5 ],tx_datain_sig[(cw*6 )-1:cw*5 ]}),.outclk(tx_clkout_sig[5 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*6 )-1:tw*5 ],tx_datain_sig_txepcf[(cw*6 )-1:cw*5 ]}));
		if	(serial_channels >=7 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_7 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*7 )-1:tw*6 ],tx_datain_sig[(cw*7 )-1:cw*6 ]}),.outclk(tx_clkout_sig[6 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*7 )-1:tw*6 ],tx_datain_sig_txepcf[(cw*7 )-1:cw*6 ]}));
		if	(serial_channels >=8 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_8 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*8 )-1:tw*7 ],tx_datain_sig[(cw*8 )-1:cw*7 ]}),.outclk(tx_clkout_sig[7 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*8 )-1:tw*7 ],tx_datain_sig_txepcf[(cw*8 )-1:cw*7 ]}));
		if	(serial_channels >=9 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_9 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*9 )-1:tw*8 ],tx_datain_sig[(cw*9 )-1:cw*8 ]}),.outclk(tx_clkout_sig[8 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*9 )-1:tw*8 ],tx_datain_sig_txepcf[(cw*9 )-1:cw*8 ]}));
		if	(serial_channels >=10) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_10	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*10)-1:tw*9 ],tx_datain_sig[(cw*10)-1:cw*9 ]}),.outclk(tx_clkout_sig[9 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*10)-1:tw*9 ],tx_datain_sig_txepcf[(cw*10)-1:cw*9 ]}));
		if	(serial_channels >=11) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_11	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*11)-1:tw*10],tx_datain_sig[(cw*11)-1:cw*10]}),.outclk(tx_clkout_sig[10]),.dataout({tx_ctrlenable_sig_txepcf[(tw*11)-1:tw*10],tx_datain_sig_txepcf[(cw*11)-1:cw*10]}));
		if	(serial_channels >=12) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_12	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*12)-1:tw*11],tx_datain_sig[(cw*12)-1:cw*11]}),.outclk(tx_clkout_sig[11]),.dataout({tx_ctrlenable_sig_txepcf[(tw*12)-1:tw*11],tx_datain_sig_txepcf[(cw*12)-1:cw*11]}));
		if	(serial_channels >=13) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_13	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*13)-1:tw*12],tx_datain_sig[(cw*13)-1:cw*12]}),.outclk(tx_clkout_sig[12]),.dataout({tx_ctrlenable_sig_txepcf[(tw*13)-1:tw*12],tx_datain_sig_txepcf[(cw*13)-1:cw*12]}));
		if	(serial_channels >=14) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_14	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*14)-1:tw*13],tx_datain_sig[(cw*14)-1:cw*13]}),.outclk(tx_clkout_sig[13]),.dataout({tx_ctrlenable_sig_txepcf[(tw*14)-1:tw*13],tx_datain_sig_txepcf[(cw*14)-1:cw*13]}));
		if	(serial_channels >=15) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_15	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*15)-1:tw*14],tx_datain_sig[(cw*15)-1:cw*14]}),.outclk(tx_clkout_sig[14]),.dataout({tx_ctrlenable_sig_txepcf[(tw*15)-1:tw*14],tx_datain_sig_txepcf[(cw*15)-1:cw*14]}));
		if	(serial_channels >=16) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_16	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*16)-1:tw*15],tx_datain_sig[(cw*16)-1:cw*15]}),.outclk(tx_clkout_sig[15]),.dataout({tx_ctrlenable_sig_txepcf[(tw*16)-1:tw*15],tx_datain_sig_txepcf[(cw*16)-1:cw*15]}));
		if	(serial_channels >=17) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_17	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*17)-1:tw*16],tx_datain_sig[(cw*17)-1:cw*16]}),.outclk(tx_clkout_sig[16]),.dataout({tx_ctrlenable_sig_txepcf[(tw*17)-1:tw*16],tx_datain_sig_txepcf[(cw*17)-1:cw*16]}));
		if	(serial_channels >=18) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_18	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*18)-1:tw*17],tx_datain_sig[(cw*18)-1:cw*17]}),.outclk(tx_clkout_sig[17]),.dataout({tx_ctrlenable_sig_txepcf[(tw*18)-1:tw*17],tx_datain_sig_txepcf[(cw*18)-1:cw*17]}));
		if	(serial_channels >=19) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_19	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*19)-1:tw*18],tx_datain_sig[(cw*19)-1:cw*18]}),.outclk(tx_clkout_sig[18]),.dataout({tx_ctrlenable_sig_txepcf[(tw*19)-1:tw*18],tx_datain_sig_txepcf[(cw*19)-1:cw*18]}));
		if	(serial_channels >=20) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_20	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*20)-1:tw*19],tx_datain_sig[(cw*20)-1:cw*19]}),.outclk(tx_clkout_sig[19]),.dataout({tx_ctrlenable_sig_txepcf[(tw*20)-1:tw*19],tx_datain_sig_txepcf[(cw*20)-1:cw*19]}));
		if	(serial_channels >=21) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_21	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*21)-1:tw*20],tx_datain_sig[(cw*21)-1:cw*20]}),.outclk(tx_clkout_sig[20]),.dataout({tx_ctrlenable_sig_txepcf[(tw*21)-1:tw*20],tx_datain_sig_txepcf[(cw*21)-1:cw*20]}));
		if	(serial_channels >=22) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_22	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*22)-1:tw*21],tx_datain_sig[(cw*22)-1:cw*21]}),.outclk(tx_clkout_sig[21]),.dataout({tx_ctrlenable_sig_txepcf[(tw*22)-1:tw*21],tx_datain_sig_txepcf[(cw*22)-1:cw*21]}));
		if	(serial_channels >=23) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_23	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*23)-1:tw*22],tx_datain_sig[(cw*23)-1:cw*22]}),.outclk(tx_clkout_sig[22]),.dataout({tx_ctrlenable_sig_txepcf[(tw*23)-1:tw*22],tx_datain_sig_txepcf[(cw*23)-1:cw*22]}));
		if	(serial_channels >=24) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_24	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*24)-1:tw*23],tx_datain_sig[(cw*24)-1:cw*23]}),.outclk(tx_clkout_sig[23]),.dataout({tx_ctrlenable_sig_txepcf[(tw*24)-1:tw*23],tx_datain_sig_txepcf[(cw*24)-1:cw*23]}));
	end    


endgenerate


// SDI x2 3G Megacore -- Arria
generate 
	if	((component_type == "SDI_3G") && (serial_channels == 2))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];
		alt2gxb_sdi_xcvr_board_3G_x2	alt2gxb_sdi_xcvr_board_3G_x2_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_enable ( gxb_enable_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.rx_bitslip ( 0 ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_coreclk ( {2{tx_clkout_sig[0]}} ),
			.tx_datain ( tx_datain_sig ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_sig ),
			.rx_pll_locked (  ),
			.rx_rlv (  ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);
	end    
endgenerate

generate 
	if	((component_type == "S2GX_SDI_3G") && (serial_channels == 2))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];
		alt2gxb_sdi_s2gx_xcvr_x2
			alt2gxb_sdi_s2gx_xcvr_x2_inst
		(
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_enable ( gxb_enable_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.rx_bitslip ( 0 ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			//.tx_coreclk ( {2{tx_clkout_sig[0]}} ),
			.tx_datain ( tx_datain_sig ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_sig ),
			.rx_pll_locked (  ),
			.rx_rlv (  ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);
	end    
endgenerate


// SDI x2 AES -- To simplify, make this look like SDI_3G SERDES (at much lower rate)
// Implement a simple SERDES:
reg	[mux_sel_width-1:0]			serdes_ctr_tx;
reg	[mux_sel_width-1:0]			serdes_ctr_rx;
reg	[data_width-1:0]			serdes_tx;
reg	[data_width-1:0]			serdes_rx;
reg	[data_width-1:0]			rx_dataout_sig_reg;

reg								serdes_tx_pld_clk;
reg								serdes_rx_pld_clk;
reg	[io_width-1:0]				tx_p_reg;
integer							serdes_int_tx;
integer							serdes_int_rx;
generate 
	if	(component_type == "SDI_AES")
	begin  	
		// TX SERDES:
		always	@(posedge refclk_p)
		begin
			case	(serdes_ctr_tx)
				(deserializer_width/2)-1:	begin
												serdes_tx_pld_clk	<=	1;
												serdes_ctr_tx		<=	serdes_ctr_tx + 1;
											end
				deserializer_width-1:		begin
												serdes_tx_pld_clk	<=	0;
												serdes_ctr_tx		<=	0;
												serdes_tx			<=	 tx_datain_sig;
											end
				default:					begin
												serdes_ctr_tx		<=	serdes_ctr_tx + 1;
											end
			endcase
		end
		always	@(negedge refclk_p)
		begin
			for	(serdes_int_tx=0; serdes_int_tx<io_width; serdes_int_tx=serdes_int_tx+1)
			begin
				tx_p_reg[serdes_int_tx]	<=	serdes_tx[(channel_width*serdes_int_tx) + serdes_ctr_tx];
			end
		end
		assign	tx_ss_clk_out	=	refclk_p;
		assign	tx_p			=	tx_p_reg;
		// RX SERDES:
		always	@(negedge rx_ss_clk_in)
		begin	
			case	(serdes_ctr_rx)
				(deserializer_width/2)-1:	begin
												serdes_rx_pld_clk	<=	1;
												serdes_ctr_rx		<=	serdes_ctr_rx + 1;
											end
				deserializer_width-1:		begin
												serdes_rx_pld_clk	<=	0;
												serdes_ctr_rx		<=	0;
												rx_dataout_sig_reg	<=	serdes_rx;
											end
				default:					begin
												serdes_ctr_rx		<=	serdes_ctr_rx + 1;
											end
			endcase
		end
		
		always	@(posedge rx_ss_clk_in)
		begin
			for	(serdes_int_rx=0; serdes_int_rx<io_width; serdes_int_rx=serdes_int_rx+1)
			begin
				serdes_rx[(channel_width*serdes_int_rx) + serdes_ctr_rx]	<=	rx_p[serdes_int_rx];
			end
		end

		assign		rx_dataout_sig		=	rx_dataout_sig_reg;
		assign		rx_clkout_sig		=	{serial_channels{rx_pld_clk}};
		assign		tx_pld_clk			=	serdes_tx_pld_clk;		
		assign		rx_pld_clk			=	serdes_rx_pld_clk;
		assign		pll_locked_sig		=	{num_quads{1'b1}};
		assign		rx_freqlocked_sig	=	{serial_channels{1'b1}};
	end    
endgenerate

// Support insert / delete ordered sets on the RX (eg for XPAK & X1 modules)
wire	rx_ready;
generate
	if	(
			(component_type != "XPAK")	 &&
			(component_type != "X2")	 &&
			(component_type != "X2_S4GX") &&
			(component_type != "X2_A2GX")
		)
		assign	rx_ready	=	1'b1;
endgenerate


// XPAK Megacore - Use for SGX board with altgxb  
wire coreclk_out_sig;
generate 
	if	((component_type == "XPAK") && (serial_channels == 4))
	begin 
		assign	tx_pld_clk	=	coreclk_out_sig;
		assign	rx_pld_clk 	=	coreclk_out_sig; 	
		assign	rx_ready	=	!(|rx_ctrldetect_sig);	// Preamble, only 1 ctrl is on
		altgxb_xpak	altgxb_xpak_inst
		(
			.inclk ( refclk_p ),
			.rx_in ( rx_p ),
			//.rx_slpbk ( seriallpbken ),
			.rxdigitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.tx_ctrlenable ( tx_ctrlenable_sig ),
			.tx_in ( tx_datain_sig ),
			.txdigitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.coreclk_out ( coreclk_out_sig ),
			.pll_locked ( pll_locked_sig ),
			.rx_channelaligned (  ), 
			.rx_ctrldetect ( rx_ctrldetect_sig_0 ), // Only compare D characters (Use this to control RX Ready
			.rx_disperr ( ),
			.rx_errdetect ( ),
			.rx_freqlocked ( rx_freqlocked_sig ),
			.rx_locked ( ),
			.rx_out ( rx_dataout_sig_0 ),
			.rx_patterndetect ( rx_patterndetect_sig ),
			.rx_syncstatus (  ),
			.tx_out ( tx_p )
		);			

// Byte order block - since doesn't exist for Stratix GX ALTGXB:
			xpak_byte_swap64 	xpak_byte_swap64
			(
				.clk			(rx_pld_clk),
				.reset_n		(reset_n),
				.rx_ctrl		(rx_ctrldetect_sig_0),
				.data_in		(rx_dataout_sig_0),
				.rx_ctrl_out	(rx_ctrldetect_sig),
				.data_out		(rx_dataout_sig)
			);
   end
endgenerate
                                 
// X2 Megacore Use ALT2GXG for S2GX Device
wire							coreclk_out_sig_x2;
generate 
	if	((component_type == "X2") && (serial_channels == 4) && (map_dprio == 0))
	begin 
		assign	tx_pld_clk	=	coreclk_out_sig_x2;
		assign	rx_pld_clk 	=	coreclk_out_sig_x2; 	
		assign	rx_ready	=	!(|rx_ctrldetect_sig);	// Preamble, only 1 ctrl is on

		alt2gxb_x2	alt2gxb_x2_inst
		(
			.rx_analogreset (avalon_dip[3]),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_enable ( gxb_enable_sig ),                                                 
			.gxb_powerdown ( gxb_powerdown_sig ),			
			.pll_inclk ( refclk_p ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_locktodata ( 0 ),	// Set locktodata & locktorefclk both to 0 for automatic lock mode
			.rx_locktorefclk ( 0 ),			
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_ctrlenable ( tx_ctrlenable_sig ),
			.tx_datain ( tx_datain_sig ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.coreclkout ( coreclk_out_sig_x2 ),
			.pll_locked ( pll_locked_sig ),
			.rx_channelaligned (  ),
			.rx_ctrldetect ( rx_ctrldetect_sig_0 ),
			.rx_dataout ( rx_dataout_sig_0 ),
			.rx_disperr ( ),
			.rx_errdetect ( ),
			.rx_freqlocked ( rx_freqlocked_sig ),
			.rx_patterndetect ( rx_patterndetect_sig ),	// Use if byte swap is done in xs_wrapper_bts
			.rx_pll_locked ( ),
			.rx_rlv ( ),
			.rx_syncstatus ( ),
			.tx_dataout ( tx_p )
		);
			
// Byte order block:
			xpak_byte_swap64 	xpak_byte_swap64
			(
				.clk			(rx_pld_clk),
				.reset_n		(reset_n),
				.rx_ctrl		(rx_ctrldetect_sig_0),
				.data_in		(rx_dataout_sig_0),
				.rx_ctrl_out	(rx_ctrldetect_sig),
				.data_out		(rx_dataout_sig)
			);			
			
   end
endgenerate

// X2 Megacore Use ALT2GXG for S4GX Device
generate 
	if	((component_type == "X2_S4GX") && (serial_channels == 4))
	begin 
   
		assign	tx_pld_clk	=	coreclk_out_sig_x2;
		assign	rx_pld_clk 	=	coreclk_out_sig_x2; 	
		assign	rx_ready	=	!(|rx_ctrldetect_sig);	// Preamble, only 1 ctrl is on

		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);

		altgx_s4gx_x2	altgx_s4gx_x2_inst (
			.cal_blk_clk ( cal_blk_clk_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_analogreset ( rx_analogreset_altgx ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_ctrlenable ( tx_ctrlenable_sig ),
			.tx_datain ( tx_datain_sig ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.coreclkout ( coreclk_out_sig_x2 ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_channelaligned (  ),
			.rx_ctrldetect ( rx_ctrldetect_sig_0 ),
			.rx_dataout ( rx_dataout_sig_0 ),
			.rx_disperr (  ),
			.rx_errdetect (  ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_patterndetect ( rx_patterndetect_sig ),
			.rx_pll_locked (  ),
			.rx_rmfifodatadeleted (  ),
			.rx_rmfifodatainserted (  ),
			.rx_rmfifoempty (  ),
			.rx_rmfifofull (  ),
			.rx_runningdisp (  ),
			.rx_syncstatus (  ),
			.tx_dataout ( tx_p )
	);

/*
		altgx_s4gx_x2	altgx_s4gx_x2_inst
		(
			.rx_analogreset (avalon_dip[3]),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_enable ( gxb_enable_sig ),                                                 
			.gxb_powerdown ( gxb_powerdown_sig ),			
			.pll_inclk ( refclk_p ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_locktodata ( 0 ),	// Set locktodata & locktorefclk both to 0 for automatic lock mode
			.rx_locktorefclk ( 0 ),			
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_ctrlenable ( tx_ctrlenable_sig ),
			.tx_datain ( tx_datain_sig ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.coreclkout ( coreclk_out_sig_x2 ),
			.pll_locked ( pll_locked_sig ),
			.rx_channelaligned (  ),
			.rx_ctrldetect ( rx_ctrldetect_sig_0 ),
			.rx_dataout ( rx_dataout_sig_0 ),
			.rx_disperr ( ),
			.rx_errdetect ( ),
			.rx_freqlocked ( rx_freqlocked_sig ),
			.rx_patterndetect ( rx_patterndetect_sig ),	// Use if byte swap is done in xs_wrapper_bts
			.rx_pll_locked ( ),
			.rx_rlv ( ),
			.rx_syncstatus ( ),
			.tx_dataout ( tx_p )
		);
*/
		
		altgx_s4gx_xcvr_reconfig_x4	altgx_s4gx_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);		
			
// Byte order block:

			xpak_byte_swap64 	xpak_byte_swap64
			(
				.clk			(rx_pld_clk),
				.reset_n		(reset_n),
				.rx_ctrl		(rx_ctrldetect_sig_0),
				.data_in		(rx_dataout_sig_0),
				.rx_ctrl_out	(rx_ctrldetect_sig),
				.data_out		(rx_dataout_sig)
			);		

   end
endgenerate

// X2 Megacore Use ALT2GXG for A2GX Device
generate 
	if	((component_type == "X2_A2GX") && (serial_channels == 4))
	begin 
   
		assign	tx_pld_clk	=	coreclk_out_sig_x2;
		assign	rx_pld_clk 	=	coreclk_out_sig_x2; 	
		assign	rx_ready	=	!(|rx_ctrldetect_sig);	// Preamble, only 1 ctrl is on

		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);

		altgx_a2gx_x2	altgx_a2gx_x2_inst (
			.cal_blk_clk ( cal_blk_clk_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_analogreset ( rx_analogreset_altgx ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_ctrlenable ( tx_ctrlenable_sig ),
			.tx_datain ( tx_datain_sig ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.coreclkout ( coreclk_out_sig_x2 ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_channelaligned (  ),
			.rx_ctrldetect ( rx_ctrldetect_sig_0 ),
			.rx_dataout ( rx_dataout_sig_0 ),
			.rx_disperr (  ),
			.rx_errdetect (  ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_patterndetect ( rx_patterndetect_sig ),
			.rx_pll_locked (  ),
			.rx_rmfifodatadeleted (  ),
			.rx_rmfifodatainserted (  ),
			.rx_rmfifoempty (  ),
			.rx_rmfifofull (  ),
			.rx_runningdisp (  ),
			.rx_syncstatus (  ),
			.tx_dataout ( tx_p )
	);
		
		
		assign	tx_preemp_0t_out_sig	=	0;
		assign	tx_preemp_2t_out_sig	=	0;	

		altgx_a2gx_xcvr_reconfig_x4	altgx_a2gx_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp ( tx_preemp_1t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_out ( tx_preemp_1t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);		
		
		
		
			
// Byte order block:

			xpak_byte_swap64 	xpak_byte_swap64
			(
				.clk			(rx_pld_clk),
				.reset_n		(reset_n),
				.rx_ctrl		(rx_ctrldetect_sig_0),
				.data_in		(rx_dataout_sig_0),
				.rx_ctrl_out	(rx_ctrldetect_sig),
				.data_out		(rx_dataout_sig)
			);		

   end
endgenerate


// XPAK Megacore with DPRIO VOD, Pre-emphasis, & RX Equalization
// X2 Megacore Use ALT2GXG for S2GX Device
generate 
	if	((component_type == "X2") && (serial_channels == 4) && (map_dprio == 1))
	begin 
		assign	tx_pld_clk	=	coreclk_out_sig_x2;
		assign	rx_pld_clk 	=	coreclk_out_sig_x2; 	
		assign	rx_ready	=	!(|rx_ctrldetect_sig);	// Preamble, only 1 ctrl is on


		alt2gxb_x2_dprio	alt2gxb_x2_dprio_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_enable ( gxb_enable_sig ),                                                 
			.gxb_powerdown ( gxb_powerdown_sig ),			
			.pll_inclk ( refclk_p ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_locktodata ( {serial_channels{avalon_dip[6]}} ),
			.rx_locktorefclk ( {serial_channels{avalon_dip[5]}} ),			
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_ctrlenable ( tx_ctrlenable_sig ),
			.tx_datain ( tx_datain_sig ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.coreclkout ( coreclk_out_sig_x2 ),
			.pll_locked ( pll_locked_sig ),
			.rx_channelaligned (  ),
			.rx_ctrldetect ( rx_ctrldetect_sig_0 ),
			.rx_dataout ( rx_dataout_sig_0 ),
			.rx_disperr ( ),
			.rx_errdetect ( ),
			.rx_freqlocked ( rx_freqlocked_sig ),
			.rx_patterndetect ( rx_patterndetect_sig ),	// Use if byte swap is done in xs_wrapper_bts
			.rx_pll_locked ( ),
			.rx_rlv ( ),
			.rx_syncstatus ( ),
			.tx_dataout ( tx_p ),
			.reconfig_clk ( reconfig_clk_sig ), 
			.reconfig_togxb (reconfig_togxb_sig[2:0] ) ,
			.reconfig_fromgxb ( reconfig_fromgxb_sig[0] )			
		);
			
		alt2gxb_reconfig_x4	alt2gxb_reconfig_x4_inst
		(
			.read				( read_sig					),
			.reconfig_clk		( reconfig_clk_sig  		),
			.reconfig_fromgxb	( reconfig_fromgxb_sig[0]	),
			.rx_eqctrl			( rx_eqctrl_sig				),
			.rx_eqdcgain		( rx_eqdcgain_sig			),
			.tx_preemp_0t		( tx_preemp_0t_sig			),
			.tx_preemp_1t		( tx_preemp_1t_sig			),
			.tx_preemp_2t		( tx_preemp_2t_sig			),
			.tx_vodctrl			( tx_vodctrl_sig			),
			.write_all			( write_all_sig				),
			.busy				( busy_sig					),
			.data_valid			( data_valid_sig			),
			.reconfig_togxb		( reconfig_togxb_sig[2:0]	),
			.rx_eqctrl_out		( rx_eqctrl_out_sig			),
			.rx_eqdcgain_out	( rx_eqdcgain_out_sig		),
			.tx_preemp_0t_out	( tx_preemp_0t_out_sig		),
			.tx_preemp_1t_out	( tx_preemp_1t_out_sig		),
			.tx_preemp_2t_out	( tx_preemp_2t_out_sig		),
			.tx_vodctrl_out		( tx_vodctrl_out_sig		)
		);   		

	
			
// Byte order block:
			xpak_byte_swap64 	xpak_byte_swap64
			(
				.clk			(rx_pld_clk),
				.reset_n		(reset_n),
				.rx_ctrl		(rx_ctrldetect_sig_0),
				.data_in		(rx_dataout_sig_0),
				.rx_ctrl_out	(rx_ctrldetect_sig),
				.data_out		(rx_dataout_sig)
			);
			
   end
endgenerate



// PCI Express Megacore x4 for Arria GX Board
wire	coreclkout_sig;
generate 
	if	((component_type == "PCIE") && (serial_channels == 4))
	begin  
		assign	tx_pld_clk	=	coreclkout_sig;
		assign	rx_pld_clk 	=	coreclkout_sig;  
		alt2gxb_agx_pcie_x4_2_5G	alt2gxb_agx_pcie_x4_2_5G_i
		( 
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.fixedclk ( cal_blk_clk_sig ),
			.gxb_enable ( gxb_enable_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pipe8b10binvpolarity ( 0 ),
			.pll_inclk ( refclk_p ),
			.powerdn ( 0 ),
			.rx_analogreset ( rx_analogreset_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain (  rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			// Remove port to enable fast recovery mode
			//.rx_locktodata ( rx_locktodata_sig ),
			//.rx_locktorefclk ( rx_locktorefclk_sig ),
			.tx_ctrlenable ( tx_ctrlenable_sig ),
			.tx_datain ( tx_datain_sig ),
			.tx_detectrxloop ( 0 ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.tx_forcedispcompliance ( 0 ),
			.tx_forceelecidle ( 0 ),
			.coreclkout ( coreclkout_sig ),
			.pipedatavalid (  ),
			.pipeelecidle (  ),
			.pipephydonestatus (  ),
			.pipestatus (  ),
			.pll_locked ( pll_locked_sig ),
			.rx_ctrldetect (  ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_disperr (  ),
			.rx_errdetect (  ),
			.rx_freqlocked ( rx_freqlocked_sig ),
			.rx_patterndetect ( rx_patterndetect_sig ),
			.rx_pll_locked (  ),
			.rx_rlv (  ),
			.rx_signaldetect (  ),
			.rx_syncstatus (  ),
			.tx_dataout ( tx_p ), 
			.tx_invpolarity ( 0 )  
		);
   end
endgenerate

// PCI Express 2.5 G Megacore x8 for Stratix IV GX Board
generate 
	if	((component_type == "PCIE_S4GX_2P5G") && (serial_channels == 8))
	begin  
		assign	tx_pld_clk	=	coreclkout_sig;
		assign	rx_pld_clk 	=	coreclkout_sig;  
		
		altgx_s4gx_pcie_x8_2p5G	altgx_s4gx_pcie_x8_2p5G_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.fixedclk ( cal_blk_clk_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pipe8b10binvpolarity ( 0 ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_sig ),
			.powerdn ( 0 ),
			.reconfig_clk ( 0 ),
			.reconfig_togxb ( 0 ),
			.rx_analogreset ( rx_analogreset_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.tx_ctrlenable ( tx_ctrlenable_sig ),
			.tx_datain ( tx_datain_sig ),
			.tx_detectrxloop ( 0 ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.tx_forcedispcompliance ( 0 ),
			.tx_forceelecidle ( 0 ),
			.tx_invpolarity ( 0 ),
			.coreclkout ( coreclkout_sig ),
			.pipedatavalid (  ),
			.pipeelecidle (  ),
			.pipephydonestatus (  ),
			.pipestatus (  ),
			.pll_locked ( pll_locked_sig ),
			.reconfig_fromgxb (  ),
			.rx_ctrldetect (  ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_disperr (  ),
			.rx_errdetect (  ),
			.rx_freqlocked ( rx_freqlocked_sig ),
			.rx_patterndetect ( rx_patterndetect_sig ),
			.rx_pll_locked (  ),
			.rx_rlv (  ),
			.rx_signaldetect (  ),
			.rx_syncstatus (  ),
			.tx_dataout ( tx_p )
		);
   end
endgenerate

// PCI Express 5 G Megacore x8 for Stratix IV GX Board
generate 
	if	((component_type == "PCIE_S4GX_5G") && (serial_channels == 8))
	begin  
		assign	tx_pld_clk	=	coreclkout_sig;
		assign	rx_pld_clk 	=	coreclkout_sig;  
		
		altgx_s4gx_pcie_x8_5G	altgx_s4gx_pcie_x8_5G_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.fixedclk ( cal_blk_clk_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pipe8b10binvpolarity ( 0 ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_sig ),
			.powerdn ( 0 ),
			.reconfig_clk ( 0 ),
			.reconfig_togxb ( 0 ),
			.rx_analogreset ( rx_analogreset_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.tx_ctrlenable ( tx_ctrlenable_sig ),
			.tx_datain ( tx_datain_sig ),
			.tx_detectrxloop ( 0 ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.tx_forcedispcompliance ( 0 ),
			.tx_forceelecidle ( 0 ),
			.tx_invpolarity ( 0 ),
			.coreclkout ( coreclkout_sig ),
			.pipedatavalid (  ),
			.pipeelecidle (  ),
			.pipephydonestatus (  ),
			.pipestatus (  ),
			.pll_locked ( pll_locked_sig ),
			.reconfig_fromgxb (  ),
			.rx_ctrldetect (  ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_disperr (  ),
			.rx_errdetect (  ),
			.rx_freqlocked ( rx_freqlocked_sig ),
			.rx_patterndetect ( rx_patterndetect_sig ),
			.rx_pll_locked (  ),
			.rx_rlv (  ),
			.rx_signaldetect (  ),
			.rx_syncstatus (  ),
			.tx_dataout ( tx_p ),
			// Signals added for PCIe Gen 2:
			.rateswitch ( {8{1'b1}} ),				// 0=Gen1, 1=Gen2
			.tx_pipedeemph ( {8{1'b0}} ),			// 0=-6db 1=-3.5db de-emphasis
			.tx_pipemargin ( {8{3'b000}} ),			// Normal operation
			.tx_pipeswing ( {8{1'b0}} ),			// 0=full swing, 1=low swing	
			.rx_elecidleinfersel ( {8{3'b100}} )	// LO- Absence of skip ordered
													// set in 128 s window			
		);
   end
endgenerate

// HSM x4 3.75 Gbps Basic Mode XCVR A2GX		
generate 
	if	((component_type == "A2GX_HSM_XCVR") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_a2gx_basic_x4_3p75G	altgx_a2gx_basic_x4_3p75G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p ),
			.rx_enapatternalign ( 0 )
		);


		assign	tx_preemp_0t_out_sig	=	0;
		assign	tx_preemp_2t_out_sig	=	0;	

		altgx_a2gx_xcvr_reconfig_x4	altgx_a2gx_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp ( tx_preemp_1t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_out ( tx_preemp_1t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
	
	end    
endgenerate

// Cyclone 4 GX Host Dev Kit HSMC
generate
	if	((component_type == "c4gx_f896_host_hsmc_xcvrs_XCVR") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];
		
		c4gx_basic_x4_3G	c4gx_basic_x4_3G_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_areset ( pll_powerdown_altgx ),
			.pll_inclk ( refclk_p ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_analogreset ( rx_analogreset_altgx ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.tx_coreclk ( {serial_channels{tx_pld_clk}} ),
			.tx_datain ( tx_datain_sig ),
			.tx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);

		
		assign	tx_preemp_0t_out_sig	=	{serial_channels{5'b10000}};
		assign	tx_preemp_2t_out_sig	=	{serial_channels{5'b10000}};	
		//assign	rx_eqdcgain_out_sig	=	0;

		c4gx_x4_altreconfig	c4gx_x4_altreconfig_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp ( tx_preemp_1t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_out ( tx_preemp_1t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);
	

	end    
endgenerate
	
// Cyclone 4 GX Host Dev Kit PCIe
generate
	if	((component_type == "c4gx_f896_host_pcie_xcvrs_XCVR") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];		

		c4gx_basic_pcie_2p5G	c4gx_basic_pcie_2p5G_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_areset ( pll_powerdown_altgx ),
			.pll_inclk ( refclk_p ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_analogreset ( rx_analogreset_altgx ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.tx_coreclk ( {serial_channels{tx_pld_clk}} ),
			.tx_datain ( tx_datain_sig ),
			.tx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);
	
		assign	tx_preemp_0t_out_sig	=	{serial_channels{5'b10000}};
		assign	tx_preemp_2t_out_sig	=	{serial_channels{5'b10000}};	
		//assign	rx_eqdcgain_out_sig	=	0;		

		c4gx_x4_altreconfig	c4gx_x4_altreconfig_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp ( tx_preemp_1t_sig ),
			//.tx_preemp_1t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			//.tx_preemp_1t_out ( tx_preemp_2t_out_sig ),
			.tx_preemp_out ( tx_preemp_1t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);
	
	end    
endgenerate



// Cyclone 4 GX 2.5 Gbps Basic Mode XCVR



generate 
	if	((component_type == "C4GX_XCVR_2P5G_100MHz_REFCLK") && (serial_channels == 1))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];
		assign		busy_sig	=	0;


		c4gx_basic_x1_2p5G_100REFCLK	c4gx_basic_x1_2p5G_100REFCLK_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( 0 ), // ( reconfig_clk_sig ),
			.reconfig_togxb ( 0 ), // ( reconfig_togxb_sig ),
			.rx_analogreset ( rx_analogreset_altgx ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.tx_datain ( tx_datain_sig ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( ), // ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);
/*
		c4gx_x1_altreconfig	c4gx_x1_altreconfig_inst
		(
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.busy ( busy_sig ),
			.reconfig_togxb ( reconfig_togxb_sig )
		);
*/


	end    
endgenerate


generate 
	if	((component_type == "C4GX_XCVR_2P5G") && (serial_channels == 1))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];
		assign		busy_sig	=	0;


		c4gx_basic_x1_2p5G	c4gx_basic_x1_2p5G_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( 0 ), // ( reconfig_clk_sig ),
			.reconfig_togxb ( 0 ), // ( reconfig_togxb_sig ),
			.rx_analogreset ( rx_analogreset_altgx ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.tx_datain ( tx_datain_sig ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( ), // ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


	end    
endgenerate

// Stratix 4 GX 2.5 Gbps Basic PCIE Mode XCVR - 100 MHz REFCLK
// For PCIe host adaptor for Cyclone IV GX F169 Starter Board 
generate 
	if	((component_type == "S4GX_PCIE_XCVR_2P5G") && (serial_channels == 1))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];


		s4gx_basic_pcie_x1_2p5G	s4gx_basic_pcie_x1_2p5G_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_analogreset ( rx_analogreset_altgx ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);
	
		// For offset cancellation:
		altgx_s4gx_reconfig_x1	altgx_s4gx_reconfig_x1_inst
		(
			.reconfig_clk 		( reconfig_clk_sig ),
			.reconfig_fromgxb 	( reconfig_fromgxb_sig ),
			.busy 				( busy_sig ),
			.reconfig_togxb 	( reconfig_togxb_sig )
		);



	end    
endgenerate

generate 
	if	((component_type == "A2GX_HSM_XCVR_6G") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		// Now a 6G ALTGX:
		altgx_a2gx_basic_x4_3p75G	altgx_a2gx_basic_x4_3p75G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p ),
			//.rx_enapatternalign ( 0 )
		);


		assign	tx_preemp_0t_out_sig	=	0;
		assign	tx_preemp_2t_out_sig	=	0;	

		altgx_a2gx_xcvr_reconfig_x4	altgx_a2gx_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp ( tx_preemp_1t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_out ( tx_preemp_1t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
	
	end    
endgenerate


// HSM x4 2.5 Gbps Basic Mode XCVR A2GX		
generate 
	if	((component_type == "A2GX_HSM_XCVR_2_5G") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_a2gx_basic_x4_2p5G	altgx_a2gx_basic_x4_2p5G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p ),
			.rx_enapatternalign ( 0 )			
		);


		assign	tx_preemp_1t_out_sig	=	0;
		assign	tx_preemp_2t_out_sig	=	0;	

		altgx_a2gx_xcvr_reconfig_x4	altgx_a2gx_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp ( tx_preemp_0t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_out ( tx_preemp_0t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
	
	end    
endgenerate


// HSM x4 2.5 Gbps Basic Mode XCVR -- use to test s4gx pcie gen 1 to a2gx pcie
// board to board in basic mode		
generate 
	if	((component_type == "S4GX_HSM_XCVR_2_5G") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gx_basic_x4_2p5G	altgx_s4gx_basic_x4_2p5G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gx_xcvr_reconfig_x4	altgx_s4gx_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
	
	end    
endgenerate



/*
altgx_s4gx_pcie_x8_5G	altgx_s4gx_pcie_x8_5G_inst (
	.cal_blk_clk ( cal_blk_clk_sig ),
	.cal_blk_powerdown ( cal_blk_powerdown_sig ),
	.fixedclk ( fixedclk_sig ),
	.gxb_powerdown ( gxb_powerdown_sig ),
	.pipe8b10binvpolarity ( pipe8b10binvpolarity_sig ),
	.pll_inclk ( pll_inclk_sig ),
	.pll_powerdown ( pll_powerdown_sig ),
	.powerdn ( powerdn_sig ),
	.rateswitch ( rateswitch_sig ),
	.reconfig_clk ( reconfig_clk_sig ),
	.reconfig_togxb ( reconfig_togxb_sig ),
	.rx_analogreset ( rx_analogreset_sig ),
	.rx_cruclk ( rx_cruclk_sig ),
	.rx_datain ( rx_datain_sig ),
	.rx_digitalreset ( rx_digitalreset_sig ),
	.rx_elecidleinfersel ( rx_elecidleinfersel_sig ),
	.tx_ctrlenable ( tx_ctrlenable_sig ),
	.tx_datain ( tx_datain_sig ),
	.tx_detectrxloop ( tx_detectrxloop_sig ),
	.tx_digitalreset ( tx_digitalreset_sig ),
	.tx_forcedispcompliance ( tx_forcedispcompliance_sig ),
	.tx_forceelecidle ( tx_forceelecidle_sig ),
	.tx_invpolarity ( tx_invpolarity_sig ),
	.tx_pipedeemph ( tx_pipedeemph_sig ),
	.tx_pipemargin ( tx_pipemargin_sig ),
	.tx_pipeswing ( tx_pipeswing_sig ),
	.coreclkout ( coreclkout_sig ),
	.pipedatavalid ( pipedatavalid_sig ),
	.pipeelecidle ( pipeelecidle_sig ),
	.pipephydonestatus ( pipephydonestatus_sig ),
	.pipestatus ( pipestatus_sig ),
	.pll_locked ( pll_locked_sig ),
	.reconfig_fromgxb ( reconfig_fromgxb_sig ),
	.rx_ctrldetect ( rx_ctrldetect_sig ),
	.rx_dataout ( rx_dataout_sig ),
	.rx_disperr ( rx_disperr_sig ),
	.rx_errdetect ( rx_errdetect_sig ),
	.rx_freqlocked ( rx_freqlocked_sig ),
	.rx_patterndetect ( rx_patterndetect_sig ),
	.rx_pll_locked ( rx_pll_locked_sig ),
	.rx_rlv ( rx_rlv_sig ),
	.rx_signaldetect ( rx_signaldetect_sig ),
	.rx_syncstatus ( rx_syncstatus_sig ),
	.tx_dataout ( tx_dataout_sig )
	);
		
*/


// x1 8.5 Gbps Basic Mode XCVR:			
generate 
	if	((component_type == "S4GX_SMA_XCVR") && (serial_channels == 1))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gx_sma_xcvr	altgx_s4gx_sma_xcvr_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_sig ),
			.reconfig_clk ( 0 ), // ( reconfig_clk_sig ),
			.reconfig_togxb ( 0 ), // ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_sig ),
			.reconfig_fromgxb ( ), // ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.rx_rlv ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);

		/*
		altgx_s4gx_sma_xcvr_reconfig	altgx_s4gx_sma_xcvr_reconfig_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);
		*/

	end    
endgenerate

// 28NM TC - S4GT x1 10Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GT_X1_28NMTC") && (serial_channels == 1))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_basic_x1_11G	altgx_s4gt_basic_x1_11G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx | pll_powerdown_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);

		altgx_s4gt_xcvr_reconfig_x1	altgx_s4gt_xcvr_reconfig_x1_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
	
	end    
endgenerate


// GT x4 10Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_basic_x4_10G	altgx_s4gt_basic_x4_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x4	altgx_s4gt_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
	
	end    
endgenerate

// GT x3 10Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT") && (serial_channels == 3))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_basic_x3_10G	altgx_s4gt_basic_x3_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x3	altgx_s4gt_xcvr_reconfig_x3_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
	
	end    
endgenerate


// GT x2 10Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT") && (serial_channels == 2))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_basic_x2_10G	altgx_s4gt_basic_x2_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x2	altgx_s4gt_xcvr_reconfig_x2_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
	
	end    
endgenerate


// GT x4 8.5Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT_8P5G") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_basic_x4_8p5G	altgx_s4gt_basic_x4_8p5G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x4	altgx_s4gt_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
	
	end    
endgenerate


// GT x1 10Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT") && (serial_channels == 1))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];


		altgx_s4gt_basic_x1_10G	altgx_s4gt_basic_x1_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( 0 ),
			.reconfig_togxb ( 0 ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);
		assign	busy_sig	=	0;

/*
		altgx_s4gt_basic_x1_10G	altgx_s4gt_basic_x1_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x1	altgx_s4gt_xcvr_reconfig_x1_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);
*/

		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));	
	end    
endgenerate

// GT x1 10Gbps Basic Mode XCVR	-- Analog Reset directly from top level	
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT_DIRECTRST") && (serial_channels == 1))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(), // (pll_powerdown_altgx),
			.rxanalog_reset			() // (rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		assign	rx_analogreset_altgx	=	{serial_channels{!xcvr_reset_pb_n}};
		assign	pll_powerdown_altgx		=	{num_quads{!xcvr_reset_pb_n}};

		altgx_s4gt_basic_x1_10G	altgx_s4gt_basic_x1_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( 0 ),
			.reconfig_togxb ( 0 ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig ), // ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);
		assign	busy_sig	=	0;

/*
		altgx_s4gt_basic_x1_10G	altgx_s4gt_basic_x1_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x1	altgx_s4gt_xcvr_reconfig_x1_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);
*/

		//if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));	
	end    
endgenerate




// GT x4 10+Gbps Basic Mode XCVR Optical Demo Board		
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT_ODB") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_basic_x4_10G_odb	altgx_s4gt_basic_x4_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x4	altgx_s4gt_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
	
	end    
endgenerate

generate
	if	((component_type == "S4GX_HSM_XCVR_GT_ODB_OPNEXT") && (serial_channels == 10))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		assign rx_dataout_sig = (opnext_loopback == 0) ? rx_dataout_sig_0 : ~rx_dataout_sig_0;
		assign	opnext_loopback	=	seriallpbken_dip[1];
		
		altgx_s4gt_basic_x10_10G_odb	altgx_s4gt_basic_x10_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( {serial_channels{seriallpbken_dip[0]}} ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig_0 ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x10	altgx_s4gt_xcvr_reconfig_x10_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
		if	(serial_channels >=6 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_6 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*6 )-1:tw*5 ],tx_datain_sig[(cw*6 )-1:cw*5 ]}),.outclk(tx_clkout_sig[5 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*6 )-1:tw*5 ],tx_datain_sig_txepcf[(cw*6 )-1:cw*5 ]}));
		if	(serial_channels >=7 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_7 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*7 )-1:tw*6 ],tx_datain_sig[(cw*7 )-1:cw*6 ]}),.outclk(tx_clkout_sig[6 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*7 )-1:tw*6 ],tx_datain_sig_txepcf[(cw*7 )-1:cw*6 ]}));
		if	(serial_channels >=8 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_8 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*8 )-1:tw*7 ],tx_datain_sig[(cw*8 )-1:cw*7 ]}),.outclk(tx_clkout_sig[7 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*8 )-1:tw*7 ],tx_datain_sig_txepcf[(cw*8 )-1:cw*7 ]}));
		if	(serial_channels >=9 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_9 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*9 )-1:tw*8 ],tx_datain_sig[(cw*9 )-1:cw*8 ]}),.outclk(tx_clkout_sig[8 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*9 )-1:tw*8 ],tx_datain_sig_txepcf[(cw*9 )-1:cw*8 ]}));
		if	(serial_channels >=10) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_10 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*10)-1:tw*9 ],tx_datain_sig[(cw*10)-1:cw*9 ]}),.outclk(tx_clkout_sig[9 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*10)-1:tw*9 ],tx_datain_sig_txepcf[(cw*10)-1:cw*9 ]}));


	
	end    
endgenerate



// GT x4 10+Gbps Basic Mode XCVR Optical Demo Board		
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT_ODB_OPNEXT_ALTGT_FIFOS") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx | {~avalon_dip[3],~avalon_dip[2],~avalon_dip[1],~avalon_dip[0]}),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
/*	 
		always	@(*)
		begin
			casex	(avalon_dip[3:0])
				4'bxxx1:	rx_pld_clk_w	=	rx_clkout_sig[0];
				4'bxx10:	rx_pld_clk_w	=	rx_clkout_sig[1];
				4'bx100:	rx_pld_clk_w	=	rx_clkout_sig[2];
				4'b1000:	rx_pld_clk_w	=	rx_clkout_sig[3];
				default:	rx_pld_clk_w	=	rx_clkout_sig[0];
			endcase
		end
*/		
		always @(*)
		begin
			rx_pld_clk_w	=	rx_clkout_sig[0]; // rx_clkout_sig[0];
		end
	
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_pld_clk_w;	// rx_clkout_sig[0];

		assign rx_dataout_sig = (opnext_loopback == 0) ? rx_dataout_sig_0 : ~rx_dataout_sig_0;
		assign	opnext_loopback	=	seriallpbken_dip[1];
		
		altgx_s4gt_basic_x4_10G_gt_host	altgx_s4gt_basic_x4_10G_gt_host_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( {serial_channels{seriallpbken_dip[0]}} | avalon_seriallpbken ), // Let avalon control serial loopback
			.tx_datain ( tx_datain_sig), // ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig_0 ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p ),
			.rx_coreclk ( {serial_channels{rx_pld_clk}} ),
			.tx_coreclk ( {serial_channels{tx_pld_clk}} ),
			
		);

		altgx_s4gt_xcvr_reconfig_x4	altgx_s4gt_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);
		
		
		assign	aci_edm_dip	=	{	// Mask out disabled channels
									{channel_width{~avalon_dip[3]}},
									{channel_width{~avalon_dip[2]}},
									{channel_width{~avalon_dip[1]}},
									{channel_width{~avalon_dip[0]}}
								};
								
		assign	aci_sdm_dip	=	{	// Mask out disabled channels
									{channel_width{~avalon_dip[3]}},
									{channel_width{~avalon_dip[2]}},
									{channel_width{~avalon_dip[1]}},
									{channel_width{~avalon_dip[0]}}
								};		
		
		
//	assign	busy_sig	=	0;

/*
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
*/
	
	end    
endgenerate

// GT x4 10+Gbps Basic Mode XCVR Optical Demo Board - RX FIFO in the FPGA Fabric
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT_ODB_OPNEXT_FPGA_FIFO") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(	rx_freqlocked_altgx | 
											//{~avalon_dip[3],~avalon_dip[2],~avalon_dip[1],~avalon_dip[0]} |
											rxpll_disable_w[serial_channels-1:0]
										),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
	 
		always	@(*)
		begin
			casex	(rxpll_disable_w[serial_channels-1:0])	// (avalon_dip[3:0])
				4'bxxx1:	rx_pld_clk_w	=	rx_clkout_sig[0];
				4'bxx10:	rx_pld_clk_w	=	rx_clkout_sig[1];
				4'bx100:	rx_pld_clk_w	=	rx_clkout_sig[2];
				4'b1000:	rx_pld_clk_w	=	rx_clkout_sig[3];
				default:	rx_pld_clk_w	=	rx_clkout_sig[0];
			endcase
		end
	

/*	
		always @(*)
		begin
			rx_pld_clk_w	=	rx_clkout_sig[0]; // rx_clkout_sig[0];
		end
*/
	
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_pld_clk_w;	// rx_clkout_sig[0];

		assign rx_dataout_sig = (opnext_loopback == 0) ? rx_dataout_sig_0 : ~rx_dataout_sig_0;
		assign	opnext_loopback	=	seriallpbken_dip[1];
		
		altgx_s4gt_basic_x4_10G_gt_host_txfifo	altgx_s4gt_basic_x4_10G_gt_host_txfifo_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( {serial_channels{seriallpbken_dip[0]}} | avalon_seriallpbken),
			.tx_datain ( tx_datain_sig), // ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig_0 ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p ),
			//.rx_coreclk ( {serial_channels{rx_pld_clk}} ),
			.tx_coreclk ( {serial_channels{tx_pld_clk}} ),
			
		);

		altgx_s4gt_xcvr_reconfig_x4	altgx_s4gt_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);
		
		
		assign	aci_edm_dip	=	{	// Mask out disabled channels
									{channel_width{~avalon_dip[3]}},
									{channel_width{~avalon_dip[2]}},
									{channel_width{~avalon_dip[1]}},
									{channel_width{~avalon_dip[0]}}
								};
								
		assign	aci_sdm_dip	=	{	// Mask out disabled channels
									{channel_width{~avalon_dip[3]}},
									{channel_width{~avalon_dip[2]}},
									{channel_width{~avalon_dip[1]}},
									{channel_width{~avalon_dip[0]}}
								};		
		// Disable rx_pll if the error detect mask and sync detect fully mask the serial channel		
		assign	rxpll_disable	=
						{
							((&aci_edm[(channel_width-1)+(channel_width*3):0+(channel_width*3)]) 	&
							 (&aci_sdm[(channel_width-1)+(channel_width*3):0+(channel_width*3)])),
							((&aci_edm[(channel_width-1)+(channel_width*2):0+(channel_width*2)]) 	&
							 (&aci_sdm[(channel_width-1)+(channel_width*2):0+(channel_width*2)])),
							((&aci_edm[(channel_width-1)+(channel_width*1):0+(channel_width*1)]) 	&
							 (&aci_sdm[(channel_width-1)+(channel_width*1):0+(channel_width*1)])),
							((&aci_edm[(channel_width-1)+(channel_width*0):0+(channel_width*0)]) 	&
							 (&aci_sdm[(channel_width-1)+(channel_width*0):0+(channel_width*0)]))
						};
												
		// User Dipsw will fully mask a channel OR NIOS can fully mask the sdm and edm which will disable the respect rxpll checks
		assign	rxpll_disable_w	=	rxpll_disable[serial_channels-1:0] |
												{~avalon_dip[serial_channels-1:0]};
		
		
//	assign	busy_sig	=	0;

/*
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
*/
	
	end    
endgenerate




// GT x4 10+Gbps Basic Mode XCVR Optical Demo Board		
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT_ODB_OPNEXT") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		assign rx_dataout_sig = (opnext_loopback == 0) ? rx_dataout_sig_0 : ~rx_dataout_sig_0;
		assign	opnext_loopback	=	seriallpbken_dip[1];
		
		altgx_s4gt_basic_x4_10G_odb	altgx_s4gt_basic_x4_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( {serial_channels{seriallpbken_dip[0]}} ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig_0 ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x4	altgx_s4gt_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
	
	end    
endgenerate



// GT x3 10+Gbps Basic Mode XCVR Optical Demo Board		
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT_ODB") && (serial_channels == 3))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_basic_x3_10G_odb	altgx_s4gt_basic_x3_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x3	altgx_s4gt_xcvr_reconfig_x3_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
	
	end    
endgenerate


// GT x2 10G+bps Basic Mode XCVR Optical Demo Board		
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT_ODB") && (serial_channels == 2))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_basic_x2_10G_odb	altgx_s4gt_basic_x2_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x2	altgx_s4gt_xcvr_reconfig_x2_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
	
	end    
endgenerate

wire	opnext_loopback;
// GT x2 10G+bps Basic Mode XCVR Optical Demo Board		
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT_ODB_OPNEXT") && (serial_channels == 2))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];
		
		assign rx_dataout_sig 	= 	(opnext_loopback == 0) ? rx_dataout_sig_0 : ~rx_dataout_sig_0;
		assign	opnext_loopback	=	seriallpbken_dip[1];

		altgx_s4gt_basic_x2_10G_odb	altgx_s4gt_basic_x2_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( {serial_channels{seriallpbken_dip[0]}} ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig_0 ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x2	altgx_s4gt_xcvr_reconfig_x2_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
	
	end    
endgenerate

// GT x1 10+Gbps Basic Mode XCVR Optical Demo Board	
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT_ODB") && (serial_channels == 1))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];


		altgx_s4gt_basic_x1_10G_odb	altgx_s4gt_basic_x1_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( 0 ),
			.reconfig_togxb ( 0 ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);
		assign	busy_sig	=	0;

/*
		altgx_s4gt_basic_x1_10G	altgx_s4gt_basic_x1_10G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x1	altgx_s4gt_xcvr_reconfig_x1_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);
*/

		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));	
	end    
endgenerate





// GT x4 6.375Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT_6P375") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_basic_x4_6p375G	altgx_s4gt_basic_x4_6p375G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x4	altgx_s4gt_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
	
	end    
endgenerate

// GT x1 6.375Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR_GT_6P375") && (serial_channels == 1))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_basic_x1_6p375G	altgx_s4gt_basic_x1_6p375G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gt_xcvr_reconfig_x1	altgx_s4gt_xcvr_reconfig_x1_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);
		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));	
	end    
endgenerate



// GT x20 Interlaken in PMA Direct Mode		
generate 
	if	((component_type == "INTERLAKEN_PMA_DIRECT_GT_XN") && (serial_channels == 20))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_pma_direct_xn	altgx_s4gt_pma_direct_xn_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			//.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			//.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig_0 ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig_0 ),
			.tx_dataout ( tx_p )
		);      

// Use fewer clocks		
		assign	rx_clkout_sig[0]	=	rx_clkout_sig_0[2];
		assign	rx_clkout_sig[1]	=	rx_clkout_sig_0[2];
		assign	rx_clkout_sig[2]	=	rx_clkout_sig_0[2];
		assign	rx_clkout_sig[3]	=	rx_clkout_sig_0[2];
		assign	rx_clkout_sig[4]	=	rx_clkout_sig_0[2];

		assign	rx_clkout_sig[5]	=	rx_clkout_sig_0[7];
		assign	rx_clkout_sig[6]	=	rx_clkout_sig_0[7];
		assign	rx_clkout_sig[7]	=	rx_clkout_sig_0[7];
		assign	rx_clkout_sig[8]	=	rx_clkout_sig_0[7];
		assign	rx_clkout_sig[9]	=	rx_clkout_sig_0[7];
		
		assign	rx_clkout_sig[10]	=	rx_clkout_sig_0[12];
		assign	rx_clkout_sig[11]	=	rx_clkout_sig_0[12];
		assign	rx_clkout_sig[12]	=	rx_clkout_sig_0[12];
		assign	rx_clkout_sig[13]	=	rx_clkout_sig_0[12];
		assign	rx_clkout_sig[14]	=	rx_clkout_sig_0[12];

		assign	rx_clkout_sig[15]	=	rx_clkout_sig_0[17];
		assign	rx_clkout_sig[16]	=	rx_clkout_sig_0[17];
		assign	rx_clkout_sig[17]	=	rx_clkout_sig_0[17];
		assign	rx_clkout_sig[18]	=	rx_clkout_sig_0[17];
		assign	rx_clkout_sig[19]	=	rx_clkout_sig_0[17];
		
		assign	tx_clkout_sig[0]	=	tx_clkout_sig_0[2];
		assign	tx_clkout_sig[1]	=	tx_clkout_sig_0[2];
		assign	tx_clkout_sig[2]	=	tx_clkout_sig_0[2];
		assign	tx_clkout_sig[3]	=	tx_clkout_sig_0[2];
		assign	tx_clkout_sig[4]	=	tx_clkout_sig_0[2];

		assign	tx_clkout_sig[5]	=	tx_clkout_sig_0[7];
		assign	tx_clkout_sig[6]	=	tx_clkout_sig_0[7];
		assign	tx_clkout_sig[7]	=	tx_clkout_sig_0[7];
		assign	tx_clkout_sig[8]	=	tx_clkout_sig_0[7];
		assign	tx_clkout_sig[9]	=	tx_clkout_sig_0[7];
		
		assign	tx_clkout_sig[10]	=	tx_clkout_sig_0[12];
		assign	tx_clkout_sig[11]	=	tx_clkout_sig_0[12];
		assign	tx_clkout_sig[12]	=	tx_clkout_sig_0[12];
		assign	tx_clkout_sig[13]	=	tx_clkout_sig_0[12];
		assign	tx_clkout_sig[14]	=	tx_clkout_sig_0[12];

		assign	tx_clkout_sig[15]	=	tx_clkout_sig_0[17];
		assign	tx_clkout_sig[16]	=	tx_clkout_sig_0[17];
		assign	tx_clkout_sig[17]	=	tx_clkout_sig_0[17];
		assign	tx_clkout_sig[18]	=	tx_clkout_sig_0[17];
		assign	tx_clkout_sig[19]	=	tx_clkout_sig_0[17];		
		     
		altgx_s4gt_reconfig_pma_direct	altgx_s4gt_reconfig_pma_direct_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
		if	(serial_channels >=6 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_6 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*6 )-1:tw*5 ],tx_datain_sig[(cw*6 )-1:cw*5 ]}),.outclk(tx_clkout_sig[5 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*6 )-1:tw*5 ],tx_datain_sig_txepcf[(cw*6 )-1:cw*5 ]}));
		if	(serial_channels >=7 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_7 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*7 )-1:tw*6 ],tx_datain_sig[(cw*7 )-1:cw*6 ]}),.outclk(tx_clkout_sig[6 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*7 )-1:tw*6 ],tx_datain_sig_txepcf[(cw*7 )-1:cw*6 ]}));
		if	(serial_channels >=8 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_8 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*8 )-1:tw*7 ],tx_datain_sig[(cw*8 )-1:cw*7 ]}),.outclk(tx_clkout_sig[7 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*8 )-1:tw*7 ],tx_datain_sig_txepcf[(cw*8 )-1:cw*7 ]}));
		if	(serial_channels >=9 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_9 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*9 )-1:tw*8 ],tx_datain_sig[(cw*9 )-1:cw*8 ]}),.outclk(tx_clkout_sig[8 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*9 )-1:tw*8 ],tx_datain_sig_txepcf[(cw*9 )-1:cw*8 ]}));
		if	(serial_channels >=10) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_10 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*10)-1:tw*9 ],tx_datain_sig[(cw*10)-1:cw*9 ]}),.outclk(tx_clkout_sig[9 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*10)-1:tw*9 ],tx_datain_sig_txepcf[(cw*10)-1:cw*9 ]}));
		if	(serial_channels >=11) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_11	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*11)-1:tw*10],tx_datain_sig[(cw*11)-1:cw*10]}),.outclk(tx_clkout_sig[10]),.dataout({tx_ctrlenable_sig_txepcf[(tw*11)-1:tw*10],tx_datain_sig_txepcf[(cw*11)-1:cw*10]}));
		if	(serial_channels >=12) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_12	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*12)-1:tw*11],tx_datain_sig[(cw*12)-1:cw*11]}),.outclk(tx_clkout_sig[11]),.dataout({tx_ctrlenable_sig_txepcf[(tw*12)-1:tw*11],tx_datain_sig_txepcf[(cw*12)-1:cw*11]}));
		if	(serial_channels >=13) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_13	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*13)-1:tw*12],tx_datain_sig[(cw*13)-1:cw*12]}),.outclk(tx_clkout_sig[12]),.dataout({tx_ctrlenable_sig_txepcf[(tw*13)-1:tw*12],tx_datain_sig_txepcf[(cw*13)-1:cw*12]}));
		if	(serial_channels >=14) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_14	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*14)-1:tw*13],tx_datain_sig[(cw*14)-1:cw*13]}),.outclk(tx_clkout_sig[13]),.dataout({tx_ctrlenable_sig_txepcf[(tw*14)-1:tw*13],tx_datain_sig_txepcf[(cw*14)-1:cw*13]}));
		if	(serial_channels >=15) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_15	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*15)-1:tw*14],tx_datain_sig[(cw*15)-1:cw*14]}),.outclk(tx_clkout_sig[14]),.dataout({tx_ctrlenable_sig_txepcf[(tw*15)-1:tw*14],tx_datain_sig_txepcf[(cw*15)-1:cw*14]}));
		if	(serial_channels >=16) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_16	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*16)-1:tw*15],tx_datain_sig[(cw*16)-1:cw*15]}),.outclk(tx_clkout_sig[15]),.dataout({tx_ctrlenable_sig_txepcf[(tw*16)-1:tw*15],tx_datain_sig_txepcf[(cw*16)-1:cw*15]}));
		if	(serial_channels >=17) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_17	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*17)-1:tw*16],tx_datain_sig[(cw*17)-1:cw*16]}),.outclk(tx_clkout_sig[16]),.dataout({tx_ctrlenable_sig_txepcf[(tw*17)-1:tw*16],tx_datain_sig_txepcf[(cw*17)-1:cw*16]}));
		if	(serial_channels >=18) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_18	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*18)-1:tw*17],tx_datain_sig[(cw*18)-1:cw*17]}),.outclk(tx_clkout_sig[17]),.dataout({tx_ctrlenable_sig_txepcf[(tw*18)-1:tw*17],tx_datain_sig_txepcf[(cw*18)-1:cw*17]}));
		if	(serial_channels >=19) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_19	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*19)-1:tw*18],tx_datain_sig[(cw*19)-1:cw*18]}),.outclk(tx_clkout_sig[18]),.dataout({tx_ctrlenable_sig_txepcf[(tw*19)-1:tw*18],tx_datain_sig_txepcf[(cw*19)-1:cw*18]}));
		if	(serial_channels >=20) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_20	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*20)-1:tw*19],tx_datain_sig[(cw*20)-1:cw*19]}),.outclk(tx_clkout_sig[19]),.dataout({tx_ctrlenable_sig_txepcf[(tw*20)-1:tw*19],tx_datain_sig_txepcf[(cw*20)-1:cw*19]}));

	
	end    
endgenerate


// GT x20 Interlaken in PMA Direct Mode		
generate 
	if	((component_type == "INTERLAKEN_PMA_DIRECT_GT") && (serial_channels == 20))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_pma_direct	altgx_s4gt_pma_direct_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			//.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			//.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig_0 ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig_0 ),
			.tx_dataout ( tx_p )
		);      

// Use fewer clocks		
		assign	rx_clkout_sig[0]	=	rx_clkout_sig_0[2];
		assign	rx_clkout_sig[1]	=	rx_clkout_sig_0[2];
		assign	rx_clkout_sig[2]	=	rx_clkout_sig_0[2];
		assign	rx_clkout_sig[3]	=	rx_clkout_sig_0[2];
		assign	rx_clkout_sig[4]	=	rx_clkout_sig_0[2];

		assign	rx_clkout_sig[5]	=	rx_clkout_sig_0[7];
		assign	rx_clkout_sig[6]	=	rx_clkout_sig_0[7];
		assign	rx_clkout_sig[7]	=	rx_clkout_sig_0[7];
		assign	rx_clkout_sig[8]	=	rx_clkout_sig_0[7];
		assign	rx_clkout_sig[9]	=	rx_clkout_sig_0[7];
		
		assign	rx_clkout_sig[10]	=	rx_clkout_sig_0[12];
		assign	rx_clkout_sig[11]	=	rx_clkout_sig_0[12];
		assign	rx_clkout_sig[12]	=	rx_clkout_sig_0[12];
		assign	rx_clkout_sig[13]	=	rx_clkout_sig_0[12];
		assign	rx_clkout_sig[14]	=	rx_clkout_sig_0[12];

		assign	rx_clkout_sig[15]	=	rx_clkout_sig_0[17];
		assign	rx_clkout_sig[16]	=	rx_clkout_sig_0[17];
		assign	rx_clkout_sig[17]	=	rx_clkout_sig_0[17];
		assign	rx_clkout_sig[18]	=	rx_clkout_sig_0[17];
		assign	rx_clkout_sig[19]	=	rx_clkout_sig_0[17];
		
		assign	tx_clkout_sig[0]	=	tx_clkout_sig_0[2];
		assign	tx_clkout_sig[1]	=	tx_clkout_sig_0[2];
		assign	tx_clkout_sig[2]	=	tx_clkout_sig_0[2];
		assign	tx_clkout_sig[3]	=	tx_clkout_sig_0[2];
		assign	tx_clkout_sig[4]	=	tx_clkout_sig_0[2];

		assign	tx_clkout_sig[5]	=	tx_clkout_sig_0[7];
		assign	tx_clkout_sig[6]	=	tx_clkout_sig_0[7];
		assign	tx_clkout_sig[7]	=	tx_clkout_sig_0[7];
		assign	tx_clkout_sig[8]	=	tx_clkout_sig_0[7];
		assign	tx_clkout_sig[9]	=	tx_clkout_sig_0[7];
		
		assign	tx_clkout_sig[10]	=	tx_clkout_sig_0[12];
		assign	tx_clkout_sig[11]	=	tx_clkout_sig_0[12];
		assign	tx_clkout_sig[12]	=	tx_clkout_sig_0[12];
		assign	tx_clkout_sig[13]	=	tx_clkout_sig_0[12];
		assign	tx_clkout_sig[14]	=	tx_clkout_sig_0[12];

		assign	tx_clkout_sig[15]	=	tx_clkout_sig_0[17];
		assign	tx_clkout_sig[16]	=	tx_clkout_sig_0[17];
		assign	tx_clkout_sig[17]	=	tx_clkout_sig_0[17];
		assign	tx_clkout_sig[18]	=	tx_clkout_sig_0[17];
		assign	tx_clkout_sig[19]	=	tx_clkout_sig_0[17];		
		     
		altgx_s4gt_reconfig_pma_direct	altgx_s4gt_reconfig_pma_direct_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
		if	(serial_channels >=6 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_6 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*6 )-1:tw*5 ],tx_datain_sig[(cw*6 )-1:cw*5 ]}),.outclk(tx_clkout_sig[5 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*6 )-1:tw*5 ],tx_datain_sig_txepcf[(cw*6 )-1:cw*5 ]}));
		if	(serial_channels >=7 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_7 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*7 )-1:tw*6 ],tx_datain_sig[(cw*7 )-1:cw*6 ]}),.outclk(tx_clkout_sig[6 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*7 )-1:tw*6 ],tx_datain_sig_txepcf[(cw*7 )-1:cw*6 ]}));
		if	(serial_channels >=8 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_8 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*8 )-1:tw*7 ],tx_datain_sig[(cw*8 )-1:cw*7 ]}),.outclk(tx_clkout_sig[7 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*8 )-1:tw*7 ],tx_datain_sig_txepcf[(cw*8 )-1:cw*7 ]}));
		if	(serial_channels >=9 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_9 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*9 )-1:tw*8 ],tx_datain_sig[(cw*9 )-1:cw*8 ]}),.outclk(tx_clkout_sig[8 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*9 )-1:tw*8 ],tx_datain_sig_txepcf[(cw*9 )-1:cw*8 ]}));
		if	(serial_channels >=10) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_10 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*10)-1:tw*9 ],tx_datain_sig[(cw*10)-1:cw*9 ]}),.outclk(tx_clkout_sig[9 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*10)-1:tw*9 ],tx_datain_sig_txepcf[(cw*10)-1:cw*9 ]}));
		if	(serial_channels >=11) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_11	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*11)-1:tw*10],tx_datain_sig[(cw*11)-1:cw*10]}),.outclk(tx_clkout_sig[10]),.dataout({tx_ctrlenable_sig_txepcf[(tw*11)-1:tw*10],tx_datain_sig_txepcf[(cw*11)-1:cw*10]}));
		if	(serial_channels >=12) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_12	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*12)-1:tw*11],tx_datain_sig[(cw*12)-1:cw*11]}),.outclk(tx_clkout_sig[11]),.dataout({tx_ctrlenable_sig_txepcf[(tw*12)-1:tw*11],tx_datain_sig_txepcf[(cw*12)-1:cw*11]}));
		if	(serial_channels >=13) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_13	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*13)-1:tw*12],tx_datain_sig[(cw*13)-1:cw*12]}),.outclk(tx_clkout_sig[12]),.dataout({tx_ctrlenable_sig_txepcf[(tw*13)-1:tw*12],tx_datain_sig_txepcf[(cw*13)-1:cw*12]}));
		if	(serial_channels >=14) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_14	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*14)-1:tw*13],tx_datain_sig[(cw*14)-1:cw*13]}),.outclk(tx_clkout_sig[13]),.dataout({tx_ctrlenable_sig_txepcf[(tw*14)-1:tw*13],tx_datain_sig_txepcf[(cw*14)-1:cw*13]}));
		if	(serial_channels >=15) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_15	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*15)-1:tw*14],tx_datain_sig[(cw*15)-1:cw*14]}),.outclk(tx_clkout_sig[14]),.dataout({tx_ctrlenable_sig_txepcf[(tw*15)-1:tw*14],tx_datain_sig_txepcf[(cw*15)-1:cw*14]}));
		if	(serial_channels >=16) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_16	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*16)-1:tw*15],tx_datain_sig[(cw*16)-1:cw*15]}),.outclk(tx_clkout_sig[15]),.dataout({tx_ctrlenable_sig_txepcf[(tw*16)-1:tw*15],tx_datain_sig_txepcf[(cw*16)-1:cw*15]}));
		if	(serial_channels >=17) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_17	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*17)-1:tw*16],tx_datain_sig[(cw*17)-1:cw*16]}),.outclk(tx_clkout_sig[16]),.dataout({tx_ctrlenable_sig_txepcf[(tw*17)-1:tw*16],tx_datain_sig_txepcf[(cw*17)-1:cw*16]}));
		if	(serial_channels >=18) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_18	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*18)-1:tw*17],tx_datain_sig[(cw*18)-1:cw*17]}),.outclk(tx_clkout_sig[17]),.dataout({tx_ctrlenable_sig_txepcf[(tw*18)-1:tw*17],tx_datain_sig_txepcf[(cw*18)-1:cw*17]}));
		if	(serial_channels >=19) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_19	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*19)-1:tw*18],tx_datain_sig[(cw*19)-1:cw*18]}),.outclk(tx_clkout_sig[18]),.dataout({tx_ctrlenable_sig_txepcf[(tw*19)-1:tw*18],tx_datain_sig_txepcf[(cw*19)-1:cw*18]}));
		if	(serial_channels >=20) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_20	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*20)-1:tw*19],tx_datain_sig[(cw*20)-1:cw*19]}),.outclk(tx_clkout_sig[19]),.dataout({tx_ctrlenable_sig_txepcf[(tw*20)-1:tw*19],tx_datain_sig_txepcf[(cw*20)-1:cw*19]}));

	
	end    
endgenerate

// GT x5 Interlaken in PMA Direct Mode		
generate 
	if	((component_type == "INTERLAKEN_PMA_DIRECT_GT") && (serial_channels == 5))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_pma_direct_x5	altgx_s4gt_pma_direct_x5_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			//.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf_0 ),
			//.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig_0 ),
			.rx_dataout ( rx_dataout_sig_0 ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig_0 ),
			.tx_dataout ( tx_p )
		);      
		
		     
		altgx_s4gt_reconfig_pma_direct_x5	altgx_s4gt_reconfig_pma_direct_x5_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		// All RX clocks required
		assign	rx_clkout_sig[0]	=	rx_clkout_sig_0[0];
		assign	rx_clkout_sig[1]	=	rx_clkout_sig_0[1];
		assign	rx_clkout_sig[2]	=	rx_clkout_sig_0[2];
		assign	rx_clkout_sig[3]	=	rx_clkout_sig_0[3];
		assign	rx_clkout_sig[4]	=	rx_clkout_sig_0[4];
		

		assign	tx_clkout_sig[0]	=	tx_clkout_sig_0[0];
		assign	tx_clkout_sig[1]	=	tx_clkout_sig_0[1];
		assign	tx_clkout_sig[2]	=	tx_clkout_sig_0[2];
		assign	tx_clkout_sig[3]	=	tx_clkout_sig_0[3];
		assign	tx_clkout_sig[4]	=	tx_clkout_sig_0[4];

	/*
		// 1 TX clock per block required, so don't use a tx phase comp fifo
		// all can be   tx_clkout_sig_0[0]:	
		assign	tx_clkout_sig[0]	=	tx_clkout_sig_0[0];
		assign	tx_clkout_sig[1]	=	tx_clkout_sig_0[0];
		assign	tx_clkout_sig[2]	=	tx_clkout_sig_0[0];
		assign	tx_clkout_sig[3]	=	tx_clkout_sig_0[0];
		assign	tx_clkout_sig[4]	=	tx_clkout_sig_0[0];
	*/
	
	assign	launch_edge_rx	=	1'b0; // avalon_dip[5];	// 1=+edge 0=-edge // avalon_dip[5];	//	may be remapped from avalon_dip[1];	// 1'b1
	assign	launch_edge_tx	=	1'b0; // avalon_dip[6];	// 1=+edge 0=-edge // avalon_dip[6];	// may be remapped // 1'b0
	
	// launch edge = 0, negative edge
	// launch edge = 1, positive edge
	defparam	retimer_ff_rx0.data_width		=	channel_width;
	defparam	retimer_ff_rx0.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_rx0
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[0]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(1*channel_width)-1:(0*channel_width)]),
		.dataout		(rx_dataout_sig[(1*channel_width)-1:(0*channel_width)])
	); 

	defparam	retimer_ff_rx1.data_width		=	channel_width;
	defparam	retimer_ff_rx1.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_rx1
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[1]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(2*channel_width)-1:(1*channel_width)]),
		.dataout		(rx_dataout_sig[(2*channel_width)-1:(1*channel_width)])
	); 	

	defparam	retimer_ff_rx2.data_width		=	channel_width;
	defparam	retimer_ff_rx2.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_rx2
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[2]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(3*channel_width)-1:(2*channel_width)]),
		.dataout		(rx_dataout_sig[(3*channel_width)-1:(2*channel_width)])
	); 
		
	defparam	retimer_ff_rx3.data_width		=	channel_width;
	defparam	retimer_ff_rx3.reset_value	=	1'b0;					
	retimer_ff		retimer_ff_rx3
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[3]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(4*channel_width)-1:(3*channel_width)]),
		.dataout		(rx_dataout_sig[(4*channel_width)-1:(3*channel_width)])
	); 

	defparam	retimer_ff_rx4.data_width		=	channel_width;
	defparam	retimer_ff_rx4.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_rx4
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[4]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(5*channel_width)-1:(4*channel_width)]),
		.dataout		(rx_dataout_sig[(5*channel_width)-1:(4*channel_width)])
	); 		


	defparam	retimer_ff_tx0.data_width		=	channel_width;
	defparam	retimer_ff_tx0.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx0
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[0]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(1*channel_width)-1:(0*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(1*channel_width)-1:(0*channel_width)])
	); 

	defparam	retimer_ff_tx1.data_width		=	channel_width;
	defparam	retimer_ff_tx1.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx1
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[1]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(2*channel_width)-1:(1*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(2*channel_width)-1:(1*channel_width)])
	); 
	
	defparam	retimer_ff_tx2.data_width		=	channel_width;
	defparam	retimer_ff_tx2.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx2
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[2]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(3*channel_width)-1:(2*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(3*channel_width)-1:(2*channel_width)])
	); 
	
	defparam	retimer_ff_tx3.data_width		=	channel_width;
	defparam	retimer_ff_tx3.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx3
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[3]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(4*channel_width)-1:(3*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(4*channel_width)-1:(3*channel_width)])
	); 

	defparam	retimer_ff_tx4.data_width		=	channel_width;
	defparam	retimer_ff_tx4.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx4
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[4]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(5*channel_width)-1:(4*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(5*channel_width)-1:(4*channel_width)])
	); 

/*
	defparam	retimer_ff_tx01234.data_width		=	serial_channels*channel_width;
	defparam	retimer_ff_tx01234.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx01234
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_pld_clk),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig),
		.dataout		(tx_datain_sig_txepcf)
	); 
*/
	


		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));

	
	end    
endgenerate



// GT x5 Interlaken in PMA Direct Mode		
generate 
	if	((component_type == "INTERLAKEN_PMA_DIRECT_GT_NOTXFIFO") && (serial_channels == 5))
	begin  

		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_pma_direct_x5	altgx_s4gt_pma_direct_x5_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			//.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			//.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig_0 ),
			.rx_dataout ( rx_dataout_sig_0 ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig_0 ),
			.tx_dataout ( tx_p )
		);      
		
		     
		altgx_s4gt_reconfig_pma_direct_x5	altgx_s4gt_reconfig_pma_direct_x5_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		// All RX clocks required
		assign	rx_clkout_sig[0]	=	rx_clkout_sig_0[0];
		assign	rx_clkout_sig[1]	=	rx_clkout_sig_0[1];
		assign	rx_clkout_sig[2]	=	rx_clkout_sig_0[2];
		assign	rx_clkout_sig[3]	=	rx_clkout_sig_0[3];
		assign	rx_clkout_sig[4]	=	rx_clkout_sig_0[4];
		
		// 1 TX clock per block required, so don't use a tx phase comp fifo
		// all can be   tx_clkout_sig_0[0]:
	/*
		assign	tx_clkout_sig[0]	=	tx_clkout_sig_0[0];
		assign	tx_clkout_sig[1]	=	tx_clkout_sig_0[1];
		assign	tx_clkout_sig[2]	=	tx_clkout_sig_0[2];
		assign	tx_clkout_sig[3]	=	tx_clkout_sig_0[3];
		assign	tx_clkout_sig[4]	=	tx_clkout_sig_0[4];
	*/

		assign	tx_clkout_sig[0]	=	tx_clkout_sig_0[0];
		assign	tx_clkout_sig[1]	=	tx_clkout_sig_0[0];
		assign	tx_clkout_sig[2]	=	tx_clkout_sig_0[0];
		assign	tx_clkout_sig[3]	=	tx_clkout_sig_0[0];
		assign	tx_clkout_sig[4]	=	tx_clkout_sig_0[0];
	
	
	assign	launch_edge_rx	=	0; // avalon_dip[5];	// 1=+edge 0=-edge // avalon_dip[5];	//	may be remapped from avalon_dip[1];	// 1'b1
	assign	launch_edge_tx	=	0; // avalon_dip[6];	// 1=+edge 0=-edge // avalon_dip[6];	// may be remapped // 1'b0
	
		
	// launch edge = 0, negative edge
	// launch edge = 1, positive edge
	defparam	retimer_ff_rx0.data_width		=	channel_width;
	defparam	retimer_ff_rx0.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_rx0
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[0]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(1*channel_width)-1:(0*channel_width)]),
		.dataout		(rx_dataout_sig[(1*channel_width)-1:(0*channel_width)])
	); 

	defparam	retimer_ff_rx1.data_width		=	channel_width;
	defparam	retimer_ff_rx1.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_rx1
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[1]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(2*channel_width)-1:(1*channel_width)]),
		.dataout		(rx_dataout_sig[(2*channel_width)-1:(1*channel_width)])
	); 	

	defparam	retimer_ff_rx2.data_width		=	channel_width;
	defparam	retimer_ff_rx2.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_rx2
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[2]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(3*channel_width)-1:(2*channel_width)]),
		.dataout		(rx_dataout_sig[(3*channel_width)-1:(2*channel_width)])
	); 
		
	defparam	retimer_ff_rx3.data_width		=	channel_width;
	defparam	retimer_ff_rx3.reset_value	=	1'b0;					
	retimer_ff		retimer_ff_rx3
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[3]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(4*channel_width)-1:(3*channel_width)]),
		.dataout		(rx_dataout_sig[(4*channel_width)-1:(3*channel_width)])
	); 

	defparam	retimer_ff_rx4.data_width		=	channel_width;
	defparam	retimer_ff_rx4.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_rx4
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[4]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(5*channel_width)-1:(4*channel_width)]),
		.dataout		(rx_dataout_sig[(5*channel_width)-1:(4*channel_width)])
	); 		

	// launch edge = 0, negative edge
	// launch edge = 1, positive edge
/*
	defparam	retimer_ff_tx0.data_width		=	channel_width;
	defparam	retimer_ff_tx0.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx0
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[0]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(1*channel_width)-1:(0*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(1*channel_width)-1:(0*channel_width)])
	); 

	defparam	retimer_ff_tx1.data_width		=	channel_width;
	defparam	retimer_ff_tx1.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx1
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[1]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(2*channel_width)-1:(1*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(2*channel_width)-1:(1*channel_width)])
	); 
	
	defparam	retimer_ff_tx2.data_width		=	channel_width;
	defparam	retimer_ff_tx2.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx2
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[2]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(3*channel_width)-1:(2*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(3*channel_width)-1:(2*channel_width)])
	); 
	
	defparam	retimer_ff_tx3.data_width		=	channel_width;
	defparam	retimer_ff_tx3.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx3
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[3]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(4*channel_width)-1:(3*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(4*channel_width)-1:(3*channel_width)])
	); 

	defparam	retimer_ff_tx4.data_width		=	channel_width;
	defparam	retimer_ff_tx4.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx4
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[4]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(5*channel_width)-1:(4*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(5*channel_width)-1:(4*channel_width)])
	); 
*/	

	defparam	retimer_ff_tx01234.data_width		=	serial_channels*channel_width;
	defparam	retimer_ff_tx01234.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx01234
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_pld_clk),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig),
		.dataout		(tx_datain_sig_txepcf)
	); 

	

/*
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
*/
	
	end    
endgenerate



/*
// GT x5 Interlaken in PMA Direct Mode	-- full TX FIFOs
generate 
	if	((component_type == "INTERLAKEN_PMA_DIRECT_GT") && (serial_channels == 5))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gt_pma_direct_x5	altgx_s4gt_pma_direct_x5_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			//.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf_0 ),
			//.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig_0 ),
			.rx_dataout ( rx_dataout_sig_0 ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig_0 ),
			.tx_dataout ( tx_p )
		);      
		
		     
		altgx_s4gt_reconfig_pma_direct_x5	altgx_s4gt_reconfig_pma_direct_x5_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		// All RX clocks required
		assign	rx_clkout_sig[0]	=	rx_clkout_sig_0[0];
		assign	rx_clkout_sig[1]	=	rx_clkout_sig_0[1];
		assign	rx_clkout_sig[2]	=	rx_clkout_sig_0[2];
		assign	rx_clkout_sig[3]	=	rx_clkout_sig_0[3];
		assign	rx_clkout_sig[4]	=	rx_clkout_sig_0[4];
		
		// 1 TX clock per block required, so don't use a tx phase comp fifo
		// all can be   tx_clkout_sig_0[0]:
		assign	tx_clkout_sig[0]	=	tx_clkout_sig_0[0];
		assign	tx_clkout_sig[1]	=	tx_clkout_sig_0[1];
		assign	tx_clkout_sig[2]	=	tx_clkout_sig_0[2];
		assign	tx_clkout_sig[3]	=	tx_clkout_sig_0[3];
		assign	tx_clkout_sig[4]	=	tx_clkout_sig_0[4];

	assign	launch_edge_rx	=	1'b1; // avalon_dip[1];
	assign	launch_edge_tx	=	1'b1; // avalon_dip[6];
	
		
	// launch edge = 0, negative edge
	// launch edge = 1, positive edge
	defparam	retimer_ff_rx0.data_width		=	channel_width;
	defparam	retimer_ff_rx0.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_rx0
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[0]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(1*channel_width)-1:(0*channel_width)]),
		.dataout		(rx_dataout_sig[(1*channel_width)-1:(0*channel_width)])
	); 
						
	defparam	retimer_ff_rx1.data_width		=	channel_width;
	defparam	retimer_ff_rx1.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_rx1
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[1]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(2*channel_width)-1:(1*channel_width)]),
		.dataout		(rx_dataout_sig[(2*channel_width)-1:(1*channel_width)])
	); 	

	defparam	retimer_ff_rx2.data_width		=	channel_width;
	defparam	retimer_ff_rx2.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_rx2
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[2]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(3*channel_width)-1:(2*channel_width)]),
		.dataout		(rx_dataout_sig[(3*channel_width)-1:(2*channel_width)])
	); 
		
	defparam	retimer_ff_rx3.data_width		=	channel_width;
	defparam	retimer_ff_rx3.reset_value	=	1'b0;					
	retimer_ff		retimer_ff_rx3
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[3]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(4*channel_width)-1:(3*channel_width)]),
		.dataout		(rx_dataout_sig[(4*channel_width)-1:(3*channel_width)])
	); 
		
	defparam	retimer_ff_rx4.data_width		=	channel_width;
	defparam	retimer_ff_rx4.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_rx4
	(
		.reset_n		(!rx_digital_reset_sync_dly),
		.clk			(rx_clkout_sig[4]),
		.launch_edge	(launch_edge_rx),
		.datain			(rx_dataout_sig_0[(5*channel_width)-1:(4*channel_width)]),
		.dataout		(rx_dataout_sig[(5*channel_width)-1:(4*channel_width)])
	); 		

	// launch edge = 0, negative edge
	// launch edge = 1, positive edge
	defparam	retimer_ff_tx0.data_width		=	channel_width;
	defparam	retimer_ff_tx0.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx0
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[0]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(1*channel_width)-1:(0*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(1*channel_width)-1:(0*channel_width)])
	); 

	defparam	retimer_ff_tx1.data_width		=	channel_width;
	defparam	retimer_ff_tx1.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx1
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[1]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(2*channel_width)-1:(1*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(2*channel_width)-1:(1*channel_width)])
	); 
	
	defparam	retimer_ff_tx2.data_width		=	channel_width;
	defparam	retimer_ff_tx2.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx2
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[2]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(3*channel_width)-1:(2*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(3*channel_width)-1:(2*channel_width)])
	); 
	
	defparam	retimer_ff_tx3.data_width		=	channel_width;
	defparam	retimer_ff_tx3.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx3
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[3]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(4*channel_width)-1:(3*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(4*channel_width)-1:(3*channel_width)])
	); 

	defparam	retimer_ff_tx4.data_width		=	channel_width;
	defparam	retimer_ff_tx4.reset_value	=	1'b0;					
	retimer_ff	retimer_ff_tx4
	(
		.reset_n		(!tx_digital_reset_sync_dly),
		.clk			(tx_clkout_sig[4]),
		.launch_edge	(launch_edge_tx),
		.datain			(tx_datain_sig_txepcf[(5*channel_width)-1:(4*channel_width)]),
		.dataout		(tx_datain_sig_txepcf_0[(5*channel_width)-1:(4*channel_width)])
	); 
	

//	defparam	retimer_ff_tx01234.data_width		=	serial_channels*channel_width;
//	defparam	retimer_ff_tx01234.reset_value	=	1'b0;					
//	retimer_ff	retimer_ff_tx01234
//	(
//		.reset_n		(!tx_digital_reset_sync_dly),
//		.clk			(!tx_pld_clk),
//		.datain			(tx_datain_sig),
//		.dataout		(tx_datain_sig_txepcf)
//	); 

	


		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));

	end    
endgenerate

*/

// HSM x1 3G S4GX FPGA Dev Kit SATA
generate 
	if	((component_type == "S4GX_HSM_XCVR_SATA") && (serial_channels == 1))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;	
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
		
		

		altgx_s4gx_basic_x1_SATA	altgx_s4gx_basic_x1_SATA_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),	
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);
		
		
		altgx_s4gx_xcvr_reconfig_x1	altgx_s4gx_xcvr_reconfig_x1_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);		
	
	end    
endgenerate

// HSM x4 3Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR_SATA") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx | {~avalon_dip[3],~avalon_dip[2],~avalon_dip[1],~avalon_dip[0]}),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gx_basic_x4_SATA	altgx_s4gx_basic_x4_SATA_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gx_xcvr_reconfig_x4	altgx_s4gx_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);
		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));

		assign	aci_edm_dip	=	{	// Mask out disabled channels
									{channel_width{~avalon_dip[3]}},
									{channel_width{~avalon_dip[2]}},
									{channel_width{~avalon_dip[1]}},
									{channel_width{~avalon_dip[0]}}
								};
								
		assign	aci_sdm_dip	=	{	// Mask out disabled channels
									{channel_width{~avalon_dip[3]}},
									{channel_width{~avalon_dip[2]}},
									{channel_width{~avalon_dip[1]}},
									{channel_width{~avalon_dip[0]}}
								};
	end
endgenerate





// HSM x14 8.5 Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR") && (serial_channels == 14))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;	
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);

		altgx_s4gx_basic_x14_8p5G	altgx_s4gx_basic_x14_8p5G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),	
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			//.rx_coreclk ( {serial_channels{rx_pld_clk}} ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			//.tx_coreclk ( {serial_channels{tx_pld_clk}} ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);

		//defparam altgx_s4gx_xcvr_reconfig_x14_inst.alt_system_interface = "true";
		altgx_s4gx_xcvr_reconfig_x14	altgx_s4gx_xcvr_reconfig_x14_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
		if	(serial_channels >=6 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_6 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*6 )-1:tw*5 ],tx_datain_sig[(cw*6 )-1:cw*5 ]}),.outclk(tx_clkout_sig[5 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*6 )-1:tw*5 ],tx_datain_sig_txepcf[(cw*6 )-1:cw*5 ]}));
		if	(serial_channels >=7 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_7 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*7 )-1:tw*6 ],tx_datain_sig[(cw*7 )-1:cw*6 ]}),.outclk(tx_clkout_sig[6 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*7 )-1:tw*6 ],tx_datain_sig_txepcf[(cw*7 )-1:cw*6 ]}));
		if	(serial_channels >=8 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_8 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*8 )-1:tw*7 ],tx_datain_sig[(cw*8 )-1:cw*7 ]}),.outclk(tx_clkout_sig[7 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*8 )-1:tw*7 ],tx_datain_sig_txepcf[(cw*8 )-1:cw*7 ]}));
		if	(serial_channels >=9 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_9 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*9 )-1:tw*8 ],tx_datain_sig[(cw*9 )-1:cw*8 ]}),.outclk(tx_clkout_sig[8 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*9 )-1:tw*8 ],tx_datain_sig_txepcf[(cw*9 )-1:cw*8 ]}));
		if	(serial_channels >=10) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_10	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*10)-1:tw*9 ],tx_datain_sig[(cw*10)-1:cw*9 ]}),.outclk(tx_clkout_sig[9 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*10)-1:tw*9 ],tx_datain_sig_txepcf[(cw*10)-1:cw*9 ]}));
		if	(serial_channels >=11) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_11	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*11)-1:tw*10],tx_datain_sig[(cw*11)-1:cw*10]}),.outclk(tx_clkout_sig[10]),.dataout({tx_ctrlenable_sig_txepcf[(tw*11)-1:tw*10],tx_datain_sig_txepcf[(cw*11)-1:cw*10]}));
		if	(serial_channels >=12) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_12	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*12)-1:tw*11],tx_datain_sig[(cw*12)-1:cw*11]}),.outclk(tx_clkout_sig[11]),.dataout({tx_ctrlenable_sig_txepcf[(tw*12)-1:tw*11],tx_datain_sig_txepcf[(cw*12)-1:cw*11]}));
		if	(serial_channels >=13) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_13	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*13)-1:tw*12],tx_datain_sig[(cw*13)-1:cw*12]}),.outclk(tx_clkout_sig[12]),.dataout({tx_ctrlenable_sig_txepcf[(tw*13)-1:tw*12],tx_datain_sig_txepcf[(cw*13)-1:cw*12]}));
		if	(serial_channels >=14) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_14	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*14)-1:tw*13],tx_datain_sig[(cw*14)-1:cw*13]}),.outclk(tx_clkout_sig[13]),.dataout({tx_ctrlenable_sig_txepcf[(tw*14)-1:tw*13],tx_datain_sig_txepcf[(cw*14)-1:cw*13]}));

	end    
endgenerate

generate 
	if	((component_type == "S4GX_HSM_XCVR_NO_ALTRECONFIG") && (serial_channels == 14))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;	
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	
		assign	busy_sig	=	0;

		altgx_s4gx_basic_x14_8p5G	altgx_s4gx_basic_x14_8p5G_inst
		(
			.rx_analogreset ( !xcvr_reset_pb_n), // ( rx_analogreset_altgx ),	
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( !xcvr_reset_pb_n ), // ( pll_powerdown_altgx ),
			.reconfig_clk ( 0 ),
			.reconfig_togxb ( 0 ),
			//.rx_coreclk ( {serial_channels{rx_pld_clk}} ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			//.tx_coreclk ( {serial_channels{tx_pld_clk}} ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb (  ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);



		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
		if	(serial_channels >=6 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_6 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*6 )-1:tw*5 ],tx_datain_sig[(cw*6 )-1:cw*5 ]}),.outclk(tx_clkout_sig[5 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*6 )-1:tw*5 ],tx_datain_sig_txepcf[(cw*6 )-1:cw*5 ]}));
		if	(serial_channels >=7 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_7 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*7 )-1:tw*6 ],tx_datain_sig[(cw*7 )-1:cw*6 ]}),.outclk(tx_clkout_sig[6 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*7 )-1:tw*6 ],tx_datain_sig_txepcf[(cw*7 )-1:cw*6 ]}));
		if	(serial_channels >=8 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_8 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*8 )-1:tw*7 ],tx_datain_sig[(cw*8 )-1:cw*7 ]}),.outclk(tx_clkout_sig[7 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*8 )-1:tw*7 ],tx_datain_sig_txepcf[(cw*8 )-1:cw*7 ]}));
		if	(serial_channels >=9 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_9 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*9 )-1:tw*8 ],tx_datain_sig[(cw*9 )-1:cw*8 ]}),.outclk(tx_clkout_sig[8 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*9 )-1:tw*8 ],tx_datain_sig_txepcf[(cw*9 )-1:cw*8 ]}));
		if	(serial_channels >=10) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_10	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*10)-1:tw*9 ],tx_datain_sig[(cw*10)-1:cw*9 ]}),.outclk(tx_clkout_sig[9 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*10)-1:tw*9 ],tx_datain_sig_txepcf[(cw*10)-1:cw*9 ]}));
		if	(serial_channels >=11) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_11	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*11)-1:tw*10],tx_datain_sig[(cw*11)-1:cw*10]}),.outclk(tx_clkout_sig[10]),.dataout({tx_ctrlenable_sig_txepcf[(tw*11)-1:tw*10],tx_datain_sig_txepcf[(cw*11)-1:cw*10]}));
		if	(serial_channels >=12) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_12	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*12)-1:tw*11],tx_datain_sig[(cw*12)-1:cw*11]}),.outclk(tx_clkout_sig[11]),.dataout({tx_ctrlenable_sig_txepcf[(tw*12)-1:tw*11],tx_datain_sig_txepcf[(cw*12)-1:cw*11]}));
		if	(serial_channels >=13) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_13	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*13)-1:tw*12],tx_datain_sig[(cw*13)-1:cw*12]}),.outclk(tx_clkout_sig[12]),.dataout({tx_ctrlenable_sig_txepcf[(tw*13)-1:tw*12],tx_datain_sig_txepcf[(cw*13)-1:cw*12]}));
		if	(serial_channels >=14) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_14	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*14)-1:tw*13],tx_datain_sig[(cw*14)-1:cw*13]}),.outclk(tx_clkout_sig[13]),.dataout({tx_ctrlenable_sig_txepcf[(tw*14)-1:tw*13],tx_datain_sig_txepcf[(cw*14)-1:cw*13]}));

	end    
endgenerate


// HSM x16 8.5 Gbps Basic Mode XCVR	
generate 
	if	((component_type == "S4GX_HSM_XCVR") && (serial_channels == 16))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;	
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			//.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_in		(rx_freqlocked_altgx | 
										{
											~avalon_dip[15],
											~avalon_dip[14],
											~avalon_dip[13],
											~avalon_dip[12],
											~avalon_dip[11],
											~avalon_dip[10],
											~avalon_dip[9],
											~avalon_dip[8],
											~avalon_dip[7],
											~avalon_dip[6],
											~avalon_dip[5],
											~avalon_dip[4],
											~avalon_dip[3],
											~avalon_dip[2],
											~avalon_dip[1],
											~avalon_dip[0]
										}),
			
			
			
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);

		altgx_s4gx_basic_x16_8p5G	altgx_s4gx_basic_x16_8p5G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),	
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			//.rx_coreclk ( {serial_channels{rx_pld_clk}} ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			//.tx_coreclk ( {serial_channels{tx_pld_clk}} ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);



		altgx_s4gx_xcvr_reconfig_x16	altgx_s4gx_xcvr_reconfig_x16_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
		if	(serial_channels >=6 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_6 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*6 )-1:tw*5 ],tx_datain_sig[(cw*6 )-1:cw*5 ]}),.outclk(tx_clkout_sig[5 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*6 )-1:tw*5 ],tx_datain_sig_txepcf[(cw*6 )-1:cw*5 ]}));
		if	(serial_channels >=7 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_7 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*7 )-1:tw*6 ],tx_datain_sig[(cw*7 )-1:cw*6 ]}),.outclk(tx_clkout_sig[6 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*7 )-1:tw*6 ],tx_datain_sig_txepcf[(cw*7 )-1:cw*6 ]}));
		if	(serial_channels >=8 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_8 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*8 )-1:tw*7 ],tx_datain_sig[(cw*8 )-1:cw*7 ]}),.outclk(tx_clkout_sig[7 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*8 )-1:tw*7 ],tx_datain_sig_txepcf[(cw*8 )-1:cw*7 ]}));
		if	(serial_channels >=9 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_9 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*9 )-1:tw*8 ],tx_datain_sig[(cw*9 )-1:cw*8 ]}),.outclk(tx_clkout_sig[8 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*9 )-1:tw*8 ],tx_datain_sig_txepcf[(cw*9 )-1:cw*8 ]}));
		if	(serial_channels >=10) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_10	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*10)-1:tw*9 ],tx_datain_sig[(cw*10)-1:cw*9 ]}),.outclk(tx_clkout_sig[9 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*10)-1:tw*9 ],tx_datain_sig_txepcf[(cw*10)-1:cw*9 ]}));
		if	(serial_channels >=11) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_11	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*11)-1:tw*10],tx_datain_sig[(cw*11)-1:cw*10]}),.outclk(tx_clkout_sig[10]),.dataout({tx_ctrlenable_sig_txepcf[(tw*11)-1:tw*10],tx_datain_sig_txepcf[(cw*11)-1:cw*10]}));
		if	(serial_channels >=12) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_12	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*12)-1:tw*11],tx_datain_sig[(cw*12)-1:cw*11]}),.outclk(tx_clkout_sig[11]),.dataout({tx_ctrlenable_sig_txepcf[(tw*12)-1:tw*11],tx_datain_sig_txepcf[(cw*12)-1:cw*11]}));
		if	(serial_channels >=13) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_13	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*13)-1:tw*12],tx_datain_sig[(cw*13)-1:cw*12]}),.outclk(tx_clkout_sig[12]),.dataout({tx_ctrlenable_sig_txepcf[(tw*13)-1:tw*12],tx_datain_sig_txepcf[(cw*13)-1:cw*12]}));
		if	(serial_channels >=14) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_14	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*14)-1:tw*13],tx_datain_sig[(cw*14)-1:cw*13]}),.outclk(tx_clkout_sig[13]),.dataout({tx_ctrlenable_sig_txepcf[(tw*14)-1:tw*13],tx_datain_sig_txepcf[(cw*14)-1:cw*13]}));
		if	(serial_channels >=15) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_15	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*15)-1:tw*14],tx_datain_sig[(cw*15)-1:cw*14]}),.outclk(tx_clkout_sig[14]),.dataout({tx_ctrlenable_sig_txepcf[(tw*15)-1:tw*14],tx_datain_sig_txepcf[(cw*15)-1:cw*14]}));
		if	(serial_channels >=16) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_16	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*16)-1:tw*15],tx_datain_sig[(cw*16)-1:cw*15]}),.outclk(tx_clkout_sig[15]),.dataout({tx_ctrlenable_sig_txepcf[(tw*16)-1:tw*15],tx_datain_sig_txepcf[(cw*16)-1:cw*15]}));
	
		assign	aci_edm_dip	=	{	// Mask out disabled channels
									{channel_width{~avalon_dip[15]}},
									{channel_width{~avalon_dip[14]}},
									{channel_width{~avalon_dip[13]}},
									{channel_width{~avalon_dip[12]}},
									{channel_width{~avalon_dip[11]}},
									{channel_width{~avalon_dip[10]}},
									{channel_width{~avalon_dip[9]}},
									{channel_width{~avalon_dip[8]}},
									{channel_width{~avalon_dip[7]}},
									{channel_width{~avalon_dip[6]}},
									{channel_width{~avalon_dip[5]}},
									{channel_width{~avalon_dip[4]}},
									{channel_width{~avalon_dip[3]}},
									{channel_width{~avalon_dip[2]}},
									{channel_width{~avalon_dip[1]}},
									{channel_width{~avalon_dip[0]}}
								};
								
		assign	aci_sdm_dip	=	{	// Mask out disabled channels
									{channel_width{~avalon_dip[15]}},
									{channel_width{~avalon_dip[14]}},
									{channel_width{~avalon_dip[13]}},
									{channel_width{~avalon_dip[12]}},
									{channel_width{~avalon_dip[11]}},
									{channel_width{~avalon_dip[10]}},
									{channel_width{~avalon_dip[9]}},
									{channel_width{~avalon_dip[8]}},
									{channel_width{~avalon_dip[7]}},
									{channel_width{~avalon_dip[6]}},
									{channel_width{~avalon_dip[5]}},
									{channel_width{~avalon_dip[4]}},
									{channel_width{~avalon_dip[3]}},
									{channel_width{~avalon_dip[2]}},
									{channel_width{~avalon_dip[1]}},
									{channel_width{~avalon_dip[0]}}
								};
	end
endgenerate


// HSM x16 8.5 Gbps Basic Mode XCVR	
generate 
	if	((component_type == "S4GX_HSM_XCVR_NO_ALTRECONFIG") && (serial_channels == 16))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;	
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			//.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_in		(rx_freqlocked_altgx | 
										{
											~avalon_dip[15],
											~avalon_dip[14],
											~avalon_dip[13],
											~avalon_dip[12],
											~avalon_dip[11],
											~avalon_dip[10],
											~avalon_dip[9],
											~avalon_dip[8],
											~avalon_dip[7],
											~avalon_dip[6],
											~avalon_dip[5],
											~avalon_dip[4],
											~avalon_dip[3],
											~avalon_dip[2],
											~avalon_dip[1],
											~avalon_dip[0]
										}),
			
			
			
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);

		assign	busy_sig	=	0;


		altgx_s4gx_basic_x16_8p5G	altgx_s4gx_basic_x16_8p5G_inst
		(
			.rx_analogreset ( !xcvr_reset_pb_n), // ( rx_analogreset_altgx ),	
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( !xcvr_reset_pb_n ), // ( pll_powerdown_altgx ),
			.reconfig_clk ( 0 ), // ( reconfig_clk_sig ),
			.reconfig_togxb ( 0 ), // ( reconfig_togxb_sig ),
			//.rx_coreclk ( {serial_channels{rx_pld_clk}} ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			//.tx_coreclk ( {serial_channels{tx_pld_clk}} ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( ), // ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);



		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
		if	(serial_channels >=6 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_6 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*6 )-1:tw*5 ],tx_datain_sig[(cw*6 )-1:cw*5 ]}),.outclk(tx_clkout_sig[5 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*6 )-1:tw*5 ],tx_datain_sig_txepcf[(cw*6 )-1:cw*5 ]}));
		if	(serial_channels >=7 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_7 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*7 )-1:tw*6 ],tx_datain_sig[(cw*7 )-1:cw*6 ]}),.outclk(tx_clkout_sig[6 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*7 )-1:tw*6 ],tx_datain_sig_txepcf[(cw*7 )-1:cw*6 ]}));
		if	(serial_channels >=8 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_8 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*8 )-1:tw*7 ],tx_datain_sig[(cw*8 )-1:cw*7 ]}),.outclk(tx_clkout_sig[7 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*8 )-1:tw*7 ],tx_datain_sig_txepcf[(cw*8 )-1:cw*7 ]}));
		if	(serial_channels >=9 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_9 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*9 )-1:tw*8 ],tx_datain_sig[(cw*9 )-1:cw*8 ]}),.outclk(tx_clkout_sig[8 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*9 )-1:tw*8 ],tx_datain_sig_txepcf[(cw*9 )-1:cw*8 ]}));
		if	(serial_channels >=10) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_10	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*10)-1:tw*9 ],tx_datain_sig[(cw*10)-1:cw*9 ]}),.outclk(tx_clkout_sig[9 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*10)-1:tw*9 ],tx_datain_sig_txepcf[(cw*10)-1:cw*9 ]}));
		if	(serial_channels >=11) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_11	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*11)-1:tw*10],tx_datain_sig[(cw*11)-1:cw*10]}),.outclk(tx_clkout_sig[10]),.dataout({tx_ctrlenable_sig_txepcf[(tw*11)-1:tw*10],tx_datain_sig_txepcf[(cw*11)-1:cw*10]}));
		if	(serial_channels >=12) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_12	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*12)-1:tw*11],tx_datain_sig[(cw*12)-1:cw*11]}),.outclk(tx_clkout_sig[11]),.dataout({tx_ctrlenable_sig_txepcf[(tw*12)-1:tw*11],tx_datain_sig_txepcf[(cw*12)-1:cw*11]}));
		if	(serial_channels >=13) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_13	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*13)-1:tw*12],tx_datain_sig[(cw*13)-1:cw*12]}),.outclk(tx_clkout_sig[12]),.dataout({tx_ctrlenable_sig_txepcf[(tw*13)-1:tw*12],tx_datain_sig_txepcf[(cw*13)-1:cw*12]}));
		if	(serial_channels >=14) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_14	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*14)-1:tw*13],tx_datain_sig[(cw*14)-1:cw*13]}),.outclk(tx_clkout_sig[13]),.dataout({tx_ctrlenable_sig_txepcf[(tw*14)-1:tw*13],tx_datain_sig_txepcf[(cw*14)-1:cw*13]}));
		if	(serial_channels >=15) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_15	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*15)-1:tw*14],tx_datain_sig[(cw*15)-1:cw*14]}),.outclk(tx_clkout_sig[14]),.dataout({tx_ctrlenable_sig_txepcf[(tw*15)-1:tw*14],tx_datain_sig_txepcf[(cw*15)-1:cw*14]}));
		if	(serial_channels >=16) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_16	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*16)-1:tw*15],tx_datain_sig[(cw*16)-1:cw*15]}),.outclk(tx_clkout_sig[15]),.dataout({tx_ctrlenable_sig_txepcf[(tw*16)-1:tw*15],tx_datain_sig_txepcf[(cw*16)-1:cw*15]}));
	
		assign	aci_edm_dip	=	{	// Mask out disabled channels
									{channel_width{~avalon_dip[15]}},
									{channel_width{~avalon_dip[14]}},
									{channel_width{~avalon_dip[13]}},
									{channel_width{~avalon_dip[12]}},
									{channel_width{~avalon_dip[11]}},
									{channel_width{~avalon_dip[10]}},
									{channel_width{~avalon_dip[9]}},
									{channel_width{~avalon_dip[8]}},
									{channel_width{~avalon_dip[7]}},
									{channel_width{~avalon_dip[6]}},
									{channel_width{~avalon_dip[5]}},
									{channel_width{~avalon_dip[4]}},
									{channel_width{~avalon_dip[3]}},
									{channel_width{~avalon_dip[2]}},
									{channel_width{~avalon_dip[1]}},
									{channel_width{~avalon_dip[0]}}
								};
								
		assign	aci_sdm_dip	=	{	// Mask out disabled channels
									{channel_width{~avalon_dip[15]}},
									{channel_width{~avalon_dip[14]}},
									{channel_width{~avalon_dip[13]}},
									{channel_width{~avalon_dip[12]}},
									{channel_width{~avalon_dip[11]}},
									{channel_width{~avalon_dip[10]}},
									{channel_width{~avalon_dip[9]}},
									{channel_width{~avalon_dip[8]}},
									{channel_width{~avalon_dip[7]}},
									{channel_width{~avalon_dip[6]}},
									{channel_width{~avalon_dip[5]}},
									{channel_width{~avalon_dip[4]}},
									{channel_width{~avalon_dip[3]}},
									{channel_width{~avalon_dip[2]}},
									{channel_width{~avalon_dip[1]}},
									{channel_width{~avalon_dip[0]}}
								};
	end
endgenerate


// HSM x10 8.5 Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR") && (serial_channels == 10))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;	
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);

		altgx_s4gx_basic_x10_8p5G	altgx_s4gx_basic_x10_8p5G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),	
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			//.rx_coreclk ( {serial_channels{rx_pld_clk}} ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			//.tx_coreclk ( {serial_channels{tx_pld_clk}} ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		//defparam altgx_s4gx_xcvr_reconfig_x10_inst.alt_system_interface = "true";
		altgx_s4gx_xcvr_reconfig_x10	altgx_s4gx_xcvr_reconfig_x10_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
		if	(serial_channels >=6 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_6 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*6 )-1:tw*5 ],tx_datain_sig[(cw*6 )-1:cw*5 ]}),.outclk(tx_clkout_sig[5 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*6 )-1:tw*5 ],tx_datain_sig_txepcf[(cw*6 )-1:cw*5 ]}));
		if	(serial_channels >=7 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_7 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*7 )-1:tw*6 ],tx_datain_sig[(cw*7 )-1:cw*6 ]}),.outclk(tx_clkout_sig[6 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*7 )-1:tw*6 ],tx_datain_sig_txepcf[(cw*7 )-1:cw*6 ]}));
		if	(serial_channels >=8 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_8 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*8 )-1:tw*7 ],tx_datain_sig[(cw*8 )-1:cw*7 ]}),.outclk(tx_clkout_sig[7 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*8 )-1:tw*7 ],tx_datain_sig_txepcf[(cw*8 )-1:cw*7 ]}));
		if	(serial_channels >=9 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_9 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*9 )-1:tw*8 ],tx_datain_sig[(cw*9 )-1:cw*8 ]}),.outclk(tx_clkout_sig[8 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*9 )-1:tw*8 ],tx_datain_sig_txepcf[(cw*9 )-1:cw*8 ]}));
		if	(serial_channels >=10) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_10	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*10)-1:tw*9 ],tx_datain_sig[(cw*10)-1:cw*9 ]}),.outclk(tx_clkout_sig[9 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*10)-1:tw*9 ],tx_datain_sig_txepcf[(cw*10)-1:cw*9 ]}));

	end    
endgenerate


// HSM x10 8.5 Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR_NO_ALTRECONFIG") && (serial_channels == 10))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;	
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
		
		assign	busy_sig	=	0;

		altgx_s4gx_basic_x10_8p5G	altgx_s4gx_basic_x10_8p5G_inst
		(
			.rx_analogreset ( !xcvr_reset_pb_n), // ( rx_analogreset_altgx ),	
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( !xcvr_reset_pb_n ), // ( pll_powerdown_altgx ),
			.reconfig_clk ( 0 ),
			.reconfig_togxb ( 0 ),
			//.rx_coreclk ( {serial_channels{rx_pld_clk}} ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			//.tx_coreclk ( {serial_channels{tx_pld_clk}} ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb (  ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
		if	(serial_channels >=6 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_6 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*6 )-1:tw*5 ],tx_datain_sig[(cw*6 )-1:cw*5 ]}),.outclk(tx_clkout_sig[5 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*6 )-1:tw*5 ],tx_datain_sig_txepcf[(cw*6 )-1:cw*5 ]}));
		if	(serial_channels >=7 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_7 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*7 )-1:tw*6 ],tx_datain_sig[(cw*7 )-1:cw*6 ]}),.outclk(tx_clkout_sig[6 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*7 )-1:tw*6 ],tx_datain_sig_txepcf[(cw*7 )-1:cw*6 ]}));
		if	(serial_channels >=8 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_8 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*8 )-1:tw*7 ],tx_datain_sig[(cw*8 )-1:cw*7 ]}),.outclk(tx_clkout_sig[7 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*8 )-1:tw*7 ],tx_datain_sig_txepcf[(cw*8 )-1:cw*7 ]}));
		if	(serial_channels >=9 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_9 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*9 )-1:tw*8 ],tx_datain_sig[(cw*9 )-1:cw*8 ]}),.outclk(tx_clkout_sig[8 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*9 )-1:tw*8 ],tx_datain_sig_txepcf[(cw*9 )-1:cw*8 ]}));
		if	(serial_channels >=10) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_10	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*10)-1:tw*9 ],tx_datain_sig[(cw*10)-1:cw*9 ]}),.outclk(tx_clkout_sig[9 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*10)-1:tw*9 ],tx_datain_sig_txepcf[(cw*10)-1:cw*9 ]}));

	end    
endgenerate


// HSM x4 ~6Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR_SATA_6G") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx | {~avalon_dip[3],~avalon_dip[2],~avalon_dip[1],~avalon_dip[0]}),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gx_basic_x4_SATA_6G	altgx_s4gx_basic_x4_SATA_6G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gx_xcvr_reconfig_x4	altgx_s4gx_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);
		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));

		assign	aci_edm_dip	=	{	// Mask out disabled channels
									{channel_width{~avalon_dip[3]}},
									{channel_width{~avalon_dip[2]}},
									{channel_width{~avalon_dip[1]}},
									{channel_width{~avalon_dip[0]}}
								};
								
		assign	aci_sdm_dip	=	{	// Mask out disabled channels
									{channel_width{~avalon_dip[3]}},
									{channel_width{~avalon_dip[2]}},
									{channel_width{~avalon_dip[1]}},
									{channel_width{~avalon_dip[0]}}
								};
	end
endgenerate

generate
	if	((component_type == "S4GX_HSM_XCVR") && (serial_channels == 6))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;	
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);

		//defparam altgx_s4gx_basic_x6_8p5G_inst.alt_system_interface = "true";
		altgx_s4gx_basic_x6_8p5G	altgx_s4gx_basic_x6_8p5G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),	
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			//.rx_coreclk ( {serial_channels{rx_pld_clk}} ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			//.tx_coreclk ( {serial_channels{tx_pld_clk}} ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);

		altgx_s4gx_xcvr_reconfig_x6	altgx_s4gx_xcvr_reconfig_x6_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
		if	(serial_channels >=6 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_6 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*6 )-1:tw*5 ],tx_datain_sig[(cw*6 )-1:cw*5 ]}),.outclk(tx_clkout_sig[5 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*6 )-1:tw*5 ],tx_datain_sig_txepcf[(cw*6 )-1:cw*5 ]}));

	end    
endgenerate

generate
	if	((component_type == "S4GX_HSM_XCVR_NO_ALTRECONFIG") && (serial_channels == 6))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;	
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
		
		assign	busy_sig	=	0;

		altgx_s4gx_basic_x6_8p5G	altgx_s4gx_basic_x6_8p5G_inst
		(
			.rx_analogreset ( !xcvr_reset_pb_n), // ( rx_analogreset_altgx ),	
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( !xcvr_reset_pb_n ), // ( pll_powerdown_altgx ),
			.reconfig_clk ( 0 ),
			.reconfig_togxb ( 0 ),
			//.rx_coreclk ( {serial_channels{rx_pld_clk}} ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			//.tx_coreclk ( {serial_channels{tx_pld_clk}} ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
		if	(serial_channels >=6 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_6 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*6 )-1:tw*5 ],tx_datain_sig[(cw*6 )-1:cw*5 ]}),.outclk(tx_clkout_sig[5 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*6 )-1:tw*5 ],tx_datain_sig_txepcf[(cw*6 )-1:cw*5 ]}));

	end    
endgenerate


// HSM x8 8.5 Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR") && (serial_channels == 8))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;	
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);

		altgx_s4gx_basic_x8_8p5G	altgx_s4gx_basic_x8_8p5G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),	
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			//.rx_coreclk ( {serial_channels{rx_pld_clk}} ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			//.tx_coreclk ( {serial_channels{tx_pld_clk}} ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);



		altgx_s4gx_xcvr_reconfig_x8	altgx_s4gx_xcvr_reconfig_x8_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
		if	(serial_channels >=6 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_6 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*6 )-1:tw*5 ],tx_datain_sig[(cw*6 )-1:cw*5 ]}),.outclk(tx_clkout_sig[5 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*6 )-1:tw*5 ],tx_datain_sig_txepcf[(cw*6 )-1:cw*5 ]}));
		if	(serial_channels >=7 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_7 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*7 )-1:tw*6 ],tx_datain_sig[(cw*7 )-1:cw*6 ]}),.outclk(tx_clkout_sig[6 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*7 )-1:tw*6 ],tx_datain_sig_txepcf[(cw*7 )-1:cw*6 ]}));
		if	(serial_channels >=8 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_8 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*8 )-1:tw*7 ],tx_datain_sig[(cw*8 )-1:cw*7 ]}),.outclk(tx_clkout_sig[7 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*8 )-1:tw*7 ],tx_datain_sig_txepcf[(cw*8 )-1:cw*7 ]}));
	

	end    
endgenerate

// HSM x8 8.5 Gbps Basic Mode XCVR		
// xN lines not supported in ES
generate 
	if	((component_type == "S4GX_HSM_XCVR_DISABLE") && (serial_channels == 8))
	begin  
		assign		tx_pld_clk			=	0;
		assign		rx_pld_clk			=	0;
		assign		rx_dataout_sig 		=	0;
		assign		pll_locked_sig 		=	{num_quads{1'b1}};
		assign		rx_freqlocked_sig	=	{serial_channels{1'b1}};
		assign		tx_p				=	0;
		assign		rx_clkout_sig[0]	=	0;
	end    
endgenerate



// HSM x4 8.5 Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gx_basic_x4_8p5G	altgx_s4gx_basic_x4_8p5G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gx_xcvr_reconfig_x4	altgx_s4gx_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
	
	end    
endgenerate

// HSM x4 8.5 Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR_NO_ALTRECONFIG") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		assign	busy_sig	=	1'b0;

		altgx_s4gx_basic_x4_8p5G	altgx_s4gx_basic_x4_8p5G_inst
		(
			.rx_analogreset ( !xcvr_reset_pb_n ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( !xcvr_reset_pb_n ),
			.reconfig_clk ( 0 ),
			.reconfig_togxb ( 0 ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
	
	end    
endgenerate



// For HSP P/N mismatch on S4GX IDA board
// HSM x4 8.5 Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR_PN_TWIST") && (serial_channels == 4))
	begin  
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
	 
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		// Compensate for P/N mistmatch by twisting the loopback cable
		// and inverting the deserialized result
		assign		rx_dataout_sig	=	~rx_dataout_sig_0;
		altgx_s4gx_basic_x4_8p5G	altgx_s4gx_basic_x4_8p5G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),  //( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig_0 ),
			.rx_freqlocked ( rx_freqlocked_altgx ), // ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gx_xcvr_reconfig_x4	altgx_s4gx_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
	
	end    
endgenerate



// HSM x2 8.5 Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR") && (serial_channels == 2))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		// Resets
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);

		//defparam altgx_s4gx_basic_x2_8p5G_inst.alt_system_interface = "true";
		altgx_s4gx_basic_x2_8p5G	altgx_s4gx_basic_x2_8p5G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( pll_powerdown_altgx ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gx_xcvr_reconfig_x2	altgx_s4gx_xcvr_reconfig_x2_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
	
	end    
endgenerate


// HSM x2 8.5 Gbps Basic Mode XCVR		
generate 
	if	((component_type == "S4GX_HSM_XCVR") && (serial_channels == 1))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		// Resets
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);

		altgx_s4gx_basic_x1_8p5G	altgx_s4gx_basic_x1_8p5G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( pll_powerdown_altgx ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);


		altgx_s4gx_xcvr_reconfig_x1	altgx_s4gx_xcvr_reconfig_x1_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
	
	end    
endgenerate
	
// HSM x2 8.5 Gbps Basic Mode XCVR	
	
generate 
	if	((component_type == "S4GX_HSM_XCVR_NO_ALTRECONFIG") && (serial_channels == 2))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		// Resets
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
		
		assign	busy_sig	=	0;

		altgx_s4gx_basic_x2_8p5G	altgx_s4gx_basic_x2_8p5G_inst
		(
			.rx_analogreset ( !xcvr_reset_pb_n), // ( rx_analogreset_altgx ),	
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( !xcvr_reset_pb_n ), // ( pll_powerdown_altgx ),
			.reconfig_clk ( 0 ), // ( reconfig_clk_sig ),
			.reconfig_togxb ( 0 ), // ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( ), // ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);

		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
	
	end    
endgenerate


	
generate 
	if	((component_type == "S4GX_TX_RX_XCVR") && (serial_channels == 1))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		// Resets
		
		// Note - review the reset controller and modify this section
		
		defparam	xcvr_reset_sequencer_bts_i.num_quads		=	num_quads;
		defparam	xcvr_reset_sequencer_bts_i.serial_channels	=	serial_channels;		
		xcvr_reset_sequencer_bts	xcvr_reset_sequencer_bts_i
		(
			.reset_n				(xcvr_reset_pb_n),
			.clk					(cal_blk_clk_sig),			
			.busy					(0), // (busy_sig),
			.pll_locked_in			(pll_locked_altgx),
			.pll_locked_out			(pll_locked_sig),
			.rx_freqlocked_in		(rx_freqlocked_altgx),
			.rx_freqlocked_out		(rx_freqlocked_sig),
			.pll_powerdown			(pll_powerdown_altgx),
			.rxanalog_reset			(rx_analogreset_altgx)
		);
		
	
/*
		altgx_s4gx_basic_x1_8p5G	altgx_s4gx_basic_x1_8p5G_inst
		(
			.rx_analogreset ( rx_analogreset_altgx ),
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( pll_powerdown_altgx ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);
*/

		altgx_s4gx_basic_x1_8p5G_tx	altgx_s4gx_basic_x1_8p5G_tx_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( pll_powerdown_altgx ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_sig ),
			.reconfig_clk ( 0 ),
			.reconfig_togxb ( 0 ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_altgx ),
			.reconfig_fromgxb ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p )
		);

		altgx_s4gx_basic_x1_8p5G_rx	altgx_s4gx_basic_x1_8p5G_rx_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( pll_powerdown_altgx ),
			.reconfig_clk ( 0 ),
			.reconfig_togxb ( 0 ),
			//.rx_analogreset ( rx_analogreset_altgx ),
			.rx_cruclk (  {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.reconfig_fromgxb ( ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_altgx ),
			.rx_pll_locked ( )
		);
		
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
	
	end    
endgenerate


// x4 SMA 8.5 Gbps Basic Mode XCVR:			
generate 
	if	((component_type == "S4GX_SMA_XCVR") && (serial_channels == 4))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gx_sma_xcvr_x4	altgx_s4gx_sma_xcvr_x4_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig_txepcf ),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.rx_rlv ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p ),
			.tx_coreclk	({serial_channels{tx_pld_clk}})
		);

		altgx_s4gx_xcvr_reconfig_x4	altgx_s4gx_xcvr_reconfig_x4_inst
		(
			.read ( read_sig ),
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig ),
			.rx_eqctrl ( rx_eqctrl_sig ),
			.rx_eqdcgain ( rx_eqdcgain_sig ),
			.tx_preemp_0t ( tx_preemp_0t_sig ),
			.tx_preemp_1t ( tx_preemp_1t_sig ),
			.tx_preemp_2t ( tx_preemp_2t_sig ),
			.tx_vodctrl ( tx_vodctrl_sig ),
			.write_all ( write_all_sig ),
			.busy ( busy_sig ),
			.data_valid ( data_valid_sig ),
			.error (  ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.rx_eqctrl_out ( rx_eqctrl_out_sig ),
			.rx_eqdcgain_out ( rx_eqdcgain_out_sig ),
			.tx_preemp_0t_out ( tx_preemp_0t_out_sig ),
			.tx_preemp_1t_out ( tx_preemp_1t_out_sig ),
			.tx_preemp_2t_out ( tx_preemp_2t_out_sig ),
			.tx_vodctrl_out ( tx_vodctrl_out_sig )
		);
  
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
	end 

endgenerate


// x1 4.25 Gbps Basic PMA Direct Mode XCVR:			
generate 
	if	((component_type == "S4GX_SMA_XCVR_PMA_DIRECT") && (serial_channels == 1))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		altgx_s4gx_sma_xcvr_pma_direct	altgx_s4gx_sma_xcvr_pma_direct_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.pll_powerdown ( pll_powerdown_sig ),
			//.reconfig_clk ( 0 ),
			//.reconfig_togxb ( 0 ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_seriallpbken ( seriallpbken ),
			.tx_datain ( tx_datain_sig ),
			.pll_locked ( pll_locked_sig ),
			//.reconfig_fromgxb ( ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_dataout ( rx_dataout_sig ),
			.rx_freqlocked ( rx_freqlocked_sig ),
			.rx_pll_locked ( ),
			.rx_rlv ( ),
			.tx_clkout ( tx_clkout_sig),
			.tx_dataout ( tx_p )
		);

	end    
endgenerate

// x6 6Gbps Basic Mode XCVRS with DPRIO:			
generate 
	if	((component_type == "XCVRS") && (serial_channels == 6))
	begin  
		assign		tx_pld_clk	=	tx_clkout_sig[0];		
		assign		rx_pld_clk	=	rx_clkout_sig[0];
		alt2gxb_reconfig_x6	alt2gxb_reconfig_x6_inst
		(
			.read				( read_sig				),
			.reconfig_clk		( reconfig_clk_sig		),
			.reconfig_fromgxb	( reconfig_fromgxb_sig	),
			.rx_eqctrl			( rx_eqctrl_sig			),
			.rx_eqdcgain		( rx_eqdcgain_sig		),
			.tx_preemp_0t		( tx_preemp_0t_sig		),
			.tx_preemp_1t		( tx_preemp_1t_sig		),
			.tx_preemp_2t		( tx_preemp_2t_sig		),
			.tx_vodctrl			( tx_vodctrl_sig		),
			.write_all			( write_all_sig			),
			.busy				( busy_sig				),
			.data_valid			( data_valid_sig		),
			.reconfig_togxb		( reconfig_togxb_sig	),
			.rx_eqctrl_out		( rx_eqctrl_out_sig		),
			.rx_eqdcgain_out	( rx_eqdcgain_out_sig	),
			.tx_preemp_0t_out	( tx_preemp_0t_out_sig	),
			.tx_preemp_1t_out	( tx_preemp_1t_out_sig	),
			.tx_preemp_2t_out	( tx_preemp_2t_out_sig	),
			.tx_vodctrl_out		( tx_vodctrl_out_sig	)
		);     
 
		alt2gxb_basic_x6_dprio	alt2gxb_basic_x6_dprio_inst
		(
			.cal_blk_clk ( cal_blk_clk_sig ),
			.cal_blk_powerdown ( cal_blk_powerdown_sig ),
			.gxb_enable ( gxb_enable_sig ),                                                 
			.gxb_powerdown ( gxb_powerdown_sig ),
			.pll_inclk ( refclk_p ),
			.rx_analogreset ( rx_analogreset_sig ),
			.rx_cruclk ( {serial_channels{refclk_p}} ),
			.rx_datain ( rx_p ),
			.rx_digitalreset ( {serial_channels{rx_digital_reset_sync_dly}} ),
			.rx_enapatternalign ( {serial_channels{en_cdet_sync}} ),
			.rx_locktodata ( rx_locktodata_sig ),
			.rx_locktorefclk ( rx_locktorefclk_sig ),
			.rx_seriallpbken (seriallpbken),
			.tx_ctrlenable ( tx_ctrlenable_sig_txepcf ),											
			.tx_datain (tx_datain_sig_txepcf),
			.tx_digitalreset ( {serial_channels{tx_digital_reset_sync_dly}} ),
			.pll_locked ( pll_locked_sig ),
			.rx_byteorderalignstatus ( ),
			.rx_clkout ( rx_clkout_sig ),
			.rx_ctrldetect ( ), 
			.rx_dataout ( rx_dataout_sig ),
			.rx_disperr ( ),
			.rx_errdetect ( ),
			.rx_freqlocked ( rx_freqlocked_sig ),
			.rx_patterndetect ( rx_patterndetect_sig ),
			.rx_pll_locked ( ),
			.rx_rlv ( ),
			//.rx_signaldetect ( ),
			.rx_syncstatus ( ),
			.tx_clkout ( tx_clkout_sig ),
			.tx_dataout ( tx_p ),
			// dprio
			.reconfig_clk ( reconfig_clk_sig ),
			.reconfig_togxb ( reconfig_togxb_sig ),
			.reconfig_fromgxb ( reconfig_fromgxb_sig )														
		);

		// Some versions of Quartus requires separate clock on each TX in Basic Mode:
		// Pass through data and control - Replace with a direct connect
		if	(serial_channels >=1 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_1 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*1 )-1:tw*0 ],tx_datain_sig[(cw*1 )-1:cw*0 ]}),.outclk(tx_clkout_sig[0 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*1 )-1:tw*0 ],tx_datain_sig_txepcf[(cw*1 )-1:cw*0 ]}));
		if	(serial_channels >=2 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_2 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*2 )-1:tw*1 ],tx_datain_sig[(cw*2 )-1:cw*1 ]}),.outclk(tx_clkout_sig[1 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*2 )-1:tw*1 ],tx_datain_sig_txepcf[(cw*2 )-1:cw*1 ]}));
		if	(serial_channels >=3 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_3 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*3 )-1:tw*2 ],tx_datain_sig[(cw*3 )-1:cw*2 ]}),.outclk(tx_clkout_sig[2 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*3 )-1:tw*2 ],tx_datain_sig_txepcf[(cw*3 )-1:cw*2 ]}));
		if	(serial_channels >=4 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_4 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*4 )-1:tw*3 ],tx_datain_sig[(cw*4 )-1:cw*3 ]}),.outclk(tx_clkout_sig[3 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*4 )-1:tw*3 ],tx_datain_sig_txepcf[(cw*4 )-1:cw*3 ]}));
		if	(serial_channels >=5 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_5 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*5 )-1:tw*4 ],tx_datain_sig[(cw*5 )-1:cw*4 ]}),.outclk(tx_clkout_sig[4 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*5 )-1:tw*4 ],tx_datain_sig_txepcf[(cw*5 )-1:cw*4 ]}));
		if	(serial_channels >=6 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_6 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*6 )-1:tw*5 ],tx_datain_sig[(cw*6 )-1:cw*5 ]}),.outclk(tx_clkout_sig[5 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*6 )-1:tw*5 ],tx_datain_sig_txepcf[(cw*6 )-1:cw*5 ]}));
		if	(serial_channels >=7 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_7 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*7 )-1:tw*6 ],tx_datain_sig[(cw*7 )-1:cw*6 ]}),.outclk(tx_clkout_sig[6 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*7 )-1:tw*6 ],tx_datain_sig_txepcf[(cw*7 )-1:cw*6 ]}));
		if	(serial_channels >=8 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_8 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*8 )-1:tw*7 ],tx_datain_sig[(cw*8 )-1:cw*7 ]}),.outclk(tx_clkout_sig[7 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*8 )-1:tw*7 ],tx_datain_sig_txepcf[(cw*8 )-1:cw*7 ]}));
		if	(serial_channels >=9 ) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_9 	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*9 )-1:tw*8 ],tx_datain_sig[(cw*9 )-1:cw*8 ]}),.outclk(tx_clkout_sig[8 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*9 )-1:tw*8 ],tx_datain_sig_txepcf[(cw*9 )-1:cw*8 ]}));
		if	(serial_channels >=10) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_10	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*10)-1:tw*9 ],tx_datain_sig[(cw*10)-1:cw*9 ]}),.outclk(tx_clkout_sig[9 ]),.dataout({tx_ctrlenable_sig_txepcf[(tw*10)-1:tw*9 ],tx_datain_sig_txepcf[(cw*10)-1:cw*9 ]}));
		if	(serial_channels >=11) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_11	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*11)-1:tw*10],tx_datain_sig[(cw*11)-1:cw*10]}),.outclk(tx_clkout_sig[10]),.dataout({tx_ctrlenable_sig_txepcf[(tw*11)-1:tw*10],tx_datain_sig_txepcf[(cw*11)-1:cw*10]}));
		if	(serial_channels >=12) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_12	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*12)-1:tw*11],tx_datain_sig[(cw*12)-1:cw*11]}),.outclk(tx_clkout_sig[11]),.dataout({tx_ctrlenable_sig_txepcf[(tw*12)-1:tw*11],tx_datain_sig_txepcf[(cw*12)-1:cw*11]}));
		if	(serial_channels >=13) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_13	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*13)-1:tw*12],tx_datain_sig[(cw*13)-1:cw*12]}),.outclk(tx_clkout_sig[12]),.dataout({tx_ctrlenable_sig_txepcf[(tw*13)-1:tw*12],tx_datain_sig_txepcf[(cw*13)-1:cw*12]}));
		if	(serial_channels >=14) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_14	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*14)-1:tw*13],tx_datain_sig[(cw*14)-1:cw*13]}),.outclk(tx_clkout_sig[13]),.dataout({tx_ctrlenable_sig_txepcf[(tw*14)-1:tw*13],tx_datain_sig_txepcf[(cw*14)-1:cw*13]}));
		if	(serial_channels >=15) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_15	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*15)-1:tw*14],tx_datain_sig[(cw*15)-1:cw*14]}),.outclk(tx_clkout_sig[14]),.dataout({tx_ctrlenable_sig_txepcf[(tw*15)-1:tw*14],tx_datain_sig_txepcf[(cw*15)-1:cw*14]}));
		if	(serial_channels >=16) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_16	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*16)-1:tw*15],tx_datain_sig[(cw*16)-1:cw*15]}),.outclk(tx_clkout_sig[15]),.dataout({tx_ctrlenable_sig_txepcf[(tw*16)-1:tw*15],tx_datain_sig_txepcf[(cw*16)-1:cw*15]}));
		if	(serial_channels >=17) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_17	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*17)-1:tw*16],tx_datain_sig[(cw*17)-1:cw*16]}),.outclk(tx_clkout_sig[16]),.dataout({tx_ctrlenable_sig_txepcf[(tw*17)-1:tw*16],tx_datain_sig_txepcf[(cw*17)-1:cw*16]}));
		if	(serial_channels >=18) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_18	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*18)-1:tw*17],tx_datain_sig[(cw*18)-1:cw*17]}),.outclk(tx_clkout_sig[17]),.dataout({tx_ctrlenable_sig_txepcf[(tw*18)-1:tw*17],tx_datain_sig_txepcf[(cw*18)-1:cw*17]}));
		if	(serial_channels >=19) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_19	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*19)-1:tw*18],tx_datain_sig[(cw*19)-1:cw*18]}),.outclk(tx_clkout_sig[18]),.dataout({tx_ctrlenable_sig_txepcf[(tw*19)-1:tw*18],tx_datain_sig_txepcf[(cw*19)-1:cw*18]}));
		if	(serial_channels >=20) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_20	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*20)-1:tw*19],tx_datain_sig[(cw*20)-1:cw*19]}),.outclk(tx_clkout_sig[19]),.dataout({tx_ctrlenable_sig_txepcf[(tw*20)-1:tw*19],tx_datain_sig_txepcf[(cw*20)-1:cw*19]}));
		if	(serial_channels >=21) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_21	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*21)-1:tw*20],tx_datain_sig[(cw*21)-1:cw*20]}),.outclk(tx_clkout_sig[20]),.dataout({tx_ctrlenable_sig_txepcf[(tw*21)-1:tw*20],tx_datain_sig_txepcf[(cw*21)-1:cw*20]}));
		if	(serial_channels >=22) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_22	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*22)-1:tw*21],tx_datain_sig[(cw*22)-1:cw*21]}),.outclk(tx_clkout_sig[21]),.dataout({tx_ctrlenable_sig_txepcf[(tw*22)-1:tw*21],tx_datain_sig_txepcf[(cw*22)-1:cw*21]}));
		if	(serial_channels >=23) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_23	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*23)-1:tw*22],tx_datain_sig[(cw*23)-1:cw*22]}),.outclk(tx_clkout_sig[22]),.dataout({tx_ctrlenable_sig_txepcf[(tw*23)-1:tw*22],tx_datain_sig_txepcf[(cw*23)-1:cw*22]}));
		if	(serial_channels >=24) phasecomp_fifo	#(cw+tw)	txe_phasecomp_fifo_24	(.reset_n(!tx_digital_reset_sync_dly),.inclk(tx_pld_clk),.datain({tx_ctrlenable_sig[(tw*24)-1:tw*23],tx_datain_sig[(cw*24)-1:cw*23]}),.outclk(tx_clkout_sig[23]),.dataout({tx_ctrlenable_sig_txepcf[(tw*24)-1:tw*23],tx_datain_sig_txepcf[(cw*24)-1:cw*23]}));
	end    
endgenerate

// LVDS SERDES:
wire	rx_ss_locked;
wire	rx_clkout;        
generate
	if	((component_type == "LVDS") && (serial_channels == 17))
    begin	
		assign		tx_pld_clk	=	refclk_p;		
		assign		rx_pld_clk	=	rx_clkout;
		ss_spi42_tx_17	ss_spi42_tx_17_i
		(                                      
			.tx_in			(tx_datain_sig),
			.tx_inclock		(tx_pld_clk),
			.tx_locked		(pll_locked_sig),
			.tx_out			(tx_p),
			.tx_outclock    (tx_ss_clk_out)
		); 			
		ss_spi42_rx_17	ss_spi42_rx_17_i
		(
			.rx_fifo_reset	({serial_channels{rx_digital_reset_sync_dly}}),	// Reset after rx is reset
			.rx_in			(rx_p),
			.rx_inclock		(rx_ss_clk_in),
			.rx_reset		({serial_channels{tx_digital_reset_sync_dly}}), // Reset rx after tx locks
			.rx_locked		(rx_ss_locked),
			.rx_out			(rx_dataout_sig),
			.rx_outclock	(rx_clkout)
		); 
		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate       

 generate
	if	((component_type == "LVDS_1_25G") && (serial_channels == 17))
    begin	
		assign		tx_pld_clk	=	refclk_p;		
		assign		rx_pld_clk	=	rx_clkout;
		ss_spi42_tx_x17_1_25G	ss_spi42_tx_x17_1_25G_i
		(                                      
			.tx_in			(tx_datain_sig),
			.tx_inclock		(tx_pld_clk),
			.tx_locked		(pll_locked_sig),
			.tx_out			(tx_p),
			.tx_outclock    (tx_ss_clk_out)
		); 			
		ss_spi42_rx_x17_1_25G	ss_spi42_rx_x17_1_25G_i
		(
			.rx_fifo_reset	({serial_channels{rx_digital_reset_sync_dly}}),	// Reset after rx is reset
			.rx_in			(rx_p),
			.rx_inclock		(rx_ss_clk_in),
			.rx_reset		({serial_channels{tx_digital_reset_sync_dly}}), // Reset rx after tx locks
			.rx_locked		(rx_ss_locked),
			.rx_out			(rx_dataout_sig),
			.rx_outclock	(rx_clkout),
			.rx_dpll_enable	({serial_channels{  rx_digital_reset_sync_dly  | avalon_dip[4] }}),
			.rx_dpll_hold	({serial_channels{(!rx_digital_reset_sync_dly) & avalon_dip[5] }})
		); 
		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate       
  
// For S4GX RDIMM Board
generate
	if	((component_type == "LVDS_1_6G") && (serial_channels == 16))
    begin	               
		assign		tx_pld_clk	=	tx_coreclock_sig;		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		ss_spi42_tx_x16_1_6G	ss_spi42_tx_x16_1_6G_inst
		(
			.tx_in 			( tx_datain_sig ),
			.tx_inclock 	( refclk_p ),
			.tx_coreclock 	( tx_coreclock_sig ),
			.tx_locked 		( pll_locked_sig ),
			.tx_out 		( tx_p ),
			.tx_outclock 	( tx_ss_clk_out )
		);
		
		ss_spi42_rx_x16_1_6G	ss_spi42_rx_x16_1_6G_inst
		(
			//.dpa_pll_recal 		( tx_digital_reset_sync_dly ),						// Reset after TX PLL locks
			//.rx_dpa_lock_reset 	( {serial_channels{tx_digital_reset_sync_dly}} ), 	// Reset after TX PLL locks
			.rx_fifo_reset 		({serial_channels{rx_digital_reset_sync_dly}}),		// Reset after TX then RX plls phase locks
			.rx_in 				( rx_p ),
			.rx_inclock 		( rx_ss_clk_in ),
			.rx_reset 			({serial_channels{tx_digital_reset_sync_dly}}), 	// Reset rx after tx locks
			.rx_dpa_locked 		( ),
			.rx_locked 			( rx_ss_locked ),
			.rx_out 			( rx_dataout_sig ),
			.rx_outclock 		( rx_clkout_sig[0] )
		);

		
		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate  
  
// For S4GX RDIMM Board
generate
	if	((component_type == "LVDS_1_6G_ES") && (serial_channels == 16))
    begin	               
		assign		tx_pld_clk	=	tx_coreclock_sig;		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		// TX same for ES & Production Device:
		//ss_spi42_tx_x16_1_6G_ES	ss_spi42_tx_x16_1_6G_ES_inst
		ss_spi42_tx_x16_1_6G	ss_spi42_tx_x16_1_6G_inst
		(
			.tx_in 			( tx_datain_sig ),
			.tx_inclock 	( refclk_p ),
			.tx_coreclock 	( tx_coreclock_sig ),
			.tx_locked 		( pll_locked_sig ),
			.tx_out 		( tx_p ),
			.tx_outclock 	( tx_ss_clk_out )
		);
		
		// Turn on calibration mode for ES device:
		ss_spi42_rx_x16_1_6G_ES	ss_spi42_rx_x16_1_6G__ES_inst
		(
			//.dpa_pll_recal 		( tx_digital_reset_sync_dly ),						// Reset after TX PLL locks
			//.rx_dpa_lock_reset 	( {serial_channels{tx_digital_reset_sync_dly}} ), 	// Reset after TX PLL locks
			.rx_fifo_reset 		({serial_channels{rx_digital_reset_sync_dly}}),		// Reset after TX then RX plls phase locks
			.rx_in 				( rx_p ),
			.rx_inclock 		( rx_ss_clk_in ),
			.rx_reset 			({serial_channels{tx_digital_reset_sync_dly}}), 	// Reset rx after tx locks
			.rx_dpa_locked 		( ),
			.rx_locked 			( rx_ss_locked ),
			.rx_out 			( rx_dataout_sig ),
			.rx_outclock 		( rx_clkout_sig[0] )
		);

		
		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate  


 wire	tx_coreclock_sig;
 generate
	if	((component_type == "LVDS_1_6G") && (serial_channels == 17))
    begin	               
		assign		tx_pld_clk	=	tx_coreclock_sig;		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		ss_spi42_tx_x17_1_6G	ss_spi42_tx_x17_1_6G_inst
		(
			.tx_in 			( tx_datain_sig ),
			.tx_inclock 	( refclk_p ),
			.tx_coreclock 	( tx_coreclock_sig ),
			.tx_locked 		( pll_locked_sig ),
			.tx_out 		( tx_p ),
			.tx_outclock 	( tx_ss_clk_out )
		);
		
		ss_spi42_rx_x17_1_6G	ss_spi42_rx_x17_1_6G_inst
		(
			//.dpa_pll_recal 		( tx_digital_reset_sync_dly ),						// Reset after TX PLL locks
			//.rx_dpa_lock_reset 	( {serial_channels{tx_digital_reset_sync_dly}} ), 	// Reset after TX PLL locks
			.rx_fifo_reset 		({serial_channels{rx_digital_reset_sync_dly}}),		// Reset after TX then RX plls phase locks
			.rx_in 				( rx_p ),
			.rx_inclock 		( rx_ss_clk_in ),
			.rx_reset 			({serial_channels{tx_digital_reset_sync_dly}}), 	// Reset rx after tx locks
			.rx_dpa_locked 		( ),
			.rx_locked 			( rx_ss_locked ),
			.rx_out 			( rx_dataout_sig ),
			.rx_outclock 		( rx_clkout_sig[0] )
		);

		
		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate   

// Production silicon, turn off calibration in the RX SERDES as is unneeded
 generate
	if	((component_type == "LVDS_1_6G_S4GX") && (serial_channels == 17))
    begin	               
		assign		tx_pld_clk	=	tx_coreclock_sig;		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		ss_spi42_tx_x17_1_6G	ss_spi42_tx_x17_1_6G_inst
		(
			.tx_in 			( tx_datain_sig ),
			.tx_inclock 	( refclk_p ),
			.tx_coreclock 	( tx_coreclock_sig ),
			.tx_locked 		( pll_locked_sig ),
			.tx_out 		( tx_p ),
			.tx_outclock 	( tx_ss_clk_out )
		);
		
		ss_spi42_rx_x17_1_6G_s4gx	ss_spi42_rx_x17_1_6G_s4gx_inst
		(
			//.dpa_pll_recal 		( tx_digital_reset_sync_dly ),						// Reset after TX PLL locks
			//.rx_dpa_lock_reset 	( {serial_channels{tx_digital_reset_sync_dly}} ), 	// Reset after TX PLL locks
			.rx_fifo_reset 		({serial_channels{rx_digital_reset_sync_dly}}),		// Reset after TX then RX plls phase locks
			.rx_in 				( rx_p ),
			.rx_inclock 		( rx_ss_clk_in ),
			.rx_reset 			({serial_channels{tx_digital_reset_sync_dly}}), 	// Reset rx after tx locks
			.rx_dpa_locked 		( ),
			.rx_locked 			( rx_ss_locked ),
			.rx_out 			( rx_dataout_sig ),
			.rx_outclock 		( rx_clkout_sig[0] )
		);

		
		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate   

generate
	if	((component_type == "LVDS_1_6G_S4E") && (serial_channels == 17))
    begin	               
		assign		tx_pld_clk	=	tx_coreclock_sig;		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		ss_spi42_tx_x17_1_6G_s4e	ss_spi42_tx_x17_1_6G_s4e_inst
		(
			.tx_in 			( tx_datain_sig ),
			.tx_inclock 	( refclk_p ),
			.tx_coreclock 	( tx_coreclock_sig ),
			.tx_locked 		( pll_locked_sig ),
			.tx_out 		( tx_p ),
			.tx_outclock 	( tx_ss_clk_out )
		);
		
		ss_spi42_rx_x17_1_6G_s4e	ss_spi42_rx_x17_1_6G_s4e_inst
		(
			//.dpa_pll_recal 		( tx_digital_reset_sync_dly ),						// Reset after TX PLL locks
			//.rx_dpa_lock_reset 	( {serial_channels{tx_digital_reset_sync_dly}} ), 	// Reset after TX PLL locks
			.rx_fifo_reset 		({serial_channels{rx_digital_reset_sync_dly}}),		// Reset after TX then RX plls phase locks
			.rx_in 				( rx_p ),
			.rx_inclock 		( rx_ss_clk_in ),
			.rx_reset 			({serial_channels{tx_digital_reset_sync_dly}}), 	// Reset rx after tx locks
			.rx_dpa_locked 		( ),
			.rx_locked 			( rx_ss_locked ),
			.rx_out 			( rx_dataout_sig ),
			.rx_outclock 		( rx_clkout_sig[0] )
		);

		
		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate   


 generate
	if	((component_type == "LVDS_A2GX") && (serial_channels == 17))
    begin	               
		assign		tx_pld_clk	=	tx_coreclock_sig;		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		ss_spi42_tx_x17_1G_a2gx	ss_spi42_tx_x17_1G_a2gx_inst
		(
			.tx_in 			( tx_datain_sig ),
			.tx_inclock 	( refclk_p ),
			.tx_coreclock 	( tx_coreclock_sig ),
			.tx_locked 		( pll_locked_sig ),
			.tx_out 		( tx_p ),
			.tx_outclock 	( tx_ss_clk_out )
		);
		
		ss_spi42_rx_x17_1G_a2gx	ss_spi42_rx_x17_1G_a2gx_inst
		(
			//.dpa_pll_recal 		( tx_digital_reset_sync_dly ),						// Reset after TX PLL locks
			//.rx_dpa_lock_reset 	( {serial_channels{tx_digital_reset_sync_dly}} ), 	// Reset after TX PLL locks
			.rx_fifo_reset 		({serial_channels{rx_digital_reset_sync_dly}}),		// Reset after TX then RX plls phase locks
			.rx_in 				( rx_p ),
			.rx_inclock 		( rx_ss_clk_in ),
			.rx_reset 			({serial_channels{tx_digital_reset_sync_dly}}), 	// Reset rx after tx locks
			.rx_dpa_locked 		( ),
			.rx_locked 			( rx_ss_locked ),
			.rx_out 			( rx_dataout_sig ),
			.rx_outclock 		( rx_clkout_sig[0] )
		);

		
		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate   



 generate
	if	((component_type == "LVDS_A2GX_1P25G") && (serial_channels == 17))
    begin	               
		assign		tx_pld_clk	=	tx_coreclock_sig;		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		ss_spi42_tx_x17_1p25G_a2gx	ss_spi42_tx_x17_1p25G_a2gx_inst
		(
			.tx_in 			( tx_datain_sig ),
			.tx_inclock 	( refclk_p ),
			.tx_coreclock 	( tx_coreclock_sig ),
			.tx_locked 		( pll_locked_sig ),
			.tx_out 		( tx_p ),
			.tx_outclock 	( tx_ss_clk_out )
		);
		
		ss_spi42_rx_x17_1p25G_a2gx	ss_spi42_rx_x17_1p25G_a2gx_inst
		(
			//.dpa_pll_recal 		( tx_digital_reset_sync_dly ),						// Reset after TX PLL locks
			//.rx_dpa_lock_reset 	( {serial_channels{tx_digital_reset_sync_dly}} ), 	// Reset after TX PLL locks
			.rx_fifo_reset 		({serial_channels{rx_digital_reset_sync_dly}}),		// Reset after TX then RX plls phase locks
			.rx_in 				( rx_p ),
			.rx_inclock 		( rx_ss_clk_in ),
			.rx_reset 			({serial_channels{tx_digital_reset_sync_dly}}), 	// Reset rx after tx locks
			.rx_dpa_locked 		( ),
			.rx_locked 			( rx_ss_locked ),
			.rx_out 			( rx_dataout_sig ),
			.rx_outclock 		( rx_clkout_sig[0] )
		);

		
		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate   

 generate
	if	((component_type == "LVDS_A2GX_1P05G") && (serial_channels == 17))
    begin	               
		assign		tx_pld_clk	=	tx_coreclock_sig;		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		ss_spi42_tx_x17_1p05g_a2gx	ss_spi42_tx_x17_1p05G_a2gx_inst
		(
			.tx_in 			( tx_datain_sig ),
			.tx_inclock 	( refclk_p ),
			.tx_coreclock 	( tx_coreclock_sig ),
			.tx_locked 		( pll_locked_sig ),
			.tx_out 		( tx_p ),
			.tx_outclock 	( tx_ss_clk_out )
		);
		
		ss_spi42_rx_x17_1p05g_a2gx	ss_spi42_rx_x17_1p05G_a2gx_inst
		(
			//.dpa_pll_recal 		( tx_digital_reset_sync_dly ),						// Reset after TX PLL locks
			//.rx_dpa_lock_reset 	( {serial_channels{tx_digital_reset_sync_dly}} ), 	// Reset after TX PLL locks
			.rx_fifo_reset 		({serial_channels{rx_digital_reset_sync_dly}}),		// Reset after TX then RX plls phase locks
			.rx_in 				( rx_p ),
			.rx_inclock 		( rx_ss_clk_in ),
			.rx_reset 			({serial_channels{tx_digital_reset_sync_dly}}), 	// Reset rx after tx locks
			.rx_dpa_locked 		( ),
			.rx_locked 			( rx_ss_locked ),
			.rx_out 			( rx_dataout_sig ),
			.rx_outclock 		( rx_clkout_sig[0] )
		);

		
		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate   

// Disable LVDS portion
 generate
	if	((component_type == "LVDS_1_6G_DISABLE") && (serial_channels == 17))
    begin
		assign	tx_pld_clk			=	0;
		assign	rx_pld_clk			=	0;
		assign	pll_locked_sig		=	0; 
		assign	tx_p				=	0;
		assign	tx_ss_clk_out		=	0;
		assign	rx_ss_locked		=	0;
		assign	rx_dataout_sig		=	0;
		assign	rx_clkout_sig[0]	=	0;
		assign	rx_freqlocked_sig	=	0;
	end
endgenerate   


 generate
	if	((component_type == "LVDS_1_6G") && (serial_channels == 8))
    begin	               
		assign		tx_pld_clk	=	tx_coreclock_sig;		
		assign		rx_pld_clk	=	rx_clkout_sig[0];

		ss_spi42_tx_x8_1_6G	ss_spi42_tx_x8_1_6G_inst
		(
			.tx_in 			( tx_datain_sig ),
			.tx_inclock 	( refclk_p ),
			.tx_coreclock 	( tx_coreclock_sig ),
			.tx_locked 		( pll_locked_sig ),
			.tx_out 		( tx_p ),
			.tx_outclock 	( tx_ss_clk_out )
		);
		
		ss_spi42_rx_x8_1_6G	ss_spi42_rx_x8_1_6G_inst
		(
			//.dpa_pll_recal 		( tx_digital_reset_sync_dly ),						// Reset after TX PLL locks
			//.rx_dpa_lock_reset 	( {serial_channels{tx_digital_reset_sync_dly}} ), 	// Reset after TX PLL locks
			.rx_fifo_reset 		({serial_channels{rx_digital_reset_sync_dly}}),		// Reset after TX then RX plls phase locks
			.rx_in 				( rx_p ),
			.rx_inclock 		( rx_ss_clk_in ),
			.rx_reset 			({serial_channels{tx_digital_reset_sync_dly}}), 	// Reset rx after tx locks
			.rx_dpa_locked 		( ),
			.rx_locked 			( rx_ss_locked ),
			.rx_out 			( rx_dataout_sig ),
			.rx_outclock 		( rx_clkout_sig[0] )
		);

		
		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate   



reg	[data_width-1:0]	tx_datain_rg;
reg	[data_width-1:0]	rx_dataout_rg;
wire	[data_width-1:0]	rx_out_sig;
generate
		//assign		tx_pld_clk	=	refclk_p;		
		//assign		rx_pld_clk	=	rx_clkout;
	if	((component_type == "LVDS_1_25G_EXTPLL") && (serial_channels == 17))
    begin	
		assign	rx_dataout_sig	=	rx_dataout_rg;		
	
		//assign		tx_pld_clk	=	refclk_p;		
		//assign		rx_pld_clk	=	rx_clkout;
		assign	tx_pld_clk	=	txrx_pld_clk;
		assign	rx_pld_clk	=	txrx_pld_clk;

		// LVDS SERDES with external PLL required rx parallel data to be registered:
		always	@(posedge rx_pld_clk or negedge reset_n)
		begin
			if	(!reset_n)
			begin
				rx_dataout_rg	<=	0;
			end else begin
				rx_dataout_rg	<=	rx_out_sig;
			end
		end
		
		// LVDS SERDES with external PLL required tx parallel data to be registered:
		always	@(posedge tx_pld_clk or negedge reset_n)
		begin
			if	(!reset_n)
			begin
				tx_datain_rg	<=	0;
			end else begin
				tx_datain_rg	<=	tx_datain_sig;
			end
		end
		
		pll_156_25MHz_to_1_25GHz_lvds_extpll	pll_156_25MHz_to_1_25GHz_lvds_extpll (
			.inclk0		( refclk_p ),
			.c0			( txrx_inclock ),	// High speed serial clock
			.c1			( txrx_enable ),	// Load pulse
			.c2			( txrx_pld_clk ),	// Low speed parallel clock
			.locked		( pll_locked_sig )
		);
				
	
		// NOTE -- rs_ss_locked, rx_ss_clk_in [forwarded parallel clock], and rx_outclock [rx_pld_clk] unavailable in this mode!
		assign	rx_ss_locked	= pll_locked_sig;

		ss_spi42_tx_x17_1_25G_extpll	ss_spi42_tx_x17_1_25G_extpll_inst (
			.tx_enable			( txrx_enable ),
			.tx_in				( tx_datain_rg ),
			.tx_inclock			( txrx_inclock ),
			.tx_out				( tx_p ),
			.tx_outclock		( tx_ss_clk_out )
		);
	
		ss_spi42_rx_x17_1_25G_extpll	ss_spi42_rx_x17_1_25G_extpll_inst (
			.rx_dpll_enable		( {serial_channels{  rx_digital_reset_sync_dly  | avalon_dip[4] }} ),
			.rx_dpll_hold		( {serial_channels{(!rx_digital_reset_sync_dly) & avalon_dip[5] }} ),
			.rx_enable			( txrx_enable ),
			.rx_fifo_reset		({serial_channels{rx_digital_reset_sync_dly}}),	// Reset after rx is reset
			.rx_in				( rx_p ),
			.rx_inclock			( txrx_inclock ),
			.rx_reset			({serial_channels{tx_digital_reset_sync_dly}}), // Reset rx after tx locks
			.rx_dpa_locked		(  ),
			.rx_out				( rx_out_sig )
		);

		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate   


/*
reg	[data_width-1:0]	tx_datain_rg;
reg	[data_width-1:0]	rx_dataout_rg;
wire	[data_width-1:0]	rx_out_sig;
generate
		//assign		tx_pld_clk	=	refclk_p;		
		//assign		rx_pld_clk	=	rx_clkout;
	if	((component_type == "LVDS_1_25G_EXTPLL") && (serial_channels == 17))
    begin	
		assign	rx_dataout_sig	=	rx_dataout_rg;		
	
		// LVDS SERDES with external PLL required rx parallel data to be registered:
		always	@(posedge rx_pld_clk or negedge reset_n)
		begin
			if	(!reset_n)
			begin
				rx_dataout_rg	<=	0;
			end else begin
				rx_dataout_rg	<=	rx_out_sig;
			end
		end
		
		// LVDS SERDES with external PLL required tx parallel data to be registered:
		always	@(posedge tx_pld_clk or negedge reset_n)
		begin
			if	(!reset_n)
			begin
				tx_datain_rg	<=	0;
			end else begin
				tx_datain_rg	<=	tx_datain_sig;
			end
		end
		
		pll_156_25MHz_to_1_25GHz_lvds_extpll	pll_156_25MHz_to_1_25GHz_lvds_extpll_tx (
			.inclk0		( refclk_p ),
			.c0			( tx_inclock ),	// High speed serial clock
			.c1			( tx_enable ),	// Load pulse
			.c2			( tx_pld_clk ),	// Low speed parallel clock
			.locked		( pll_locked_sig )
		);
		
		pll_156_25MHz_to_1_25GHz_lvds_extpll	pll_156_25MHz_to_1_25GHz_lvds_extpll_rx (
			.inclk0		( rx_ss_clk_in ),
			.c0			( rx_inclock ),	// High speed serial clock
			.c1			( rx_enable ),	// Load pulse
			.c2			( rx_pld_clk ),	// Low speed parallel clock
			.locked		( rx_ss_locked )
		);		
				
		ss_spi42_tx_x17_1_25G_extpll	ss_spi42_tx_x17_1_25G_extpll_inst (
			.tx_enable			( tx_enable ),
			.tx_in				( tx_datain_rg ),
			.tx_inclock			( tx_inclock ),
			.tx_out				( tx_p ),
			.tx_outclock		( tx_ss_clk_out )
		);
	
		ss_spi42_rx_x17_1_25G_extpll	ss_spi42_rx_x17_1_25G_extpll_inst (
			.rx_dpll_enable		( {serial_channels{  rx_digital_reset_sync_dly  | avalon_dip[4] }} ),
			.rx_dpll_hold		( {serial_channels{(!rx_digital_reset_sync_dly) & avalon_dip[5] }} ),
			.rx_enable			( rx_enable ),
			.rx_fifo_reset		({serial_channels{rx_digital_reset_sync_dly}}),	// Reset after rx is reset
			.rx_in				( rx_p ),
			.rx_inclock			( rx_inclock ),
			.rx_reset			({serial_channels{tx_digital_reset_sync_dly}}), // Reset rx after tx locks
			.rx_dpa_locked		(  ),
			.rx_out				( rx_out_sig )
		);

		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate 
*/

generate
	if	((component_type == "AGX_LVDS") && (serial_channels == 17))
    begin	
		assign		tx_pld_clk	=	refclk_p;		
		assign		rx_pld_clk	=	rx_clkout;
		ss_spi42_tx_17_840M	ss_spi42_tx_17_840M_i
		(                                      
			.tx_in			(tx_datain_sig),
			.tx_inclock		(tx_pld_clk),
			.tx_locked		(pll_locked_sig),
			.tx_out			(tx_p),
			.tx_outclock    (tx_ss_clk_out)
		); 			
		ss_spi42_rx_17_840M	ss_spi42_rx_17_840M_i
		(
			.rx_fifo_reset	({serial_channels{rx_digital_reset_sync_dly}}),	// Reset after rx is reset
			.rx_in			(rx_p),
			.rx_inclock		(rx_ss_clk_in),
			.rx_reset		({serial_channels{tx_digital_reset_sync_dly}}), // Reset rx after tx locks
			.rx_locked		(rx_ss_locked),
			.rx_out			(rx_dataout_sig),
			.rx_outclock	(rx_clkout)
		); 
		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate 


generate
	if	((component_type == "LVDS_C3LS") && (serial_channels == 17))
    begin	
		assign		tx_pld_clk	=	tx_coreclock_sig;		
		assign		rx_pld_clk	=	rx_clkout_sig[0];
		
		hsmc_lvds_c3ls_x17_tx	hsmc_lvds_c3ls_x17_tx_inst
		(
			.tx_in			( tx_datain_sig ),
			.tx_inclock		( refclk_p ),
			.tx_coreclock	( tx_coreclock_sig ),
			.tx_locked		( pll_locked_sig ),
			.tx_out			( tx_p ),
			.tx_outclock 	( tx_ss_clk_out )
		);

		hsmc_lvds_c3ls_x17_rx	hsmc_lvds_c3ls_x17_rx_inst
		(
			.rx_in ( rx_p ),
			.rx_inclock ( rx_ss_clk_in ),
			.rx_locked ( rx_ss_locked ),
			.rx_out ( rx_dataout_sig ),
			.rx_outclock ( rx_clkout_sig[0] )
		);

		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate 


generate
	if	((component_type == "C3_LVDS_SDI_3G") && (serdes_2d == 1) && (serial_channels == 5))
    begin	
		if	(enable_tx == 1)
		begin
			ss_sdi_tx_x5_cyclone3	ss_sdi_tx_x5_cyclone3_i
			(                                      
				.tx_in			(tx_datain_sig),
				.tx_inclock		(refclk_p),
				.tx_locked		(pll_locked_sig),
				.tx_out			(tx_p),
				.tx_outclock    (tx_ss_clk_out),
				.tx_coreclock	(tx_pld_clk)			
			); 			
		end else begin
			assign	pll_locked_sig	=	1;
			assign	tx_p			=	0;
			assign	tx_ss_clk_out	=	0;
			assign	tx_pld_clk		=	refclk_p;
		end
		if	(enable_rx == 1)
		begin
			ss_sdi_rx_x5_cyclone3	ss_sdi_rx_x5_cyclone3_i
			(
				.rx_in			(rx_p),
				.rx_inclock		(rx_ss_clk_in),
				.rx_locked		(rx_ss_locked),
				.rx_out			(rx_dataout_sig),
				.rx_outclock	(rx_clkout)
			); 
			assign	rx_pld_clk			=	rx_clkout;		
		end else begin
			assign	rx_ss_locked	=	1'b1;
			assign	rx_dataout_sig	=	0;
			assign	rx_clkout		=	refclk_p;
			assign	rx_pld_clk		=	rx_clkout;				
		end	
		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
		
	end
endgenerate 


generate
	if	((component_type == "C3_LVDS") && (serial_channels == 17))
    begin	
		assign		rx_pld_clk	=	rx_clkout;
		ss_spi42_tx_17_cyclone3	ss_spi42_tx_17_cyclone3_i
		(                                      
			.tx_in			(tx_datain_sig),
			.tx_inclock		(refclk_p),
			.tx_locked		(pll_locked_sig),
			.tx_out			(tx_p),
			.tx_outclock    (tx_ss_clk_out),
			.tx_coreclock	(tx_pld_clk)			
		); 			
		ss_spi42_rx_17_cyclone3	ss_spi42_rx_17_cyclone3_i
		(
			.rx_in			(rx_p),
			.rx_inclock		(rx_ss_clk_in),
			.rx_locked		(rx_ss_locked),
			.rx_out			(rx_dataout_sig),
			.rx_outclock	(rx_clkout)
		); 
		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate 




 generate
	if	((component_type == "c4gx_f896_host_hsmc_lvds_LVDS") && (serial_channels == 17))
    begin

	 	assign		tx_pld_clk	=	tx_coreclock_sig;		
		assign		rx_pld_clk	=	rx_clkout_sig[0];
		
		lvds_serdes_tx	lvds_serdes_tx_inst
		(
			.tx_in ( tx_datain_sig ),
			.tx_inclock ( refclk_p ),
			.tx_coreclock ( tx_coreclock_sig ),
			.tx_locked ( pll_locked_sig ),
			.tx_out ( tx_p ),
			.tx_outclock ( tx_ss_clk_out )
		);

		lvds_serdes_rx	lvds_serdes_rx_inst
		(
			.rx_data_align ( 0 ),
			.rx_in ( rx_p ),
			.rx_inclock ( rx_ss_clk_in ),
			.rx_locked ( rx_ss_locked ),
			.rx_out ( rx_dataout_sig ),
			.rx_outclock ( rx_clkout_sig[0] )
		);

		assign	rx_freqlocked_sig	=	{serial_channels{rx_ss_locked}}; // Can combine with dpa_locked signal if available
	end
endgenerate 




// Parallel Loopback
generate
	if	(component_type == "PARALLEL")
	begin	
		assign		tx_pld_clk			=	refclk_p;      				// Input + clock tree delay should match rx_p input delay
		assign		rx_pld_clk			=	rx_ss_clk_in;				// Can Use DDIO to match rx_p delay
		assign		pll_locked_sig		=	{num_quads{1'b1}};			// No TX CSU in parallel design
		assign		tx_p				=	tx_datain_sig;				// Use DDIO or fast registers
		assign		rx_dataout_sig		=	rx_p;						// Use fast registers in input (clocked by rx_ss_clk_in=rx_pld_clk)   
		assign		rx_freqlocked_sig	=	{serial_channels{1'b1}};	// No RX CDR in parallel design      

		//assign		tx_ss_clk_out		=	!refclk_p;              // Can use DDIO to match tx_p delay
		ddio_out_x1	ddio_out_clk
		(
			.datain_h(1'b0),	// Invert clock before outputting
			.datain_l(1'b1),
			.outclock(refclk_p),
			.dataout(tx_ss_clk_out)
		);
	end           
endgenerate

	always @(posedge tx_pld_clk)
	begin
		lf_clk	<=	!lf_clk;
	end

// Push Buttons Debouncers for XCVR
switch_debouncer	#(0,debounce_prd)	switch_debouncer_xcvr_fpga_en_cdet	(.reset_n(xcvr_reset_n),.clk(rx_pld_clk),.data_in(en_cdet),.data_out(en_cdet_sync));
switch_debouncer	#(0,debounce_prd)	switch_debouncer_xcvr_chn_bond		(.reset_n(xcvr_reset_n),.clk(rx_pld_clk),.data_in(channel_bond),.data_out(channel_bond_sync));
switch_debouncer	#(0,debounce_prd)	switch_debouncer_xcvr_bts_start		(.reset_n(xcvr_reset_n),.clk(tx_pld_clk),.data_in(start),.data_out(bts_start_sync));
switch_debouncer	#(0,debounce_prd)	switch_debouncer_xcvr_create_error	(.reset_n(xcvr_reset_n),.clk(tx_pld_clk),.data_in(create_error),.data_out(create_error_sync)); 
// 1st parameter enables positive edge detect
// 2nd parameter enables negative edge detect
// 3rd parameter is pulse polarity (1=positive, 0=negative)
// Creates single clock pulse from edge
edge_detector		#(1,0,1)			edge_detector_create_single_error(.reset_n(xcvr_reset_n),.clk(tx_pld_clk),.data_in(create_error_sync),.data_out(create_single_error)); // Single error event

// Metastability synchronizers for XCVR
// 1st parameter is synchonizer depth, 2nd parameter is the preset value
synchronizer	#(4,0)		synchonizer_pll_locked_sync	(.reset_n(1'b1),.clk(lf_clk),.data_in(plls_in_locked && tx_plls_locked),.data_out(tx_plls_locked_sync));
synchronizer	#(4,0)		synchonizer_rx_fifo_rst_n	(.reset_n(1'b1),.clk(lf_clk),.data_in(plls_in_locked && rx_tx_plls_locked),.data_out(rx_tx_plls_locked_sync));

/* XCVR Auto Reset (eg Allow PLLs to lock, then reset phase comp FIFOs) */
assign	tx_plls_locked		= &pll_locked_sig;
assign	rx_tx_plls_locked	= (&rx_freqlocked_sig) && (&pll_locked_sig);



// 1st parameter is the asserted value, 2nd is pulse width, 3rd is delay from rising edge on trig
// On rising edge of trig, a pulse is asserted, otherwise data_in passes thru.  On reset the asserted value passes thru. 
// Adds delay so PLLs can fully phase lock:
// Trig only assert once, so use + trigger
delayed_reset	#(1,delayed_rst_pw,delayed_rst_prd)	delayed_reset_txdigitalreset 	(.clk(lf_clk),.reset_n(xcvr_reset_n),.trig(tx_plls_locked_sync),.data_in(!tx_plls_locked_sync),.data_out(tx_digital_reset_sync_dly));
// Trig only assert once, so use + trigger
delayed_reset	#(1,delayed_rst_pw,delayed_rst_prd)	delayed_reset_rxdigitalreset	(.clk(lf_clk),.reset_n(xcvr_reset_n),.trig(rx_tx_plls_locked_sync),.data_in(!rx_tx_plls_locked_sync),.data_out(rx_digital_reset_sync_dly));
// Use negative edge of XCVR reset pulse to trigger next pulse                                                                                                                                                
delayed_reset	#(0,delayed_rst_pw,delayed_rst_prd)	delayed_reset_rxfifo_rst_n		(.clk(lf_clk),.reset_n(xcvr_reset_n),.trig(!rx_digital_reset_sync_dly),.data_in(!rx_digital_reset_sync_dly),.data_out(rx_fifos_rst_sync_dly_n));

parameter	enable_gdn_pdn				=	0;			// Set to 1 to enable dynamic ALTGX and PLL power down control from NIOS
parameter		dc_bits										=	2;
parameter		eq_bits										=	4;   
parameter		pre_bits									=	4;
parameter		vod_bits									=	3; 
parameter	altreconfig_offset	=	1;

parameter	serial_channels_ar	=	serial_channels * altreconfig_offset;
parameter	dc_width					=	serial_channels_ar * dc_bits;	
parameter	eq_width					=	serial_channels_ar * eq_bits;
parameter	pre_width				=	serial_channels_ar * pre_bits;
parameter	vod_width				=	serial_channels_ar * vod_bits; 




// Skew delay across serial channels:
parameter	xcvr_skew_num_barrels				=	10;														// >=2
parameter	xcvr_skew_num_barrels_m1_log2		=	4;														// // ceil(log2(skew_num_barrels-1)) (ceil means round up)
parameter	xcvr_data_skew_width				=	channel_width_log2 + xcvr_skew_num_barrels_m1_log2;		// >= ceil(log2(data channel width*(skew_num_barrels-1))) 

/*
// If using 8B/10B, must be multiple of 8 (can only delay to the next byte boundary):  
parameter	xcvr_data_skew_0					=	0;
parameter	xcvr_data_skew_1					=	8;  
parameter	xcvr_data_skew_2					=	16;
parameter	xcvr_data_skew_3					=	24;
parameter	xcvr_data_skew_4					=	32;     
parameter	xcvr_data_skew_5					=	40; 
wire		[(xcvr_data_skew_width*serial_channels)-1:0]						  xcvr_data_skew;
assign		xcvr_data_skew[(xcvr_data_skew_width*1)-1:xcvr_data_skew_width*0]	= xcvr_data_skew_0;
assign		xcvr_data_skew[(xcvr_data_skew_width*2)-1:xcvr_data_skew_width*1]	= xcvr_data_skew_1;
assign		xcvr_data_skew[(xcvr_data_skew_width*3)-1:xcvr_data_skew_width*2]	= xcvr_data_skew_2;
assign		xcvr_data_skew[(xcvr_data_skew_width*4)-1:xcvr_data_skew_width*3]	= xcvr_data_skew_3;
assign		xcvr_data_skew[(xcvr_data_skew_width*5)-1:xcvr_data_skew_width*4]	= xcvr_data_skew_4;
assign		xcvr_data_skew[(xcvr_data_skew_width*6)-1:xcvr_data_skew_width*5]	= xcvr_data_skew_5;   
*/
  
parameter	map_dprio				=	1;   
parameter	rx_fifo_enable			=	1;
parameter	word_aligner_enable		=	0;
parameter	channel_bond_enable		=	1;
parameter	convert_endian			=	0;      
parameter	flip_data_bits			=	0;  	
parameter	skew_msb_1st			=	0;	


// Special parameter to enable HSSDC2 wire twist on Stratix GX Development Kit:
defparam	xs_xcvr_wrapper_bts_i.stratix_gx_dev_kit_xcvrs = 0;          
// Top level xs_wrapper_bts:
defparam	xs_xcvr_wrapper_bts_i.rx_fifo_enable			=	rx_fifo_enable		;
defparam	xs_xcvr_wrapper_bts_i.word_aligner_enable		=	word_aligner_enable	;
defparam	xs_xcvr_wrapper_bts_i.channel_bond_enable		=	channel_bond_enable	;
defparam	xs_xcvr_wrapper_bts_i.convert_endian			=	convert_endian		;                   
defparam	xs_xcvr_wrapper_bts_i.flip_data_bits			=	flip_data_bits		;  						// Use if not done in XCVR (1 if MSB sent 1st (SONET), 0 lsb transmitted 1st (8B/10B)
defparam	xs_xcvr_wrapper_bts_i.num_endians				=	num_endians;    
defparam	xs_xcvr_wrapper_bts_i.cw						=	cw;					 		// Channel Width 
defparam	xs_xcvr_wrapper_bts_i.sync_pattern_width		=	sync_pattern_width;	 		// Not needed if word_aligner_enable=0 
defparam	xs_xcvr_wrapper_bts_i.mux_sel_width				=	mux_sel_width;				// Not needed if word_aligner_enable=0 
defparam	xs_xcvr_wrapper_bts_i.mux_sel_step				=	mux_sel_step;				// Set to 1 for word aligner &  byte ordering in core, set to deserializer_width for byte ordering only in core
defparam	xs_xcvr_wrapper_bts_i.word_align_pattern		=	word_align_pattern;			// Output by tx_wrapper_bts, and also used by word_aligner if word_aligner_enable=1
defparam	xs_xcvr_wrapper_bts_i.byte_order_pattern		=	byte_order_pattern;			// For byte re-ordering in the rx 
defparam	xs_xcvr_wrapper_bts_i.bonded_word_width			=	bonded_word_width;  								
// altera_bts:
defparam	xs_xcvr_wrapper_bts_i.nios_width				=	nios_width;		 			// For transaction counter
defparam	xs_xcvr_wrapper_bts_i.data_width				=	data_width; 				// Make equal to bonded_word_width!
defparam	xs_xcvr_wrapper_bts_i.prbs_depth				=	prbs_depth; 				// Use as seed for PRBS
defparam	xs_xcvr_wrapper_bts_i.prbs_seed_param			=	prbs_seed_param;
defparam	xs_xcvr_wrapper_bts_i.prbs_coeff_param			=	prbs_coeff_param;
defparam	xs_xcvr_wrapper_bts_i.prbs_type					=	prbs_type;
defparam	xs_xcvr_wrapper_bts_i.prbs_width				=	prbs_width;
// tx_wrapper_bts:
defparam	xs_xcvr_wrapper_bts_i.serial_channels			=	serial_channels;			// if 20 use hssdc twist	
defparam	xs_xcvr_wrapper_bts_i.tx_wrapper_mode			=	tx_wrapper_mode;			// After reset, truncate repeating pattern to this value  (0 for max count)
// channel_aligner:                                                             		
defparam	xs_xcvr_wrapper_bts_i.deskew_depth				=	deskew_depth;				// Used for channel bonder & in tx_wrapper_bts
defparam	xs_xcvr_wrapper_bts_i.deskew_ptr_width			= 	deskew_depth_log2;			// log2(deskew_depth)  -> Not needed if channel_bond_enable=0
// adj_skew_bts
defparam	xs_xcvr_wrapper_bts_i.skew_num_barrels			=	xcvr_skew_num_barrels;		// >= 2
defparam	xs_xcvr_wrapper_bts_i.channel_data_skew_width	=	xcvr_data_skew_width;		// >= ceil(log2(data channel width*(skew_num_barrels-1)))  
defparam	xs_xcvr_wrapper_bts_i.skew_msb_1st				=	skew_msb_1st;							// skew_msb_1st=1 (Source Synchronous LVDS are MSB 1st. skew_msb_1st=0 (XCVRS are LSB 1st) 
// misc                                                                         		
defparam	xs_xcvr_wrapper_bts_i.test_length_bts			=	test_length_bts;			// Number of data_gen_readdata to transmit before asserting testcomplete
defparam	xs_xcvr_wrapper_bts_i.test_complete_ctr_width	=	test_complete_ctr_width;	// > log2(test_length_bts)	  
defparam	xs_xcvr_wrapper_bts_i.bts_idle					=	bts_idle;
defparam	xs_xcvr_wrapper_bts_i.component_type			=	component_type;
defparam	xs_xcvr_wrapper_bts_i.serdes_2d					=	serdes_2d;					// Set to 2 levels of SERDES synchronization

xs_wrapper_bts	xs_xcvr_wrapper_bts_i
(
	.reset_n				    		(xcvr_reset_n), 
	.rx_fifos_rst_n						(rx_fifos_rst_sync_dly_n), 		// Not needed if rx_fifo_enable=0
	.tx_pld_clk							(tx_pld_clk),             	
	.tx_datain							(tx_datain_sig),           	
	.tx_ctrlenable					    (tx_ctrlenable_sig),	   		// Not needed unless using out of band signaling (eg K28.5 in 8B/10B)
	.tx_ready							(1'b1),                    	
	.rx_pld_clk							(rx_pld_clk),             	
	.rx_clkout			    			(rx_clkout_sig),		   		// Not needed if rx_fifo_enable=0
	.rx_patterndetect					(rx_patterndetect_sig),			// Not needed if enable_word_aligner=1 or enable_channel_bond = 0
	.rx_dataout							(rx_dataout_sig),            
	.rx_ready							(rx_ready),
	.en_cdet				    		(en_cdet_sync),			  	 	// Not needed if word_aligner_enable=0 
	.channel_bond						(channel_bond_sync),	  	 	// Not needed if channel_bond_enable=0
	.start								(bts_start_sync),
	.create_error						(create_single_error),
	.error_latch			    		(error_latch),
	.data_error							(data_error),
	.pattern_sync_acquired				(pattern_sync_acquired),
	.test_complete						(test_complete),
	// BTS System                                   
	.data_skew							(aci_tds),
	.error_mask                         (aci_edm | aci_edm_dip),  // 1st used in S2GX PCIE & Audio-Video Boards DPRIO Designs  
	.sync_mask                          (aci_sdm | aci_sdm_dip),  // 1st used in S2GX PCIE & Audio-Video Boards DPRIO Designs 
	.aci_prs                         	(aci_prs),
	.aci_prd                         	(aci_prd),
	.aci_prc                         	(aci_prc),
	.aci_prt							(aci_prt),
	.aci_cem                         	(aci_cem),
	.aci_ttc                         	(aci_ttc),
	.aci_rtc                         	(aci_rtc),
	.aci_wec                         	(aci_wec),
	.aci_lec                         	(aci_lec),
	.aci_ips						 	(aci_ips),
	.aci_cla						 	(aci_cla),
	.aci_cls							(aci_cls),
	.aci_pass						 	(aci_pass)												
);                            
 				                                                         
// NIOS System  
 
	// Avalon Control Interface      	                   	
parameter	nios_map_width			=	20;         
parameter	addrs_per_reg			=	256;
parameter	addrs_per_reg_log2		=	8;   
parameter	enable_nios_bts 		=	0;
parameter	tec_words				=	2;					// Transaction/Error Counter Nios words
parameter	num_lanes				=	serial_channels;	// Number of error lanes (eg 9 for 72 bit DDR2 with 9 byte lanes)
parameter	ssd_bits				=	8;					// Number of bits per SSD
parameter	num_ssds				=	4;					// Number of Seven Segment Display addresses
parameter	bts_id1					=	32'h0000_0000;
parameter	bts_id2					=	32'h0000_0001;
parameter	bts_id3					=	32'h0000_0002;
parameter	bts_id4					=	32'h0000_0003;
parameter	bts_id5					=	32'hffff_fffc;
parameter	bts_id6					=	32'hffff_fffd;
parameter	bts_id7					=	32'hffff_fffe;
parameter	bts_id8					=	32'hffff_ffff;
parameter	board_id	    		=	" Stratix II GX PCI Express Board";
parameter	port_id					=	"            HSMA x6 Transceivers";
parameter	generic_id				=	"  NIOS Board Test System Rev 1.0";
parameter	id_words	    		=	8;  
   
// 1 SSD per NIOS address.												
													
//wire	[nios_width-1:0]								aci_bcr;
wire	[prbs_width-1:0]								aci_prs;
wire	[nios_width-1:0]								aci_prd;
wire	[prbs_width-1:0]								aci_prc;
wire	[nios_width-1:0]								aci_prt;
wire	[bonded_word_width-1:0]							aci_edm;
wire	[bonded_word_width-1:0]							aci_sdm;    
wire	[bonded_word_width-1:0]							aci_edm_dip; // user dips control masking of errors
wire	[bonded_word_width-1:0]							aci_sdm_dip; // user dips control masking of sync   
wire	[bonded_word_width-1:0]							aci_cem; 
wire	[(xcvr_data_skew_width*serial_channels)-1:0]	aci_tds;
//wire	[:]												aci_xvc;
wire	[(nios_width*tec_words)-1:0]					aci_ttc;
wire	[(nios_width*tec_words)-1:0]					aci_rtc;
wire	[(nios_width*tec_words)-1:0]					aci_wec;   // Use wec for accurate system error rate
wire	[(nios_width*tec_words*num_lanes)-1:0]			aci_lec;   // Individual Lane Error Counter, width of earch error counter is nios_width
//wire	[nios_width-1:0]								aci_bio;
wire	[nios_width-1:0]								aci_pfa;
wire	[(num_ssds*ssd_bits)-1:0] 						aci_ssd;	// Number of seven segment displays                      	                		
wire													aci_cls;
wire													aci_cla;
wire													aci_pllr;
wire													aci_pllt;
wire													aci_pass;
wire													aci_eirq;
wire													aci_ips;   
wire													aci_gdn;
wire													aci_pdn;

defparam	avalon_control_interface_nios_bts.altreconfig_offset	=	altreconfig_offset;
defparam	avalon_control_interface_nios_bts.serial_channels_ar	=	serial_channels_ar;
defparam	avalon_control_interface_nios_bts.dc_width				=	dc_width;
defparam	avalon_control_interface_nios_bts.eq_width				=	eq_width;
defparam	avalon_control_interface_nios_bts.pre_width				=	pre_width;
defparam	avalon_control_interface_nios_bts.vod_width				=	vod_width;
defparam	avalon_control_interface_nios_bts.dc_bits					=	dc_bits;
defparam	avalon_control_interface_nios_bts.eq_bits					=	eq_bits;
defparam	avalon_control_interface_nios_bts.pre_bits					=	pre_bits;
defparam	avalon_control_interface_nios_bts.vod_bits					=	vod_bits;
defparam	avalon_control_interface_nios_bts.map_dprio 				=	map_dprio;
defparam	avalon_control_interface_nios_bts.serial_channels 			=	serial_channels; 
defparam	avalon_control_interface_nios_bts.channel_width				=	channel_width;      
defparam	avalon_control_interface_nios_bts.serial_channels 			=	serial_channels; 
defparam    avalon_control_interface_nios_bts.xcvr_data_skew_width 		=	xcvr_data_skew_width;
defparam    avalon_control_interface_nios_bts.tec_words 				=	tec_words;
defparam    avalon_control_interface_nios_bts.num_lanes 				=	num_lanes; // eg: Set to 9 for 72 bit DDR2 with 9 byte lanes                
defparam    avalon_control_interface_nios_bts.ssd_bits 					=	ssd_bits; 
defparam    avalon_control_interface_nios_bts.num_ssds 					=	num_ssds; 
defparam    avalon_control_interface_nios_bts.nios_width 				=	nios_width; 
// Default values for PRBS Generator
defparam	avalon_control_interface_nios_bts.prbs_depth_init			=	prbs_depth; 				// Use as seed for PRBS
defparam	avalon_control_interface_nios_bts.prbs_seed_init			=	prbs_seed_param;
defparam	avalon_control_interface_nios_bts.prbs_coeff_init			=	prbs_coeff_param;
defparam	avalon_control_interface_nios_bts.prbs_width				=	prbs_width;
defparam	avalon_control_interface_nios_bts.prbs_type_init			=	prt_init;
// Address map
defparam	avalon_control_interface_nios_bts.nios_map_width			=	nios_map_width;
defparam	avalon_control_interface_nios_bts.addrs_per_reg				=	addrs_per_reg;		
defparam	avalon_control_interface_nios_bts.addrs_per_reg_log2		=	addrs_per_reg_log2; 
// Board & Avalon Control Interface ID
defparam	avalon_control_interface_nios_bts.bts_id1					=	bts_id1;		
defparam	avalon_control_interface_nios_bts.bts_id2					=	bts_id2;		
defparam	avalon_control_interface_nios_bts.bts_id3					=	bts_id3;		
defparam	avalon_control_interface_nios_bts.bts_id4					=	bts_id4;		
defparam	avalon_control_interface_nios_bts.bts_id5					=	bts_id5;		
defparam	avalon_control_interface_nios_bts.bts_id6					=	bts_id6;		
defparam	avalon_control_interface_nios_bts.bts_id7					=	bts_id7;		
defparam	avalon_control_interface_nios_bts.bts_id8					=	bts_id8;		
defparam	avalon_control_interface_nios_bts.board_id	    			=	board_id;	
defparam	avalon_control_interface_nios_bts.port_id					=	port_id;		
defparam	avalon_control_interface_nios_bts.generic_id				=	generic_id;	
defparam	avalon_control_interface_nios_bts.id_words	    			=	id_words;	
defparam	avalon_control_interface_nios_bts.component_type				=	component_type;	   
avalon_control_interface_nios_bts avalon_control_interface_nios_bts
  (
      .avalon_channel_bond           (avalon_channel_bond		),
      .avalon_create_error           (avalon_create_error		),
      .avalon_dip                    (avalon_dip				),                    	
      .avalon_dprio_busy             (busy_sig					),
      .avalon_dprio_data_valid       (data_valid_sig			),
      .avalon_dprio_read             (read_sig					),
      .avalon_dprio_rx_eqctrl        (rx_eqctrl_sig				),
      .avalon_dprio_rx_eqctrl_out    (rx_eqctrl_out_sig			),
      .avalon_dprio_rx_eqdcgain      (rx_eqdcgain_sig			),
      .avalon_dprio_rx_eqdcgain_out  (rx_eqdcgain_out_sig		),
      .avalon_dprio_tx_preemp_0t     (tx_preemp_0t_sig    		),
      .avalon_dprio_tx_preemp_0t_out (tx_preemp_0t_out_sig		),   
      .avalon_dprio_tx_preemp_1t     (tx_preemp_1t_sig    		),
      .avalon_dprio_tx_preemp_1t_out (tx_preemp_1t_out_sig		),
      .avalon_dprio_tx_preemp_2t     (tx_preemp_2t_sig    		),
      .avalon_dprio_tx_preemp_2t_out (tx_preemp_2t_out_sig		),
      .avalon_dprio_tx_vodctrl       (tx_vodctrl_sig      		),
      .avalon_dprio_tx_vodctrl_out   (tx_vodctrl_out_sig  		),
      .avalon_dprio_write_all        (write_all_sig       		),
      .avalon_en_cdet                (avalon_en_cdet			),
      .avalon_error_latch            (error_latch				),
	  .avalon_data_error			 (data_error				),
      .avalon_led                    (avalon_led				),
      .avalon_pattern_sync_acquired  (pattern_sync_acquired		),
      .avalon_pb                     (avalon_pb					),
      .avalon_plls_locked            (plls_locked				),
      .avalon_reset_n                (avalon_reset_n			),  
      .avalon_seriallpbken           (avalon_seriallpbken		),
      .avalon_start                  (avalon_start				),
      .avalon_test_complete          (test_complete				),
	  .aci_pllt						 (aci_pllt					),
	  .aci_pllr						 (aci_pllr					),
	  .aci_ips						 (aci_ips					),
	.aci_gdn                                                  (aci_gdn                                      ),
	.aci_pdn	                                          (aci_pdn                                      ),
	  .aci_cla						 (aci_cla					),
	  .aci_cls						 (aci_cls					),
	  .aci_pass						 (aci_pass					),
	  .aci_eirq						 (aci_eirq					),
	  .aci_prs						 (aci_prs					),
	  .aci_prd						 (aci_prd					),
	  .aci_prc						 (aci_prc					),
	  .aci_prt						 (aci_prt					),
      .aci_edm						 (aci_edm					),
	  .aci_sdm						 (aci_sdm					),
	  .aci_cem						 (aci_cem					),
	  .aci_tds						 (aci_tds					),
	  .aci_ttc						 (aci_ttc					),
	  .aci_rtc						 (aci_rtc					),
	  .aci_wec						 (aci_wec					),
	  .aci_lec						 (aci_lec					),
	  .aci_ssd						 (aci_ssd					),
	  .aci_pfa						 (aci_pfa					),
      .nios_address                  (nios_address   			), 
      .nios_chipselect               (nios_chipselect			), 
      .nios_clk                      (nios_clk       			), 
      .nios_read                     (nios_read      			), 
      .nios_readdata                 (nios_readdata  			), 
      .nios_reset_n                  (nios_reset_n   			), 
      .nios_write                    (nios_write     			), 
      .nios_writedata                (nios_writedata 			),
	  .nios_waitrequest				 (nios_waitrequest			),
	  .tx_pld_clk					 (tx_pld_clk				),
	  .rx_pld_clk					 (rx_pld_clk				)
    );                                                                                                                                                                                                            

	// nios bts:
	defparam	xs_xcvr_wrapper_bts_i.enable_nios_bts	=	enable_nios_bts;
	defparam	xs_xcvr_wrapper_bts_i.tec_words			=	tec_words;					// Transaction/Error Counter Nios width words  
	defparam	xs_xcvr_wrapper_bts_i.num_lanes			=	num_lanes;   

 	
	assign	error_irq									=	0; // aci_eirq;   Disable interrupts for now by setting to 0       
	assign	aci_pllr									=	&rx_freqlocked_sig;
	assign	aci_pllt									=	&pll_locked_sig;
	wire	xcvr_reset_n								=	avalon_reset_n		&	xcvr_reset_pb_n;
	wire	en_cdet										=	avalon_en_cdet		|	en_cdet_pb;
	wire	channel_bond								=	avalon_channel_bond	|	channel_bond_pb;
	wire	start										=	avalon_start		|	start_pb;
	wire	create_error								=	avalon_create_error	|	create_error_pb;
	wire	[serial_channels-1:0]	seriallpbken		=	seriallpbken_dip	|	avalon_seriallpbken;
  
endmodule